!function(e){const n=e.bg=e.bg||{};n.dictionary=Object.assign(n.dictionary||{},{"Connected users":"Свързани потребители","Document outline":"Вид на документа","Enter fullscreen mode":"Вход в режим на цял екран","Fullscreen mode":"Режим на цял екран","Hide left sidebar":"Скрий лявата странична лента","Leave fullscreen mode":"Изход от режима на цял екран","Show left sidebar":"Покажи лявата странична лента","Toggle fullscreen mode":"Превключване на режима на цял екран","Toggle sidebar":"Превключи на странична лента"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));