!function(t){const i=t.fi=t.fi||{};i.dictionary=Object.assign(i.dictionary||{},{"Edit source":"Muokkaa lähdettä","Empty snippet content":"Tyhjä pätkäsisältö","HTML snippet":"HTML-pätkä","Insert HTML":"Liitä HTML","No preview available":"Esikatseluversiota ei ole saatavilla","Paste raw HTML here...":"Liitä raaka-HTML tähän...","Save changes":"Tallenna muutokset"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));