!function(i){const e=i.tr=i.tr||{};e.dictionary=Object.assign(e.dictionary||{},{"Bulleted List":"Simgeli Liste","Bulleted list styles toolbar":"Madde işaretli liste stilleri araç çubuğu",Circle:"Çember",Decimal:"Ondalık","Decimal with leading zero":"0'la başlayan ondalık","Decrease list item indent":"Liste ögesi girintisini azalt",Disc:"Disk","Entering a to-do list":"Yapılacaklar listesi girmek","Increase list item indent":"Liste ögesi girintisini artır","Invalid start index value.":"Geçersiz başlangıç ​​dizini değeri.","Keystrokes that can be used in a list":"Listedeki kullanılabilecek tuş vuruşları","Leaving a to-do list":"Yapılacaklar listesinden ayrılma","List properties":"Liste özellikleri","Lower-latin":"Küçük Latin harfleri","Lower–roman":"Küçük Roma rakamları","Numbered List":"Numaralı Liste","Numbered list styles toolbar":"Numaralandırılmış liste stilleri araç çubuğu","Reversed order":"Ters sıralama",Square:"Kare","Start at":"Şununla başla","Start index must be greater than 0.":"Başlangıç dizini 0'dan büyük olmalıdır.","To-do List":"Yapılacaklar Listesi","Toggle the circle list style":"Çember liste stilini değiştir","Toggle the decimal list style":"Ondalık liste stilini değiştir","Toggle the decimal with leading zero list style":"0'la başlayan ondalık liste stilini değiştir","Toggle the disc list style":"Disk liste stilini değiştir","Toggle the lower–latin list style":"Küçük Latin harfleri liste stilini değiştir","Toggle the lower–roman list style":"Küçük Roma rakamları liste stilini değiştir","Toggle the square list style":"Kare liste stilini değiştir","Toggle the upper–latin list style":"Büyük Latin harfleri liste stilini değiştir","Toggle the upper–roman list style":"Büyük Roma rakamları liste stilini değiştir","Upper-latin":"Büyük Latin harfleri","Upper-roman":"Büyük Roma rakamları"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));