{#
/**
 * @file
 * Default theme implementation to display a Drupal installation page.
 *
 * All available variables are mirrored in page.html.twig.
 * Some may be blank but they are provided for consistency.
 *
 * @see \Drupal\Core\Theme\ThemePreprocess::preprocessInstallPage()
 *
 * @ingroup themeable
 */
#}
  <div class="layout-container">

    <header role="banner">
      {% if site_name or site_slogan %}
        <div class="name-and-slogan">
          {% if site_name %}
            <h1>{{ site_name }}</h1>
          {% endif %}
          {% if site_slogan %}
            <div class="site-slogan">{{ site_slogan }}</div>
          {% endif %}
        </div>{# /.name-and-slogan #}
      {% endif %}
    </header>

    <main role="main">
      {% if title %}
        <h2>{{ title }}</h2>
      {% endif %}
      {{ page.highlighted }}
      {{ page.content }}
    </main>

    {% if page.sidebar_first %}
      <aside class="layout-sidebar-first" role="complementary">
        {{ page.sidebar_first }}
      </aside>{# /.layout-sidebar-first #}
    {% endif %}

    {% if page.sidebar_second %}
      <aside class="layout-sidebar-second" role="complementary">
        {{ page.sidebar_second }}
      </aside>{# /.layout-sidebar-second #}
    {% endif %}

    {% if page.footer %}
      <footer role="contentinfo">
        {{ page.footer }}
      </footer>
    {% endif %}

  </div>{# /.layout-container #}
