{#
/**
 * @file
 * Template for a generic layout.
 */
#}
{%
  set classes = [
    'layout',
    'layout--' ~ layout.id|clean_class,
  ]
%}
{% if content %}
  <div{{ attributes.addClass(classes) }}>
    {% for region in layout.getRegionNames %}
      {% if content[region] %}
        <div {{ region_attributes[region].addClass('layout__region', 'layout__region--' ~ region|clean_class) }}>
          {{ content[region] }}
        </div>
      {% endif %}
    {% endfor %}
  </div>
{% endif %}
