<?php

namespace Drupal\Core\Entity\Sql;

use Drupal\Core\Entity\EntityStorageException;

/**
 * Exception thrown when a SQL storage operation fails.
 */
class SqlContentEntityStorageException extends EntityStorageException {
}
