<div>link via the link generator: {{ link('register', test_url) }}</div>
<div>link via the link generator: {{ link('register', test_url, {'foo': 'bar'}) }}</div>
<div>link via the link generator: {{ link('register', test_url_attribute, {'id': 'kitten'}) }}</div>
<div>link via the link generator: {{ link('register', 'route:user.register', {'id': 'kitten'}) }}</div>
<div>link via the link generator: {{ link('register', 'route:user.register', attributes) }}</div>
{% set title %}<span>register</span>{% endset %}
<div>link via the link generator: {{ link(title, test_url) }}</div>
{% set title %}<span>register</span><svg></svg>{% endset %}
<div>link via the link generator: {{ link(title, test_url) }}</div>
<div>link via the link generator: {{ link('register', test_url_attribute) }} {{ link('register', test_url_attribute) }}</div>
