!function(e){const t=e.ug=e.ug||{};t.dictionary=Object.assign(t.dictionary||{},{"Bulleted List":"بەلگە تىزىملىك","Bulleted list styles toolbar":"تۈر بەلگە تىزىمى ئۇسلۇبىدىكى قورال بالداق",Circle:"چەمبەر",Decimal:"ئونلۇق كەسىر","Decimal with leading zero":"نۆل بىلەن باشلانغان ئونلۇق كەسىر","Decrease list item indent":"تىزىم تۈرىنى تارايتىشنى كېمەيتىدۇ",Disc:"ئۇيۇل چەمبەر","Entering a to-do list":"ئىش تىزىمىنى كىرگۈزۈۋاتىدۇ","Increase list item indent":"تىزىم تۈرىنى تارايتىشنى ئاشۇرىدۇ","Invalid start index value.":"ئىناۋەتسىز باشلىنىش ئىندېكىس قىممىتى","Keystrokes that can be used in a list":"تىزىمدا ئىشلەتكىلى بولىدىغان كۇنۇپكا بېسىلىشى","Leaving a to-do list":"ئىش تىزىمىدىن ئايرىلىۋاتىدۇ","List properties":"تىزىم خاسلىقى","Lower-latin":"كىچىك لاتىن رەقىمى","Lower–roman":"كىچىك رىم رەقىمى","Numbered List":"نومۇرلۇق تىزىملىك","Numbered list styles toolbar":"تۈر نومۇرى تىزىمى ئۇسلۇبىدىكى قورال بالداق","Reversed order":"ئەكسىچە تەرتىپ",Square:"چاسا","Start at":"باشلىنىشى","Start index must be greater than 0.":"باشلىنىدىغان ئىندېكىس قىممىتى چوقۇم 0 دىن چوڭ بولۇشى كېرەك.","To-do List":"ئىش تىزىمى","Toggle the circle list style":"بوش چەمبەر تىزىم ئۇسلۇبىغا ئالماشتۇر","Toggle the decimal list style":"ئونلۇق سان تىزىم ئۇسلۇبىغا ئالماشتۇر","Toggle the decimal with leading zero list style":"نۆل بىلەن باشلانغان ئونلۇق سان تىزىم ئۇسلۇبىغا ئالماشتۇر","Toggle the disc list style":"ئۇيۇل چەمبەر تىزىم ئۇسلۇبىغا ئالماشتۇر","Toggle the lower–latin list style":"كىچىك لاتىن رەقىمى تىزىم ئۇسلۇبىغا ئالماشتۇر","Toggle the lower–roman list style":"كىچىك رىم رەقىمى تىزىم ئۇسلۇبىغا ئالماشتۇر","Toggle the square list style":"ئۇيۇل كۋادرات تىزىم ئۇسلۇبىغا ئالماشتۇر","Toggle the upper–latin list style":"چوڭ لاتىن رەقىمى تىزىم ئۇسلۇبىغا ئالماشتۇر","Toggle the upper–roman list style":"چوڭ رىم رەقىمى تىزىم ئۇسلۇبىغا ئالماشتۇر","Upper-latin":"چوڭ لاتىن رەقىمى","Upper-roman":"چوڭ رىم رەقىمى"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));