!function(e){const l=e.ca=e.ca||{};l.dictionary=Object.assign(l.dictionary||{},{"Bulleted List":"Llista amb vinyetes","Bulleted list styles toolbar":"Barra d'eines d'estils de llista amb vinyetes",Circle:"Cercle",Decimal:"Decimal","Decimal with leading zero":"Decimal amb un zero al davant","Decrease list item indent":"Redueix el sagnat de l'element de la llista",Disc:"Disc","Entering a to-do list":"Introduint una llista de tasques pendents","Increase list item indent":"Augmenta el sagnat de l'element de la llista","Invalid start index value.":"Valor del número inicial no vàlid.","Keystrokes that can be used in a list":"Tecles que es poden emprar en una llista","Leaving a to-do list":"Sortint d'una llista de tasques pendents","List properties":"Llista de propietats","Lower-latin":"Lletres llatines en minúscules","Lower–roman":"Numerals romans en minúscules","Numbered List":"Llista numerada","Numbered list styles toolbar":"Barra d'eines d'estils de llista numerada","Reversed order":"Ordre invertit",Square:"Quadrat","Start at":"Començar a","Start index must be greater than 0.":"L'índex inicial ha de ser superior a 0.","To-do List":"Llista de tasques pendents","Toggle the circle list style":"Canviar l'estil de llista de cercles","Toggle the decimal list style":"Canviar l'estil de llista decimal","Toggle the decimal with leading zero list style":"Commutar l'estil de la llista de decimals amb un zero al davant","Toggle the disc list style":"Canviar l'estil de llista de discs","Toggle the lower–latin list style":"Canviar l'estil de la llista a lletres llatines en minúscules","Toggle the lower–roman list style":"Canviar l'estil de la llista a numerals romans en minúscules","Toggle the square list style":"Canviar l'estil de llista quadrada","Toggle the upper–latin list style":"Canviar l'estil de la llista a lletres llatines en majúscules","Toggle the upper–roman list style":"Canviar l'estil de la llista a numerals romans en majúscules","Upper-latin":"Lletres llatines en majúscules","Upper-roman":"Numerals romans en majúscules"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));