<?php

/**
 * @file
 * Generic transliteration data for the PhpTransliteration class.
 */

$base = [
  0x00 => 'tai', 'han', NULL, 'dian', 'ji', 'jie', NULL, 'zuan', NULL, 'xie', 'lai', 'fan', 'huo', 'xi', 'nie', 'mi',
  0x10 => 'ran', 'cuan', 'yin', 'mi', NULL, 'jue', 'qu', 'tong', 'wan', 'zhe', 'li', 'shao', 'kong', 'xian', 'zhe', 'zhi',
  0x20 => 'tiao', 'shu', 'bei', 'ye', 'pian', 'chan', 'hu', 'ken', 'jiu', 'an', 'chun', 'qian', 'bei', 'ba', 'fen', 'ke',
  0x30 => 'tuo', 'tuo', 'zuo', 'ling', NULL, 'gui', 'yan', 'shi', 'hou', 'lie', 'sha', 'si', NULL, 'bei', 'ren', 'du',
  0x40 => 'bo', 'liang', 'qian', 'fei', 'ji', 'zong', 'hui', 'he', 'li', 'yuan', 'yue', 'xiu', 'chan', 'di', 'lei', 'jin',
  0x50 => 'chong', 'si', 'pu', 'yao', 'jiang', 'huan', 'huan', 'tao', 'ru', 'weng', 'ying', 'rao', 'yin', 'shi', 'yin', 'jue',
  0x60 => 'tun', 'xuan', 'jia', 'zhong', 'qie', 'zhu', 'diao', NULL, 'you', NULL, NULL, 'yi', 'shi', 'yi', 'mo', NULL,
  0x70 => NULL, 'que', 'xiao', 'wu', 'geng', 'ying', 'ting', 'shi', 'ni', 'geng', 'ta', 'wo', 'ju', 'chan', 'piao', 'zhuo',
  0x80 => 'hu', 'nao', 'yan', 'gou', 'yu', 'hou', NULL, 'si', 'chi', 'hu', 'yang', 'weng', 'xian', 'pin', 'rong', 'lou',
  0x90 => 'lao', 'shan', 'xiao', 'ze', 'hai', 'fan', 'han', 'chan', 'zhan', NULL, 'ta', 'zhu', 'nong', 'han', 'yu', 'zhuo',
  0xA0 => 'you', 'li', 'huo', 'xi', 'xian', 'chan', 'lian', NULL, 'si', 'jiu', 'pu', 'qiu', 'gong', 'zi', 'yu', NULL,
  0xB0 => NULL, 'reng', 'niu', 'mei', 'ba', 'jiu', NULL, 'xu', 'ping', 'bian', 'mao', NULL, NULL, NULL, NULL, 'yi',
  0xC0 => 'yu', NULL, 'ping', 'qu', 'bao', 'hui', NULL, NULL, NULL, 'bu', 'mang', 'la', 'tu', 'wu', 'li', 'ling',
  0xD0 => NULL, 'ji', 'jun', 'zou', 'duo', 'jue', 'dai', 'bei', NULL, NULL, NULL, NULL, NULL, 'la', 'bin', 'sui',
  0xE0 => 'tu', 'xue', NULL, NULL, NULL, NULL, NULL, 'duo', NULL, NULL, 'sui', 'bi', 'tu', 'se', 'can', 'tu',
  0xF0 => 'mian', 'jin', 'lu', NULL, NULL, 'zhan', 'bi', 'ji', 'zen', 'xuan', 'li', NULL, NULL, 'sui', 'yong', 'shu',
];
