!function(e){const t=e.ja=e.ja||{};t.dictionary=Object.assign(t.dictionary||{},{"Insert media":"メディアの挿入",Media:"メディア","Media embed":"メディアの埋め込み","Media toolbar":"メディア","Media URL":"メディアURL","media widget":"メディアウィジェット","Open media in new tab":"新しいタブでメディアを開く","Paste the media URL in the input.":"URLを入力欄にコピー","The URL must not be empty.":"空のURLは許可されていません。","This media URL is not supported.":"このメディアのURLはサポートされていません。","Tip: Paste the URL into the content to embed faster.":"ヒント：より迅速に埋め込むには、コンテンツへURLを貼り付けてください。"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));