<?php

/**
 * @file
 * Generic transliteration data for the PhpTransliteration class.
 */

$base = [
  0x00 => 'ji', 'zhuo', 'lu', 'jun', 'xian', 'xi', 'cai', 'wen', 'zhi', 'zi', 'kun', 'cong', 'tian', 'chu', 'di', 'chun',
  0x10 => 'qiu', 'zhe', 'zha', 'rou', 'bin', 'ji', 'xi', 'zhu', 'jue', 'ge', 'ji', 'da', 'chen', 'suo', 'ruo', 'xiang',
  0x20 => 'huang', 'qi', 'zhu', 'sun', 'chai', 'weng', 'ke', 'kao', 'gu', 'gai', 'fan', 'cong', 'cao', 'zhi', 'chan', 'lei',
  0x30 => 'xiu', 'zhai', 'zhe', 'yu', 'gui', 'gong', 'zan', 'dan', 'huo', 'sou', 'tan', 'gu', 'xi', 'man', 'duo', 'ao',
  0x40 => 'pi', 'wu', 'ai', 'meng', 'pi', 'meng', 'yang', 'zhi', 'bo', 'ying', 'wei', 'rang', 'lan', 'yan', 'chan', 'quan',
  0x50 => 'zhen', 'pu', NULL, 'tai', 'fei', 'shu', NULL, 'dang', 'cuo', 'tan', 'tian', 'chi', 'ta', 'jia', 'shun', 'huang',
  0x60 => 'liao', NULL, NULL, 'chen', 'jin', 'e', 'gou', 'fu', 'duo', NULL, 'e', 'beng', 'tao', 'di', NULL, 'di',
  0x70 => 'bu', 'wan', 'zhao', 'lun', 'qi', 'mu', 'qian', NULL, 'zong', 'sou', NULL, 'you', 'zhou', 'ta', NULL, 'su',
  0x80 => 'bu', 'xi', 'jiang', 'cao', 'fu', 'teng', 'che', 'fu', 'fei', 'wu', 'xi', 'yang', 'ming', 'pang', 'mang', 'seng',
  0x90 => 'meng', 'cao', 'tiao', 'kai', 'bai', 'xiao', 'xin', 'qi', NULL, NULL, 'shao', 'huan', 'niu', 'xiao', 'chen', 'dan',
  0xA0 => 'feng', 'yin', 'ang', 'ran', 'ri', 'man', 'fan', 'qu', 'shi', 'he', 'bian', 'dai', 'mo', 'deng', NULL, NULL,
  0xB0 => 'kuang', NULL, 'cha', 'duo', 'you', 'hao', NULL, 'gua', 'xue', 'lei', 'jin', 'qi', 'qu', 'wang', 'yi', 'liao',
  0xC0 => NULL, NULL, 'yan', 'yi', 'yin', 'qi', 'zhe', 'xi', 'yi', 'ye', 'wu', 'zhi', 'zhi', 'han', 'chuo', 'fu',
  0xD0 => 'chun', 'ping', 'kuai', 'chou', NULL, 'tuo', 'qiong', 'cong', 'gao', 'kua', 'qu', 'qu', 'zhi', 'meng', 'li', 'zhou',
  0xE0 => 'ta', 'zhi', 'gu', 'liang', 'hu', 'la', 'dian', 'ci', 'ying', NULL, NULL, 'qi', NULL, 'cha', 'mao', 'du',
  0xF0 => 'yin', 'chai', 'rui', 'hen', 'ruan', 'fu', 'lai', 'xing', 'jian', 'yi', 'mei', NULL, 'mang', 'ji', 'suo', 'han',
];
