!function(o){const a=o.ti=o.ti||{};a.dictionary=Object.assign(a.dictionary||{},{Bookmark:"","Bookmark must not be empty.":"","Bookmark name":"","Bookmark name already exists.":"","Bookmark name cannot contain space characters.":"","Bookmark toolbar":"","bookmark widget":"",Bookmarks:"","Edit bookmark":"","Enter the bookmark name without spaces.":"","No bookmarks available.":"","Remove bookmark":"","Scroll to bookmark":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));