!function(e){const a=e.eu=e.eu||{};a.dictionary=Object.assign(a.dictionary||{},{"Break text":"","Caption for image: %0":"","Caption for the image":"","Centered image":"Zentratutako irudia","Change image text alternative":"Aldatu irudiaren ordezko testua",Custom:"","Custom image size":"","Enter image caption":"Sartu irudiaren epigrafea","Error during image upload":"","From computer":"","Full size image":"Tamaina osoko irudia",Image:"","Image from computer":"","Image Resize":"","Image resize list":"","Image toolbar":"","Image upload complete":"","Image via URL":"","image widget":"irudi widgeta","In line":"","Insert image":"Txertatu irudia","Insert image via URL":"","Insert via URL":"","Left aligned image":"Ezkerrean lerrokatutako irudia",Original:"","Replace from computer":"","Replace image":"","Replace image from computer":"","Resize image":"","Resize image (in %0)":"","Resize image to %0":"","Resize image to the original size":"","Right aligned image":"Eskuinean lerrokatutako irudia","Side image":"Alboko irudia","Text alternative":"Ordezko testua","Text Alternative":"","The value must not be empty.":"","The value should be a plain number.":"","Update image URL":"","Upload failed":"Kargatzeak huts egin du","Upload from computer":"","Upload image from computer":"","Uploading image":"","Via URL":"","Wrap text":"","You have no image upload permissions.":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));