!function(t){const e=t.ne=t.ne||{};e.dictionary=Object.assign(e.dictionary||{},{Bold:"बोल्ड","Bold text":"",Code:"कोड",Italic:"इटालिक","Italic text":"","Move out of an inline code style":"",Strikethrough:"स्ट्राइकथ्रू","Strikethrough text":"",Subscript:"सबस्क्रिप्ट",Superscript:"सुपरस्क्रिप्ट",Underline:"रेखांकन","Underline text":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));