{#
/**
 * @file
 * Default theme implementation for the message when fetching data fails.
 *
 * Available variables:
 * - error_message: A render array containing the appropriate error message.
 *
 * @see template_preprocess_update_fetch_error_message()
 *
 * @ingroup themeable
 */
#}
{{ error_message }}
