!function(e){const n=e.bn=e.bn||{};n.dictionary=Object.assign(n.dictionary||{},{"Connected users":"সংযুক্ত ব্যবহারকারী","Document outline":"ডকুমেন্ট আউটলাইন","Enter fullscreen mode":"ফুলস্ক্রীন মোডে প্রবেশ করুন","Fullscreen mode":"ফুলস্ক্রিন মোড","Hide left sidebar":"বাম সাইডবার লুকান","Leave fullscreen mode":"ফুলস্ক্রিন মোড ছেড়ে যান","Show left sidebar":"বাম সাইডবার দেখান","Toggle fullscreen mode":"ফুলস্ক্রিন মোড টগল করুন","Toggle sidebar":"সাইডবার টগল করুন"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));