!function(t){const i=t.tk=t.tk||{};i.dictionary=Object.assign(i.dictionary||{},{Bold:"Galyň","Bold text":"",Code:"Kod",Italic:"Italik","Italic text":"","Move out of an inline code style":"",Strikethrough:"Üsti çyzykly","Strikethrough text":"",Subscript:"Aşaky ýazgy",Superscript:"Üst ýazgy",Underline:"Aşagy çyzykly","Underline text":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));