!function(t){const i=t.ur=t.ur||{};i.dictionary=Object.assign(i.dictionary||{},{Bold:"جلّی","Bold text":"",Code:"کوڈ",Italic:"ترچھا","Italic text":"","Move out of an inline code style":"",Strikethrough:"خط کشیدہ","Strikethrough text":"",Subscript:"زير نوشت",Superscript:"بالا نوشت",Underline:"ترچھا","Underline text":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));