!function(e){const t=e.af=e.af||{};t.dictionary=Object.assign(t.dictionary||{},{Bold:"Vet","Bold text":"",Code:"Bronkode",Italic:"Kursief","Italic text":"","Move out of an inline code style":"",Strikethrough:"Deurstreep","Strikethrough text":"",Subscript:"Onderskrif",Superscript:"Boskrif",Underline:"Onderstreep","Underline text":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));