!function(o){const n=o.ne=o.ne||{};n.dictionary=Object.assign(n.dictionary||{},{Big:"ठूलो",Default:"पूर्वनिर्धारित","Document colors":"कागजात रंग","Font Background Color":"पृष्ठभूमिको फन्ट रंग","Font Color":"फन्ट रंग","Font Family":"फन्ट परिवार","Font Size":"फन्ट आकार",Huge:"विशाल",Small:"सानो",Tiny:"सानो"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));