!function(i){const n=i.sq=i.sq||{};n.dictionary=Object.assign(n.dictionary||{},{"Choose heading":"Përzgjidh nëntitullin",Heading:"Nëntitulli","Heading 1":"Nëntitulli 1","Heading 2":"Nëntitulli 2","Heading 3":"Nëntitulli 3","Heading 4":"Titulli 4","Heading 5":"Titulli 5","Heading 6":"Titulli 6","Type or paste your content here.":"Shkruaj ose kopjo përmbajtjen këtu.","Type your title":"Shkruani titullin tuaj"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));