!function(i){const e=i.sl=i.sl||{};e.dictionary=Object.assign(e.dictionary||{},{"Could not insert image at the current position.":"Slike ni mogoče vstaviti na trenutni položaj.","Could not obtain resized image URL.":"Ne morem pridobiti spremenjenega URL-ja slike.","Insert image or file":"Vstavi sliko ali datoteko","Inserting image failed":"Vstavljanje slike ni uspelo","Selecting resized image failed":"Izbira spremenjene slike ni uspela"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));