!function(e){const r=e.pt=e.pt||{};r.dictionary=Object.assign(r.dictionary||{},{"Connected users":"Utilizadores ligados","Document outline":"Esboço do documento","Enter fullscreen mode":"Entrar em modo de ecrã inteiro","Fullscreen mode":"Modo de ecrã inteiro","Hide left sidebar":"Ocultar a barra lateral esquerda","Leave fullscreen mode":"Sair de modo de ecrã inteiro","Show left sidebar":"Exibir a barra lateral esquerda","Toggle fullscreen mode":"Alternar modo de ecrã inteiro","Toggle sidebar":"Exibir/Ocultar a barra lateral"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));