!function(i){const t=i.lv=i.lv||{};t.dictionary=Object.assign(t.dictionary||{},{"Could not insert image at the current position.":"Pašreizējā pozīcijā attēlu nevarēja ievietot.","Could not obtain resized image URL.":"Nevarēja iegūt mērogotā attēla adresi.","Insert image or file":"Ievietot attēlu vai failu","Inserting image failed":"Nevarēja ievietot attēlu","Selecting resized image failed":"Nevarēja izvēlēties mērogoto attēlu."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));