!function(n){const e=n.ru=n.ru||{};e.dictionary=Object.assign(e.dictionary||{},{"Code block":"Блок кодов","Entering %0 code snippet":"Ввод фрагмента кода %0","Entering code snippet":"Ввод фрагмента кода","Insert code block":"Вставить код","Leaving %0 code snippet":"Оставляем фрагмент кода %0","Leaving code snippet":"Оставляем фрагмент кода","Plain text":"Простой текст"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));