<?php

declare(strict_types=1);

namespace Drupal\file_test\StreamWrapper;

/**
 * Helper class for testing the stream wrapper registry.
 *
 * Dummy stream wrapper implementation (dummy1://, dummy2://).
 */
class DummyMultipleStreamWrapper extends DummyStreamWrapper {}
