<div {{ attributes }}>
  <h2 class="component--my-card__header">{{ header }}</h2>
  <div class="component--my-card__body">
    {% block card_body %}
      Default contents for a card
    {% endblock %}
  </div>
</div>
