!function(i){const e=i.sq=i.sq||{};e.dictionary=Object.assign(e.dictionary||{},{"Disable block editing":"","Disable editing":"","Disable inline editing":"","Enable block editing":"","Enable editing":"Munëso redaktimin","Enable inline editing":"","Navigate editable regions":"Navigo rajonet e redaktueshme","Next editable region":"Rajoni i redaktueshëm i radhës","Previous editable region":"Rajoni paraprak i redaktueshëm"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));