!function(i){const e=i.he=i.he||{};e.dictionary=Object.assign(e.dictionary||{},{"Disable block editing":"ביטול אפשרות עריכת בלוקים","Disable editing":"הפוך עריכה ללא זמינה","Disable inline editing":"ביטול אפשרות עריכה בתוך השורה","Enable block editing":"אפשר עריכת בלוקים","Enable editing":"אפשר עריכה","Enable inline editing":"אפשר עריכה בתוך השורה","Navigate editable regions":"ניווט באזורים הניתנים לעריכה","Next editable region":"האזור הבא שניתן לעריכה","Previous editable region":"האזור הקודם שניתן לעריכה"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));