!function(e){const a=e["sr-latn"]=e["sr-latn"]||{};a.dictionary=Object.assign(a.dictionary||{},{"Break text":"Prelom teksta","Caption for image: %0":"Natpis za sliku:%0","Caption for the image":"Natpis za sliku","Centered image":"Slika u sredini","Change image text alternative":"Izmena alternativnog teksta",Custom:"Prilagodjen","Custom image size":"Prilagođena veličina slike","Enter image caption":"Odredi tekst ispod slike","Error during image upload":"Greška tokom otpremanja slike","From computer":"Sa kompjutera","Full size image":"Slika u punoj veličini",Image:"Slika","Image from computer":"Slika sa računara","Image Resize":"","Image resize list":"Lista veličine slike","Image toolbar":"Slika traka sa alatkama","Image upload complete":"Otpremanje slike je završeno","Image via URL":"Slika preko URL-a","image widget":"modul sa slikom","In line":"U redu","Insert image":"Dodaj sliku","Insert image via URL":"Ubaci sliku preko URL-a","Insert via URL":"Ubaci preko URL-a","Left aligned image":"Leva slika",Original:"Original","Replace from computer":"Zamenite sa računara","Replace image":"Zamenite sliku","Replace image from computer":"Zamenite sliku sa računara","Resize image":"Promenite veličinu slike","Resize image (in %0)":"Promenite veličinu slike (u %0)","Resize image to %0":"Promenite veličinu slike na% 0","Resize image to the original size":"Promenite veličinu slike do originalne veličine","Right aligned image":"Desna slika","Side image":"Bočna slika","Text alternative":"Alternativni tekst","Text Alternative":"","The value must not be empty.":"Vrednost ne sme biti prazna.","The value should be a plain number.":"Vrednost treba da bude običan broj.","Update image URL":"Ažuriraj URL slike","Upload failed":"Postavljanje neuspešno","Upload from computer":"Otpremite sa računara","Upload image from computer":"Otpremite sliku sa računara","Uploading image":"Otpremanje slike","Via URL":"Preko URL-a","Wrap text":"Prelomiti tekst","You have no image upload permissions.":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));