/*
 * DO NOT EDIT THIS FILE.
 * See the following change record for more information,
 * https://www.drupal.org/node/3084859
 * @preserve
 */

:is(#extra-specificity-hack, [data-drupal-admin-styles]) {
  &.admin-toolbar .layout-builder-block {
    padding: 0;
  }
}

.layout-builder.layout-builder--navigation {
  border: none;
}

/* Label when "content preview" is disabled. */

.navigation-layout .layout-builder-block__content-preview-placeholder-label {
  font-size: 0.79rem;
  overflow-wrap: anywhere;
}
