!function(i){const n=i.fi=i.fi||{};n.dictionary=Object.assign(n.dictionary||{},{Big:"Suuri",Default:"Oletus","Document colors":"Asiakirjan värit","Font Background Color":"Fontin taustaväri","Font Color":"Fontin väri","Font Family":"Fonttiperhe","Font Size":"Fontin koko",Huge:"Hyvin suuri",Small:"Pieni",Tiny:"Hyvin pieni"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));