{#
/**
 * @file
 * Default theme implementation for a local task link.
 *
 * Available variables:
 * - attributes: HTML attributes for the wrapper element.
 * - is_active: Whether the task item is an active tab.
 * - link: A rendered link element.
 *
 * Note: This template renders the content for each task item in
 * menu-local-tasks.html.twig.
 *
 * @see \Drupal\Core\Menu\MenuPreprocess::preprocessMenuLocalTask()
 *
 * @ingroup themeable
 */
#}
<li{{ attributes }}>{{ link }}</li>
