!function(e){const r=e.pt=e.pt||{};r.dictionary=Object.assign(r.dictionary||{},{"Blue marker":"Marcador azul","Green marker":"Marcador verde","Green pen":"Caneta verde",Highlight:"Realçar","Pink marker":"Marcador rosa","Red pen":"Caneta vermelha","Remove highlight":"Remover realce","Text highlight toolbar":"Barra de ferramentas de realces de texto","Yellow marker":"Marcador amarelo"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));