!function(i){const e=i.pt=i.pt||{};e.dictionary=Object.assign(e.dictionary||{},{"Could not insert image at the current position.":"Não é possível inserir a imagem na posição atual.","Could not obtain resized image URL.":"Não é possível obter o URL da imagem redimensionada.","Insert image or file":"Inserir imagem ou ficheiro","Inserting image failed":"A inserção da imagem falhou","Selecting resized image failed":"A seleção da imagem redimensionada falhou"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));