!function(e){const t=e.be=e.be||{};t.dictionary=Object.assign(t.dictionary||{},{"Bulleted List":"Маркіраваны спіс","Bulleted list styles toolbar":"Стылі маркіраваных спісаў",Circle:"Круг",Decimal:"Дзесятковы","Decimal with leading zero":"Дзесятковы з нулямі ў пачатку","Decrease list item indent":"Паменшыць адступ элемента спісу",Disc:"Дыск","Entering a to-do list":"Увод спісу задач","Increase list item indent":"Павялічыць адступ элемента спісу","Invalid start index value.":"Памылковае значэнне пачатковага індэкса.","Keystrokes that can be used in a list":"Спалучэнні клавіш, якія можна выкарыстоўваць у спісе","Leaving a to-do list":"Выхад з спісу задач","List properties":"Уласцівасці спіса","Lower-latin":"Малыя лацінскія","Lower–roman":"Малыя рымскія","Numbered List":"Нумарыроўваны спіс","Numbered list styles toolbar":"Стылі нумарыроўваных спісаў","Reversed order":"Адваротны парадак",Square:"Квадрат","Start at":"Пачаць з","Start index must be greater than 0.":"Пачатковы індэкс павінен быць большым за 0.","To-do List":"Спіс задач","Toggle the circle list style":"Пераключыць на стыль маркіраваных спісаў з кругам","Toggle the decimal list style":"Пераключыць на стыль маркіраваных спісаў з лікамі","Toggle the decimal with leading zero list style":"Пераключыць стыль спісу з дзесятковым лікам, які пачынаецца з нуля","Toggle the disc list style":"Пераключыць на стыль маркіраваных спісаў з дыскам","Toggle the lower–latin list style":"Пераключыць на стыль маркіраваных спісаў з малымі лацінскімі лікамі","Toggle the lower–roman list style":"Пераключыць на стыль маркіраваных спісаў з малымі рымскімі лікамі","Toggle the square list style":"Пераключыць на стыль маркіраваных спісаў з квадратам","Toggle the upper–latin list style":"Пераключыць на стыль маркіраваных спісаў з вялікімі лацінскімі лікамі","Toggle the upper–roman list style":"Пераключыць на стыль маркіраваных спісаў з вялікімі рымскімі лікамі","Upper-latin":"Вялікія лацінскія","Upper-roman":"Вялікія рымскія"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));