{#
/**
 * @file
 * Theme override for testing the presence of all user data.
 *
 * This template is used when viewing a registered user's page,
 * e.g., example.com/user/123. 123 being the user's ID.
 *
 * Available variables:
 * - content: A list of content items. Use 'content' to print all content, or
 *   print a subset such as 'content.field_example'. Fields attached to a user
 *   such as 'user_picture' are available as 'content.user_picture'.
 * - attributes: HTML attributes for the container element.
 * - user: A Drupal User entity.
 *
 * @see \Drupal\user\UserThemeHooks::preprocessUser()
 */
#}
<article{{ attributes.addClass('profile') }}>
  {% if content %}
    {{ content }}
  {% endif %}
  {% if user %}
    <p>{{ user.mail.value }}</p>
  {% endif  %}
</article>
