!function(e){const a=e.id=e.id||{};a.dictionary=Object.assign(a.dictionary||{},{"Align cell text to the bottom":"Sejajarkan teks sel ke bawah","Align cell text to the center":"Sejajarkan teks sel ke tengah","Align cell text to the left":"Sejajarkan teks sel ke kiri","Align cell text to the middle":"Sejajarkan teks sel ke tengah","Align cell text to the right":"Sejajarkan teks sel ke kanan","Align cell text to the top":"Sejajarkan teks sel ke atas","Align table to the left":"Sejajarkan teks sel ke kiri","Align table to the left with no text wrapping":"Ratakan tabel ke kiri tanpa pembungkusan teks","Align table to the left with text wrapping":"Ratakan tabel ke kiri dengan pembungkusan teks","Align table to the right":"Sejajarkan teks sel ke kanan","Align table to the right with no text wrapping":"Ratakan tabel ke kanan tanpa pembungkusan teks","Align table to the right with text wrapping":"Ratakan tabel ke kanan dengan pembungkusan teks",Alignment:"Penjajaran",Background:"Latar belakang",Border:"Garis batas","Cell properties":"Properti sel","Center table":"Tengahkan tabel","Center table with no text wrapping":"Pusatkan tabel tanpa pembungkusan teks","Choose table type":"Pilih jenis tabel",Color:"Warna",Column:"Kolom","Content table":"Tabel konten",Dashed:"Garis putus-putus","Delete column":"Hapus kolom","Delete row":"Hapus baris",Dimensions:"Dimensi",Dotted:"Titik titik",Double:"Ganda","Enter table caption":"Masukkan keterangan tabel",Groove:"Groove","Header column":"Kolom tajuk","Header row":"Baris tajuk",Height:"Tinggi","Horizontal text alignment toolbar":"Bilah alat penjajaran teks horizontal","Insert a new table row (when in the last cell of a table)":"Sisipkan baris tabel baru (saat berada di sel terakhir tabel)","Insert column left":"Sisipkan kolom ke kiri","Insert column right":"Sisipkan kolom ke kanan","Insert row above":"Sisipkan baris ke atas","Insert row below":"Sisipkan baris ke bawah","Insert table":"Sisipkan tabel","Insert table layout":"Sisipkan tata letak tabel",Inset:"Inset","Justify cell text":"Ratakan teks sel","Keystrokes that can be used in a table cell":"Penekanan tombol yang dapat digunakan di sel tabel","Layout table":"Tabel tata letak","Merge cell down":"Gabungkan sel ke bawah","Merge cell left":"Gabungkan sel ke kiri","Merge cell right":"Gabungkan sel ke kanan","Merge cell up":"Gabungkan sel ke atas","Merge cells":"Gabungkan sel","Move the selection to the next cell":"Pindahkan pilihan ke sel berikutnya","Move the selection to the previous cell":"Pindahkan pilihan ke sel sebelumnya","Navigate through the table":"Menjelajahi tabel",None:"Tidak ada",Outset:"Outset",Padding:"Padding",Ridge:"Ridge",Row:"Baris","Select column":"Seleksi kolom","Select row":"Seleksi baris",Solid:"Garis utuh","Split cell horizontally":"Bagikan sel secara horizontal","Split cell vertically":"Bagikan sel secara vertikal",Style:"Gaya",Table:"Tabel","Table Alignment":"Perataan Tabel","Table alignment toolbar":"Bilah alat penjajaran tabel","Table cell text alignment":"Penjajaran teks sel tabel","Table layout":"Tata letak tabel","Table properties":"Properti tabel","Table toolbar":"Alat tabel","Table type":"Jenis tabel","Table type options":"Opsi jenis tabel",'The color is invalid. Try "#FF0000" or "rgb(255,0,0)" or "red".':'Warna tidak valid. Coba "#FF0000" atau "rgb(255,0,0)" atau "red".','The value is invalid. Try "10px" or "2em" or simply "2".':'Nilai tidak valid. Coba "10px" atau "2em" atau hanya "2".',"Vertical text alignment toolbar":"Bilah alat penjajaran teks vertikal",Width:"Lebar"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));