!function(e){const t=e.ja=e.ja||{};t.dictionary=Object.assign(t.dictionary||{},{"Align cell text to the bottom":"セルのテキストを下に寄せる","Align cell text to the center":"セルのテキストを中央へ揃える","Align cell text to the left":"セルのテキストを左へ寄せる","Align cell text to the middle":"セルのテキストを中央へ揃える","Align cell text to the right":"セルのテキストを右へ寄せる","Align cell text to the top":"セルのテキストを上に寄せる","Align table to the left":"テーブルを左へ寄せる","Align table to the left with no text wrapping":"表を左揃え（テキストの折り返しなし）","Align table to the left with text wrapping":"表を左揃え（テキストの折り返しあり）","Align table to the right":"テーブルを右へ寄せる","Align table to the right with no text wrapping":"表を右揃え（テキストの折り返しなし）","Align table to the right with text wrapping":"表を右揃え（テキストの折り返しあり）",Alignment:"配置",Background:"背景",Border:"罫線","Cell properties":"セルのプロパティ","Center table":"テーブルを中央へ寄せる","Center table with no text wrapping":"表を中央揃え（テキストの折り返しなし）","Choose table type":"表の種類を選択",Color:"色",Column:"列","Content table":"コンテンツ表",Dashed:"破線","Delete column":"列を削除","Delete row":"行を削除",Dimensions:"寸法",Dotted:"点線",Double:"2本線","Enter table caption":"テーブルキャプションを入力",Groove:"立体的にくぼんだ線","Header column":"見出し列","Header row":"見出し行",Height:"高さ","Horizontal text alignment toolbar":"水平方向のテキスト配置ツールバー","Insert a new table row (when in the last cell of a table)":"新しいテーブル行を挿入する（テーブルの最終セルにある場合）","Insert column left":"左に列を挿入","Insert column right":"右に列を挿入","Insert row above":"上に行を挿入","Insert row below":"下に行を挿入","Insert table":"表の挿入","Insert table layout":"表のレイアウトを挿入",Inset:"内側全体がくぼんだ線","Justify cell text":"セルのテキストを両端へ揃える","Keystrokes that can be used in a table cell":"テーブルセルで使用できるキーストローク","Layout table":"表のレイアウト","Merge cell down":"下のセルと結合","Merge cell left":"左のセルと結合","Merge cell right":"右のセルと結合","Merge cell up":"上のセルと結合","Merge cells":"セルを結合","Move the selection to the next cell":"選択範囲を次のセルに移動させる","Move the selection to the previous cell":"選択範囲を前のセルに移動させる","Navigate through the table":"テーブル内を移動する",None:"なし",Outset:"内側全体が隆起した線",Padding:"パディング",Ridge:"立体的に隆起した線",Row:"行","Select column":"列を選択","Select row":"行を選択",Solid:"1本線","Split cell horizontally":"縦にセルを分離","Split cell vertically":"横にセルを分離",Style:"スタイル",Table:"テーブル","Table Alignment":"表の配置","Table alignment toolbar":"テーブル配置ツールバー","Table cell text alignment":"テーブルセルのテキスト配置","Table layout":"表のレイアウト","Table properties":"テーブルのプロパティ","Table toolbar":"テーブルのツールバー","Table type":"表の種類","Table type options":"表の種類のオプション",'The color is invalid. Try "#FF0000" or "rgb(255,0,0)" or "red".':"この色は無効です。「#FF0000」、「rgb（255,0,0」または「赤」をお試しください。",'The value is invalid. Try "10px" or "2em" or simply "2".':"この値は無効です。「10px」、「2em」または単純に「2」をお試しください。","Vertical text alignment toolbar":"垂直方向のテキスト配置ツールバー",Width:"幅"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));