!function(n){const e=n.id=n.id||{};e.dictionary=Object.assign(e.dictionary||{},{"Code block":"Blok kode","Entering %0 code snippet":"memasuki %0 cuplikan kode","Entering code snippet":"memasuki cuplikan kode","Insert code block":"Sisipkan blok kode","Leaving %0 code snippet":"meninggalkan %0 cuplikan kode","Leaving code snippet":"meninggalkan cuplikan kode","Plain text":"Teks mentah"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));