!function(i){const n=i.sr=i.sr||{};n.dictionary=Object.assign(n.dictionary||{},{"Create link":"Napravi vezu","Displayed text":"Prikazani tekst",Downloadable:"Могуће преузимање","Edit link":"Исправи линк",Link:"Линк","Link image":"Линк слике","Link properties":"Svojstva veze","Link URL":"УРЛ линк","Link URL must not be empty.":"URL linka ne sme biti prazan.","Move out of a link":"Idi sa veze","No links available":"Nema dostupnih veza","Open in a new tab":"Отвори у новој картици","Open link in new tab":"Отвори линк у новом прозору",Unlink:"Отклони линк"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));