!function(e){const s=e.hu=e.hu||{};s.dictionary=Object.assign(s.dictionary||{},{Activities:"Tevékenységek","Animals & Nature":"Állatok és természet","Dark skin tone":"Sötét bőrszín","Default skin tone":"Alapértelmezett bőrszín",Emoji:"Emoji","Emoji picker":"Emojiválasztó","Find an emoji (min. 2 characters)":"Emoji keresése (min. 2 karakter)",Flags:"Zászlók","Food & Drinks":"Étel és ital","Gestures & People":"Gesztusok és emberek","Keep on typing to see the emoji.":"Írjon tovább az emoji megjelenéséhez.","Light skin tone":"Világos bőrszín","Medium Dark skin tone":"Közepesen sötét bőrszín","Medium Light skin tone":"Közepesen világos bőrszín","Medium skin tone":"Közepes bőrszín",'No emojis were found matching "%0".':"Nem található emoji „%0” kifejezésre.",Objects:"Tárgyak","Select skin tone":"Válasszon bőrszínt","Show all emoji...":"Minden emoji megjelenítése...","Smileys & Expressions":"Smiley-k és arckifejezések",Symbols:"Jelek","The query must contain at least two characters.":"A kérés legalább 2 karaktert kell tartalmazzon.","Travel & Places":"Utazás és helyek"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));