!function(e){const t=e.af=e.af||{};t.dictionary=Object.assign(t.dictionary||{},{"Advanced options":"",Find:"Soek","Find and replace":"Soek en vervang","Find in text…":"Soek in teks …","Find in the document":"","Match case":"Hooflettersensitief","Next result":"Volgende resultaat","Previous result":"Vorige resultaat",Replace:"Vervang","Replace all":"Vervang alles","Replace with…":"Vervang met ...","Text to find must not be empty.":"Soekteks mag nie leeg wees nie.","Tip: Find some text first in order to replace it.":"Wenk: Soek eers 'n bietjie teks om dit te vervang.","Whole words only":"Slegs hele woorde"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));