!function(i){const e=i.el=i.el||{};e.dictionary=Object.assign(e.dictionary||{},{"Disable block editing":"Απενεργοποίηση επεξεργασίας μπλοκ","Disable editing":"Απενεργοποίηση επεξεργασίας","Disable inline editing":"Απενεργοποίηση ενσωματωμένης επεξεργασίας","Enable block editing":"Ενεργοποίηση επεξεργασίας μπλοκ","Enable editing":"Ενεργοποίηση επεξεργασίας","Enable inline editing":"Ενεργοποίηση ενσωματωμένης επεξεργασίας","Navigate editable regions":"Πλοήγηση στις επεξεργάσιμες περιοχές","Next editable region":"Επόμενη επεξεργάσιμη περιοχή","Previous editable region":"Προηγούμενη επεξεργάσιμη περιοχή"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));