!function(i){const e=i["sr-latn"]=i["sr-latn"]||{};e.dictionary=Object.assign(e.dictionary||{},{"Disable block editing":"","Disable editing":"","Disable inline editing":"","Enable block editing":"","Enable editing":"Omogući uređivanje","Enable inline editing":"","Navigate editable regions":"Pronađi regije za uređivanje","Next editable region":"Sledeća regija za uređivanje","Previous editable region":"Prethodna regija za uređivanje"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));