!function(e){const t=e.ru=e.ru||{};t.dictionary=Object.assign(t.dictionary||{},{"Align cell text to the bottom":"Выровнять текст ячейки по нижнему краю","Align cell text to the center":"Выровнять текст по центру","Align cell text to the left":"Выровнять текст по левому краю","Align cell text to the middle":"Выровнять текст ячейки по центру","Align cell text to the right":"Выровнять текст по правому краю","Align cell text to the top":"Выровнять текст ячейки по верхнему краю","Align table to the left":"Выровнять таблицу по левому краю","Align table to the left with no text wrapping":"Выровнять таблицу по левому краю без обтекания текстом","Align table to the left with text wrapping":"Выровнять таблицу по левому краю с обтеканием текстом","Align table to the right":"Выровнять таблицу по правому краю","Align table to the right with no text wrapping":"Выровнять таблицу по правому краю без обтекания текстом","Align table to the right with text wrapping":"Выровнять таблицу по правому краю с обтеканием текстом",Alignment:"Выравнивание",Background:"Фон",Border:"Граница","Cell properties":"Свойства ячейки","Center table":"Выровнять таблицу по центру","Center table with no text wrapping":"Выровнять таблицу по центру без обтекания текстом","Choose table type":"Выберите тип таблицы",Color:"Цвет",Column:"Столбец","Content table":"Таблица содержимого",Dashed:"Пунктирная","Delete column":"Удалить столбец","Delete row":"Удалить строку",Dimensions:"Размеры",Dotted:"Точечная",Double:"Двойная","Enter table caption":"Подпись таблицы",Groove:"Желобчатая","Header column":"Столбец заголовков","Header row":"Строка заголовков",Height:"Высота","Horizontal text alignment toolbar":"Панель инструментов горизонтального выравнивания текста","Insert a new table row (when in the last cell of a table)":"Вставить новую строку таблицы (в последней ячейке таблицы)","Insert column left":"Вставить столбец слева","Insert column right":"Вставить столбец справа","Insert row above":"Вставить строку выше","Insert row below":"Вставить строку ниже","Insert table":"Вставить таблицу","Insert table layout":"Вставить табличный макет",Inset:"Вдавленная","Justify cell text":"Выровнять текст по ширине","Keystrokes that can be used in a table cell":"Нажатия клавиш, которые можно использовать в ячейке таблицы","Layout table":"Таблица макетов","Merge cell down":"Объединить с ячейкой снизу","Merge cell left":"Объединить с ячейкой слева","Merge cell right":"Объединить с ячейкой справа","Merge cell up":"Объединить с ячейкой сверху","Merge cells":"Объединить ячейки","Move the selection to the next cell":"Переместить выделение в следующую ячейку","Move the selection to the previous cell":"Переместить выделение на предыдущую ячейку","Navigate through the table":"Навигация по таблице",None:"Нет",Outset:"Выпуклая",Padding:"Отступ",Ridge:"Ребристая",Row:"Строка","Select column":"Выбрать столбец","Select row":"Выбрать строку",Solid:"Сплошная","Split cell horizontally":"Разделить ячейку горизонтально","Split cell vertically":"Разделить ячейку вертикально",Style:"Стиль",Table:"Таблица","Table Alignment":"Выравнивание таблицы","Table alignment toolbar":"Панель инструментов выравнивания таблицы","Table cell text alignment":"Выравнивание текста в ячейке таблицы","Table layout":"Табличный макет","Table properties":"Свойства таблицы","Table toolbar":"Панель инструментов таблицы","Table type":"Тип таблицы","Table type options":"Параметры типа таблицы",'The color is invalid. Try "#FF0000" or "rgb(255,0,0)" or "red".':'Неверный цвет. Попробуйте "#FF0000" или "rgb(255,0,0)" или "red".','The value is invalid. Try "10px" or "2em" or simply "2".':'Неверное значение. Попробуйте "10px" или "2em" или просто "2".',"Vertical text alignment toolbar":"Панель инструментов вертикального выравнивания текста",Width:"Ширина"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));