!function(e){const i=e.tk=e.tk||{};i.dictionary=Object.assign(i.dictionary||{},{"Insert media":"Mediýa goýuň",Media:"","Media embed":"","Media toolbar":"Mediýa gurallar paneli","Media URL":"Media URL","media widget":"media widjeti","Open media in new tab":"","Paste the media URL in the input.":"Media URL-ni girişde goýuň.","The URL must not be empty.":"URL boş bolmaly däldir.","This media URL is not supported.":"Bu media URL goldanok.","Tip: Paste the URL into the content to embed faster.":"Maslahat: Has çalt ýerleşdirmek üçin URL-i mazmuna goýuň."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));