!function(t){const e=t.cs=t.cs||{};e.dictionary=Object.assign(e.dictionary||{},{Activities:"Aktivity","Animals & Nature":"Zvířata a příroda","Dark skin tone":"Tmavý tón pleti","Default skin tone":"Výchozí tón pleti",Emoji:"Emotikon","Emoji picker":"Výběr emotikonů","Find an emoji (min. 2 characters)":"Najít emotikon (min. 2 znaky)",Flags:"Vlajky","Food & Drinks":"Jídlo a pití","Gestures & People":"Gesta a lidé","Keep on typing to see the emoji.":"Pokud chcete zobrazit emotikon, pokračujte v psaní.","Light skin tone":"Světlý tón pleti","Medium Dark skin tone":"Středně tmavý tón pleti","Medium Light skin tone":"Středně světlý tón pleti","Medium skin tone":"Střední tón pleti",'No emojis were found matching "%0".':"Nebyly nalezeny žádné emotikony odpovídající „%0“.",Objects:"Objekty","Select skin tone":"Vyberte tón pleti","Show all emoji...":"Zobrazit všechny emotikony…","Smileys & Expressions":"Smajlíci a výrazy",Symbols:"Symboly","The query must contain at least two characters.":"Dotaz musí obsahovat alespoň dva znaky.","Travel & Places":"Cestování a místa"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));