!function(e){const i=e.bn=e.bn||{};i.dictionary=Object.assign(i.dictionary||{},{"Blue marker":"নীল মার্কার","Green marker":"সবুজ মার্কার","Green pen":"সবুজ কলম",Highlight:"হাইলাইট","Pink marker":"গোলাপী মার্কার","Red pen":"লাল কলম","Remove highlight":"হাইলাইট মুছুন","Text highlight toolbar":"টেক্সট হাইলাইট টুলবার","Yellow marker":"হলুদ মার্কার"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));