!function(i){const e=i.cs=i.cs||{};e.dictionary=Object.assign(e.dictionary||{},{"Disable block editing":"Zakázat blokové úpravy","Disable editing":"Zakázat úpravy","Disable inline editing":"Zakázat úpravy v textu","Enable block editing":"Povolit blokové úpravy","Enable editing":"Povolit úpravy","Enable inline editing":"Povolit inline úpravy","Navigate editable regions":"Procházet upravitelnými oblastmi","Next editable region":"Následující upravitelná oblast","Previous editable region":"Předchozí upravitelná oblast"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));