!function(e){const i=e.af=e.af||{};i.dictionary=Object.assign(i.dictionary||{},{"Could not insert image at the current position.":"Beeld kan nie in die posisie toegevoeg word nie.","Could not obtain resized image URL.":"Kon nie die beeld URL vir die aanpassing kry nie.","Insert image or file":"Voeg beeld of lêer in","Inserting image failed":"Beeld kan nie toegevoeg word nie","Selecting resized image failed":"Kon nie die beeld se grootte verander nie"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));