!function(n){const i=n.ne=n.ne||{};i.dictionary=Object.assign(i.dictionary||{},{"Create link":"","Displayed text":"",Downloadable:"डाउनलोड योग्य","Edit link":"लिङ्क सम्पादन गर्नुहोस्",Link:"लिङ्क","Link image":"","Link properties":"","Link URL":"लिङ्क यूआरएल","Link URL must not be empty.":"","Move out of a link":"","No links available":"","Open in a new tab":"नयाँ ट्याबमा खोल्न","Open link in new tab":"नयाँ ट्याबमा लिङ्क खोल्नुहोस्",Unlink:"अनलिङ्क गर्नुहोस्"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));