!function(a){const e=a.id=a.id||{};e.dictionary=Object.assign(e.dictionary||{},{Activities:"Aktifitas","Animals & Nature":"Hewan & Alam","Dark skin tone":"Warna kulit gelap","Default skin tone":"Warna kulit standar",Emoji:"Emoji","Emoji picker":"Pemilih emoji","Find an emoji (min. 2 characters)":"Temukan emoji (min. 2 karakter)",Flags:"Bendera","Food & Drinks":"Makanan & Minuman","Gestures & People":"Gerakan & Orang","Keep on typing to see the emoji.":"Teruskan mengetik untuk melihat emoji.","Light skin tone":"Warna kulit cerah","Medium Dark skin tone":"Warna kulit gelap sedang","Medium Light skin tone":"Warna kulit cerah sedang","Medium skin tone":"Warna kulit sedang",'No emojis were found matching "%0".':'Tidak ditemukan emoji yang cocok dengan "%0".',Objects:"Objek","Select skin tone":"Pilih warna kulit","Show all emoji...":"Tampilkan semua emoji…","Smileys & Expressions":"Smiley & Ekspresi",Symbols:"Simbol","The query must contain at least two characters.":"Kueri harus berisi setidaknya dua karakter.","Travel & Places":"Perjalanan & Tempat"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));