!function(n){const e=n.nb=n.nb||{};e.dictionary=Object.assign(e.dictionary||{},{"Create link":"","Displayed text":"",Downloadable:"","Edit link":"Rediger lenke",Link:"Lenke","Link image":"","Link properties":"","Link URL":"URL for lenke","Link URL must not be empty.":"","Move out of a link":"","No links available":"","Open in a new tab":"","Open link in new tab":"Åpne lenke i ny fane",Unlink:"Fjern lenke"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));