!function(i){const e=i.ku=i.ku||{};e.dictionary=Object.assign(e.dictionary||{},{"Could not insert image at the current position.":"نەتوانرا وێنە دابنرێت لەم شوێنە.","Could not obtain resized image URL.":"نەتوانرا بەستەری وێنەی قەبارە گۆڕاو بەدەست بێت.","Insert image or file":"وێنە یان پەڕگە دابنێ","Inserting image failed":"دانانی وێنە سەرکەوتوو نەبوو","Selecting resized image failed":"دیاریکردنی وێنەی قەبارە گۆڕاو سەرکەوتوو نەبوو"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));