!function(a){const n=a.ms=a.ms||{};n.dictionary=Object.assign(n.dictionary||{},{"Advanced options":"Pilihan lanjutan",Find:"Cari","Find and replace":"Cari dan ganti","Find in text…":"Cari dalam teks...","Find in the document":"Cari dalam dokumen","Match case":"Sesuaikan kes","Next result":"Keputusan seterusnya","Previous result":"Keputusan sebelumnya",Replace:"Ganti","Replace all":"Ganti semua","Replace with…":"Gantikan dengan...","Text to find must not be empty.":"Teks yang hendak dicari tidak boleh ditinggalkan kosong.","Tip: Find some text first in order to replace it.":"Petua: Cari teks terlebih dahulu untuk menggantikannya.","Whole words only":"Perkataan penuh sahaja"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));