!function(n){const e=n.ar=n.ar||{};e.dictionary=Object.assign(e.dictionary||{},{"Choose heading":"اختر عنوان",Heading:"عنوان","Heading 1":"عنوان 1","Heading 2":"عنوان 2","Heading 3":"عنوان 3","Heading 4":"عنوان 4","Heading 5":"عنوان 5","Heading 6":"عنوان 6","Type or paste your content here.":"أكتب المحتوى أو ألصقه هنا.","Type your title":"أدخل العنوان"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));