{#
/**
 * @file
 * Default theme implementation to display a file form widget.
 *
 * Available variables:
 * - element: Form element for the file upload.
 * - attributes: HTML attributes for the containing element.
 *
 * @see \Drupal\file\Hook\FileThemeHooks::preprocessManagedFile()
 *
 * @ingroup themeable
 */
#}
{%
  set classes = [
    'js-form-managed-file',
    'form-managed-file',
  ]
%}
<div{{ attributes.addClass(classes) }}>
  {{ element }}
</div>
