!function(e){const t=e.hu=e.hu||{};t.dictionary=Object.assign(t.dictionary||{},{"Cannot access default workspace.":"Nem lehetséges hozzáférni az alapértelmezett munkaterülethez.","Cannot determine a category for the uploaded file.":"Nem sikerült meghatározni a feltöltött fájl kategóriáját.","Edit image":"Kép szerkesztése","Failed to determine category of edited image.":"Nem sikerült meghatározni a szerkesztett kép kategóriáját.","Open file manager":"Fájlkezelő megnyitása","Processing the edited image.":"A szerkesztett kép feldolgozása.","Server failed to process the image.":"A szerver nem tudta feldolgozni a képet.","You have no image editing permissions.":"Nincs képszerkesztési engedélyed."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));