!function(e){const i=e.it=e.it||{};i.dictionary=Object.assign(i.dictionary||{},{Activities:"Attività","Animals & Nature":"Animali e natura","Dark skin tone":"Tonalità di pelle scura","Default skin tone":"Tonalità di pelle predefinita",Emoji:"Emoji","Emoji picker":"Selettore di emoji","Find an emoji (min. 2 characters)":"Trova un emoji (min. 2 caratteri)",Flags:"Bandiere","Food & Drinks":"Cibo e bevande","Gestures & People":"Gesti e persone","Keep on typing to see the emoji.":"Continua a digitare per vedere l'emoji.","Light skin tone":"Tonalità di pelle chiara","Medium Dark skin tone":"Tonalità di pelle medio-scura","Medium Light skin tone":"Tonalità di pelle medio-chiara","Medium skin tone":"Tonalità di pelle media",'No emojis were found matching "%0".':'Non sono state trovati emoji corrispondenti a "%0".',Objects:"Oggetti","Select skin tone":"Seleziona la tonalità della pelle","Show all emoji...":"Mostra tutti gli emoji...","Smileys & Expressions":"Faccine ed espressioni",Symbols:"Simboli","The query must contain at least two characters.":"La ricerca deve contenere almeno due caratteri.","Travel & Places":"Viaggi e luoghi"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));