<?php

/**
 * @file
 * Generic transliteration data for the PhpTransliteration class.
 */

$base = [
  0x00 => 'kwon', 'kwonj', 'kwonh', 'kwod', 'kwol', 'kwolg', 'kwolm', 'kwolb', 'kwols', 'kwolt', 'kwolp', 'kwolh', 'kwom', 'kwob', 'kwobs', 'kwos',
  0x10 => 'kwoss', 'kwong', 'kwoj', 'kwoch', 'kwok', 'kwot', 'kwop', 'kwoh', 'kwe', 'kweg', 'kwekk', 'kwegs', 'kwen', 'kwenj', 'kwenh', 'kwed',
  0x20 => 'kwel', 'kwelg', 'kwelm', 'kwelb', 'kwels', 'kwelt', 'kwelp', 'kwelh', 'kwem', 'kweb', 'kwebs', 'kwes', 'kwess', 'kweng', 'kwej', 'kwech',
  0x30 => 'kwek', 'kwet', 'kwep', 'kweh', 'kwi', 'kwig', 'kwikk', 'kwigs', 'kwin', 'kwinj', 'kwinh', 'kwid', 'kwil', 'kwilg', 'kwilm', 'kwilb',
  0x40 => 'kwils', 'kwilt', 'kwilp', 'kwilh', 'kwim', 'kwib', 'kwibs', 'kwis', 'kwiss', 'kwing', 'kwij', 'kwich', 'kwik', 'kwit', 'kwip', 'kwih',
  0x50 => 'kyu', 'kyug', 'kyukk', 'kyugs', 'kyun', 'kyunj', 'kyunh', 'kyud', 'kyul', 'kyulg', 'kyulm', 'kyulb', 'kyuls', 'kyult', 'kyulp', 'kyulh',
  0x60 => 'kyum', 'kyub', 'kyubs', 'kyus', 'kyuss', 'kyung', 'kyuj', 'kyuch', 'kyuk', 'kyut', 'kyup', 'kyuh', 'keu', 'keug', 'keukk', 'keugs',
  0x70 => 'keun', 'keunj', 'keunh', 'keud', 'keul', 'keulg', 'keulm', 'keulb', 'keuls', 'keult', 'keulp', 'keulh', 'keum', 'keub', 'keubs', 'keus',
  0x80 => 'keuss', 'keung', 'keuj', 'keuch', 'keuk', 'keut', 'keup', 'keuh', 'kui', 'kuig', 'kuikk', 'kuigs', 'kuin', 'kuinj', 'kuinh', 'kuid',
  0x90 => 'kuil', 'kuilg', 'kuilm', 'kuilb', 'kuils', 'kuilt', 'kuilp', 'kuilh', 'kuim', 'kuib', 'kuibs', 'kuis', 'kuiss', 'kuing', 'kuij', 'kuich',
  0xA0 => 'kuik', 'kuit', 'kuip', 'kuih', 'ki', 'kig', 'kikk', 'kigs', 'kin', 'kinj', 'kinh', 'kid', 'kil', 'kilg', 'kilm', 'kilb',
  0xB0 => 'kils', 'kilt', 'kilp', 'kilh', 'kim', 'kib', 'kibs', 'kis', 'kiss', 'king', 'kij', 'kich', 'kik', 'kit', 'kip', 'kih',
  0xC0 => 'ta', 'tag', 'takk', 'tags', 'tan', 'tanj', 'tanh', 'tad', 'tal', 'talg', 'talm', 'talb', 'tals', 'talt', 'talp', 'talh',
  0xD0 => 'tam', 'tab', 'tabs', 'tas', 'tass', 'tang', 'taj', 'tach', 'tak', 'tat', 'tap', 'tah', 'tae', 'taeg', 'taekk', 'taegs',
  0xE0 => 'taen', 'taenj', 'taenh', 'taed', 'tael', 'taelg', 'taelm', 'taelb', 'taels', 'taelt', 'taelp', 'taelh', 'taem', 'taeb', 'taebs', 'taes',
  0xF0 => 'taess', 'taeng', 'taej', 'taech', 'taek', 'taet', 'taep', 'taeh', 'tya', 'tyag', 'tyakk', 'tyags', 'tyan', 'tyanj', 'tyanh', 'tyad',
];
