!function(e){const i=e.el=e.el||{};i.dictionary=Object.assign(i.dictionary||{},{"Cannot access default workspace.":"Δεν είναι δυνατή η πρόσβαση στον προεπιλεγμένο χώρο εργασίας","Cannot determine a category for the uploaded file.":"Δεν μπορεί να οριστεί κατηγορία για το αρχείο που μεταφορτώθηκε.","Edit image":"Επεξεργασία εικόνας","Failed to determine category of edited image.":"Απέτυχε ο προσδιορισμός της κατηγορίας της επεξεργασμένης εικόνας.","Open file manager":"Άνοιγμα διαχειριστή αρχείων","Processing the edited image.":"Επεξεργασία της επεξεργασμένης εικόνας","Server failed to process the image.":"Ο διακομιστής απέτυχε να επεξεργαστεί την εικόνα.","You have no image editing permissions.":"Δεν έχεις άδειες επεξεργασίας εικόνας."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));