!function(e){const t=e.bn=e.bn||{};t.dictionary=Object.assign(t.dictionary||{},{"Insert media":"মিডিয়া ঢোকান",Media:"মিডিয়া","Media embed":"মিডিয়া এম্বেড","Media toolbar":"মিডিয়া টুলবার","Media URL":"মিডিয়া URL","media widget":"মিডিয়া উইজেট","Open media in new tab":"নতুন ট্যাবে মিডিয়া খুলুন","Paste the media URL in the input.":"ইনপুটে মিডিয়া URL পেস্ট করুন।","The URL must not be empty.":"URLটি খালি হওয়া উচিত নয়৷","This media URL is not supported.":"এই মিডিয়া URL সমর্থিত নয়।","Tip: Paste the URL into the content to embed faster.":"টিপ: দ্রুত এম্বেড করতে কন্টেন্টে URL পেস্ট করুন।"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));