!function(e){const i=e.be=e.be||{};i.dictionary=Object.assign(i.dictionary||{},{"Cannot access default workspace.":"Немагчыма атрымаць доступ да прадвызначанай працоўнай прасторы.","Cannot determine a category for the uploaded file.":"Немагчыма вызначыць катэгорыю для запампаванага файла.","Edit image":"Рэдагаваць відарыс","Failed to determine category of edited image.":"Не здольны вызначыць катэгорыю адрэдагаванага відарыса.","Open file manager":"Адкрыць менеджэр файлаў","Processing the edited image.":"Апрацоўка адрэдагаванага відарыса.","Server failed to process the image.":"Сервер не здольны апрацаваць відарыс.","You have no image editing permissions.":"У вас няма дозволу на рэдагаванне відарыса."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));