!function(n){const e=n.nl=n.nl||{};e.dictionary=Object.assign(e.dictionary||{},{"Create link":"Creëer link","Displayed text":"Weergegeven tekst",Downloadable:"Downloadbaar","Edit link":"Bewerk link",Link:"Link","Link image":"Link afbeelding","Link properties":"Linkeigenschappen","Link URL":"Link URL","Link URL must not be empty.":"URL-link mag niet leeg zijn.","Move out of a link":"Uit een link gaan","No links available":"Geen links beschikbaar","Open in a new tab":"Open een nieuw tabblad","Open link in new tab":"Open link in nieuw tabblad",Unlink:"Verwijder link"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));