!function(e){const t=e.fr=e.fr||{};t.dictionary=Object.assign(t.dictionary||{},{"Advanced options":"Options avancées",Find:"Rechercher","Find and replace":"Rechercher et remplacer","Find in text…":"Rechercher dans le texte...","Find in the document":"Rechercher dans le document","Match case":"Sensible à la casse","Next result":"Résultat suivant","Previous result":"Résultat précédent",Replace:"Remplacer","Replace all":"Remplacer tout","Replace with…":"Remplacer par ...","Text to find must not be empty.":"L'expression à rechercher ne doit pas être vide","Tip: Find some text first in order to replace it.":"Astuce : rechercher une expression afin de la remplacer","Whole words only":"Mots entiers uniquement"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));