!function(i){const e=i["en-au"]=i["en-au"]||{};e.dictionary=Object.assign(e.dictionary||{},{"Disable block editing":"Disable block editing","Disable editing":"Disable editing","Disable inline editing":"Disable inline editing","Enable block editing":"Enable block editing","Enable editing":"Enable editing","Enable inline editing":"Enable inline editing","Navigate editable regions":"Navigate editable regions","Next editable region":"Next editable region","Previous editable region":"Previous editable region"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));