!function(i){const e=i["sr-latn"]=i["sr-latn"]||{};e.dictionary=Object.assign(e.dictionary||{},{"Could not insert image at the current position.":"Nemoguće je dodati sliku na ovo mesto.","Could not obtain resized image URL.":"URL slikа promenjeniih dimenzija nije dostupna.","Insert image or file":"Dodaj sliku ili fajl","Inserting image failed":"Dodavanje slike je neuspešno","Selecting resized image failed":"Odabir slike promenjenih dimenzija nije uspešnо"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));