!function(a){const t=a.uz=a.uz||{};t.dictionary=Object.assign(t.dictionary||{},{"Align cell text to the bottom":"Hujayra matnini pastga tekislash","Align cell text to the center":"Matnni markazga tekislash","Align cell text to the left":"Matnni chapga tekislash","Align cell text to the middle":"Hujayra matnini markazga tekislash","Align cell text to the right":"Matnni o'ngga tekislash","Align cell text to the top":"Hujayra matnini tepaga tekislash","Align table to the left":"Jadvalni chap tomonga tekislash","Align table to the left with no text wrapping":"","Align table to the left with text wrapping":"","Align table to the right":"Jadvalni o'ngga tekislash","Align table to the right with no text wrapping":"","Align table to the right with text wrapping":"",Alignment:"Tekislash",Background:"Fon",Border:"Chegara","Cell properties":"Hujayra xususiyatlari","Center table":"Jadvalni markazga tekislash","Center table with no text wrapping":"","Choose table type":"",Color:"Rang",Column:"Ustun","Content table":"",Dashed:"Nuqtali","Delete column":"Ustunni o'chirish","Delete row":"Satrni o'chirish",Dimensions:"O'lchamlar",Dotted:"Nuqta",Double:"Ikkitalik","Enter table caption":"",Groove:"Yivli","Header column":"Ustun sarlavhalari","Header row":"Sarlavhalar satri",Height:"Balandligi","Horizontal text alignment toolbar":"Matnni gorizontal tekislash asboblar paneli","Insert a new table row (when in the last cell of a table)":"","Insert column left":"Ustunni chapga kiritish","Insert column right":"Ustunni o'ngga kiritish","Insert row above":"Yuqoriga qatorni kiritish","Insert row below":"Pastga qatorni kiritish","Insert table":"Jadvalni kiritish","Insert table layout":"",Inset:"Tushkunlikka tushgan","Justify cell text":"Matnni kenglikka tekislash","Keystrokes that can be used in a table cell":"","Layout table":"","Merge cell down":"Pastdagi katak bilan birlashtirish","Merge cell left":"Chapdagi katakcha bilan birlashtirish","Merge cell right":"O'ngdagi katakcha bilan birlashtirish","Merge cell up":"Yuqoridagi katak bilan birlashtirish","Merge cells":"Hujayralarni birlashtirish","Move the selection to the next cell":"","Move the selection to the previous cell":"","Navigate through the table":"",None:"Yo'q",Outset:"Qavariq",Padding:"Chekinish",Ridge:"Qirrali",Row:"Satr","Select column":"Ustunni tanlash","Select row":"Satrni tanlang",Solid:"Qattiq","Split cell horizontally":"Hujayrani gorizontal ravishda ajratish","Split cell vertically":"Hujayrani vertikal ravishda ajratish",Style:"Uslub",Table:"","Table Alignment":"","Table alignment toolbar":"Jadvalni tekislash asboblar paneli","Table cell text alignment":"Jadval katakchasidagi matnni tekislash","Table layout":"","Table properties":"Jadvalning xususiyatlari","Table toolbar":"Jadval asboblar paneli","Table type":"","Table type options":"",'The color is invalid. Try "#FF0000" or "rgb(255,0,0)" or "red".':'Noto\'g\'ri rang. \\ "# FF0000 \\" yoki \\ "rgb (255,0,0) \\" yoki \\ "red \\" ni sinab ko\'ring.','The value is invalid. Try "10px" or "2em" or simply "2".':'Noto\'g\'ri qiymat. \\ "10px \\" yoki \\ "2em \\" yoki shunchaki \\ "2 \\" ni sinab ko\'ring.',"Vertical text alignment toolbar":"Vertikal matnni tekislash asboblar paneli",Width:"Kengligi"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));