!function(a){const e=a.ms=a.ms||{};e.dictionary=Object.assign(e.dictionary||{},{"Bulleted List":"Senarai Berbulet","Bulleted list styles toolbar":"Bar capaian gaya senarai berbulet",Circle:"Bula",Decimal:"Perpuluhan","Decimal with leading zero":"Perpuluhan dengan kosong pendahulu","Decrease list item indent":"Kurangkan inden item senarai",Disc:"Cakera","Entering a to-do list":"Memasukkan senarai tugas untuk dilakukan","Increase list item indent":"Tingkatkan inden item senarai","Invalid start index value.":"Nilai indeks permulaan tidak sah.","Keystrokes that can be used in a list":"Ketukan kekunci yang boleh digunakan dalam senarai","Leaving a to-do list":"Meninggalkan senarai tugas untuk dilakukan","List properties":"Senarai sifat","Lower-latin":"Latin bawah","Lower–roman":"Roman bawah","Numbered List":"Senarai Bernombor","Numbered list styles toolbar":"Bar alat capaian gaya senarai bernombor","Reversed order":"Susunan terbalik",Square:"Petak","Start at":"Mula pada","Start index must be greater than 0.":"Indeks mula hendaklah lebih besar daripada 0.","To-do List":"Senarai Untuk Dilakukan","Toggle the circle list style":"Buka gaya senarai bulatan","Toggle the decimal list style":"Buka gaya senarai titik perpuluhan","Toggle the decimal with leading zero list style":"Buka titik perpuluhan dengan gaya senarai kosong pendahulu","Toggle the disc list style":"Buka gaya senarai cakera","Toggle the lower–latin list style":"Buka gaya senarai latin bawah","Toggle the lower–roman list style":"Buka gaya senarai roman bawah","Toggle the square list style":"Buka gaya senarai petak","Toggle the upper–latin list style":"Buka gaya senarai latin atas","Toggle the upper–roman list style":"Buka gaya senarai roman atas","Upper-latin":"Latin atas","Upper-roman":"Roman atas"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));