!function(e){const n=e.fa=e.fa||{};n.dictionary=Object.assign(n.dictionary||{},{"Edit source":"ویرایش سورس","Empty snippet content":"","HTML snippet":"","Insert HTML":"وارد کردن html","No preview available":"پیش نمایشی موجود نیست","Paste raw HTML here...":"","Save changes":"ذخیره تغییرات"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));