!function(e){const r=e["de-ch"]=e["de-ch"]||{};r.dictionary=Object.assign(r.dictionary||{},{"Blue marker":"Blauer Marker","Green marker":"Grüner Marker","Green pen":"Grüne Schriftfarbe",Highlight:"Texthervorhebung","Pink marker":"Pinker Marker","Red pen":"Rote Schriftfarbe","Remove highlight":"Texthervorhebung entfernen","Text highlight toolbar":"Texthervorhebung Werkzeugleiste","Yellow marker":"Gelber Marker"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));