!function(e){const t=e.sl=e.sl||{};t.dictionary=Object.assign(t.dictionary||{},{"Insert media":"",Media:"","Media embed":"","Media toolbar":"","Media URL":"","media widget":"","Open media in new tab":"","Paste the media URL in the input.":"","The URL must not be empty.":"","This media URL is not supported.":"","Tip: Paste the URL into the content to embed faster.":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));