!function(t){const i=t.jv=t.jv||{};i.dictionary=Object.assign(i.dictionary||{},{Bold:"Kandhel","Bold text":"",Code:"Kode",Italic:"Miring","Italic text":"","Move out of an inline code style":"",Strikethrough:"Seratan dicoret","Strikethrough text":"",Subscript:"",Superscript:"",Underline:"Garis ngandhap","Underline text":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));