!function(o){const a=o.bn=o.bn||{};a.dictionary=Object.assign(a.dictionary||{},{Bookmark:"বুকমার্ক","Bookmark must not be empty.":"বুকমার্ক খালি রাখা যাবে না।","Bookmark name":"বুকমার্কের নাম","Bookmark name already exists.":"বুকমার্কের নাম আগে থেকেই বিদ্যমান।","Bookmark name cannot contain space characters.":"বুকমার্কের নামে স্পেস ক্যারেক্টার রাখা যাবে না।","Bookmark toolbar":"বুকমার্ক টুলবার","bookmark widget":"বুকমার্ক উইজেট",Bookmarks:"বুকমার্কস","Edit bookmark":"বুকমার্ক এডিট করুন","Enter the bookmark name without spaces.":"স্পেস ছাড়া বুকমার্কের নাম লিখুন।","No bookmarks available.":"কোনো বুকমার্কস উপলব্ধ নেই।","Remove bookmark":"বুকমার্ক অপসারণ করুন","Scroll to bookmark":"বুকমার্কে স্ক্রল করুন"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));