!function(e){const i=e.da=e.da||{};i.dictionary=Object.assign(i.dictionary||{},{"Could not insert image at the current position.":"Kunne ikke indsætte billede på aktuel position.","Could not obtain resized image URL.":"Kunne ikke hente URL på ændret billede.","Insert image or file":"Indsæt billede eller fil","Inserting image failed":"Indsætning af billede fejlede","Selecting resized image failed":"Valg af ændret billede fejlede"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));