!function(e){const t=e.sr=e.sr||{};t.dictionary=Object.assign(t.dictionary||{},{"Align cell text to the bottom":"Поравнајте текст ћелије према доле","Align cell text to the center":"Поравнајте текст ћелије у средину","Align cell text to the left":"Поравнајте текст ћелије лево","Align cell text to the middle":"Поравнајте текст ћелије у средину","Align cell text to the right":"Поравнајте текст ћелије десно","Align cell text to the top":"Поравнајте текст ћелије према горе","Align table to the left":"Поравнајте табелу на леву страну","Align table to the left with no text wrapping":"Poravnaj tabelu levo bez preloma teksta","Align table to the left with text wrapping":"Poravnaj tabelu levo sa prelomom teksta","Align table to the right":"Поравнајте табелу на десну страну","Align table to the right with no text wrapping":"Poravnaj tabelu desno bez preloma teksta","Align table to the right with text wrapping":"Poravnaj tabelu desno sa prelomom teksta",Alignment:"Поравнање",Background:"Позадина",Border:"Граница","Cell properties":"Својства ћелије","Center table":"Центар табеле","Center table with no text wrapping":"Centriraj tabelu bez preloma teksta","Choose table type":"Izaberite tip tabele",Color:"Боја",Column:"Колона","Content table":"Tabela sadržaja",Dashed:"Разбијено","Delete column":"Бриши колону","Delete row":"Бриши ред",Dimensions:"Димензија",Dotted:"Са тачкама",Double:"Двоструко","Enter table caption":"Унесите наслов табеле ",Groove:"Колосек","Header column":"Колона за заглавље","Header row":"Ред за заглавлје",Height:"Висина","Horizontal text alignment toolbar":"Хоризонтална трака са алаткама за поравнање текста","Insert a new table row (when in the last cell of a table)":"Umetni novi red u tabeli (kada je u poslednjoj ćeliji tabele)","Insert column left":"Додај колону лево","Insert column right":"Додај колону десно","Insert row above":"Додај ред изнад","Insert row below":"Додај ред испод","Insert table":"Додај табелу","Insert table layout":"Umetnite raspored tabele",Inset:"Прилог","Justify cell text":"Оправдајте текст ћелије","Keystrokes that can be used in a table cell":"Tasteri koji se mogu koristiti u ćeliji tabele","Layout table":"Tabela rasporeda","Merge cell down":"Спој ћелије на доле","Merge cell left":"Cпој ћелије на лево","Merge cell right":"Спој ћелије на десно","Merge cell up":"Спој ћелије на горе","Merge cells":"Спој ћелије","Move the selection to the next cell":"Pomeri odabir u sledeću ćeliju","Move the selection to the previous cell":"Pomeri odabir u prethodnu ćeliju","Navigate through the table":"Kretanje kroz tabelu",None:"Ниједан",Outset:"Почетак",Padding:"Постављање",Ridge:"Гребен",Row:"Ред","Select column":"Изабери колону","Select row":"Изабери ред",Solid:"Чврст","Split cell horizontally":"Дели ћелије водоравно","Split cell vertically":"Дели ћелије усправно",Style:"Стил",Table:"Tabela","Table Alignment":"Poravnjavanje tabele","Table alignment toolbar":"Трака са алаткама за поравнање табеле","Table cell text alignment":"Поравнај тексту табели","Table layout":"Raspored tabele","Table properties":"Својства табеле","Table toolbar":"Табела трака са алаткама","Table type":"Tip tabele","Table type options":"Opcije tipa tabele",'The color is invalid. Try "#FF0000" or "rgb(255,0,0)" or "red".':'Боја је неважећа. Покушајте са "#FF0000" или "rgb(255,0,0)" или "црвена".','The value is invalid. Try "10px" or "2em" or simply "2".':'Вредност је неважећа. Покушајте са "10px" или "2em" или једноставно "2".',"Vertical text alignment toolbar":"Вертикална трака са алаткама за поравнање текста",Width:"Ширина"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));