/**
 * Image style configuration pages.
 */
.image-style-new,
.image-style-new div {
  display: inline;
}
.image-style-preview .preview-image-wrapper {
  top: 50%;
  float: left;
  width: 48%;
  padding-bottom: 2em;
  text-align: center;
}
.image-style-preview .preview-image {
  position: relative;
  margin: 0.5em auto auto auto;
}
.image-style-preview .preview-image a,
.image-style-preview .preview-image a img {
  display: block;
}
.image-style-preview .preview-image .width {
  position: absolute;
  bottom: -6px;
  left: -1px;
  box-sizing: content-box;
  height: 2px;
  border: 1px solid #666;
  border-top: none;
}
.image-style-preview .preview-image .width span {
  position: relative;
  top: 4px;
}
.image-style-preview .preview-image .height {
  position: absolute;
  top: -1px;
  right: -6px;
  box-sizing: content-box;
  width: 2px;
  border: 1px solid #666;
  border-left: none;
}
.image-style-preview .preview-image .height span {
  position: absolute;
  top: 50%;
  left: 10px;
  height: 2em;
  margin-top: -1em;
}

/**
 * Improve image style preview on narrow viewports.
 */
@media screen and (max-width: 470px) {
  .image-style-preview .preview-image-wrapper {
    float: none;
    margin-bottom: 1em;
  }
  .image-style-preview .preview-image-wrapper:last-child {
    margin-bottom: 0;
  }
}

/**
 * Image anchor element.
 */
.image-anchor {
  width: auto;
}
.image-anchor tr {
  background: none;
}
.image-anchor td {
  border: 1px solid #ccc;
}
