!function(e){const i=e.id=e.id||{};i.dictionary=Object.assign(i.dictionary||{},{"Insert media":"Sisipkan media",Media:"Media","Media embed":"Sematkan media","Media toolbar":"Alat media","Media URL":"URL Media","media widget":"widget media","Open media in new tab":"Buka media di tab baru","Paste the media URL in the input.":"Tempelkan URL ke dalam bidang masukan.","The URL must not be empty.":"URL tidak boleh kosong.","This media URL is not supported.":"URL media ini tidak didukung.","Tip: Paste the URL into the content to embed faster.":"Tip: Tempelkan URL ke bagian konten untuk sisip cepat."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));