!function(i){const o=i.lt=i.lt||{};o.dictionary=Object.assign(o.dictionary||{},{"Create link":"Sukurti nuorodą","Displayed text":"Rodomas tekstas",Downloadable:"Parsisiunčiamas","Edit link":"Keisti nuorodą",Link:"Pridėti nuorodą","Link image":"Susieti paveikslėlį","Link properties":"Nuorodos savybės","Link URL":"Nuorodos URL","Link URL must not be empty.":"Nuorodos URL negali būti tuščias.","Move out of a link":"Išeiti iš nuorodos","No links available":"Nėra jokių nuorodų","Open in a new tab":"Atverti naujoje kortelėje","Open link in new tab":"Atidaryti nuorodą naujame skirtuke",Unlink:"Pašalinti nuorodą"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));