!function(o){const a=o.ru=o.ru||{};a.dictionary=Object.assign(a.dictionary||{},{Bookmark:"Закладка","Bookmark must not be empty.":"Закладка не должна быть пустой.","Bookmark name":"Имя закладки","Bookmark name already exists.":"Имя закладки уже существует.","Bookmark name cannot contain space characters.":"Имя закладки не может содержать пробелы.","Bookmark toolbar":"Панель инструментов закладок","bookmark widget":"виджет закладок",Bookmarks:"Закладки","Edit bookmark":"Изменить закладку","Enter the bookmark name without spaces.":"Введите имя закладки без пробелов.","No bookmarks available.":"Нет доступных закладок.","Remove bookmark":"Удалить закладку","Scroll to bookmark":"Прокрутить до закладки"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));