!function(i){const e=i.es=i.es||{};e.dictionary=Object.assign(e.dictionary||{},{"Disable block editing":"Deshabilitar la edición de bloques","Disable editing":"Deshabilitar la edición","Disable inline editing":"Deshabilitar la edición en línea","Enable block editing":"Habilitar la edición de bloques","Enable editing":"Habilitar edición","Enable inline editing":"Habilitar la edición en línea","Navigate editable regions":"Navegar por las zonas editables","Next editable region":"Siguiente zona editable","Previous editable region":"Anterior zona editable"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));