!function(e){const i=e.de=e.de||{};i.dictionary=Object.assign(i.dictionary||{},{"Cannot access default workspace.":"Zugriff auf Standardarbeitsbereich nicht möglich.","Cannot determine a category for the uploaded file.":"Die Kategorie für die hochgeladene Datei konnte nicht bestimmt werden.","Edit image":"Bild bearbeiten","Failed to determine category of edited image.":"Es konnte für das bearbeitete Bild keine Kategorie ermittelt werden.","Open file manager":"Dateimanager öffnen","Processing the edited image.":"Das bearbeitete Bild wird verarbeitet.","Server failed to process the image.":"Der Server konnte das Bild nicht verarbeiten.","You have no image editing permissions.":"Sie haben keine Berechtigung zum Bearbeiten von Bildern."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));