!function(o){const a=o.uk=o.uk||{};a.dictionary=Object.assign(a.dictionary||{},{Bookmark:"Закладка","Bookmark must not be empty.":"Закладка не може бути порожньою.","Bookmark name":"Назва закладки","Bookmark name already exists.":"Назва закладки вже існує.","Bookmark name cannot contain space characters.":"Назва закладки не може містити пробілів.","Bookmark toolbar":"Панель закладок","bookmark widget":"віджет закладок",Bookmarks:"Закладки","Edit bookmark":"Редагувати закладку","Enter the bookmark name without spaces.":"Введіть назву закладки без пробілів.","No bookmarks available.":"Немає доступних закладок.","Remove bookmark":"Видалити закладку","Scroll to bookmark":"Прокрутити до закладки"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));