!function(n){const e=n.he=n.he||{};e.dictionary=Object.assign(e.dictionary||{},{"Decrease indent":"הקטנת הזחה","Increase indent":"הגדלת הזחה"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));