!function(i){const e=i.bg=i.bg||{};e.dictionary=Object.assign(e.dictionary||{},{"Could not insert image at the current position.":"На текущата позиция не може да се вмъкне изображение.","Could not obtain resized image URL.":"Не може да бъде придобит промененият уеб адрес на изображението.","Insert image or file":"Вмъкни изображение или файл","Inserting image failed":"Вмъкването на изображение не е успешно","Selecting resized image failed":"Избирането на промененото изображение не е успешно"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));