!function(e){const i=e.ms=e.ms||{};i.dictionary=Object.assign(i.dictionary||{},{Bold:"Tebal","Bold text":"Teks tebal",Code:"Kod",Italic:"Italik","Italic text":"Teks huruf condong","Move out of an inline code style":"Alih keluar daripada gaya kod sebaris",Strikethrough:"Garis lorek","Strikethrough text":"Teks coretan",Subscript:"Subskrip",Superscript:"Superskrip",Underline:"Garis bawah","Underline text":"Teks garis bawah"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));