<?php

namespace Drupal\views\Plugin\views\sort;

use Drupal\views\Attribute\ViewsSort;

/**
 * Default implementation of the base sort plugin.
 *
 * @ingroup views_sort_handlers
 */
#[ViewsSort("standard")]
class Standard extends SortPluginBase {

}
