!function(e){const i=e.gl=e.gl||{};i.dictionary=Object.assign(i.dictionary||{},{"Edit source":"Editar a orixe","Empty snippet content":"Contido de fragmento de código baleiro","HTML snippet":"Fragmento HTML","Insert HTML":"Inserir HTML","No preview available":"Non hai vista previa dispoñíbel","Paste raw HTML here...":"Pegar HTML en bruto aquí…","Save changes":"Gardar cambios"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));