!function(e){const i=e.th=e.th||{};i.dictionary=Object.assign(i.dictionary||{},{"Blue marker":"มาร์กเกอร์สีน้ำเงิน","Green marker":"มาร์กเกอร์สีเขียว","Green pen":"ปากกาสีเขียว",Highlight:"ไฮไลต์","Pink marker":"มาร์กเกอร์สีชมพู","Red pen":"ปากกาสีแดง","Remove highlight":"ลบไฮไลต์ออก","Text highlight toolbar":"แถบเครื่องมือไฮไลต์ข้อความ","Yellow marker":"มาร์กเกอร์สีเหลือง"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));