!function(e){const n=e.de=e.de||{};n.dictionary=Object.assign(n.dictionary||{},{"Insert media":"Medium einfügen",Media:"Medien","Media embed":"Medieneinbettung","Media toolbar":"Medien Werkzeugleiste","Media URL":"Medien-URL","media widget":"Medien-Widget","Open media in new tab":"Medien in neuem Tab öffnen","Paste the media URL in the input.":"Medien-URL in das Eingabefeld einfügen.","The URL must not be empty.":"Die URL darf nicht leer sein.","This media URL is not supported.":"Diese Medien-URL wird nicht unterstützt.","Tip: Paste the URL into the content to embed faster.":"Tipp: Zum schnelleren Einbetten können Sie die Medien-URL in den Inhalt einfügen."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));