<?php

namespace Drupal\Component\Plugin\Exception;

/**
 * An exception class to be thrown for context plugin exceptions.
 */
class ContextException extends \Exception implements ExceptionInterface {}
