!function(e){const o=e.pt=e.pt||{};o.dictionary=Object.assign(o.dictionary||{},{Activities:"Atividades","Animals & Nature":"Animais e natureza","Dark skin tone":"Tom de pele escuro","Default skin tone":"Tom de pele padrão",Emoji:"Emoji","Emoji picker":"Seletor de emojis","Find an emoji (min. 2 characters)":"Encontrar um emoji (mín. 2 carateres)",Flags:"Bandeiras","Food & Drinks":"Comida e bebidas","Gestures & People":"Gestos e pessoas","Keep on typing to see the emoji.":"Continue a escrever para ver o emoji.","Light skin tone":"Tom de pele claro","Medium Dark skin tone":"Tom de pele médio escuro","Medium Light skin tone":"Tom de pele médio claro","Medium skin tone":"Tom de pele médio",'No emojis were found matching "%0".':'Não foi encontrado nenhum emoji que corresponda a "%0".',Objects:"Objetos","Select skin tone":"Selecionar tom de pele","Show all emoji...":"Mostrar todos os emojis...","Smileys & Expressions":"Smiley e expressões",Symbols:"Símbolos","The query must contain at least two characters.":"A consulta deve conter, pelo menos, dois carateres.","Travel & Places":"Viagens e lugares"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));