!function(e){const t=e.ko=e.ko||{};t.dictionary=Object.assign(t.dictionary||{},{"Insert media":"미디어 삽입",Media:"미디어","Media embed":"미디어 ㅅ","Media toolbar":"미디어 툴바","Media URL":"미디어 URL","media widget":"미디어 위젯","Open media in new tab":"새 탭에서 미디어 열기","Paste the media URL in the input.":"미디어 URL을 입력해주세요.","The URL must not be empty.":"URL이 비어있을 수 없습니다.","This media URL is not supported.":"이 미디어 URL은 지원되지 않습니다.","Tip: Paste the URL into the content to embed faster.":"팁: URL을 붙여넣으면 더 빨리 삽입할 수 있습니다."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));