!function(e){const t=e.el=e.el||{};t.dictionary=Object.assign(t.dictionary||{},{"Bulleted List":"Λίστα με κουκκίδες","Bulleted list styles toolbar":"Γραμμή εργαλείων στυλ λιστών με κουκίδες",Circle:"Κύκλος",Decimal:"Δεκαδικό","Decimal with leading zero":"Δεκαδικό με μηδενικά","Decrease list item indent":"Μείωση εσοχής στοιχείου λίστας",Disc:"Δίσκος","Entering a to-do list":"Κάνοντας εισαγωγή μιας λίστας υποχρεώσεων","Increase list item indent":"Αύξηση εσοχής στοιχείου λίστας","Invalid start index value.":"Μη έγκυρη τιμή έναρξης ευρετηρίου.","Keystrokes that can be used in a list":"Πλήκτρα που μπορείτε να χρησιμοποιήσετε σε μια λίστα","Leaving a to-do list":"Φεύγοντας από μια λίστα υποχρεώσεων","List properties":"Ιδιότητες λίστας","Lower-latin":"Μικρά λατινικά","Lower–roman":"Μικρά ρωμαϊκά","Numbered List":"Λίστα με αριθμούς","Numbered list styles toolbar":"Γραμμή εργαλείων στυλ λιστών με αριθμούς","Reversed order":"Αντίστροφη σειρά",Square:"Τετράγωνο","Start at":"Εναρξη από","Start index must be greater than 0.":"Ο αριθμός έναρξης πρέπει να είναι μεγαλύτερος από 0.","To-do List":"Λίστα προγραμματισμένων ενεργειών","Toggle the circle list style":"Εναλλαγή του στυλ λίστας με κύκλο","Toggle the decimal list style":"Εναλλαγή του στυλ λίστας με δεκαδικά","Toggle the decimal with leading zero list style":"Εναλλαγή του στυλ λίστας δεκαδικών με μηδενικά","Toggle the disc list style":"Εναλλαγή του στυλ λίστας με δίσκο","Toggle the lower–latin list style":"Εναλλαγή του στυλ λίστας με μικρά λατινικά","Toggle the lower–roman list style":"Εναλλαγή του στυλ λίστας με μικρά ρωμαϊκά","Toggle the square list style":"Εναλαγή του στυλ λίστας με τετράγωνο","Toggle the upper–latin list style":"Εναλλαγή του στυλ λίστας με κεφαλαία λατινικά","Toggle the upper–roman list style":"Εναλλαγή του στυλ λίστας με κεφαλαία ρωμαϊκά","Upper-latin":"Κεφαλαία λατινικά","Upper-roman":"Κεφαλαία ρωμαϊκά"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));