!function(e){const t=e.lv=e.lv||{};t.dictionary=Object.assign(t.dictionary||{},{"Edit source":"Labot pirmavotu","Empty snippet content":"Tukšs fragments","HTML snippet":"HTML fragments","Insert HTML":"Ievietot HTML","No preview available":"Priekšskatījums nav pieejams","Paste raw HTML here...":"Ievietojiet HTML šeit...","Save changes":"Saglabāt izmaiņas"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));