!function(e){const a=e.sq=e.sq||{};a.dictionary=Object.assign(a.dictionary||{},{"Cannot access default workspace.":"","Cannot determine a category for the uploaded file.":"Nuk mund të përcaktohet kategori për skedarin e ngarkuar.","Edit image":"","Failed to determine category of edited image.":"","Open file manager":"Hap menaxhuesin e skedarëve","Processing the edited image.":"","Server failed to process the image.":"","You have no image editing permissions.":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));