<?php

/**
 * @file
 * Generic transliteration data for the PhpTransliteration class.
 */

$base = [
  0x00 => 'jun', 'junj', 'junh', 'jud', 'jul', 'julg', 'julm', 'julb', 'juls', 'jult', 'julp', 'julh', 'jum', 'jub', 'jubs', 'jus',
  0x10 => 'juss', 'jung', 'juj', 'juch', 'juk', 'jut', 'jup', 'juh', 'jwo', 'jwog', 'jwokk', 'jwogs', 'jwon', 'jwonj', 'jwonh', 'jwod',
  0x20 => 'jwol', 'jwolg', 'jwolm', 'jwolb', 'jwols', 'jwolt', 'jwolp', 'jwolh', 'jwom', 'jwob', 'jwobs', 'jwos', 'jwoss', 'jwong', 'jwoj', 'jwoch',
  0x30 => 'jwok', 'jwot', 'jwop', 'jwoh', 'jwe', 'jweg', 'jwekk', 'jwegs', 'jwen', 'jwenj', 'jwenh', 'jwed', 'jwel', 'jwelg', 'jwelm', 'jwelb',
  0x40 => 'jwels', 'jwelt', 'jwelp', 'jwelh', 'jwem', 'jweb', 'jwebs', 'jwes', 'jwess', 'jweng', 'jwej', 'jwech', 'jwek', 'jwet', 'jwep', 'jweh',
  0x50 => 'jwi', 'jwig', 'jwikk', 'jwigs', 'jwin', 'jwinj', 'jwinh', 'jwid', 'jwil', 'jwilg', 'jwilm', 'jwilb', 'jwils', 'jwilt', 'jwilp', 'jwilh',
  0x60 => 'jwim', 'jwib', 'jwibs', 'jwis', 'jwiss', 'jwing', 'jwij', 'jwich', 'jwik', 'jwit', 'jwip', 'jwih', 'jyu', 'jyug', 'jyukk', 'jyugs',
  0x70 => 'jyun', 'jyunj', 'jyunh', 'jyud', 'jyul', 'jyulg', 'jyulm', 'jyulb', 'jyuls', 'jyult', 'jyulp', 'jyulh', 'jyum', 'jyub', 'jyubs', 'jyus',
  0x80 => 'jyuss', 'jyung', 'jyuj', 'jyuch', 'jyuk', 'jyut', 'jyup', 'jyuh', 'jeu', 'jeug', 'jeukk', 'jeugs', 'jeun', 'jeunj', 'jeunh', 'jeud',
  0x90 => 'jeul', 'jeulg', 'jeulm', 'jeulb', 'jeuls', 'jeult', 'jeulp', 'jeulh', 'jeum', 'jeub', 'jeubs', 'jeus', 'jeuss', 'jeung', 'jeuj', 'jeuch',
  0xA0 => 'jeuk', 'jeut', 'jeup', 'jeuh', 'jui', 'juig', 'juikk', 'juigs', 'juin', 'juinj', 'juinh', 'juid', 'juil', 'juilg', 'juilm', 'juilb',
  0xB0 => 'juils', 'juilt', 'juilp', 'juilh', 'juim', 'juib', 'juibs', 'juis', 'juiss', 'juing', 'juij', 'juich', 'juik', 'juit', 'juip', 'juih',
  0xC0 => 'ji', 'jig', 'jikk', 'jigs', 'jin', 'jinj', 'jinh', 'jid', 'jil', 'jilg', 'jilm', 'jilb', 'jils', 'jilt', 'jilp', 'jilh',
  0xD0 => 'jim', 'jib', 'jibs', 'jis', 'jiss', 'jing', 'jij', 'jich', 'jik', 'jit', 'jip', 'jih', 'jja', 'jjag', 'jjakk', 'jjags',
  0xE0 => 'jjan', 'jjanj', 'jjanh', 'jjad', 'jjal', 'jjalg', 'jjalm', 'jjalb', 'jjals', 'jjalt', 'jjalp', 'jjalh', 'jjam', 'jjab', 'jjabs', 'jjas',
  0xF0 => 'jjass', 'jjang', 'jjaj', 'jjach', 'jjak', 'jjat', 'jjap', 'jjah', 'jjae', 'jjaeg', 'jjaekk', 'jjaegs', 'jjaen', 'jjaenj', 'jjaenh', 'jjaed',
];
