!function(e){const i=e.az=e.az||{};i.dictionary=Object.assign(i.dictionary||{},{"Insert media":"Media əlavə ed",Media:"","Media embed":"","Media toolbar":"Media paneli","Media URL":"Media URL","media widget":"media vidgeti","Open media in new tab":"","Paste the media URL in the input.":"Media URL-ni xanaya əlavə edin","The URL must not be empty.":"URL boş olmamalıdır.","This media URL is not supported.":"Bu media URL dəstəklənmir.","Tip: Paste the URL into the content to embed faster.":"Məsləhət: Sürətli qoşma üçün URL-i kontentə əlavə edin"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));