!function(n){const i=n.km=n.km||{};i.dictionary=Object.assign(i.dictionary||{},{"Create link":"","Displayed text":"",Downloadable:"","Edit link":"",Link:"តំណ","Link image":"","Link properties":"","Link URL":"URL តំណ","Link URL must not be empty.":"","Move out of a link":"","No links available":"","Open in a new tab":"","Open link in new tab":"",Unlink:"ផ្ដាច់​តំណ"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));