---
label: 'Setting a default contact form'
related:
  - contact.overview
---
{% set contact_link_text %}{% trans %}Contact forms{% endtrans %}{% endset %}
{% set contact_link = render_var(help_route_link(contact_link_text, 'entity.contact_form.collection')) %}
{% set contact_page_link_text %}{% trans %}the form displayed on the default contact page{% endtrans %}{% endset %}
{% set contact_page_link = render_var(help_route_link(contact_page_link_text, 'contact.site_page')) %}
<h2>{% trans %}Goal{% endtrans %}</h2>
<p>{% trans %}Set a site-wide contact form to be the default contact form ({{ contact_page_link }}; for example, <em>/contact</em>).{% endtrans %}</p>
<h2>{% trans %}Steps{% endtrans %}</h2>
<ol>
  <li>{% trans %}In the <em>Manage</em> administrative menu, navigate to <em>Structure</em> &gt; <em>{{ contact_link }}</em>.{% endtrans %}</li>
  <li>{% trans %}Click <em>Edit</em> for the site-wide form you want to be the default.{% endtrans %}</li>
  <li>{% trans %}Check <em>Make this the default form</em> and click <em>Save</em>.{% endtrans %}</li>
</ol>
