!function(e){const t=e.ca=e.ca||{};t.dictionary=Object.assign(t.dictionary||{},{"Advanced options":"Opcions avançades",Find:"Trobar","Find and replace":"Trobar i substituir","Find in text…":"Trobar al text...","Find in the document":"Busca en el document","Match case":"Coincidència de majúscules i minúscules","Next result":"Següent resultat","Previous result":"Resultat anterior",Replace:"Substituir","Replace all":"Substituir-ho tot","Replace with…":"Substituir per...","Text to find must not be empty.":"El text per cercar no pot estar buit.","Tip: Find some text first in order to replace it.":"Consell: Troba primer un text per substituir-lo.","Whole words only":"Només paraules senceres"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));