!function(n){const o=n.da=n.da||{};o.dictionary=Object.assign(o.dictionary||{},{"Choose language":"Vælg sprog",Language:"Sprog","Remove language":"Fjern sprog"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));