!function(o){const a=o.sk=o.sk||{};a.dictionary=Object.assign(a.dictionary||{},{Bookmark:"Záložka","Bookmark must not be empty.":"Záložka nesmie byť prázdna.","Bookmark name":"Názov záložky","Bookmark name already exists.":"Názov záložky už existuje.","Bookmark name cannot contain space characters.":"Názov záložky nemôže obsahovať medzery.","Bookmark toolbar":"Panel s nástrojmi záložky","bookmark widget":"widget pre záložky",Bookmarks:"Záložky","Edit bookmark":"Upraviť záložku","Enter the bookmark name without spaces.":"Zadajte názov záložky bez medzier.","No bookmarks available.":"Nie sú k dispozícii žiadne záložky.","Remove bookmark":"Odstrániť záložku","Scroll to bookmark":"Prejdite na záložku"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));