!function(i){const n=i.fr=i.fr||{};n.dictionary=Object.assign(n.dictionary||{},{"Create link":"Créer un lien","Displayed text":"Texte affiché",Downloadable:"Fichier téléchargeable","Edit link":"Modifier le lien",Link:"Lien","Link image":"Lien d'image","Link properties":"Propriétés du lien","Link URL":"URL du lien","Link URL must not be empty.":"L'URL du lien ne doit pas être vide.","Move out of a link":"Sortir d'un lien","No links available":"Aucun lien disponible","Open in a new tab":"Ouvrir dans un nouvel onglet","Open link in new tab":"Ouvrir le lien dans un nouvel onglet",Unlink:"Supprimer le lien"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));