!function(o){const n=o.si=o.si||{};n.dictionary=Object.assign(n.dictionary||{},{Big:"",Default:"","Document colors":"","Font Background Color":"","Font Color":"","Font Family":"","Font Size":"",Huge:"",Small:"",Tiny:""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));