{#
/**
 * @file
 * Default theme implementation for a page wrapper.
 *
 * For consistent wrapping to {{ page }} render in all themes. The
 * "data-off-canvas-main-canvas" attribute is required by the off-canvas dialog.
 * This is used by the core/drupal.dialog.off_canvas library to select the
 * "main canvas" page element as opposed to the "off canvas" which is the dialog
 * itself. The "main canvas" element must be resized according to the width of
 * the "off canvas" dialog so that no portion of the "main canvas" is obstructed
 * by the off-canvas dialog. The off-canvas dialog can vary in width when opened
 * and can be resized by the user. The "data-off-canvas-main-canvas" attribute
 * cannot be removed without breaking the off-canvas dialog functionality.
 *
 * Available variables:
 * - children: Contains the child elements of the page.
 *
 * @ingroup themeable
 */
#}
{% if children %}
  <div class="dialog-off-canvas-main-canvas" data-off-canvas-main-canvas>
    {{ children }}
  </div>
{% endif %}
