!function(o){const e=o.sk=o.sk||{};e.dictionary=Object.assign(e.dictionary||{},{"Could not insert image at the current position.":"Obrázok nie je možné vložiť na vybranú pozíciu.","Could not obtain resized image URL.":"Nepodarilo sa získať URL adresu zmenšeného obrázka.","Insert image or file":"Vložiť obrázok alebo súbor","Inserting image failed":"Vloženie obrázka zlyhalo","Selecting resized image failed":"Vybranie zmenšeného obrázka zlyhalo"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));