!function(t){const i=t.tt=t.tt||{};i.dictionary=Object.assign(i.dictionary||{},{Bold:"Калын","Bold text":"Калын текст",Code:"Код",Italic:"Курсив","Italic text":"Курсив текст","Move out of an inline code style":"",Strikethrough:"Сызылган","Strikethrough text":"Сызылган текст",Subscript:"",Superscript:"",Underline:"Ассызыклау","Underline text":"Ассызыклы текст"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));