<?php

/**
 * @file
 * Generic transliteration data for the PhpTransliteration class.
 */

$base = [
  0x00 => 'tong', 'yu', 'xiang', 'nao', 'shun', 'fen', 'pu', 'ling', 'ao', 'huan', 'yi', 'huan', 'meng', 'ying', 'lei', 'yan',
  0x10 => 'bao', 'die', 'ling', 'shi', 'jiao', 'lie', 'jing', 'ju', 'ti', 'pi', 'gang', 'xiao', 'wai', 'chuai', 'di', 'huan',
  0x20 => 'yao', 'li', 'mi', 'hu', 'sheng', 'jia', 'yin', 'wei', NULL, 'piao', 'lu', 'ling', 'yi', 'cai', 'shan', 'hu',
  0x30 => 'shu', 'tuo', 'mo', 'hua', 'tie', 'bing', 'peng', 'hun', 'fu', 'guo', 'bu', 'li', 'chan', 'pi', 'cuo', 'meng',
  0x40 => 'suo', 'qiang', 'zhi', 'kuang', 'bi', 'ao', 'meng', 'xian', 'ku', 'tou', 'tuan', 'wei', 'xian', NULL, 'tuan', 'lao',
  0x50 => 'chan', 'ni', 'ni', 'li', 'dong', 'ju', 'qian', 'bo', 'shai', 'zha', 'tao', 'qian', 'nong', 'yi', 'jing', 'gan',
  0x60 => 'di', 'jian', 'mei', 'da', 'jian', 'yu', 'xie', 'zai', 'mang', 'li', 'gun', 'xun', 'ta', 'zhe', 'yang', 'tuan',
  0x70 => 'shang', 'xi', 'qiao', 'wei', 'ying', 'chua', 'qu', 'wa', NULL, 'zhi', 'ting', 'gu', 'shang', 'ca', 'fu', 'tie',
  0x80 => 'ta', 'ta', 'zhuo', 'han', 'ping', 'he', 'zhui', 'zhou', 'bo', 'liu', 'nu', 'xi', 'pao', 'di', 'he', 'ti',
  0x90 => 'wai', 'ti', 'qi', 'ji', 'chi', 'ba', 'jin', 'ke', 'li', 'ju', 'qu', 'la', 'gu', 'qia', 'qi', 'xian',
  0xA0 => 'jian', 'shi', 'jian', 'ai', 'hua', 'zha', 'ze', 'yao', 'zhan', 'ji', 'cha', 'yan', 'jian', NULL, 'yan', NULL,
  0xB0 => 'jiao', 'tong', 'nan', 'yue', NULL, 'chi', NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  0xC0 => NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  0xD0 => NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  0xE0 => NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  0xF0 => NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
];
