!function(i){const k=i.fi=i.fi||{};k.dictionary=Object.assign(k.dictionary||{},{Bookmark:"Kirjanmerkki","Bookmark must not be empty.":"Kirjanmerkkiä ei saa jättää tyhjäksi.","Bookmark name":"Kirjanmerkin nimi","Bookmark name already exists.":"Kirjanmerkin nimi on jo käytössä.","Bookmark name cannot contain space characters.":"Kirjanmerkin nimessä ei saa olla välilyöntejä.","Bookmark toolbar":"Kirjanmerkkien työkalupalkki","bookmark widget":"kirjanmerkkien pienoisohjelma",Bookmarks:"Kirjanmerkit","Edit bookmark":"Muokkaa kirjanmerkkiä","Enter the bookmark name without spaces.":"Lisää kirjanmerkin nimi ilman välilyöntejä","No bookmarks available.":"Kirjanmerkkejä ei käytettävissä.","Remove bookmark":"Poista kirjanmerkki","Scroll to bookmark":"Vieritä kirjanmerkkiin"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));