<?php

/**
 * @file
 * Generic transliteration data for the PhpTransliteration class.
 */

$base = [
  0x00 => 'poss', 'pong', 'poj', 'poch', 'pok', 'pot', 'pop', 'poh', 'pwa', 'pwag', 'pwakk', 'pwags', 'pwan', 'pwanj', 'pwanh', 'pwad',
  0x10 => 'pwal', 'pwalg', 'pwalm', 'pwalb', 'pwals', 'pwalt', 'pwalp', 'pwalh', 'pwam', 'pwab', 'pwabs', 'pwas', 'pwass', 'pwang', 'pwaj', 'pwach',
  0x20 => 'pwak', 'pwat', 'pwap', 'pwah', 'pwae', 'pwaeg', 'pwaekk', 'pwaegs', 'pwaen', 'pwaenj', 'pwaenh', 'pwaed', 'pwael', 'pwaelg', 'pwaelm', 'pwaelb',
  0x30 => 'pwaels', 'pwaelt', 'pwaelp', 'pwaelh', 'pwaem', 'pwaeb', 'pwaebs', 'pwaes', 'pwaess', 'pwaeng', 'pwaej', 'pwaech', 'pwaek', 'pwaet', 'pwaep', 'pwaeh',
  0x40 => 'poe', 'poeg', 'poekk', 'poegs', 'poen', 'poenj', 'poenh', 'poed', 'poel', 'poelg', 'poelm', 'poelb', 'poels', 'poelt', 'poelp', 'poelh',
  0x50 => 'poem', 'poeb', 'poebs', 'poes', 'poess', 'poeng', 'poej', 'poech', 'poek', 'poet', 'poep', 'poeh', 'pyo', 'pyog', 'pyokk', 'pyogs',
  0x60 => 'pyon', 'pyonj', 'pyonh', 'pyod', 'pyol', 'pyolg', 'pyolm', 'pyolb', 'pyols', 'pyolt', 'pyolp', 'pyolh', 'pyom', 'pyob', 'pyobs', 'pyos',
  0x70 => 'pyoss', 'pyong', 'pyoj', 'pyoch', 'pyok', 'pyot', 'pyop', 'pyoh', 'pu', 'pug', 'pukk', 'pugs', 'pun', 'punj', 'punh', 'pud',
  0x80 => 'pul', 'pulg', 'pulm', 'pulb', 'puls', 'pult', 'pulp', 'pulh', 'pum', 'pub', 'pubs', 'pus', 'puss', 'pung', 'puj', 'puch',
  0x90 => 'puk', 'put', 'pup', 'puh', 'pwo', 'pwog', 'pwokk', 'pwogs', 'pwon', 'pwonj', 'pwonh', 'pwod', 'pwol', 'pwolg', 'pwolm', 'pwolb',
  0xA0 => 'pwols', 'pwolt', 'pwolp', 'pwolh', 'pwom', 'pwob', 'pwobs', 'pwos', 'pwoss', 'pwong', 'pwoj', 'pwoch', 'pwok', 'pwot', 'pwop', 'pwoh',
  0xB0 => 'pwe', 'pweg', 'pwekk', 'pwegs', 'pwen', 'pwenj', 'pwenh', 'pwed', 'pwel', 'pwelg', 'pwelm', 'pwelb', 'pwels', 'pwelt', 'pwelp', 'pwelh',
  0xC0 => 'pwem', 'pweb', 'pwebs', 'pwes', 'pwess', 'pweng', 'pwej', 'pwech', 'pwek', 'pwet', 'pwep', 'pweh', 'pwi', 'pwig', 'pwikk', 'pwigs',
  0xD0 => 'pwin', 'pwinj', 'pwinh', 'pwid', 'pwil', 'pwilg', 'pwilm', 'pwilb', 'pwils', 'pwilt', 'pwilp', 'pwilh', 'pwim', 'pwib', 'pwibs', 'pwis',
  0xE0 => 'pwiss', 'pwing', 'pwij', 'pwich', 'pwik', 'pwit', 'pwip', 'pwih', 'pyu', 'pyug', 'pyukk', 'pyugs', 'pyun', 'pyunj', 'pyunh', 'pyud',
  0xF0 => 'pyul', 'pyulg', 'pyulm', 'pyulb', 'pyuls', 'pyult', 'pyulp', 'pyulh', 'pyum', 'pyub', 'pyubs', 'pyus', 'pyuss', 'pyung', 'pyuj', 'pyuch',
];
