!function(t){const n=t.cs=t.cs||{};n.dictionary=Object.assign(n.dictionary||{},{Bold:"Tučné","Bold text":"Tučně",Code:"Kódový blok",Italic:"Kurzíva","Italic text":"Kurzíva","Move out of an inline code style":"Odejít ze stylu vloženého kódu",Strikethrough:"Přeškrtnuté","Strikethrough text":"Přešktnutí",Subscript:"Dolní index",Superscript:"Horní index",Underline:"Podtržené","Underline text":"Podtrhnutí"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));