!function(i){const e=i.bn=i.bn||{};e.dictionary=Object.assign(e.dictionary||{},{"Could not insert image at the current position.":"বর্তমান অবস্থানে ছবি ঢোকানো যাচ্ছে না।","Could not obtain resized image URL.":"আকার পরিবর্তন করা ছবির URL পাওয়া যাচ্ছে না।","Insert image or file":"ছবি অথবা ফাইল ঢোকান","Inserting image failed":"ছবি ঢোকানো ব্যর্থ হয়েছে।","Selecting resized image failed":"আকার পরিবর্তন করা ছবি নির্বাচন করা ব্যর্থ হয়েছে"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));