!function(e){const n=e.he=e.he||{};n.dictionary=Object.assign(n.dictionary||{},{"Connected users":"משתמשים מחוברים","Document outline":"מפת מסמך","Enter fullscreen mode":"היכנס למצב מסך מלא","Fullscreen mode":"מצב מסך מלא","Hide left sidebar":"הסתר סרגל צד שמאלי","Leave fullscreen mode":"צא ממצב מסך מלא","Show left sidebar":"הצג סרגל צד שמאלי","Toggle fullscreen mode":"החלף מצב מסך מלא","Toggle sidebar":"פתיחת/סגירת סרגל הצד"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));