!function(e){const a=e.ms=e.ms||{};a.dictionary=Object.assign(a.dictionary||{},{"Cannot access default workspace.":"Tidak dapat mengakses ruang kerja lalai.","Cannot determine a category for the uploaded file.":"Gagal menentukan kategori bagi fail yang dimuat naik.","Edit image":"Sunting imej","Failed to determine category of edited image.":"Gagal menentukan kategori imej yang disunting.","Open file manager":"Buka pengurus fail","Processing the edited image.":"Memproses imej yang telah disunting.","Server failed to process the image.":"Pelayan gagal memproses imej.","You have no image editing permissions.":"Anda tiada kebenaran untuk mengedit imej."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));