!function(i){const n=i.uz=i.uz||{};n.dictionary=Object.assign(n.dictionary||{},{"Edit source":"Kodni o'zgartirish","Empty snippet content":"","HTML snippet":"HTML snippet","Insert HTML":"HTML kiritish","No preview available":"","Paste raw HTML here...":"HTML kodini shu yerga joylashtiring...","Save changes":"O'zgarishlarni saqlash"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));