!function(n){const i=n.vi=n.vi||{};i.dictionary=Object.assign(i.dictionary||{},{"Create link":"Tạo liên kết","Displayed text":"Văn bản đã hiển thị",Downloadable:"Có thể tải về","Edit link":"Sửa liên kết",Link:"Chèn liên kết","Link image":"Liên kết của ảnh","Link properties":"Thuộc tính đường liên kết","Link URL":"Đường dẫn liên kết","Link URL must not be empty.":"Không được để trống URL đường liên kết.","Move out of a link":"Di chuyển ra khỏi một liên kết","No links available":"Không có đường liên kết khả dụng","Open in a new tab":"Mở trên tab mới","Open link in new tab":"Mở liên kết",Unlink:"Bỏ liên kết"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));