!function(o){const e=o.es=o.es||{};e.dictionary=Object.assign(e.dictionary||{},{Big:"Grande",Default:"Por defecto","Document colors":"Colores del documento","Font Background Color":"Color de Fondo","Font Color":"Color de Fuente","Font Family":"Fuente","Font Size":"Tamaño de fuente",Huge:"Enorme",Small:"Pequeño",Tiny:"Minúsculo"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));