!function(i){const e=i.it=i.it||{};e.dictionary=Object.assign(e.dictionary||{},{"Could not insert image at the current position.":"Non è stato possibile inserire l'immagine nella posizione corrente.","Could not obtain resized image URL.":"Non è stato possibile ottenere l'URL dell'immagine ridimensionata.","Insert image or file":"Inserisci immagine o file","Inserting image failed":"L'inserimento dell'immagine è fallito","Selecting resized image failed":"La selezione dell'immagine ridimensionata è fallita"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));