!function(t){const e=t["sr-latn"]=t["sr-latn"]||{};e.dictionary=Object.assign(e.dictionary||{},{Bold:"Podebljano","Bold text":"Zadebljani tekst",Code:"Kod",Italic:"Kurziv","Italic text":"Kurziv tekst","Move out of an inline code style":"Izađite iz inline stila koda",Strikethrough:"Precrtan","Strikethrough text":"Precrtan tekst",Subscript:"Index dole",Superscript:"Index gore",Underline:"Podvučen","Underline text":"Podvuci tekst"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));