<?php

declare(strict_types=1);

namespace Drupal\Tests\config\Functional;

use Drupal\Tests\BrowserTestBase;
use PHPUnit\Framework\Attributes\Group;
use PHPUnit\Framework\Attributes\RunTestsInSeparateProcesses;

/**
 * Tests configuration entity status UI functionality.
 */
#[Group('config')]
#[RunTestsInSeparateProcesses]
class ConfigEntityStatusUITest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['config_test'];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Tests status operations.
   */
  public function testCRUD(): void {
    $this->drupalLogin($this->drupalCreateUser([
      'administer site configuration',
    ]));

    $id = $this->randomMachineName();
    $edit = [
      'id' => $id,
      'label' => $this->randomMachineName(),
    ];
    $this->drupalGet('admin/structure/config_test/add');
    $this->submitForm($edit, 'Save');

    $entity = \Drupal::entityTypeManager()->getStorage('config_test')->load($id);

    // Disable an entity.
    $disable_url = $entity->toUrl('disable');
    $this->assertSession()->linkByHrefExists($disable_url->toString());
    $this->drupalGet($disable_url);
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->linkByHrefNotExists($disable_url->toString());

    // Enable an entity.
    $enable_url = $entity->toUrl('enable');
    $this->assertSession()->linkByHrefExists($enable_url->toString());
    $this->drupalGet($enable_url);
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->linkByHrefNotExists($enable_url->toString());
  }

}
