!function(e){const i=e.nl=e.nl||{};i.dictionary=Object.assign(i.dictionary||{},{Activities:"Activiteiten","Animals & Nature":"Dieren & natuur","Dark skin tone":"Donkere huidskleur","Default skin tone":"Standaard huidskleur",Emoji:"Emoji","Emoji picker":"Emoji-kiezer","Find an emoji (min. 2 characters)":"Vind een emoji (min. 2 tekens)",Flags:"Vlaggen","Food & Drinks":"Eten & drinken","Gestures & People":"Gebaren & mensen","Keep on typing to see the emoji.":"Blijf typen om de emoji te zien.","Light skin tone":"Lichte huidskleur","Medium Dark skin tone":"Redelijk donkere huidskleur","Medium Light skin tone":"Redelijk lichte huidskleur","Medium skin tone":"Gemiddeld getinte huidskleur",'No emojis were found matching "%0".':'Er zijn geen emoji gevonden overeenkomend met "%0".',Objects:"Objecten","Select skin tone":"Selecteer huidskleur","Show all emoji...":"Toon alle emoji...","Smileys & Expressions":"Smileys & gezichtsuitdrukkingen",Symbols:"Symbolen","The query must contain at least two characters.":"De zoekopdracht moet ten minste 2 tekens bevatten.","Travel & Places":"Reizen & plaatsen"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));