!function(e){const i=e["sr-latn"]=e["sr-latn"]||{};i.dictionary=Object.assign(i.dictionary||{},{"Advanced options":"Napredne opcije",Find:"Pronađji","Find and replace":"Nađji i zameni","Find in text…":"Pronađji u tekstu…","Find in the document":"Pronađite u dokumentu","Match case":"Podudaranje slučaj","Next result":"Sledeći rezultat","Previous result":"Prethodni rezultat",Replace:"Zameni","Replace all":"Zameni sve","Replace with…":"Zameni sa…","Text to find must not be empty.":"Tekst za pronalaženje ne sme biti prazan.","Tip: Find some text first in order to replace it.":"Savet: Prvo pronađjite neki tekst da biste ga zamenili.","Whole words only":"Samo cele reči"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));