!function(e){const a=e.ca=e.ca||{};a.dictionary=Object.assign(a.dictionary||{},{"Connected users":"Usuaris connectats","Document outline":"Esquema del document","Enter fullscreen mode":"Mode Entra a la pantalla completa","Fullscreen mode":"Mode de pantalla completa","Hide left sidebar":"Amagar el menú lateral esquerra","Leave fullscreen mode":"Surt del mode de pantalla completa","Show left sidebar":"Mostrar el menú lateral esquerra","Toggle fullscreen mode":"Activa el mode de pantalla completa","Toggle sidebar":"Alternar al menú lateral"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));