<?php

/**
 * @file
 * Generic transliteration data for the PhpTransliteration class.
 */

$base = [
  0x00 => 'nao', 'xi', NULL, 'dui', 'chi', 'wei', 'zhe', 'gun', 'chao', 'chi', 'zao', 'hui', 'luan', 'liao', 'lao', 'tuo',
  0x10 => 'hui', 'wu', 'ao', 'she', 'sui', 'mai', 'tan', 'xin', 'jing', 'an', 'ta', 'chan', 'wei', 'tuan', 'ji', 'chen',
  0x20 => 'che', 'yu', 'xian', 'xin', NULL, NULL, NULL, 'nao', NULL, 'yan', 'qiu', 'jiang', 'song', 'jun', 'liao', 'ju',
  0x30 => NULL, 'man', 'lie', NULL, 'chu', 'chi', 'xiang', 'qin', 'mei', 'shu', 'chai', 'chi', 'gu', 'yu', 'yin', NULL,
  0x40 => 'liu', 'lao', 'shu', 'zhe', 'shuang', 'hui', NULL, NULL, 'e', NULL, 'sha', 'zong', 'jue', 'jun', 'tuan', 'lou',
  0x50 => 'wei', 'chong', 'zhu', 'lie', NULL, 'zhe', 'zhao', NULL, 'yi', 'chu', 'ni', 'bo', 'suan', 'yi', 'hao', 'ya',
  0x60 => 'huan', 'man', 'man', 'qu', 'lao', 'hao', 'zhong', 'min', 'xian', 'zhen', 'shu', 'zuo', 'zhu', 'gou', 'xuan', 'yi',
  0x70 => 'zhi', 'xie', 'jin', 'can', NULL, 'bu', 'liang', 'zhi', 'ji', 'wan', 'guan', 'ju', 'jing', 'ai', 'fu', 'gui',
  0x80 => 'hou', 'yan', 'ruan', 'zhi', 'biao', 'yi', 'suo', 'die', 'gui', 'sheng', 'xun', 'chen', 'she', 'qing', NULL, NULL,
  0x90 => 'chun', 'hong', 'dong', 'cheng', 'wei', 'ru', 'shu', 'cai', 'ji', 'za', 'qi', 'yan', 'fu', 'yu', 'fu', 'po',
  0xA0 => 'zhi', 'tan', 'zuo', 'che', 'qu', 'you', 'he', 'hou', 'gui', 'e', 'jiang', 'yun', 'tou', 'cun', 'tu', 'fu',
  0xB0 => 'zuo', 'hu', NULL, 'bo', 'zhao', 'jue', 'tang', 'jue', 'fu', 'huang', 'chun', 'yong', 'chui', 'suo', 'chi', 'qian',
  0xC0 => 'cai', 'xiao', 'man', 'can', 'qi', 'jian', 'bi', 'ji', 'zhi', 'zhu', 'qu', 'zhan', 'ji', 'bian', NULL, 'li',
  0xD0 => 'li', 'yue', 'quan', 'cheng', 'fu', 'cha', 'tang', 'shi', 'hang', 'qie', 'qi', 'bo', 'na', 'tou', 'chu', 'cu',
  0xE0 => 'yue', 'zhi', 'chen', 'chu', 'bi', 'meng', 'ba', 'tian', 'min', 'lie', 'feng', 'cheng', 'qiu', 'tiao', 'fu', 'kuo',
  0xF0 => 'jian', NULL, NULL, NULL, 'zhen', 'qiu', 'zuo', 'chi', 'kui', 'lie', 'bei', 'du', 'wu', NULL, 'zhuo', 'lu',
];
