!function(e){const a=e.zh=e.zh||{};a.dictionary=Object.assign(a.dictionary||{},{"Break text":"上及下","Caption for image: %0":"影像標題：%0","Caption for the image":"影像的標題","Centered image":"置中圖片","Change image text alternative":"修改圖片的替代文字",Custom:"自訂","Custom image size":"自訂圖片大小","Enter image caption":"輸入圖片說明","Error during image upload":"圖片上傳期間發生錯誤","From computer":"從電腦","Full size image":"完整尺寸圖片",Image:"圖片","Image from computer":"來自電腦的圖片","Image Resize":"圖片調整大小","Image resize list":"圖片縮放清單","Image toolbar":"圖片工具","Image upload complete":"圖片上傳完成","Image via URL":"透過網址插入圖片","image widget":"圖片小工具","In line":"行中","Insert image":"插入圖片","Insert image via URL":"使用連結插入圖片","Insert via URL":"透過網址插入","Left aligned image":"向左對齊圖片",Original:"原始圖片","Replace from computer":"從電腦替換","Replace image":"替換圖片","Replace image from computer":"從電腦替換圖片","Resize image":"縮放圖片","Resize image (in %0)":"調整圖片大小（單位為 %0）","Resize image to %0":"縮放圖片到 %0","Resize image to the original size":"縮放圖片到原始尺寸","Right aligned image":"向右對齊圖片","Side image":"側邊圖片","Text alternative":"替代文字","Text Alternative":"文字替代","The value must not be empty.":"數值不得為空白。","The value should be a plain number.":"數值應為純數字。","Update image URL":"更新圖片連結","Upload failed":"上傳失敗","Upload from computer":"從電腦上傳","Upload image from computer":"從電腦上傳圖片","Uploading image":"正在上傳圖片","Via URL":"透過網址","Wrap text":"文繞圖","You have no image upload permissions.":"您沒有圖片上傳權限。"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));