!function(n){const e=n.bn=n.bn||{};e.dictionary=Object.assign(e.dictionary||{},{"Edit source":"উৎস এডিট করুন","Empty snippet content":"খালি স্নিপেট বিষয়বস্তু","HTML snippet":"HTML স্নিপেট","Insert HTML":"HTML ঢোকান","No preview available":"কোনো প্রিভিউ উপলব্ধ নেই","Paste raw HTML here...":"এখানে সারি HTML পেস্ট করুন...","Save changes":"পরিবর্তনগুলো সংরক্ষন"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));