<?php

/**
 * @file
 * Generic transliteration data for the PhpTransliteration class.
 */

$base = [
  0x00 => 'hwen', 'hwenj', 'hwenh', 'hwed', 'hwel', 'hwelg', 'hwelm', 'hwelb', 'hwels', 'hwelt', 'hwelp', 'hwelh', 'hwem', 'hweb', 'hwebs', 'hwes',
  0x10 => 'hwess', 'hweng', 'hwej', 'hwech', 'hwek', 'hwet', 'hwep', 'hweh', 'hwi', 'hwig', 'hwikk', 'hwigs', 'hwin', 'hwinj', 'hwinh', 'hwid',
  0x20 => 'hwil', 'hwilg', 'hwilm', 'hwilb', 'hwils', 'hwilt', 'hwilp', 'hwilh', 'hwim', 'hwib', 'hwibs', 'hwis', 'hwiss', 'hwing', 'hwij', 'hwich',
  0x30 => 'hwik', 'hwit', 'hwip', 'hwih', 'hyu', 'hyug', 'hyukk', 'hyugs', 'hyun', 'hyunj', 'hyunh', 'hyud', 'hyul', 'hyulg', 'hyulm', 'hyulb',
  0x40 => 'hyuls', 'hyult', 'hyulp', 'hyulh', 'hyum', 'hyub', 'hyubs', 'hyus', 'hyuss', 'hyung', 'hyuj', 'hyuch', 'hyuk', 'hyut', 'hyup', 'hyuh',
  0x50 => 'heu', 'heug', 'heukk', 'heugs', 'heun', 'heunj', 'heunh', 'heud', 'heul', 'heulg', 'heulm', 'heulb', 'heuls', 'heult', 'heulp', 'heulh',
  0x60 => 'heum', 'heub', 'heubs', 'heus', 'heuss', 'heung', 'heuj', 'heuch', 'heuk', 'heut', 'heup', 'heuh', 'hui', 'huig', 'huikk', 'huigs',
  0x70 => 'huin', 'huinj', 'huinh', 'huid', 'huil', 'huilg', 'huilm', 'huilb', 'huils', 'huilt', 'huilp', 'huilh', 'huim', 'huib', 'huibs', 'huis',
  0x80 => 'huiss', 'huing', 'huij', 'huich', 'huik', 'huit', 'huip', 'huih', 'hi', 'hig', 'hikk', 'higs', 'hin', 'hinj', 'hinh', 'hid',
  0x90 => 'hil', 'hilg', 'hilm', 'hilb', 'hils', 'hilt', 'hilp', 'hilh', 'him', 'hib', 'hibs', 'his', 'hiss', 'hing', 'hij', 'hich',
  0xA0 => 'hik', 'hit', 'hip', 'hih', NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  0xB0 => NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  0xC0 => NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  0xD0 => NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  0xE0 => NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  0xF0 => NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
];
