!function(t){const i=t.th=t.th||{};i.dictionary=Object.assign(i.dictionary||{},{Bold:"ตัวหนา","Bold text":"ทำข้อความเป็นตัวหนา",Code:"โค้ด",Italic:"ตัวเอียง","Italic text":"ข้อความเอียง","Move out of an inline code style":"ย้ายออกจากสไตล์โค้ดแบบอินไลน์",Strikethrough:"ขีดทับ","Strikethrough text":"ขีดทับข้อความ",Subscript:"ตัวห้อย",Superscript:"ตัวยก",Underline:"ขีดเส้นใต้","Underline text":"ขีดเส้นใต้ข้อความ"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));