!function(r){const e=r.da=r.da||{};e.dictionary=Object.assign(e.dictionary||{},{"Blue marker":"Blå markør","Green marker":"Grøn markør","Green pen":"Grøn pen",Highlight:"Fremhæv","Pink marker":"Lyserød markør","Red pen":"Rød pen","Remove highlight":"Fjern fremhævning","Text highlight toolbar":"Tekstfremhævning værktøjslinje","Yellow marker":"Gul markør"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));