!function(e){const t=e.ar=e.ar||{};t.dictionary=Object.assign(t.dictionary||{},{"Align cell text to the bottom":"قم بمحاذاة نص الخلية للاسفل","Align cell text to the center":"قم بمحاذاة نص الخلية إلى المركز","Align cell text to the left":"قم بمحاذاة نص الخلية إلى اليسار","Align cell text to the middle":"قم بمحاذاة نص الخلية إلى المنتصف","Align cell text to the right":"قم بمحاذاة نص الخلية إلى اليمين","Align cell text to the top":"قم بمحاذاة نص الخلية إلى الأعلى","Align table to the left":"قم بمحاذاة الجدول إلى اليسار","Align table to the left with no text wrapping":"محاذاة الجدول إلى اليسار دون التفاف النص","Align table to the left with text wrapping":"محاذاة الجدول إلى اليسار مع التفاف النص","Align table to the right":"قم بمحاذاة الجدول إلى اليمين","Align table to the right with no text wrapping":"محاذاة الجدول إلى اليمين دون التفاف النص","Align table to the right with text wrapping":"محاذاة الجدول إلى اليمين مع التفاف النص",Alignment:"المحاذاة",Background:"الخلفية",Border:"الحدود","Cell properties":"خصائص الخلية","Center table":"قم بمحاذاة الجدول إلى المنتصف","Center table with no text wrapping":"توسيط الجدول دون التفاف النص","Choose table type":"اختر نوع الجدول",Color:"اللون",Column:"عمود","Content table":"جدول المحتويات",Dashed:"متقطع","Delete column":"حذف العمود","Delete row":"حذف الصف",Dimensions:"الابعاد",Dotted:"منقط",Double:"مزدوج","Enter table caption":"أدخل التسمية التوضيحية للجدول",Groove:"إطار محفور","Header column":"عمود عنوان","Header row":"صف عنوان",Height:"الارتفاع","Horizontal text alignment toolbar":"شريط أدوات محاذاة النص الأفقي","Insert a new table row (when in the last cell of a table)":"قمْ بإدراج صف جدول جديد (في آخر خلية من الجدول)","Insert column left":"أدخل العمود إلى اليسار","Insert column right":"أدخل العمود إلى اليمين","Insert row above":"ادراج صف قبل","Insert row below":"ادراج صف بعد","Insert table":"إدراج جدول","Insert table layout":"إدراج تخطيط الجدول",Inset:"منخفض","Justify cell text":"ضبط نص الخلية","Keystrokes that can be used in a table cell":"ضغطة المفاتيح التي يمكن استخدامها في خلية الجدول","Layout table":"جدول التخطيط","Merge cell down":"دمج الخلايا للأسفل","Merge cell left":"دمج الخلايا لليسار","Merge cell right":"دمج الخلايا لليمين","Merge cell up":"دمج الخلايا للأعلى","Merge cells":"دمج الخلايا","Move the selection to the next cell":"انقلْ التحديد إلى الخلية التالية","Move the selection to the previous cell":"انقلْ التحديد إلى الخلية السابقة","Navigate through the table":"تنقّلْ عبر الجدول",None:"لا شيء",Outset:"بارز",Padding:"الحاشية",Ridge:"إطار ناتئ",Row:"صف","Select column":"حدد العمود","Select row":"حدد صفًا",Solid:"صلب","Split cell horizontally":"فصل الخلايا بشكل افقي","Split cell vertically":"فصل الخلايا بشكل عمودي",Style:"أسلوب",Table:"جدول","Table Alignment":"محاذاة الجدول","Table alignment toolbar":"شريط أدوات محاذاة الجدول","Table cell text alignment":"محاذاة نص خلية الجدول","Table layout":"تخطيط الجدول","Table properties":"خصائص الجدول","Table toolbar":"شريط أدوات الجدول","Table type":"نوع الجدول","Table type options":"خيارات نوع الجدول",'The color is invalid. Try "#FF0000" or "rgb(255,0,0)" or "red".':'هذا اللون غير صالح. جرِّب "#FF0000" أو "rgb(255,0,0)" أو "أحمر".','The value is invalid. Try "10px" or "2em" or simply "2".':'هذه القيمة غير صالحة. جرِّب "10px" أو "2em" أو "2" وحسب.',"Vertical text alignment toolbar":"شريط أدوات محاذاة النص العمودي",Width:"العرض"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));