!function(t){const s=t.lv=t.lv||{};s.dictionary=Object.assign(s.dictionary||{},{Bold:"Trekns","Bold text":"Teksts treknrakstā",Code:"Kods",Italic:"Kursīvs","Italic text":"Teksts slīprakstā","Move out of an inline code style":"Iziet no iekļautā koda stila",Strikethrough:"Nosvītrots","Strikethrough text":"Pārsvītrot teksts",Subscript:"Apakšraksts",Superscript:"Augšraksts",Underline:"Pasvītrots","Underline text":"Pasvītrot tekstu"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));