<?php

/**
 * @file
 * Generic transliteration data for the PhpTransliteration class.
 */

$base = [
  0x00 => 'jeo', 'jeog', 'jeokk', 'jeogs', 'jeon', 'jeonj', 'jeonh', 'jeod', 'jeol', 'jeolg', 'jeolm', 'jeolb', 'jeols', 'jeolt', 'jeolp', 'jeolh',
  0x10 => 'jeom', 'jeob', 'jeobs', 'jeos', 'jeoss', 'jeong', 'jeoj', 'jeoch', 'jeok', 'jeot', 'jeop', 'jeoh', 'je', 'jeg', 'jekk', 'jegs',
  0x20 => 'jen', 'jenj', 'jenh', 'jed', 'jel', 'jelg', 'jelm', 'jelb', 'jels', 'jelt', 'jelp', 'jelh', 'jem', 'jeb', 'jebs', 'jes',
  0x30 => 'jess', 'jeng', 'jej', 'jech', 'jek', 'jet', 'jep', 'jeh', 'jyeo', 'jyeog', 'jyeokk', 'jyeogs', 'jyeon', 'jyeonj', 'jyeonh', 'jyeod',
  0x40 => 'jyeol', 'jyeolg', 'jyeolm', 'jyeolb', 'jyeols', 'jyeolt', 'jyeolp', 'jyeolh', 'jyeom', 'jyeob', 'jyeobs', 'jyeos', 'jyeoss', 'jyeong', 'jyeoj', 'jyeoch',
  0x50 => 'jyeok', 'jyeot', 'jyeop', 'jyeoh', 'jye', 'jyeg', 'jyekk', 'jyegs', 'jyen', 'jyenj', 'jyenh', 'jyed', 'jyel', 'jyelg', 'jyelm', 'jyelb',
  0x60 => 'jyels', 'jyelt', 'jyelp', 'jyelh', 'jyem', 'jyeb', 'jyebs', 'jyes', 'jyess', 'jyeng', 'jyej', 'jyech', 'jyek', 'jyet', 'jyep', 'jyeh',
  0x70 => 'jo', 'jog', 'jokk', 'jogs', 'jon', 'jonj', 'jonh', 'jod', 'jol', 'jolg', 'jolm', 'jolb', 'jols', 'jolt', 'jolp', 'jolh',
  0x80 => 'jom', 'job', 'jobs', 'jos', 'joss', 'jong', 'joj', 'joch', 'jok', 'jot', 'jop', 'joh', 'jwa', 'jwag', 'jwakk', 'jwags',
  0x90 => 'jwan', 'jwanj', 'jwanh', 'jwad', 'jwal', 'jwalg', 'jwalm', 'jwalb', 'jwals', 'jwalt', 'jwalp', 'jwalh', 'jwam', 'jwab', 'jwabs', 'jwas',
  0xA0 => 'jwass', 'jwang', 'jwaj', 'jwach', 'jwak', 'jwat', 'jwap', 'jwah', 'jwae', 'jwaeg', 'jwaekk', 'jwaegs', 'jwaen', 'jwaenj', 'jwaenh', 'jwaed',
  0xB0 => 'jwael', 'jwaelg', 'jwaelm', 'jwaelb', 'jwaels', 'jwaelt', 'jwaelp', 'jwaelh', 'jwaem', 'jwaeb', 'jwaebs', 'jwaes', 'jwaess', 'jwaeng', 'jwaej', 'jwaech',
  0xC0 => 'jwaek', 'jwaet', 'jwaep', 'jwaeh', 'joe', 'joeg', 'joekk', 'joegs', 'joen', 'joenj', 'joenh', 'joed', 'joel', 'joelg', 'joelm', 'joelb',
  0xD0 => 'joels', 'joelt', 'joelp', 'joelh', 'joem', 'joeb', 'joebs', 'joes', 'joess', 'joeng', 'joej', 'joech', 'joek', 'joet', 'joep', 'joeh',
  0xE0 => 'jyo', 'jyog', 'jyokk', 'jyogs', 'jyon', 'jyonj', 'jyonh', 'jyod', 'jyol', 'jyolg', 'jyolm', 'jyolb', 'jyols', 'jyolt', 'jyolp', 'jyolh',
  0xF0 => 'jyom', 'jyob', 'jyobs', 'jyos', 'jyoss', 'jyong', 'jyoj', 'jyoch', 'jyok', 'jyot', 'jyop', 'jyoh', 'ju', 'jug', 'jukk', 'jugs',
];
