!function(t){const e=t.ca=t.ca||{};e.dictionary=Object.assign(e.dictionary||{},{Bold:"Negreta","Bold text":"Text en negreta",Code:"Codi",Italic:"Cursiva","Italic text":"Text en cursiva","Move out of an inline code style":"Surt d'un codi d'estil en línia",Strikethrough:"Marcat","Strikethrough text":"Barra el text",Subscript:"Subíndex",Superscript:"Superíndex",Underline:"Subrallat","Underline text":"Subratlla el text"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));