!function(e){const t=e.he=e.he||{};t.dictionary=Object.assign(t.dictionary||{},{"Insert media":"הכנס מדיה",Media:"מדיה","Media embed":"הטמעת מדיה","Media toolbar":"סרגל כלים של מדיה","Media URL":"כתובת URL של מדיה","media widget":"יישומון מדיה","Open media in new tab":"פתח מדיה בכרטיסיה חדשה","Paste the media URL in the input.":"הדביקו את כתובת ה-URL של המדיה בקלט.","The URL must not be empty.":"כתובת URL לא יכולה להיות ריקה.","This media URL is not supported.":"כתובת ה-URL של מדיה זו אינה נתמכת.","Tip: Paste the URL into the content to embed faster.":"טיפ: הדביקו את כתובת ה-URL לתוך התוכן כדי להטמיע מהר יותר."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));