!function(e){const i=e.es=e.es||{};i.dictionary=Object.assign(i.dictionary||{},{"Could not insert image at the current position.":"No se pudo insertar la imagen en la posición actual.","Could not obtain resized image URL.":"No se pudo obtener el URL de la imagen redimensionada.","Insert image or file":"Insertar imagen o archivo","Inserting image failed":"Error al insertar la imagen","Selecting resized image failed":"No se pudo seleccionar la imagen redimensionada"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));