!function(a){const n=a.tk=a.tk||{};n.dictionary=Object.assign(n.dictionary||{},{"Create link":"","Displayed text":"",Downloadable:"Göçürip alyp bolýar","Edit link":"Baglanyşygy üýtget",Link:"Baglanyşyk","Link image":"Surat baglanyşygy","Link properties":"","Link URL":"URL baglanyşygy","Link URL must not be empty.":"","Move out of a link":"","No links available":"","Open in a new tab":"Täze goýmada açyň","Open link in new tab":"Täze goýmada baglanyşyk açyň",Unlink:"Baglanyşygy aýyr"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));