!function(e){const n=e["es-co"]=e["es-co"]||{};n.dictionary=Object.assign(n.dictionary||{},{"Code block":"","Entering %0 code snippet":"","Entering code snippet":"","Insert code block":"Insertar bloque de código","Leaving %0 code snippet":"","Leaving code snippet":"","Plain text":"Texto plano"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));