!function(e){const i=e.he=e.he||{};i.dictionary=Object.assign(i.dictionary||{},{"Cannot access default workspace.":"אין גישה למרחב העבודה של ברירת המחדל.","Cannot determine a category for the uploaded file.":"לא ניתן לקבוע קטגוריה עבור הקובץ שהועלה.","Edit image":"עריכת תמונה","Failed to determine category of edited image.":"קביעת הקטגוריה של התמונה הערוכה נכשלה.","Open file manager":"פתח את מנהל הקבצים","Processing the edited image.":"מעבד את התמונה הערוכה.","Server failed to process the image.":"השרת לא הצליח לעבד את התמונה.","You have no image editing permissions.":"אין לך הרשאות עריכת תמונה."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));