!function(e){const i=e.ro=e.ro||{};i.dictionary=Object.assign(i.dictionary||{},{"Blue marker":"Evidențiator albastru","Green marker":"Evidențiator verde","Green pen":"Pix verde",Highlight:"Evidențiere text","Pink marker":"Evidențiator roz","Red pen":"Pix roșu","Remove highlight":"Șterge evidențiere text","Text highlight toolbar":"Bară evidențiere text","Yellow marker":"Evidențiator galben"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));