{#
/**
 * @file
 * Default theme implementation for an administrative page.
 *
 * Available variables:
 * - system_compact_link: Themed link to toggle compact view.
 * - containers: A list of administrative blocks keyed by position: left or
 *   right. Contains:
 *   - blocks: A list of blocks within a container.
 *
 * @see \Drupal\system\Theme\SystemAdminThemePreprocess::preprocessAdminPage()
 *
 * @ingroup themeable
 */
#}

<div class="clearfix">
  {{ system_compact_link }}
  {% for container in containers %}
    <div class="layout-column layout-column--half">
      {% for block in container.blocks %}
        {{ block }}
      {% endfor %}
    </div>
  {% endfor %}
</div>
