!function(e){const a=e.bn=e.bn||{};a.dictionary=Object.assign(a.dictionary||{},{"Break text":"টেক্সট ভেঙ্গে ফেলুন","Caption for image: %0":"ছবির জন্য ক্যাপশন: %0","Caption for the image":"ছবির জন্য ক্যাপশন","Centered image":"কেন্দ্রীভূত ছবি","Change image text alternative":"ছবির টেক্সট বিকল্প পরিবর্তন",Custom:"কাস্টম","Custom image size":"কাস্টম ইমেজ সাইজ","Enter image caption":"ছবির ক্যাপশন লিখুন","Error during image upload":"ইমেজ আপলোডের সময় ত্রুটি","From computer":"কম্পিউটার থেকে","Full size image":"সম্পূর্ণ আকারের ছবি",Image:"ইমেজ","Image from computer":"কম্পিউটার থেকে ছবি","Image Resize":"ছবি আকার পরিবর্তন","Image resize list":"ছবির আকার পরিবর্তনের তালিকা","Image toolbar":"ছবির টুলবার","Image upload complete":"ইমেজ আপলোড সম্পূর্ণ হয়েছে","Image via URL":"URL-এর মাধ্যমে ইমেজ","image widget":"ছবির উইজেট","In line":"সঙ্গতিপূর্ণভাবে","Insert image":"ছবি ঢোকান","Insert image via URL":"URL এর মাধ্যমে ছবি ঢোকান","Insert via URL":"URL-এর মাধ্যমে প্রবেশ করান","Left aligned image":"বাম সারিবদ্ধ ছবি",Original:"মূল","Replace from computer":"কম্পিউটার থেকে প্রতিস্থাপন করুন","Replace image":"ছবি প্রতিস্থাপন করুন","Replace image from computer":"কম্পিউটার থেকে ছবি প্রতিস্থাপন করুন","Resize image":"ছবির আকার পরিবর্তন করুন","Resize image (in %0)":"(%0 এ) ইমেজ রিসাইজ করুন","Resize image to %0":"ছবি %0 আকার পরিবর্তন করুন","Resize image to the original size":"মূল আকারে ছবির আকার পরিবর্তন করুন","Right aligned image":"ডানে সারিবদ্ধ ছবি","Side image":"পার্শ্ব ছবি","Text alternative":"টেক্সট বিকল্প","Text Alternative":"টেক্সট বিকল্প","The value must not be empty.":"মানটি খালি রাখা যাবে না।","The value should be a plain number.":"মানটি একটি সরল সংখ্যা হতে হবে।","Update image URL":"ছবির URL আপডেট করুন","Upload failed":"আপলোড ব্যর্থ হয়েছে","Upload from computer":"কম্পিউটার থেকে আপলোড করুন","Upload image from computer":"কম্পিউটার থেকে ছবি আপলোড করুন","Uploading image":"ইমেজ আপলোড হচ্ছে","Via URL":"URL-এর মাধ্যমে","Wrap text":"টেক্সট মোড়ানো","You have no image upload permissions.":"আপনার কোনও ইমেজ আপলোডের অনুমতি নেই।"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));