!function(t){const i=t.ko=t.ko||{};i.dictionary=Object.assign(i.dictionary||{},{Bold:"굵게","Bold text":"굵은 텍스트",Code:"코드",Italic:"기울임꼴","Italic text":"기울인 텍스트","Move out of an inline code style":"인라인 코드 스타일 밖으로 이동",Strikethrough:"취소선","Strikethrough text":"텍스트 취소선",Subscript:"아래 첨자",Superscript:"위 첨자",Underline:"밑줄","Underline text":"텍스트 밑줄"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));