!function(a){const o=a.ro=a.ro||{};o.dictionary=Object.assign(o.dictionary||{},{Bookmark:"Marcaj","Bookmark must not be empty.":"Marcajul nu poate fi gol.","Bookmark name":"Numele marcajului","Bookmark name already exists.":"Numele marcajului există deja.","Bookmark name cannot contain space characters.":"Numele marcajului nu poate conține spații.","Bookmark toolbar":"Bara de instrumente Marcaj","bookmark widget":"widget marcaj",Bookmarks:"Marcaje","Edit bookmark":"Editează marcajul","Enter the bookmark name without spaces.":"Introdu numele marcajului fără spații.","No bookmarks available.":"Nu sunt disponibile marcaje.","Remove bookmark":"Elimină marcajul","Scroll to bookmark":"Derulați la marcaj"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));