!function(e){const i=e.nl=e.nl||{};i.dictionary=Object.assign(i.dictionary||{},{"Disable block editing":"Blokbewerking uitschakelen","Disable editing":"Bewerken uitschakelen","Disable inline editing":"Inline-bewerking uitschakelen","Enable block editing":"Blokbewerking inschakelen","Enable editing":"Bewerken inschakelen","Enable inline editing":"Inline-bewerking inschakelen","Navigate editable regions":"Navigeer bewerkbare gebieden","Next editable region":"Volgend bewerkbaar gebied","Previous editable region":"Vorig bewerkbaar gebied"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));