!function(i){const n=i.vi=i.vi||{};n.dictionary=Object.assign(n.dictionary||{},{"Could not insert image at the current position.":"Không thể chèn ảnh ở vị trí hiện tại","Could not obtain resized image URL.":"Không thể lấy được đường dẫn ảnh đã đổi kích thước","Insert image or file":"Chèn ảnh hoặc file","Inserting image failed":"Chèn ảnh thất bại","Selecting resized image failed":"Chọn ảnh đã đổi kích thước thất bại"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));