!function(e){const n=e["en-au"]=e["en-au"]||{};n.dictionary=Object.assign(n.dictionary||{},{"Connected users":"Connected users","Document outline":"Document outline","Enter fullscreen mode":"Enter fullscreen mode","Fullscreen mode":"Fullscreen mode","Hide left sidebar":"Hide left sidebar","Leave fullscreen mode":"Leave fullscreen mode","Show left sidebar":"Show left sidebar","Toggle fullscreen mode":"Toggle fullscreen mode","Toggle sidebar":"Toggle sidebar"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));