{#
/**
 * @file
 * Default theme implementation to display a formatted responsive image field.
 *
 * Available variables:
 * - responsive_image: A collection of responsive image data.
 * - url: An optional URL the image can be linked to.
 *
 * @see template_preprocess_responsive_image_formatter()
 *
 * @ingroup themeable
 */
#}
{% if url %}
  <a href="{{ url }}">{{ responsive_image }}</a>
{% else %}
  {{ responsive_image }}
{% endif %}
