!function(e){const t=e.ar=e.ar||{};t.dictionary=Object.assign(t.dictionary||{},{"Insert media":"أدخل الوسائط",Media:"وسائط","Media embed":"تضمين الوسائط","Media toolbar":"شريط أدوات الوسائط","Media URL":"عنوان URL الوسائط","media widget":"أداة الوسائط","Open media in new tab":"فتح الوسائط في علامة تبويب جديدة","Paste the media URL in the input.":"ألصق عنوان URL الوسائط في موضع الإدخال.","The URL must not be empty.":"لا يمكن ترك بيان عنوان URL فارغاً.","This media URL is not supported.":"عنوان URL الوسائط هذا غير مدعوم.","Tip: Paste the URL into the content to embed faster.":"نصيحة: ألصق عنوان URL داخل المحتوى؛ لتسريع عملية التضمين."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));