/**
 * @file
 * Body scroll lock with css only.
 */

@import "../base/media-queries.pcss.css";

[data-admin-toolbar-body-scroll="locked"] {
  overflow: clip;

  body {
    overflow: clip;
  }

  @media (--admin-toolbar-desktop) {
    overflow: initial;

    body {
      overflow: initial;
    }
  }
}
