!function(e){const i=e.pl=e.pl||{};i.dictionary=Object.assign(i.dictionary||{},{"Bulleted List":"Lista wypunktowana","Bulleted list styles toolbar":"Pasek z narzędziami: style listy z punktorami",Circle:"Kółko",Decimal:"Dziesiętne","Decimal with leading zero":"Dziesiętne z zerem wiodącym","Decrease list item indent":"Zmniejsza wcięcie elementu listy",Disc:"Dysk","Entering a to-do list":"Wchodzenie na listę zadań do wykonania","Increase list item indent":"Zwiększa wcięcie elementu listy","Invalid start index value.":"Nieprawidłowa wartość indeksu początkowego.","Keystrokes that can be used in a list":"Klawisze, których można używać w odniesieniu do listy","Leaving a to-do list":"Opuszczenie listy zadań do wykonania","List properties":"Właściwości listy","Lower-latin":"Alfabet łaciński – małe litery","Lower–roman":"Małe cyfry rzymskie","Numbered List":"Lista numerowana","Numbered list styles toolbar":"Pasek z narzędziami: style listy numerowanej","Reversed order":"Odwrócona kolejność",Square:"Kwadrat","Start at":"Zacznij od","Start index must be greater than 0.":"Wartość początkowa musi być większa niż 0.","To-do List":"Lista rzeczy do zrobienia","Toggle the circle list style":"Włącz/wyłącz listę w stylu „kółko”","Toggle the decimal list style":"Włącz/wyłącz listę w stylu „dziesiętne”","Toggle the decimal with leading zero list style":"Włącz/wyłącz listę w stylu „dziesiętne z zerem wiodącym”","Toggle the disc list style":"Włącz/wyłącz listę w stylu „dysk”","Toggle the lower–latin list style":"Włącz/wyłącz listę w stylu „alfabet łaciński – małe litery”","Toggle the lower–roman list style":"Włącz/wyłącz listę w stylu „małe cyfry rzymskie”","Toggle the square list style":"Włącz/wyłącz listę w stylu „kwadrat”","Toggle the upper–latin list style":"Włącz/wyłącz listę w stylu „alfabet łaciński – wielkie litery”","Toggle the upper–roman list style":"Włącz/wyłącz listę w stylu „wielkie cyfry rzymskie”","Upper-latin":"Alfabet łaciński – wielkie litery","Upper-roman":"Wielkie cyfry rzymskie"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));