!function(i){const e=i.sr=i.sr||{};e.dictionary=Object.assign(e.dictionary||{},{"Could not insert image at the current position.":"Немогуће је додати слику на ово место.","Could not obtain resized image URL.":"УРЛ слика промењених димензија није доступна.","Insert image or file":"Додај слику или фајл","Inserting image failed":"Додавање слике је неуспешно","Selecting resized image failed":"Одабир слике промењених дименшија није успешно"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));