<?php

/**
 * @file
 * Generic transliteration data for the PhpTransliteration class.
 */

$base = [
  0x00 => 'syon', 'syonj', 'syonh', 'syod', 'syol', 'syolg', 'syolm', 'syolb', 'syols', 'syolt', 'syolp', 'syolh', 'syom', 'syob', 'syobs', 'syos',
  0x10 => 'syoss', 'syong', 'syoj', 'syoch', 'syok', 'syot', 'syop', 'syoh', 'su', 'sug', 'sukk', 'sugs', 'sun', 'sunj', 'sunh', 'sud',
  0x20 => 'sul', 'sulg', 'sulm', 'sulb', 'suls', 'sult', 'sulp', 'sulh', 'sum', 'sub', 'subs', 'sus', 'suss', 'sung', 'suj', 'such',
  0x30 => 'suk', 'sut', 'sup', 'suh', 'swo', 'swog', 'swokk', 'swogs', 'swon', 'swonj', 'swonh', 'swod', 'swol', 'swolg', 'swolm', 'swolb',
  0x40 => 'swols', 'swolt', 'swolp', 'swolh', 'swom', 'swob', 'swobs', 'swos', 'swoss', 'swong', 'swoj', 'swoch', 'swok', 'swot', 'swop', 'swoh',
  0x50 => 'swe', 'sweg', 'swekk', 'swegs', 'swen', 'swenj', 'swenh', 'swed', 'swel', 'swelg', 'swelm', 'swelb', 'swels', 'swelt', 'swelp', 'swelh',
  0x60 => 'swem', 'sweb', 'swebs', 'swes', 'swess', 'sweng', 'swej', 'swech', 'swek', 'swet', 'swep', 'sweh', 'swi', 'swig', 'swikk', 'swigs',
  0x70 => 'swin', 'swinj', 'swinh', 'swid', 'swil', 'swilg', 'swilm', 'swilb', 'swils', 'swilt', 'swilp', 'swilh', 'swim', 'swib', 'swibs', 'swis',
  0x80 => 'swiss', 'swing', 'swij', 'swich', 'swik', 'swit', 'swip', 'swih', 'syu', 'syug', 'syukk', 'syugs', 'syun', 'syunj', 'syunh', 'syud',
  0x90 => 'syul', 'syulg', 'syulm', 'syulb', 'syuls', 'syult', 'syulp', 'syulh', 'syum', 'syub', 'syubs', 'syus', 'syuss', 'syung', 'syuj', 'syuch',
  0xA0 => 'syuk', 'syut', 'syup', 'syuh', 'seu', 'seug', 'seukk', 'seugs', 'seun', 'seunj', 'seunh', 'seud', 'seul', 'seulg', 'seulm', 'seulb',
  0xB0 => 'seuls', 'seult', 'seulp', 'seulh', 'seum', 'seub', 'seubs', 'seus', 'seuss', 'seung', 'seuj', 'seuch', 'seuk', 'seut', 'seup', 'seuh',
  0xC0 => 'sui', 'suig', 'suikk', 'suigs', 'suin', 'suinj', 'suinh', 'suid', 'suil', 'suilg', 'suilm', 'suilb', 'suils', 'suilt', 'suilp', 'suilh',
  0xD0 => 'suim', 'suib', 'suibs', 'suis', 'suiss', 'suing', 'suij', 'suich', 'suik', 'suit', 'suip', 'suih', 'si', 'sig', 'sikk', 'sigs',
  0xE0 => 'sin', 'sinj', 'sinh', 'sid', 'sil', 'silg', 'silm', 'silb', 'sils', 'silt', 'silp', 'silh', 'sim', 'sib', 'sibs', 'sis',
  0xF0 => 'siss', 'sing', 'sij', 'sich', 'sik', 'sit', 'sip', 'sih', 'ssa', 'ssag', 'ssakk', 'ssags', 'ssan', 'ssanj', 'ssanh', 'ssad',
];
