!function(e){const a=e.es=e.es||{};a.dictionary=Object.assign(a.dictionary||{},{"Edit source":"Editar fuente","Empty snippet content":"Fragmento vacío","HTML snippet":"Fragmento HTML","Insert HTML":"Insertar HTML","No preview available":"No hay ninguna previsualización","Paste raw HTML here...":"Pegue el código HTML aquí…","Save changes":"Guardar cambios"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));