!function(e){const a=e.nl=e.nl||{};a.dictionary=Object.assign(a.dictionary||{},{"Break text":"Tekst afbreken","Caption for image: %0":"Bijschrift voor afbeelding: %0","Caption for the image":"Bijschrift voor afbeelding","Centered image":"Gecentreerde afbeelding","Change image text alternative":"Verander alt-tekst van de afbeelding",Custom:"Aangepast","Custom image size":"Aangepaste afbeeldingsgrootte","Enter image caption":"Typ een afbeeldingsbijschrift","Error during image upload":"Fout tijdens het uploaden van afbeeldingen","From computer":"Van computer","Full size image":"Afbeelding op volledige grootte",Image:"Afbeelding","Image from computer":"Afbeelding van computer","Image Resize":"Afbeeldingformaat wijzigen","Image resize list":"Lijst voor wijzigen van afbeeldingsformaat","Image toolbar":"Afbeeldingswerkbalk","Image upload complete":"Uploaden van afbeelding voltooid","Image via URL":"Afbeelding via URL","image widget":"afbeeldingswidget","In line":"In lijn","Insert image":"Afbeelding toevoegen","Insert image via URL":"Afbeelding toevoegen via URL","Insert via URL":"Via URL invoegen","Left aligned image":"Links uitgelijnde afbeelding",Original:"Origineel","Replace from computer":"Vervangen vanaf de computer","Replace image":"Afbeelding vervangen","Replace image from computer":"Afbeelding vervangen vanaf computer","Resize image":"Afbeeldingsformaat wijzigen","Resize image (in %0)":"Formaat afbeelding wijzigen (in %0)","Resize image to %0":"Afbeeldingsformaat wijzigen naar %0","Resize image to the original size":"Afbeeldingsformaat wijzigen naar originele grootte","Right aligned image":"Rechts uitgelijnde afbeelding","Side image":"Afbeelding naast tekst","Text alternative":"Alt-tekst","Text Alternative":"Tekstalternatief","The value must not be empty.":"De waarde mag niet leeg zijn.","The value should be a plain number.":"De waarde moet een gewoon getal zijn.","Update image URL":"URL van afbeelding bijwerken","Upload failed":"Uploaden afbeelding mislukt","Upload from computer":"Uploaden vanaf computer","Upload image from computer":"Afbeelding uploaden vanaf de computer","Uploading image":"Afbeelding uploaden","Via URL":"Via URL","Wrap text":"Tekstterugloop","You have no image upload permissions.":"U heeft geen toestemming om afbeeldingen te uploaden."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));