!function(i){const n=i.ro=i.ro||{};n.dictionary=Object.assign(n.dictionary||{},{"Create link":"Crearea unui link","Displayed text":"Textul afișat",Downloadable:"Descărcabil","Edit link":"Modifică link",Link:"Link","Link image":"Link imagine","Link properties":"Proprietățile linkului","Link URL":"Link URL","Link URL must not be empty.":"URL-ul linkului nu trebuie să fie necompletat.","Move out of a link":"Ieșire dintr-un link","No links available":"Niciun link disponibil","Open in a new tab":"Deschide în tab nou","Open link in new tab":"Deschide link în tab nou",Unlink:"Șterge link"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));