!function(o){const n=o.be=o.be||{};n.dictionary=Object.assign(n.dictionary||{},{Big:"Крупны",Default:"Па ўмове","Document colors":"Колеры дакумента","Font Background Color":"Колер фону шрыфту","Font Color":"Колер шрыфту","Font Family":"Сям'я шрыфтаў","Font Size":"Памер шрыфту",Huge:"Вельмі крупны",Small:"Мелкі",Tiny:"Вельмі малы"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));