!function(n){const i=n.id=n.id||{};i.dictionary=Object.assign(i.dictionary||{},{"Disable block editing":"Nonaktifkan penyuntingan blok","Disable editing":"Nonaktifkan pengeditan","Disable inline editing":"Nonaktifkan penyuntingan langsung","Enable block editing":"Aktifkan penyuntingan blok","Enable editing":"Aktifkan penyuntingan","Enable inline editing":"Aktifkan penyuntingan langsung","Navigate editable regions":"Posisikan bagian-bagian penyuntingan","Next editable region":"Bagian penyuntingan berikutnya","Previous editable region":"Bagian penyuntingan sebelumnya"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));