!function(e){const i=e.ru=e.ru||{};i.dictionary=Object.assign(i.dictionary||{},{Activities:"Развлечения","Animals & Nature":"Животные и природа","Dark skin tone":"Темный оттенок кожи","Default skin tone":"Цвет кожи по умолчанию",Emoji:"Эмодзи","Emoji picker":"Выбор эмодзи","Find an emoji (min. 2 characters)":"Найти эмодзи (мин. 2 символа)",Flags:"Флаги","Food & Drinks":"Еда и напитки","Gestures & People":"Жесты и люди","Keep on typing to see the emoji.":"Продолжайте набирать текст, чтобы увидеть эмодзи.","Light skin tone":"Светлый оттенок кожи","Medium Dark skin tone":"Средне-темный оттенок кожи","Medium Light skin tone":"Средне-светлый оттенок кожи","Medium skin tone":"Средний оттенок кожи",'No emojis were found matching "%0".':'Не найдено ни одного эмодзи, соответствующего "%0".',Objects:"Объекты","Select skin tone":"Выберите цвет кожи","Show all emoji...":"Показать все эмодзи...","Smileys & Expressions":"Смайлы и эмоции",Symbols:"Символы","The query must contain at least two characters.":"Запрос должен содержать не менее двух символов.","Travel & Places":"Путешествия и достопримечательности"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));