!function(n){const i=n.si=n.si||{};i.dictionary=Object.assign(i.dictionary||{},{"Choose language":"",Language:"","Remove language":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));