!function(e){const r=e["es-co"]=e["es-co"]||{};r.dictionary=Object.assign(r.dictionary||{},{"Blue marker":"Marcador azul","Green marker":"Marcador verde","Green pen":"Pincel verde",Highlight:"Resaltar","Pink marker":"Marcador rosa","Red pen":"Pincel rojo","Remove highlight":"Quitar resaltado","Text highlight toolbar":"Herramientas de resaltado de texto","Yellow marker":"Marcador amarillo"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));