!function(a){const m=a.lv=a.lv||{};m.dictionary=Object.assign(m.dictionary||{},{Bookmark:"Grāmatzīme","Bookmark must not be empty.":"Grāmatzīmes lauks nedrīkst būt tukšs.","Bookmark name":"Grāmatzīmes nosaukums","Bookmark name already exists.":"Šāds grāmatzīmes nosaukums jau pastāv.","Bookmark name cannot contain space characters.":"Grāmatzīmes nosaukumā nedrīkst būt atstarpes.","Bookmark toolbar":"Grāmatzīmju rīkjosla","bookmark widget":"grāmatzīmju ikona",Bookmarks:"Grāmatzīmes","Edit bookmark":"Rediģēt grāmatzīmi","Enter the bookmark name without spaces.":"Ievadiet grāmatzīmes nosaukumu bez atstarpēm.","No bookmarks available.":"Grāmatzīmes nav pieejamas.","Remove bookmark":"Aizvākt grāmatzīmi","Scroll to bookmark":"Pārvieto līdz grāmatzīmei"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));