!function(e){const a=e.el=e.el||{};a.dictionary=Object.assign(a.dictionary||{},{"Break text":"Κατάτμηση κειμένου","Caption for image: %0":"Λεζάντα για την εικόνα: %0","Caption for the image":"Λεζάντα για την εικόνα","Centered image":"Εικόνα με στοίχιση στο κέντρο","Change image text alternative":"Αλλαγή εναλλακτικού κείμενου εικόνας",Custom:"Προσαρμοσμένο","Custom image size":"Προσαρμοσμένο μέγεθος εικόνας","Enter image caption":"Εισαγωγή λεζάντας εικόνας","Error during image upload":"Σφάλμα κατά τη μεταφόρτωση της εικόνας","From computer":"Από υπολογιστή","Full size image":"Εικόνα σε πλήρες μέγεθος",Image:"Εικόνα","Image from computer":"Εικόνα από υπολογιστή","Image Resize":"Αλλαγή μεγέθους εικόνας","Image resize list":"Λίστα μεγεθών εικόνων","Image toolbar":"Γραμμή εργαλείων εικόνας","Image upload complete":"Η μεταφόρτωση της εικόνας ολοκληρώθηκε","Image via URL":"Εικόνα μέσω URL","image widget":"Γραφικό στοιχείο εικόνας","In line":"Εντός γραμμής","Insert image":"Εισαγωγή εικόνας","Insert image via URL":"Εισαγωγή εικόνας μέσω διεύθυνσης","Insert via URL":"Εισαγωγή μέσω URL","Left aligned image":"Εικόνα με αριστερή στοίχιση",Original:"Αρχικό","Replace from computer":"Αντικατάσταση από υπολογιστή","Replace image":"Αντικατάσταση εικόνας","Replace image from computer":"Αντικατάσταση εικόνας από υπολογιστή","Resize image":"Αλλαγή μεγέθους εικόνας","Resize image (in %0)":"Αλλαγή μεγέθους εικόνας (σε %0)","Resize image to %0":"Αλλαγή μεγέθους σε %0","Resize image to the original size":"Αλλαγή μεγέθους εικόνας στο αρχικό μέγεθος","Right aligned image":"Εικόνα με δεξιά στοίχιση","Side image":"Πλευρική εικόνα","Text alternative":"Εναλλακτικό κείμενο","Text Alternative":"Εναλλακτικό κείμενο","The value must not be empty.":"Η τιμή δεν μπορεί να είναι κενή.","The value should be a plain number.":"Η τιμή θα πρέπει να είναι ένας απλός αριθμός.","Update image URL":"Ενημέρωση διεύθυνσης εικόνας","Upload failed":"Η αποστολή απέτυχε","Upload from computer":"Μεταφόρτωση από υπολογιστή","Upload image from computer":"Μεταφόρτωση εικόνας από υπολογιστή","Uploading image":"Μεταφόρτωση εικόνας","Via URL":"Μέσω URL","Wrap text":"Αναδίπλωση κειμένου","You have no image upload permissions.":"Δεν έχεις άδειες μεταφόρτωσης εικόνων."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));