{#
/**
 * @file
 * Default theme implementation for a select element.
 *
 * Available variables:
 * - attributes: HTML attributes for the <select> tag.
 * - options: The <option> element children.
 *
 * @see\Drupal\Core\Form\FormPreprocess::preprocessSelect()
 *
 * @ingroup themeable
 */
#}
<select{{ attributes }}>
  {% for option in options %}
    {% if option.type == 'optgroup' %}
      <optgroup label="{{ option.label }}">
        {% for sub_option in option.options %}
          <option value="{{ sub_option.value }}"{{ sub_option.selected ? ' selected="selected"' }}>{{ sub_option.label }}</option>
        {% endfor %}
      </optgroup>
    {% elseif option.type == 'option' %}
      <option value="{{ option.value }}"{{ option.selected ? ' selected="selected"' }}>{{ option.label }}</option>
    {% endif %}
  {% endfor %}
</select>
