!function(n){const i=n.ur=n.ur||{};i.dictionary=Object.assign(i.dictionary||{},{"Create link":"","Displayed text":"",Downloadable:"ڈاؤنلوڈ ہو سکتا ہے","Edit link":"ربط کی تدوین",Link:"ربط","Link image":"","Link properties":"","Link URL":"ربط کا یو آر ایل","Link URL must not be empty.":"","Move out of a link":"","No links available":"","Open in a new tab":"نئی ٹیب کھولیں","Open link in new tab":"نئے ٹیب میں کھولیں",Unlink:"ربط حذف کریں"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));