{#
/**
 * @file
 * Default theme implementation for an 'input' #type form element.
 *
 * Available variables:
 * - attributes: A list of HTML attributes for the input element.
 * - children: Optional additional rendered elements.
 *
 * @see\Drupal\Core\Form\FormPreprocess::preprocessInput()
 *
 * @ingroup themeable
 */
#}
<input{{ attributes }} />{{ children }}
