!function(e){const i=e.hi=e.hi||{};i.dictionary=Object.assign(i.dictionary||{},{"Break text":"टेक्स्ट तोड़ें","Caption for image: %0":"छवि के लिए कैप्शन: %0","Caption for the image":"छवि के लिए कैप्शन","Centered image":"Centered image","Change image text alternative":"Change image text alternative",Custom:"कस्टम","Custom image size":"तस्वीर का कस्टम साइज़","Enter image caption":"Enter image caption","Error during image upload":"तस्वीर अपलोड के दौरान त्रुटि","From computer":"कंप्यूटर से","Full size image":"Full size image",Image:"इमेज","Image from computer":"कंप्यूटर से इमेज","Image Resize":"तस्वीर का आकर बदलें","Image resize list":"Image resize list","Image toolbar":"Image toolbar","Image upload complete":"तस्वीर का अपलोड पूरा हुआ","Image via URL":"URL के माध्यम से इमेज","image widget":"image widget","In line":"इन - लाइन","Insert image":"Insert image","Insert image via URL":"Insert image via URL","Insert via URL":"URL के माध्यम से इंसर्ट करें","Left aligned image":"Left aligned image",Original:"Original","Replace from computer":"कंप्यूटर से बदलें","Replace image":"इमेज बदलें","Replace image from computer":"कंप्यूटर से इमेज बदलें","Resize image":"Resize image","Resize image (in %0)":"(%0 में) तस्वीर का साइज़ बदलें","Resize image to %0":"Resize image to %0","Resize image to the original size":"Resize image to the original size","Right aligned image":"Right aligned image","Side image":"Side image","Text alternative":"Text alternative","Text Alternative":"टेक्स्ट का विकल्प","The value must not be empty.":"वैल्यू रिक्त नहीं होना चाहिए.","The value should be a plain number.":"वैल्यू एक प्लेन नंबर होना चाहिए.","Update image URL":"Update image URL","Upload failed":"Upload failed","Upload from computer":"कंप्यूटर से अपलोड करे","Upload image from computer":"कंप्यूटर से इमेज अपलोड करें","Uploading image":"तस्वीर अपलोड की जा रही है","Via URL":"URL के माध्यम से","Wrap text":"टेक्स्ट रैप करें","You have no image upload permissions.":"आपके पास तस्वीर को अपलोड करने की अनुमति नहीं है."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));