<?php

namespace Drupal\Core\Entity\KeyValueStore\Query;

use Drupal\Core\Config\Entity\Query\Condition as ConditionParent;

/**
 * Defines the condition class for the key value entity query.
 */
class Condition extends ConditionParent {

}
