!function(e){const i=e.ko=e.ko||{};i.dictionary=Object.assign(i.dictionary||{},{Activities:"활동","Animals & Nature":"동물 및 자연","Dark skin tone":"어두운 피부색","Default skin tone":"기본 피부색",Emoji:"이모지","Emoji picker":"이모지 선택","Find an emoji (min. 2 characters)":"이모지 찾기(최소 2자)",Flags:"깃발","Food & Drinks":"음식 및 음료","Gestures & People":"제스처 및 사람","Keep on typing to see the emoji.":"이모지를 보려면 계속 입력하세요.","Light skin tone":"밝은 피부색","Medium Dark skin tone":"약간 어두운 피부색","Medium Light skin tone":"약간 밝은 피부색","Medium skin tone":"중간 피부색",'No emojis were found matching "%0".':'"%0"에 대한 이모지를 찾을 수 없습니다.',Objects:"사물","Select skin tone":"피부색 선택","Show all emoji...":"모든 이모지 표시...","Smileys & Expressions":"스마일 및 표정",Symbols:"기호","The query must contain at least two characters.":"쿼리에는 2자 이상이 포함되어야 합니다.","Travel & Places":"여행 및 장소"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));