!function(n){const i=n.bs=n.bs||{};i.dictionary=Object.assign(i.dictionary||{},{"Create link":"","Displayed text":"",Downloadable:"","Edit link":"",Link:"","Link image":"","Link properties":"","Link URL":"","Link URL must not be empty.":"","Move out of a link":"","No links available":"","Open in a new tab":"","Open link in new tab":"",Unlink:""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));