!function(e){const t=e.he=e.he||{};t.dictionary=Object.assign(t.dictionary||{},{"Advanced options":"אפשרויות מתקדמות",Find:"חיפוש","Find and replace":"חיפוש והחלפה","Find in text…":"חיפוש בטקסט","Find in the document":"חיפוש במסמך","Match case":"התאם רישיות","Next result":"התוצאה הבאה","Previous result":"התוצאה הקודמת",Replace:"החלף","Replace all":"החלף הכל","Replace with…":"החלף ב…","Text to find must not be empty.":"הטקסט לחיפוש לא יכול להיות ריק.","Tip: Find some text first in order to replace it.":"טיפ: מצאו תחילה טקסט כדי להחליף אותו.","Whole words only":"מילים שלמות בלבד"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));