<?php

/**
 * @file
 * Generic transliteration data for the PhpTransliteration class.
 */

$base = [
  0x00 => 'ga', 'gag', 'gakk', 'gags', 'gan', 'ganj', 'ganh', 'gad', 'gal', 'galg', 'galm', 'galb', 'gals', 'galt', 'galp', 'galh',
  0x10 => 'gam', 'gab', 'gabs', 'gas', 'gass', 'gang', 'gaj', 'gach', 'gak', 'gat', 'gap', 'gah', 'gae', 'gaeg', 'gaekk', 'gaegs',
  0x20 => 'gaen', 'gaenj', 'gaenh', 'gaed', 'gael', 'gaelg', 'gaelm', 'gaelb', 'gaels', 'gaelt', 'gaelp', 'gaelh', 'gaem', 'gaeb', 'gaebs', 'gaes',
  0x30 => 'gaess', 'gaeng', 'gaej', 'gaech', 'gaek', 'gaet', 'gaep', 'gaeh', 'gya', 'gyag', 'gyakk', 'gyags', 'gyan', 'gyanj', 'gyanh', 'gyad',
  0x40 => 'gyal', 'gyalg', 'gyalm', 'gyalb', 'gyals', 'gyalt', 'gyalp', 'gyalh', 'gyam', 'gyab', 'gyabs', 'gyas', 'gyass', 'gyang', 'gyaj', 'gyach',
  0x50 => 'gyak', 'gyat', 'gyap', 'gyah', 'gyae', 'gyaeg', 'gyaekk', 'gyaegs', 'gyaen', 'gyaenj', 'gyaenh', 'gyaed', 'gyael', 'gyaelg', 'gyaelm', 'gyaelb',
  0x60 => 'gyaels', 'gyaelt', 'gyaelp', 'gyaelh', 'gyaem', 'gyaeb', 'gyaebs', 'gyaes', 'gyaess', 'gyaeng', 'gyaej', 'gyaech', 'gyaek', 'gyaet', 'gyaep', 'gyaeh',
  0x70 => 'geo', 'geog', 'geokk', 'geogs', 'geon', 'geonj', 'geonh', 'geod', 'geol', 'geolg', 'geolm', 'geolb', 'geols', 'geolt', 'geolp', 'geolh',
  0x80 => 'geom', 'geob', 'geobs', 'geos', 'geoss', 'geong', 'geoj', 'geoch', 'geok', 'geot', 'geop', 'geoh', 'ge', 'geg', 'gekk', 'gegs',
  0x90 => 'gen', 'genj', 'genh', 'ged', 'gel', 'gelg', 'gelm', 'gelb', 'gels', 'gelt', 'gelp', 'gelh', 'gem', 'geb', 'gebs', 'ges',
  0xA0 => 'gess', 'geng', 'gej', 'gech', 'gek', 'get', 'gep', 'geh', 'gyeo', 'gyeog', 'gyeokk', 'gyeogs', 'gyeon', 'gyeonj', 'gyeonh', 'gyeod',
  0xB0 => 'gyeol', 'gyeolg', 'gyeolm', 'gyeolb', 'gyeols', 'gyeolt', 'gyeolp', 'gyeolh', 'gyeom', 'gyeob', 'gyeobs', 'gyeos', 'gyeoss', 'gyeong', 'gyeoj', 'gyeoch',
  0xC0 => 'gyeok', 'gyeot', 'gyeop', 'gyeoh', 'gye', 'gyeg', 'gyekk', 'gyegs', 'gyen', 'gyenj', 'gyenh', 'gyed', 'gyel', 'gyelg', 'gyelm', 'gyelb',
  0xD0 => 'gyels', 'gyelt', 'gyelp', 'gyelh', 'gyem', 'gyeb', 'gyebs', 'gyes', 'gyess', 'gyeng', 'gyej', 'gyech', 'gyek', 'gyet', 'gyep', 'gyeh',
  0xE0 => 'go', 'gog', 'gokk', 'gogs', 'gon', 'gonj', 'gonh', 'god', 'gol', 'golg', 'golm', 'golb', 'gols', 'golt', 'golp', 'golh',
  0xF0 => 'gom', 'gob', 'gobs', 'gos', 'goss', 'gong', 'goj', 'goch', 'gok', 'got', 'gop', 'goh', 'gwa', 'gwag', 'gwakk', 'gwags',
];
