<?php

/**
 * @file
 * Generic transliteration data for the PhpTransliteration class.
 */

$base = [
  0x00 => 'ke', 'yao', 'ou', 'xiao', 'geng', 'tang', 'gui', 'hui', 'ta', NULL, 'yao', 'da', 'qi', 'jin', 'lue', 'mi',
  0x10 => 'mi', 'jian', 'lu', 'fan', 'ou', 'mi', 'jie', 'fu', 'bie', 'huang', 'su', 'yao', 'nie', 'jin', 'lian', 'bo',
  0x20 => 'jian', 'ti', 'ling', 'zuan', 'shi', 'yin', 'dao', 'chou', 'ca', 'mie', 'yan', 'lan', 'chong', 'jiao', 'shuang', 'quan',
  0x30 => 'nie', 'luo', NULL, 'shi', 'luo', 'zhu', NULL, 'chou', 'juan', 'jiong', 'er', 'yi', 'rui', 'cai', 'ren', 'fu',
  0x40 => 'lan', 'sui', 'yu', 'you', 'dian', 'ling', 'zhu', 'ta', 'ping', 'zhai', 'jiao', 'chui', 'bu', 'kou', 'cun', NULL,
  0x50 => 'han', 'han', 'mou', 'hu', 'gong', 'di', 'fu', 'xuan', 'mi', 'mei', 'lang', 'gu', 'zhao', 'ta', 'yu', 'zong',
  0x60 => 'li', 'lu', 'wu', 'lei', 'ji', 'li', 'li', NULL, 'po', 'yang', 'wa', 'tuo', 'peng', NULL, 'zhao', 'gui',
  0x70 => NULL, 'xu', 'nai', 'que', 'wei', 'zheng', 'dong', 'wei', 'bo', NULL, 'huan', 'xuan', 'zan', 'li', 'yan', 'huang',
  0x80 => 'xue', 'hu', 'bao', 'ran', 'xiao', 'po', 'liao', 'zhou', 'yi', 'xu', 'luo', 'kao', 'chu', NULL, 'na', 'han',
  0x90 => 'chao', 'lu', 'zhan', 'ta', 'fu', 'hong', 'zeng', 'qiao', 'su', 'pin', 'guan', NULL, 'hun', 'chu', NULL, 'er',
  0xA0 => 'er', 'ruan', 'qi', 'si', 'ju', NULL, 'yan', 'bang', 'ye', 'zi', 'ne', 'chuang', 'ba', 'cao', 'ti', 'han',
  0xB0 => 'zuo', 'ba', 'zhe', 'wa', 'geng', 'bi', 'er', 'zhu', 'wu', 'wen', 'zhi', 'zhou', 'lu', 'wen', 'gun', 'qiu',
  0xC0 => 'la', 'zai', 'sou', 'mian', 'di', 'qi', 'cao', 'piao', 'lian', 'shi', 'long', 'su', 'qi', 'yuan', 'feng', 'xu',
  0xD0 => 'jue', 'di', 'pian', 'guan', 'niu', 'ren', 'zhen', 'gai', 'pi', 'tan', 'chao', 'chun', 'he', 'zhuan', 'mo', 'bie',
  0xE0 => 'qi', 'shi', 'bi', 'jue', 'si', NULL, 'gua', 'na', 'hui', 'xi', 'er', 'xiu', 'mou', NULL, 'xi', 'zhi',
  0xF0 => 'run', 'ju', 'die', 'zhe', 'shao', 'meng', 'bi', 'han', 'yu', 'xian', 'pang', 'neng', 'can', 'bu', NULL, 'qi',
];
