!function(e){const a=e.th=e.th||{};a.dictionary=Object.assign(a.dictionary||{},{"Break text":"แบ่งข้อความ","Caption for image: %0":"คำบรรยายภาพ: %0","Caption for the image":"คำบรรยายภาพ","Centered image":"จัดแนวรูปกึ่งกลาง","Change image text alternative":"เปลี่ยนข้อความเมื่อไม่พบรูป",Custom:"กำหนดเอง","Custom image size":"ขนาดภาพที่กำหนดเอง","Enter image caption":"ระบุคำอธิบายภาพ","Error during image upload":"เกิดข้อผิดพลาดระหว่างอัปโหลดรูปภาพ","From computer":"จากคอมพิวเตอร์","Full size image":"รูปขนาดเต็ม",Image:"ภาพ","Image from computer":"ภาพจากคอมพิวเตอร์","Image Resize":"ปรับขนาดรูปภาพ","Image resize list":"รายการปรับขนาดภาพ","Image toolbar":"เครื่องมือรูปภาพ","Image upload complete":"อัปโหลดรูปภาพเสร็จสิ้น","Image via URL":"ภาพทาง URL","image widget":"วิดเจ็ตรูปภาพ","In line":"ในบรรทัด","Insert image":"แทรกรูป","Insert image via URL":"แทรกภาพผ่าน URL","Insert via URL":"แทรกทาง URL","Left aligned image":"จัดแนวภาพซ้าย",Original:"ดั้งเดิม","Replace from computer":"แทนที่จากคอมพิวเตอร์","Replace image":"แทนที่ภาพ","Replace image from computer":"แทนที่ภาพจากคอมพิวเตอร์","Resize image":"ปรับขนาดภาพ","Resize image (in %0)":"ปรับขนาดภาพ (ใน %0)","Resize image to %0":"ปรับขนาดภาพเป็น %0","Resize image to the original size":"ปรับขนาดภาพเป็นขนาดเดิม","Right aligned image":"จัดแนวภาพขวา","Side image":"รูปด้านข้าง","Text alternative":"ข้อความเมื่อไม่พบรูป","Text Alternative":"ข้อความแสดงแทน","The value must not be empty.":"ค่าต้องไม่ว่างเปล่า","The value should be a plain number.":"ค่าควรเป็นตัวเลขธรรมดา","Update image URL":"อัปเดต URL ภาพ","Upload failed":"อัปโหลดไม่สำเร็จ","Upload from computer":"อัปโหลดจากคอมพิวเตอร์","Upload image from computer":"อัปโหลดภาพจากคอมพิวเตอร์","Uploading image":"กำลังอัปโหลดรูปภาพ","Via URL":"ทาง URL","Wrap text":"ตัดคำข้อความ","You have no image upload permissions.":"คุณไม่มีสิทธิ์อัปโหลดรูปภาพ"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));