<?php

declare(strict_types=1);

namespace Drupal\layout_builder_override\Entity;

use Drupal\Core\Entity\Display\EntityViewDisplayInterface;
use Drupal\Core\Entity\Entity\EntityViewDisplay as BaseEntityViewDisplay;

/**
 * Stub class to evaluate the override.
 */
class EntityViewDisplay extends BaseEntityViewDisplay implements EntityViewDisplayInterface {
}
