!function(e){const o=e.cs=e.cs||{};o.dictionary=Object.assign(o.dictionary||{},{"Connected users":"Připojení uživatelé","Document outline":"Osnova dokumentu","Enter fullscreen mode":"Přejít do režimu celé obrazovky","Fullscreen mode":"Režim celé obrazovky","Hide left sidebar":"Skrýt levý postranní panel","Leave fullscreen mode":"Opustit režim celé obrazovky","Show left sidebar":"Zobrazit levý postranní panel","Toggle fullscreen mode":"Přepnout režim celé obrazovky","Toggle sidebar":"Přepnout postranní panel"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));