<?php

declare(strict_types=1);

namespace Drupal\Core\Extension;

/**
 * Helper class to target procedural implementations with hook order attributes.
 *
 * This will be removed when procedural hooks are no longer supported.
 *
 * @internal
 */
final class ProceduralCall {

}
