!function(e){const i=e.ja=e.ja||{};i.dictionary=Object.assign(i.dictionary||{},{"Cannot access default workspace.":"デフォルトワークスペースにアクセスできません。","Cannot determine a category for the uploaded file.":"アップロードされたファイルのカテゴリを特定することができません。","Edit image":"画像を編集","Failed to determine category of edited image.":"編集した画像のカテゴリーを決定できませんでした。","Open file manager":"ファイルマネージャーを開く","Processing the edited image.":"編集した画像を処理しています。","Server failed to process the image.":"サーバが画像の処理に失敗しました。","You have no image editing permissions.":"画像編集のパーミッションがありません。"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));