/**
 * Remove CSS animation effects that can cause random test failures.
 */
* {
  /* CSS transitions. */
  -o-transition: none !important;
  -webkit-transition: none !important;
  transition: none !important;
  -o-transition-property: none !important;
  -webkit-transition-property: none !important;
  transition-property: none !important;
  /* CSS animations. */
  -webkit-animation: none !important;
  -o-animation: none !important;
  animation: none !important;
}

/**
 * Prevent youtube and third party content in iFrames from affecting tests.
 *
 * @todo Remove once bug in Chromedriver is fixed.
     https://bugs.chromium.org/p/chromedriver/issues/detail?id=2758
 */
iframe.media-oembed-content {
  width: 0 !important;
  height: 0 !important;
}
