<?php

/**
 * @file
 * Generic transliteration data for the PhpTransliteration class.
 */

$base = [
  0x00 => 'qie', 'du', 'ta', 'tang', 'zhai', 'dong', 'bao', 'fu', 'xing', 'jiang', 'jian', 'kuo', 'wu', 'hu', NULL, NULL,
  0x10 => 'zhong', NULL, 'qing', NULL, NULL, 'xi', 'zhu', 'yi', 'li', 'shen', 'xiang', 'fu', 'jing', 'jing', 'yu', NULL,
  0x20 => 'qiu', NULL, 'zhu', NULL, NULL, 'yi', 'dou', NULL, NULL, NULL, 'fan', 'si', 'guan', 'he', NULL, NULL,
  0x30 => 'wu', 'seng', 'mian', 'mian', 'qin', 'bei', 'he', 'tan', 'qi', 'ping', 'mo', 'ceng', 'che', 'hui', 'kai', 'zeng',
  0x40 => 'cheng', 'min', 'ji', 'shu', 'mei', 'hai', 'zhu', 'han', 'zhu', 'zhao', 'zuo', 'bei', 'she', 'zhi', 'qi', 'you',
  0x50 => 'zu', 'zhu', 'huo', 'zhen', 'gu', 'tu', 'jie', 'lian', 'jin', 'fan', 'shu', 'zhe', 'chou', 'cao', 'cao', 'zhe',
  0x60 => 'he', 'shi', 'ye', 'jin', 'bin', 'zeng', 'chuo', 'yi', 'nan', 'xiang', 'pin', 'hui', NULL, 'guan', NULL, NULL,
  0x70 => 'bing', 'kuang', 'quan', 'xing', 'chong', 'ji', 'yong', 'shao', 'he', 'tao', 'hui', 'wa', 'zhong', 'fen', 'yan', 'ben',
  0x80 => 'bi', 'ci', 'ao', 'yi', 'cai', 'yao', 'wang', 'shen', 'yu', 'zeng', 'ao', 'cheng', 'dai', 'yu', 'sou', 'bing',
  0x90 => 'ao', 'qing', 'lang', 'wang', 'zhang', 'dai', 'sha', 'liu', 'yin', 'zi', 'han', 'jing', 'zhu', 'qiao', 'jue', 'fan',
  0xA0 => 'zhu', 'zhen', 'ci', 'hua', 'guan', 'wen', 'yi', 'sheng', 'zhi', 'juan', 'zhe', 'tian', 'tiao', 'jie', 'lei', 'tao',
  0xB0 => 'lian', 'ping', 'zhe', 'huang', 'hua', 'yun', 'qiang', 'fu', 'shi', 'diao', 'zhu', 'qing', 'ye', 'nuo', 'yu', 'jin',
  0xC0 => 'bian', 'zeng', 'shu', 'chi', 'sou', 'xing', 'zhu', 'nan', 'jing', 'bai', 'xiang', 'e', 'pin', 'zhen', 'gui', NULL,
  0xD0 => NULL, NULL, 'he', 'xie', 'jie', NULL, 'qian', 'beng', 'e', 'pang', NULL, NULL, NULL, NULL, NULL, NULL,
  0xE0 => NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  0xF0 => NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
];
