<?php

declare(strict_types=1);

namespace Drupal\Core\DefaultContent;

/**
 * Exception thrown when there is an error importing content.
 *
 * @internal
 *   This API is experimental.
 */
final class ImportException extends \RuntimeException {
}
