!function(n){const a=n["zh-cn"]=n["zh-cn"]||{};a.dictionary=Object.assign(a.dictionary||{},{"Choose language":"选择语言",Language:"语言","Remove language":"移除语言"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));