!function(e){const t=e.bg=e.bg||{};t.dictionary=Object.assign(t.dictionary||{},{"Insert media":"Вмъкни медия",Media:"Медия","Media embed":"Вмъкване на мултимедия","Media toolbar":"Лента със задачи за медия","Media URL":"Медиен уеб адрес","media widget":"Медиен компонент","Open media in new tab":"Отворете мултимедията в нов раздел","Paste the media URL in the input.":"Постави медииния уеб адрес във входа.","The URL must not be empty.":"Уеб адресът не трябва да бъде празен.","This media URL is not supported.":"Този медиен уеб адрес не се поддържа.","Tip: Paste the URL into the content to embed faster.":"Полезен съвет: Постави уеб адреса в съдържанието, за да вградите по-бързо."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));