!function(e){const t=e.el=e.el||{};t.dictionary=Object.assign(t.dictionary||{},{"Insert media":"Εισαγωγή πολυμέσου",Media:"Πολυμέσα","Media embed":"Ενσωμάτωση πολυμέσων","Media toolbar":"Γραμμή εργαλείων πολυμέσων","Media URL":"Διεύθυνση πολυμέσου","media widget":"Γραφικό στοιχείου πολυμέσου","Open media in new tab":"Άνοιγμα πολυμέσων σε νέα καρτέλα","Paste the media URL in the input.":"Επικολλήστε τη διεύθυνση του πολυμέσου στο πλαίσιο.","The URL must not be empty.":"Η διεύθυνση δεν πρέπει να είναι άδεια.","This media URL is not supported.":"Αυτή η διεύθυνση πολυμέσου δεν υποστηρίζεται.","Tip: Paste the URL into the content to embed faster.":"Επισήμανση: Επικολλήστε τη διεύθυνση στο περιεχόμενο για γρηγορότερη ενσωμάτωση."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));