!function(o){const a=o.sk=o.sk||{};a.dictionary=Object.assign(a.dictionary||{},{Big:"Veľké",Default:"Predvolené","Document colors":"Farby dokumentu","Font Background Color":"Farba zvýraznenia textu","Font Color":"Farba písma","Font Family":"Názov písma","Font Size":"Veľkosť písma",Huge:"Veľmi veľké",Small:"Malé",Tiny:"Veľmi malé"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));