!function(t){const i=t.hi=t.hi||{};i.dictionary=Object.assign(i.dictionary||{},{Bold:"Bold","Bold text":"टेक्स्ट को बोल्ड करें",Code:"Code",Italic:"Italic","Italic text":"इटैलिक टेक्स्ट","Move out of an inline code style":"इनलाइन कोड स्टाइल के बाहर जाएँ",Strikethrough:"Strikethrough","Strikethrough text":"टेक्स्ट को स्ट्राइकथ्रू करें",Subscript:"Subscript",Superscript:"Superscript",Underline:"Underline","Underline text":"टेक्स्ट को अंडरलाइन करें"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));