!function(n){const e=n.bg=n.bg||{};e.dictionary=Object.assign(e.dictionary||{},{"Code block":"Кодов блок","Entering %0 code snippet":"Въвеждане %0 отрязък от код","Entering code snippet":"Въвеждане на отрязък от код","Insert code block":"Въведи кодов блок","Leaving %0 code snippet":"Оставяне %0 отрязък от код","Leaving code snippet":"Оставяне на отрязък от код","Plain text":"Обикновен текст"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));