!function(i){const t=i.ro=i.ro||{};t.dictionary=Object.assign(t.dictionary||{},{"Choose heading":"Alege titlu",Heading:"Titlu","Heading 1":"Titlu 1","Heading 2":"Titlu 2","Heading 3":"Titlu 3","Heading 4":"Titlu 4","Heading 5":"Titlu 5","Heading 6":"Titlu 6","Type or paste your content here.":"Scrie sau inserează aici conținutul tău","Type your title":"Scrie titlul tău"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));