!function(e){const i=e.pl=e.pl||{};i.dictionary=Object.assign(i.dictionary||{},{"Blue marker":"Niebieski marker","Green marker":"Zielony marker","Green pen":"Zielony długopis",Highlight:"Podświetlenie","Pink marker":"Różowy marker","Red pen":"Czerwony długopis","Remove highlight":"Usuń podświetlenie","Text highlight toolbar":"Pasek narzędzi podświetleń","Yellow marker":"Żółty marker"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));