!function(e){const n=e.pl=e.pl||{};n.dictionary=Object.assign(n.dictionary||{},{"Connected users":"Połączeni użytkownicy","Document outline":"Zarys dokumentu","Enter fullscreen mode":"Włącz tryb pełnoekranowy","Fullscreen mode":"Tryb pełnoekranowy","Hide left sidebar":"Ukryj lewy pasek boczny","Leave fullscreen mode":"Wyłącz tryb pełnoekranowy","Show left sidebar":"Pokaż lewy pasek boczny","Toggle fullscreen mode":"Przełącz tryb pełnoekranowy","Toggle sidebar":"Przełącz pasek boczny"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));