!function(n){const a=n.eu=n.eu||{};a.dictionary=Object.assign(a.dictionary||{},{"Choose language":"",Language:"","Remove language":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));