!function(e){const t=e["zh-cn"]=e["zh-cn"]||{};t.dictionary=Object.assign(t.dictionary||{},{"(may require <kbd>Fn</kbd>)":"（可能需要用到 <kbd>Fn</kbd>键）","%0 of %1":"第 %0 步，共 %1 步",Accept:"接受",Accessibility:"可访问性","Accessibility help":"无障碍辅助功能帮助",Aquamarine:"海蓝色",Back:"返回","Below, you can find a list of keyboard shortcuts that can be used in the editor.":"您可以在下方找到可在编辑器中使用的键盘快捷键列表。",Black:"黑色",Blue:"蓝色",Cancel:"取消","Cannot upload file:":"无法上传的文件：",Clear:"清除","Click to edit block":"单击以编辑块",Close:"关闭","Close contextual balloons, dropdowns, and dialogs":"关闭上下文气泡框、下拉菜单和对话框","Color picker":"颜色选择器","Content editing keystrokes":"内容编辑按键","Copy selected content":"复制选定的内容","Dim grey":"暗灰色","Drag to move":"拖拽以移动","Dropdown menu":"下拉菜单","Dropdown toolbar":"下拉工具栏","Edit block":"编辑框","Editor block content toolbar":"编辑器块内容工具栏","Editor contextual toolbar":"编辑器上下文工具栏","Editor dialog":"编辑器对话框","Editor menu bar":"编辑器菜单栏","Editor toolbar":"编辑器工具栏","Execute the currently focused button. Executing buttons that interact with the editor content moves the focus back to the content.":"执行当前聚焦的按钮。执行与编辑器内容交互的按钮将焦点返回到内容。",File:"文件",Green:"绿色",Grey:"灰色","Help Contents. To close this dialog press ESC.":"帮助内容。要关闭此对话框，请按 ESC 键。",HEX:"十六进制",Insert:"插入","Insert a hard break (a new paragraph)":"插入硬换行（新段落）","Insert a new paragraph directly after a widget":"直接在小组件之后插入新段落","Insert a new paragraph directly before a widget":"直接在小组件之前插入新段落","Insert a soft break (a <code>&lt;br&gt;</code> element)":"插入软换行（一个<code>&lt;br&gt;</code> 元素）","Insert image with file manager":"使用文件管理器插入图片","Insert paragraph after block":"在后面插入段落","Insert paragraph before block":"在前面插入段落","Insert with file manager":"使用文件管理器插入","Keystrokes that can be used when a widget is selected (for example: image, table, etc.)":"当小组件被选中时（例如：图片、表格等）可以使用的按键","Light blue":"浅蓝色","Light green":"浅绿色","Light grey":"浅灰色",MENU_BAR_MENU_EDIT:"编辑",MENU_BAR_MENU_FILE:"文件",MENU_BAR_MENU_FONT:"字体",MENU_BAR_MENU_FORMAT:"格式",MENU_BAR_MENU_HELP:"帮助",MENU_BAR_MENU_INSERT:"插入",MENU_BAR_MENU_TEXT:"文本",MENU_BAR_MENU_TOOLS:"工具",MENU_BAR_MENU_VIEW:"查看","Move focus between form fields (inputs, buttons, etc.)":"在表单字段（输入、按钮等）之间移动焦点","Move focus from an editable area back to the parent widget":"将焦点从可编辑区域移回父窗口小组件","Move focus in and out of an active dialog window":"将焦点移入或移出活跃的对话框窗口","Move focus to the menu bar, navigate between menu bars":"将焦点移到菜单栏，在菜单栏之间导航","Move focus to the toolbar, navigate between toolbars":"将焦点移至工具栏，在工具栏之间导览","Move the caret to allow typing directly after a widget":"移动插入符，以允许在小组件之后直接输入文字","Move the caret to allow typing directly before a widget":"移动插入符，以允许在小组件之前直接输入文字","Navigate through the toolbar or menu bar":"通过工具栏或菜单栏进行导航",Next:"下一步","No results found":"未找到结果","No searchable items":"没有可搜索的项目","Open the accessibility help dialog":"打开“无障碍辅助功能帮助”对话框",Orange:"橙色",Paragraph:"段落","Paste content":"粘贴内容","Paste content as plain text":"将内容粘贴为纯文本",'Please enter a valid color (e.g. "ff0000").':"请输入有效的颜色（例如“ff0000”）。","Please try a different phrase or check the spelling.":"请尝试使用不同的短语或检查拼写。","Press %0 for help.":"按 %0 获取帮助。","Press Enter to type after or press Shift + Enter to type before the widget":"按下“Enter”键，在小组件后输入；按下“Shift+Enter”键，在小组件前输入",Previous:"上一步",Purple:"紫色",Red:"红色",Redo:"重做","Remove color":"移除颜色","Replace image with file manager":"使用文件管理器替换图片","Replace with file manager":"使用文件管理器替换","Restore default":"恢复默认","Rich Text Editor":"富文本编辑器","Rich Text Editor. Editing area: %0":"富文本编辑器。编辑区域：%0",Save:"保存","Select all":"全选","Show more items":"显示更多",Source:"源","These keyboard shortcuts allow for quick access to content editing features.":"这些键盘快捷键允许快速访问内容编辑功能。","Toggle caption off":"关闭表标题","Toggle caption on":"打开表标题",Turquoise:"青色",Undo:"撤销",Update:"更新","Upload in progress":"正在上传","Use the following keystrokes for more efficient navigation in the CKEditor 5 user interface.":"使用以下按键可以在 CKEditor 5 用户界面中进行更有效地导览。","User interface and content navigation keystrokes":"用户界面和内容导航按键",White:"白色","Widget toolbar":"小部件工具栏","With file manager":"通过文件管理器",Yellow:"黄色"}),t.getPluralForm=function(e){return 0}}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));