!function(e){const i=e.et=e.et||{};i.dictionary=Object.assign(i.dictionary||{},{"Bulleted List":"Punktidega loetelu","Bulleted list styles toolbar":"Täpploendi stiili tööriistariba",Circle:"Ring",Decimal:"Kümnend","Decimal with leading zero":"Nulliga kümnendsüsteem","Decrease list item indent":"Vähenda loendi taanet",Disc:"Ketas","Entering a to-do list":"Tulevaste tegevuste nimekirja avamine","Increase list item indent":"Suurenda loendi taanet","Invalid start index value.":"Vigane alguse väärtus.","Keystrokes that can be used in a list":"Loendis kasutatavad klahvikombinatsioonid","Leaving a to-do list":"Tulevaste tegevuste nimekirja sulgemine","List properties":"Loendi atribuudid","Lower-latin":"Ladina väiketähe","Lower–roman":"Väikse rooma numbri","Numbered List":"Nummerdatud loetelu","Numbered list styles toolbar":"Numbriloendi stiilide tööriistariba","Reversed order":"Vastupidine järjekord",Square:"Ruut","Start at":"Alusta alates","Start index must be greater than 0.":"Algindeks peab olema suurem kui 0.","To-do List":"Tööde nimekiri","Toggle the circle list style":"Lülita ringi loendistiili sisse-välja","Toggle the decimal list style":"Lülita kümnendsüsteemi loendistiili sisse-välja","Toggle the decimal with leading zero list style":"Lülita nulliga kümnendsüsteemi loendistiili sisse-välja","Toggle the disc list style":"Lülita ketta loendistiili sisse-välja","Toggle the lower–latin list style":"Lülita ladina väiketähe loendistiili sisse-välja","Toggle the lower–roman list style":"Lülita väikse rooma numbri loendistiili sisse-välja","Toggle the square list style":"Lülita ruudu loendistiili sisse-välja","Toggle the upper–latin list style":"Lülita ladina suurtähe loendistiili sisse-välja","Toggle the upper–roman list style":"Lülita suure rooma numbri loendistiili sisse-välja","Upper-latin":"Ladina suurtähe","Upper-roman":"Suure rooma numbri"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));