!function(e){const t=e.sq=e.sq||{};t.dictionary=Object.assign(t.dictionary||{},{"Insert media":"Shto Medie",Media:"","Media embed":"","Media toolbar":"Kokështrirja e mediave","Media URL":"URL e Medies","media widget":"Vegla e medies","Open media in new tab":"","Paste the media URL in the input.":"","The URL must not be empty.":"URL nuk duhet të jetë e zbrazët.","This media URL is not supported.":"URL e medies nuk mbështetet.","Tip: Paste the URL into the content to embed faster.":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));