!function(e){const t=e.sv=e.sv||{};t.dictionary=Object.assign(t.dictionary||{},{"Bulleted List":"Punktlista","Bulleted list styles toolbar":"Verktygsfält för punktlistor",Circle:"Cirkel",Decimal:"Decimal","Decimal with leading zero":"Decimal med inledande nolla","Decrease list item indent":"Minska indrag på listobjekt",Disc:"Disk","Entering a to-do list":"Fyller i en att-göra-lista","Increase list item indent":"Öka indrag på listobjekt","Invalid start index value.":"Ogiltigt startvärde på indexet.","Keystrokes that can be used in a list":"Tangenter som fungerar i en lista","Leaving a to-do list":"Lämnar en att-göra-lista","List properties":"Listegenskaper","Lower-latin":"Latinska gemener","Lower–roman":"Romerska gemener","Numbered List":"Numrerad lista","Numbered list styles toolbar":"Verktygsfält för numrerade listor","Reversed order":"Byt riktning",Square:"Fyrkant","Start at":"Börja på","Start index must be greater than 0.":"Startindex måste vara större än 0.","To-do List":"Att-göra-lista","Toggle the circle list style":"Växla till cirkellisttypen","Toggle the decimal list style":"Växla till decimallisttypen","Toggle the decimal with leading zero list style":"Växla till listtypen decimal-med-inledande-nolla","Toggle the disc list style":"Växla till disklisttypen","Toggle the lower–latin list style":"Växla till listtypen latinska gemener","Toggle the lower–roman list style":"Växla till listtypen romerska gemener","Toggle the square list style":"Växla till fyrkantslisttypen","Toggle the upper–latin list style":"Växla till listtypen latinska versaler ","Toggle the upper–roman list style":"Växla till listtypen romerska versaler","Upper-latin":"Latinska versaler","Upper-roman":"Romerska versaler"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));