<?php

declare(strict_types=1);

namespace Drupal\plugin_test;

/**
 * Provides a test plugin within a custom directory.
 *
 * @Plugin(
 *   id = "custom_example_1",
 *   custom = "Tim"
 * )
 */
class CustomDirectoryExample1 {}
