!function(i){const e=i.it=i.it||{};e.dictionary=Object.assign(e.dictionary||{},{"Cannot access default workspace.":"Impossibile accedere all'area di lavoro predefinita.","Cannot determine a category for the uploaded file.":"Impossibile determinare la categoria del file caricato.","Edit image":"Modifica immagine","Failed to determine category of edited image.":"Impossibile determinare la categoria dell'immagine modificata.","Open file manager":"Apri il gestore dei file","Processing the edited image.":"Elaborazione dell'immagine modificata.","Server failed to process the image.":"Il server non è riuscito a elaborare l'immagine.","You have no image editing permissions.":"Non disponi di autorizzazione per la modifica di immagini."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));