!function(e){const i=e.be=e.be||{};i.dictionary=Object.assign(i.dictionary||{},{"Blue marker":"Выдзяленне сінім маркерам","Green marker":"Выдзяленне зялёным маркерам","Green pen":"Зялёны колер тэксту",Highlight:"Выдзяленне","Pink marker":"Выдзяленне ружавым маркерам","Red pen":"Красны колер тэксту","Remove highlight":"Выдаліць выдзяленне","Text highlight toolbar":"Панель інструментаў выдзялення тэксту","Yellow marker":"Выдзяленне жоўтым маркерам"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));