!function(e){const t=e.hu=e.hu||{};t.dictionary=Object.assign(t.dictionary||{},{"Disable block editing":"Blokk szerkesztésének letiltása","Disable editing":"Szerkesztés letiltása","Disable inline editing":"Közvetlen szerkesztés letiltása","Enable block editing":"Blokk szerkesztésének engedélyezése","Enable editing":"Szerkesztés engedélyezése","Enable inline editing":"Közvetlen szerkesztés engedélyezése","Navigate editable regions":"Szerkeszthető területek navigálása","Next editable region":"Következő szerkeszthető terület","Previous editable region":"Előző szerkeszthető terület"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));