!function(e){const i=e.he=e.he||{};i.dictionary=Object.assign(i.dictionary||{},{Activities:"פעילויות","Animals & Nature":"בעלי חיים וטבע","Dark skin tone":"גוון עור כהה","Default skin tone":"גוון עור ברירת מחדל",Emoji:"אמוג'י","Emoji picker":"בורר אימוג'י","Find an emoji (min. 2 characters)":"חפש אמוג'י (מינימום 2 תווים)",Flags:"דגלים","Food & Drinks":"מזון ומשקאות","Gestures & People":"מחוות ואנשים","Keep on typing to see the emoji.":"המשיכו להקליד כדי לראות את האמוג'י.","Light skin tone":"גוון עור בהיר","Medium Dark skin tone":"גוון עור כהה בינוני","Medium Light skin tone":"גוון עור בהיר בינוני","Medium skin tone":"גוון עור בינוני",'No emojis were found matching "%0".':'לא נמצאו סמלי אמוג\'י התואמים ל-"%0".',Objects:"חפצים","Select skin tone":"בחר גוון עור","Show all emoji...":"הצג את כל סמלי האמוג'י...","Smileys & Expressions":"סמיילים והבעות",Symbols:"סמלים","The query must contain at least two characters.":"השאילתה חייבת להכיל לפחות שני תווים.","Travel & Places":"נסיעות ומקומות"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));