<?php

namespace Drupal\views\Annotation;

use Drupal\Component\Annotation\PluginID;

/**
 * Defines an abstract base class for all views handler annotations.
 */
abstract class ViewsHandlerAnnotationBase extends PluginID {

}
