!function(n){const i=n.th=n.th||{};i.dictionary=Object.assign(i.dictionary||{},{"Create link":"สร้างลิงก์","Displayed text":"ข้อความที่แสดง",Downloadable:"ที่สามารถดาวน์โหลดได้","Edit link":"แก้ไขลิงก์",Link:"ลิงก์","Link image":"ลิงก์ภาพ","Link properties":"คุณสมบัติลิงก์","Link URL":"ลิงก์ URL","Link URL must not be empty.":"URL ของลิงก์ต้องไม่เว้นว่าง","Move out of a link":"ย้ายออกจากลิงก์","No links available":"ไม่มีลิงก์พร้อมใช้งาน","Open in a new tab":"เปิดในแท็บใหม่","Open link in new tab":"เปิดลิงก์ในแท็บใหม่",Unlink:"ยกเลิกการลิงก์"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));