/**
 * @file
 * Visual styles for animated throbber.
 *
 * @see autocomplete.js
 */

.js input.form-autocomplete {
  background-image: url(../throbber-inactive.svg);
  background-repeat: no-repeat;
  background-position: 100% center; /* LTR */
}
.js[dir="rtl"] input.form-autocomplete {
  background-position: 0% center;
}
.js input.form-autocomplete.ui-autocomplete-loading {
  background-image: url(../throbber-active.svg);
  background-position: 100% center; /* LTR */
}
.js[dir="rtl"] input.form-autocomplete.ui-autocomplete-loading {
  background-position: 0% center;
}
