!function(e){const n=e.de=e.de||{};n.dictionary=Object.assign(n.dictionary||{},{"Edit source":"Quelle bearbeiten","Empty snippet content":"Leerer Snippet-Inhalt","HTML snippet":"HTML-Snippet","Insert HTML":"HTML einfügen","No preview available":"Keine Vorschau verfügbar","Paste raw HTML here...":"Rohes HTML hier einfügen …","Save changes":"Änderungen speichern"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));