!function(n){const a=n.jv=n.jv||{};a.dictionary=Object.assign(a.dictionary||{},{"Advanced options":"",Find:"Pados","Find and replace":"Pados lan gantos","Find in text…":"Pados ing seratan","Find in the document":"","Match case":"Samikaken aksara","Next result":"Kasil salajengipun","Previous result":"Kasil saderengipun",Replace:"Gantos","Replace all":"Gantos sedaya","Replace with…":"Gantos kaliyan...","Text to find must not be empty.":"Seratan ingkang dipunpadosi mboten angsal kosong.","Tip: Find some text first in order to replace it.":"","Whole words only":"Sedayaning ukanten"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));