!function(i){const e=i.ur=i.ur||{};e.dictionary=Object.assign(e.dictionary||{},{"Disable block editing":"","Disable editing":"","Disable inline editing":"","Enable block editing":"","Enable editing":"تدوین فعال کریں","Enable inline editing":"","Navigate editable regions":"","Next editable region":"","Previous editable region":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));