!function(t){const s=t.uk=t.uk||{};s.dictionary=Object.assign(s.dictionary||{},{"Block styles":"Стилі блоку","Multiple styles":"Кілька стилів",Styles:"Стилі","Text styles":"Стилі тексту"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));