!function(e){const i=e.sr=e.sr||{};i.dictionary=Object.assign(i.dictionary||{},{"Insert media":"Додај медиа",Media:"Medijski sadržaj","Media embed":"Ugradi medij","Media toolbar":"Медији трака са алаткама","Media URL":"Mедиа УРЛ","media widget":"Медиа wидгет","Open media in new tab":"Отворите медије у новој картици","Paste the media URL in the input.":"Налепи медијски УРЛ у поље за унос","The URL must not be empty.":"УРЛ не сме бити празан.","This media URL is not supported.":"Овај медиа УРЛ тип није подржан.","Tip: Paste the URL into the content to embed faster.":"Савет: Залепите УРЛ у садржај да би сте га брже уградили."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));