!function(e){const t=e.th=e.th||{};t.dictionary=Object.assign(t.dictionary||{},{"Advanced options":"ตัวเลือกขั้นสูง",Find:"ค้นหา","Find and replace":"ค้นหาและแทนที่","Find in text…":"ค้นหาในข้อความ...","Find in the document":"ค้นหาในเอกสาร","Match case":"ตัวใหญ่-เล็ก ตรงกัน","Next result":"ผลลัพธ์ถัดไป","Previous result":"ผลลัพธ์ก่อนหน้านี้",Replace:"แทนที่","Replace all":"แทนที่ทั้งหมด","Replace with…":"แทนที่ด้วย...","Text to find must not be empty.":"ข้อความที่จะค้นหาต้องไม่ว่างเปล่า","Tip: Find some text first in order to replace it.":"เคล็ดลับ: ค้นหาข้อความบางอย่างก่อนจึงจะแทนที่ได้","Whole words only":"ตรงกันทุกตัวอักษร"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));