/**
 * @file
 *
 * Styles for the announcements toolbar item.
 */

.toolbar .toolbar-icon.announce-canvas-link::before {
  background-image: url("../images/announcement-bell.svg");

  @media (forced-colors: active) {
    background: linktext;
    mask-image: url("../images/announcement-bell.svg");
    mask-repeat: no-repeat;
    mask-position: center;
  }
}

/* Pushes the tab to the opposite side of the page. */
.toolbar .toolbar-bar .announce-toolbar-tab.toolbar-tab {
  float: right; /* LTR */

  &:dir(rtl) {
    float: left;
  }
}
