!function(t){const i=t.fi=t.fi||{};i.dictionary=Object.assign(i.dictionary||{},{"Bulleted List":"Lista","Bulleted list styles toolbar":"Luetelmalistan työkalupalkki",Circle:"Ympyrä",Decimal:"Desimaali","Decimal with leading zero":"Nolla alussa -desimaali","Decrease list item indent":"Vähennä listanimikkeen sisennystä",Disc:"Levy","Entering a to-do list":"Syötetään tehtävälista.","Increase list item indent":"Kasvata listanimikkeen sisennystä","Invalid start index value.":"Virheellinen aloitusindeksin arvo.","Keystrokes that can be used in a list":"Listassa käytettävät painallukset","Leaving a to-do list":"Jätetään tehtävälista","List properties":"Listaominaisuudet","Lower-latin":"Pienet latinalaiset aakkoset","Lower–roman":"Pienet roomalaiset numerot","Numbered List":"Numeroitu lista","Numbered list styles toolbar":"Numerolistan työkalupalkki","Reversed order":"Päinvastainen järjestys",Square:"Neliö","Start at":"Alkaa","Start index must be greater than 0.":"Aloitusindeksin pitää olla suurempi kuin 0.","To-do List":"Tehtävälista","Toggle the circle list style":"Ympyrälistatyylin vaihdin","Toggle the decimal list style":"Desimaalilistatyylin vaihdin","Toggle the decimal with leading zero list style":"Nolla alussa -desimaalilistatyylin vaihdin","Toggle the disc list style":"Levylistatyylin vaihdin","Toggle the lower–latin list style":"Pienet latinalaiset aakkoset -listatyylin vaihdin","Toggle the lower–roman list style":"Pienet roomalaiset numerot -listatyylin vaihdin","Toggle the square list style":"Neliölistatyylin vaihdin","Toggle the upper–latin list style":"Isot latinalaiset aakkoset -listatyylin vaihdin","Toggle the upper–roman list style":"Isot roomalaiset numerot -listatyylin vaihdin","Upper-latin":"Isot latinalaiset aakkoset","Upper-roman":"Isot roomalaiset numerot"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));