!function(e){const l=e.hu=e.hu||{};l.dictionary=Object.assign(l.dictionary||{},{"Connected users":"Csatlakozott felhasználók","Document outline":"Dokumentum vázlata","Enter fullscreen mode":"Teljes képernyős mód","Fullscreen mode":"Teljes képernyős mód","Hide left sidebar":"Bal oldalsáv elrejtése","Leave fullscreen mode":"Kilépés a teljes képernyőből","Show left sidebar":"Bal oldalsáv megjelenítése","Toggle fullscreen mode":"Váltás teljes képernyőre","Toggle sidebar":"Oldalsáv megjelenítése/elrejtése"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));