!function(e){const i=e.ca=e.ca||{};i.dictionary=Object.assign(i.dictionary||{},{Activities:"Activitats","Animals & Nature":"Animals i naturalesa","Dark skin tone":"To de pell fosc","Default skin tone":"To de pell predeterminat",Emoji:"Emoji","Emoji picker":"Selector d'emojis","Find an emoji (min. 2 characters)":"Troba un emoji (mín. 2 caràcters)",Flags:"Banderes","Food & Drinks":"Menjar i beguda","Gestures & People":"Gestos i gent","Keep on typing to see the emoji.":"Continua escrivint per veure l'emoji.","Light skin tone":"To de pell clar","Medium Dark skin tone":"To de pell mitjà-fosc","Medium Light skin tone":"To de pell mitjà-clar","Medium skin tone":"To de pell mitjà",'No emojis were found matching "%0".':'No s\'ha trobat cap emoji que coincideixi amb "%0".',Objects:"Objectes","Select skin tone":"Selecciona el to de pell","Show all emoji...":"Mostra tots els emojis...","Smileys & Expressions":"Emoticones i expressions",Symbols:"Símbols","The query must contain at least two characters.":"La consulta ha de contenir com a mínim dos caràcters.","Travel & Places":"Viatges i llocs"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));