!function(i){const n=i.ti=i.ti||{};n.dictionary=Object.assign(n.dictionary||{},{"Create link":"መራኽቦ ፍጠር","Displayed text":"",Downloadable:"ዝረግፍ","Edit link":"መራኽቦ ኣርም",Link:"መራኽቦ","Link image":"","Link properties":"","Link URL":"","Link URL must not be empty.":"","Move out of a link":"","No links available":"","Open in a new tab":"","Open link in new tab":"ንመራኽቦ ኣብ ሓዱሽ ታብ ክፈት",Unlink:"መራኽቦ ኣወግድ"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));