!function(e){const n=e.pl=e.pl||{};n.dictionary=Object.assign(n.dictionary||{},{"Code block":"Blok kodu","Entering %0 code snippet":"Wchodzenie we fragment kodu %0","Entering code snippet":"Wchodzenie we fragment kodu","Insert code block":"Wstaw blok kodu","Leaving %0 code snippet":"Opuszczenie fragmentu kodu %0","Leaving code snippet":"Opuszczanie fragmentu kodu","Plain text":"Zwykły tekst"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));