!function(t){const e=t.cs=t.cs||{};e.dictionary=Object.assign(e.dictionary||{},{"Advanced options":"Pokročilé možnosti",Find:"Najít","Find and replace":"Najít a nahradit","Find in text…":"Najít v textu...","Find in the document":"Najít v dokumentu","Match case":"Rozlišovat velikost písmen","Next result":"Další výskyt","Previous result":"Předchozí výskyt",Replace:"Nahradit","Replace all":"Nahradit vše","Replace with…":"Nahradit čím...","Text to find must not be empty.":"Hledaný text nesmí být prázdný.","Tip: Find some text first in order to replace it.":"Tip: Nejprve najděte nějaký text, abyste jej mohli nahradit.","Whole words only":"Pouze celá slova"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));