!function(e){const t=e.ur=e.ur||{};t.dictionary=Object.assign(t.dictionary||{},{"Advanced options":"",Find:"تلاش","Find and replace":"تلاش و تبدیل","Find in text…":"متن میں تلاش۔۔۔","Find in the document":"","Match case":"بڑے چھوٹے حروف کا خیال رکھیں","Next result":"اگلا نتیجہ","Previous result":"گزشتہ نتیجہ",Replace:"بدل دیں","Replace all":"تمام بدل دیں","Replace with…":"اور اس سے تبدیل کریں...","Text to find must not be empty.":"تلاش کے لیے متن خالی نہیں ہونا چاہیے۔","Tip: Find some text first in order to replace it.":"نکتہ: تبدیل کرنے کے لیے پہلے متن کو تلاش کریں۔","Whole words only":"صرف الفاظ"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));