!function(e){const a=e.sk=e.sk||{};a.dictionary=Object.assign(a.dictionary||{},{"Cannot access default workspace.":"Nie je možné získať prístup k predvolenému pracovnému priestoru.","Cannot determine a category for the uploaded file.":"Neznáma kategória pre nahratý súbor.","Edit image":"Upraviť obrázok","Failed to determine category of edited image.":"Nepodarilo sa určiť kategóriu upraveného obrázka.","Open file manager":"Otvoriť manažér súborov","Processing the edited image.":"Spracovanie upraveného obrázka.","Server failed to process the image.":"Serveru sa nepodarilo spracovať obrázok.","You have no image editing permissions.":"Nemáte žiadne povolenia na úpravu obrázkov."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));