!function(e){const i=e.ne=e.ne||{};i.dictionary=Object.assign(i.dictionary||{},{"Blue marker":"नीलो मार्कर","Green marker":"हरियो मार्कर","Green pen":"हरियो कलम",Highlight:"हाइलाइट","Pink marker":"गुलाबी मार्कर","Red pen":"रातो कलम","Remove highlight":"हाइलाइट हटाउनुहोस्","Text highlight toolbar":"","Yellow marker":"पहेंलो मार्कर"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));