!function(o){const n=o.bs=o.bs||{};n.dictionary=Object.assign(n.dictionary||{},{Big:"",Default:"Zadani","Document colors":"","Font Background Color":"Boja pozadine","Font Color":"Boja","Font Family":"Font","Font Size":"Veličina fonta",Huge:"",Small:"",Tiny:""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));