!function(e){const n=e.ar=e.ar||{};n.dictionary=Object.assign(n.dictionary||{},{"Edit source":"تحرير المصدر","Empty snippet content":"محتوى القصاصة فارغ","HTML snippet":"قصاصة برمجية بلغة HTML","Insert HTML":"إدراج HTML","No preview available":"المعاينة غير متاحة","Paste raw HTML here...":"ألصق رمز HTML الأولي هنا...","Save changes":"حفظ التغييرات"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));