!function(o){const a=o.ko=o.ko||{};a.dictionary=Object.assign(a.dictionary||{},{Bookmark:"책갈피","Bookmark must not be empty.":"책갈피 이름은 비워 둘 수 없습니다.","Bookmark name":"책갈피 이름","Bookmark name already exists.":"책갈피 이름이 이미 존재합니다.","Bookmark name cannot contain space characters.":"책갈피 이름에는 공백이 포함될 수 없습니다.","Bookmark toolbar":"북마크 도구 모음","bookmark widget":"책갈피 위젯",Bookmarks:"북마크","Edit bookmark":"책갈피 편집","Enter the bookmark name without spaces.":"공백 없이 책갈피 이름을 입력하세요.","No bookmarks available.":"사용 가능한 북마크가 없습니다.","Remove bookmark":"책갈피 제거","Scroll to bookmark":"북마크로 이동"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));