---
label: 'Configuring aliases for URLs'
top_level: true
related:
  - path.creating_alias
  - path.editing_alias
---
<h2>{% trans %}What is a URL?{% endtrans %}</h2>
<p>{% trans %}URL is the abbreviation for "Uniform Resource Locator", which is the page's address on the web. It is the "name" by which a browser identifies a page to display. In the example "Visit us at <em>example.com</em>.", <em>https://example.com</em> would be the URL for the home page of your website. Users use URLs to locate content on the web.{% endtrans %}</p>
<h2>{% trans %}What is a path?{% endtrans %}</h2>
<p>{% trans %}A path is the unique, last part of the URL for a specific function or piece of content. For example, for a page whose full URL is <em>https://example.com/node/7</em>, the path is <em>node/7</em>. Here are some examples of paths you might find in your site:{% endtrans %}</p>
<ul>
  <li>{% trans %}node/7: Path to a particular content item.{% endtrans %}</li>
  <li>{% trans %}taxonomy/term/6: Path to a taxonomy term page.{% endtrans %}</li>
  <li>{% trans %}user/3: Path to a user profile page.{% endtrans %}</li>
</ul>
<h2>{% trans %}What is an alias?{% endtrans %}</h2>
<p>{% trans %}The core software allows you to provide more understandable URLs for pages on your site, which are called <em>aliases</em>. For example, if you have an "About Us" page with the path <em>node/7</em>, you can set up an alias of <em>about</em> so that your visitors will see it as <em>https://www.example.com/about</em>.{% endtrans %}</p>
<h2>{% trans %}Overview of configuring paths, aliases, and URLs{% endtrans %}</h2>
<p>{% trans %}The core Path module provides the URL aliasing functionality. The contributed <a href="https://www.drupal.org/project/pathauto">Pathauto</a> module allows you to configure automatically-generated URL aliases for content items and other pages. See the related topics listed below for specific tasks.{% endtrans %}</p>
<h2>{% trans %}Additional resources{% endtrans %}</h2>
<ul>
  <li><a href="https://www.drupal.org/docs/user_guide/en/content-paths.html">{% trans %}Concept: Paths, Aliases, and URLs (Drupal User Guide){% endtrans %}</a></li>
</ul>
