/**
 * @file
 * Vertical Tabs.
 */

.vertical-tabs {
  margin: 1em 0 1em 15em; /* LTR */
  border: 1px solid #ccc;
}
[dir="rtl"] .vertical-tabs {
  margin-right: 15em;
  margin-left: 0;
}
.vertical-tabs__menu {
  float: left; /* LTR */
  width: 15em;
  margin: -1px 0 -1px -15em; /* LTR */
  padding: 0;
  list-style: none;
  border-top: 1px solid #ccc;
}
[dir="rtl"] .vertical-tabs__menu {
  float: right;
  margin-right: -15em;
  margin-left: 0;
}
.vertical-tabs__pane {
  margin: 0;
  border: 0;
}
.vertical-tabs__pane > summary {
  display: none;
}

/* Layout of each tab. */
.vertical-tabs__menu-item {
  border: 1px solid #ccc;
  border-top: 0;
  background: #eee;
}
.vertical-tabs__menu-item a {
  display: block;
  padding: 0.5em 0.6em;
  text-decoration: none;
}
.vertical-tabs__menu-item a:focus .vertical-tabs__menu-item-title,
.vertical-tabs__menu-item a:active .vertical-tabs__menu-item-title,
.vertical-tabs__menu-item a:hover .vertical-tabs__menu-item-title {
  text-decoration: underline;
}
.vertical-tabs__menu-item a:hover {
  outline: 1px dotted;
}
.vertical-tabs__menu-item.is-selected {
  border-right-width: 0; /* LTR */
  background-color: #fff;
}
[dir="rtl"] .vertical-tabs__menu-item.is-selected {
  border-right-width: 1px;
  border-left-width: 0;
}
.vertical-tabs__menu-item.is-selected .vertical-tabs__menu-item-title {
  color: #000;
}
.vertical-tabs__menu-item-summary {
  display: block;
  margin-bottom: 0;
  line-height: normal;
}
