!function(e){const n=e.bn=e.bn||{};n.dictionary=Object.assign(n.dictionary||{},{"Advanced options":"উন্নত বিকল্পগুলি",Find:"খুঁজুন ","Find and replace":"অনুসন্ধান ও প্রতিস্থাপন করুন","Find in text…":"টেক্সটে খুঁজুন…","Find in the document":"ডকুমেন্টে খুঁজুন","Match case":"ম্যাচ কেস","Next result":"পরবর্তী ফলাফল","Previous result":"পূর্ববর্তী ফলাফল",Replace:"প্রতিস্থাপন করুন","Replace all":"সব প্রতিস্থাপন","Replace with…":"এর সাথে প্রতিস্থাপন...","Text to find must not be empty.":"খুঁজে পেতে টেক্সট খালি হওয়া উচিত নয়।","Tip: Find some text first in order to replace it.":"পরামর্শঃ এটি প্রতিস্থাপন করতে প্রথমে কিছু টেক্সট খুঁজুন।","Whole words only":"শুধুমাত্র পুরো শব্দ"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));