!function(t){const i=t.bn=t.bn||{};i.dictionary=Object.assign(i.dictionary||{},{Bold:"বোল্ড","Bold text":"বোল্ড টেক্সট",Code:"কোড",Italic:"ইটালিক","Italic text":"ইটালিক টেক্সট","Move out of an inline code style":"ইনলাইন কোড স্টাইল থেকে সরে আসুন",Strikethrough:"স্ট্রাইকথ্রু","Strikethrough text":"স্ট্রাইকথ্রু টেক্সট",Subscript:"সাবস্ক্রিপ্ট",Superscript:"সুপারস্ক্রিপ্ট",Underline:"আন্ডারলাইন","Underline text":"আন্ডারলাইন টেক্সট"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));