<?php

declare(strict_types=1);

namespace Drupal\Tests\block\Unit;

use Drupal\block\Controller\CategoryAutocompleteController;
use Drupal\Component\Utility\Html;
use Drupal\Tests\UnitTestCase;
use PHPUnit\Framework\Attributes\CoversClass;
use PHPUnit\Framework\Attributes\DataProvider;
use PHPUnit\Framework\Attributes\Group;
use Symfony\Component\HttpFoundation\Request;

/**
 * Tests Drupal\block\Controller\CategoryAutocompleteController.
 */
#[CoversClass(CategoryAutocompleteController::class)]
#[Group('block')]
class CategoryAutocompleteTest extends UnitTestCase {

  /**
   * The autocomplete controller.
   *
   * @var \Drupal\block\Controller\CategoryAutocompleteController
   */
  protected $autocompleteController;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $block_manager = $this->createMock('Drupal\Core\Block\BlockManagerInterface');
    $block_manager->expects($this->any())
      ->method('getCategories')
      ->willReturn(['Comment', 'Node', 'None & Such', 'User']);

    $this->autocompleteController = new CategoryAutocompleteController($block_manager);
  }

  /**
   * Tests the autocomplete method.
   *
   * @param string $string
   *   The string entered into the autocomplete.
   * @param array $suggestions
   *   The array of expected suggestions.
   *
   * @see \Drupal\block\Controller\CategoryAutocompleteController::autocomplete()
   */
  #[DataProvider('providerTestAutocompleteSuggestions')]
  public function testAutocompleteSuggestions($string, $suggestions): void {
    $suggestions = array_map(function ($suggestion) {
      return ['value' => $suggestion, 'label' => Html::escape($suggestion)];
    }, $suggestions);
    $result = $this->autocompleteController->autocomplete(new Request(['q' => $string]));
    $this->assertSame($suggestions, json_decode($result->getContent(), TRUE));
  }

  /**
   * Data provider for testAutocompleteSuggestions().
   *
   * @return array
   *   An array of test cases, each containing an input string and
   *   expected autocomplete suggestions.
   */
  public static function providerTestAutocompleteSuggestions() {
    $test_parameters = [];
    $test_parameters[] = [
      'string' => 'Com',
      'suggestions' => [
        'Comment',
      ],
    ];
    $test_parameters[] = [
      'string' => 'No',
      'suggestions' => [
        'Node',
        'None & Such',
      ],
    ];
    $test_parameters[] = [
      'string' => 'us',
      'suggestions' => [
        'User',
      ],
    ];
    $test_parameters[] = [
      'string' => 'Banana',
      'suggestions' => [],
    ];
    return $test_parameters;
  }

}
