{#
/**
 * @file
 * Default theme implementation to display a multi file form widget.
 *
 * Available variables:
 * - table: Table of previously uploaded files.
 * - element: The form element for uploading another file.
 *
 * @see \Drupal\file\Hook\FileThemeHooks::preprocessFileWidgetMultiple()
 *
 * @ingroup themeable
 */
#}
{{ table }}
{{ element }}
