!function(e){const o=e["pt-br"]=e["pt-br"]||{};o.dictionary=Object.assign(o.dictionary||{},{Activities:"Atividades","Animals & Nature":"Animais e Natureza","Dark skin tone":"Tom de pele escuro","Default skin tone":"Tom de pele padrão",Emoji:"Emoji","Emoji picker":"Seletor de emojis","Find an emoji (min. 2 characters)":"Encontre um emoji (mín. de 2 caracteres)",Flags:"Bandeiras","Food & Drinks":"Comidas e Bebidas","Gestures & People":"Gestos e Pessoas","Keep on typing to see the emoji.":"Continue digitando para ver o emoji.","Light skin tone":"Tom de pele claro","Medium Dark skin tone":"Tom de pele médio-escuro","Medium Light skin tone":"Tom de pele médio-claro","Medium skin tone":"Tom de pele médio",'No emojis were found matching "%0".':'Não foram encontrados emojis correspondentes a "%0".',Objects:"Objetos","Select skin tone":"Selecione o tom de pele","Show all emoji...":"Exibir todos os emojis...","Smileys & Expressions":"Rostos e Expressões",Symbols:"Símbolos","The query must contain at least two characters.":"A consulta deve conter ao menos dois caracteres.","Travel & Places":"Viagem e Lugares"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));