!function(t){const e=t.lv=t.lv||{};e.dictionary=Object.assign(e.dictionary||{},{"Advanced options":"Uzlabotas iespējas",Find:"Meklēt","Find and replace":"Meklēt un aizstāt","Find in text…":"Meklēt tekstā...","Find in the document":"Meklēt dokumentā","Match case":"Precīza atbilstība","Next result":"Nākamais rezultāts","Previous result":"Iepriekšējais rezultāts",Replace:"Aizstāt","Replace all":"Aizstāt visu","Replace with…":"Aizstāt ar...","Text to find must not be empty.":"Meklēšanas tekstam jābūt aizpildītam.","Tip: Find some text first in order to replace it.":"Padoms: Sākumā uzmeklējiet tekstu un tikai tad aizstājiet to.","Whole words only":"Tikai pilni vārdi"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));