!function(i){const a=i.uz=i.uz||{};a.dictionary=Object.assign(a.dictionary||{},{"Insert media":"Mediani joylashtiring",Media:"","Media embed":"","Media toolbar":"Media asboblar paneli","Media URL":"Media URL manzili","media widget":"media vidjeti","Open media in new tab":"","Paste the media URL in the input.":"Media URL manzilini kiritish maydoniga joylashtiring.","The URL must not be empty.":"URL bo'sh bo'lmasligi kerak.","This media URL is not supported.":"Ushbu media URL manzili qo‘llab-quvvatlanmaydi.","Tip: Paste the URL into the content to embed faster.":"Maslahat: Tez kiritish uchun URL manzilini kontentga joylashtiring."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));