!function(t){const i=t.km=t.km||{};i.dictionary=Object.assign(i.dictionary||{},{Bold:"ដិត","Bold text":"",Code:"កូដ",Italic:"ទ្រេត","Italic text":"","Move out of an inline code style":"",Strikethrough:"ឆូតកណ្ដាល","Strikethrough text":"",Subscript:"អក្សរ​តូចក្រោម",Superscript:"អក្សរ​តូចលើ",Underline:"គូស​បន្ទាត់​ក្រោម","Underline text":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));