!function(a){const e=a.ms=a.ms||{};e.dictionary=Object.assign(e.dictionary||{},{"Align cell text to the bottom":"Jajarkan teks sel kebawah","Align cell text to the center":"Jajarkan teks sel ketengah","Align cell text to the left":"Jajarkan teks sel kekiri","Align cell text to the middle":"Jajarkan teks sel ketengah","Align cell text to the right":"Jajarkan teks sel kekanan","Align cell text to the top":"Jajarkan teks sel keatas","Align table to the left":"Jajarkan jadual sel kekiri","Align table to the left with no text wrapping":"Jajarkan jadual ke kiri tanpa pembalut teks","Align table to the left with text wrapping":"Align","Align table to the right":"Jajarkan jadual sel kekanan","Align table to the right with no text wrapping":"Align","Align table to the right with text wrapping":"Jajarkan jadual ke kanan dengan pembalut teks",Alignment:"Jajaran",Background:"Latar belakang",Border:"Sempadan","Cell properties":"Sifat sel","Center table":"Ketengahkan jadual","Center table with no text wrapping":"Center","Choose table type":"Pilih jenis jadual",Color:"Warna",Column:"Kolum","Content table":"Jadual kandungan",Dashed:"Garis putus-putus","Delete column":"Padam kolum","Delete row":"Padam baris",Dimensions:"Dimensi",Dotted:"Bertitik",Double:"Dua baris","Enter table caption":"Benarkan kapsyen jadual",Groove:"Lurah","Header column":"Kolum pengepala","Header row":"Baris pengepala",Height:"Ketinggian","Horizontal text alignment toolbar":"Bar alat capaian jajaran teks melintang","Insert a new table row (when in the last cell of a table)":"Masukkan baris jadual baharu (apabila berada dalam sel terakhir jadual)","Insert column left":"Masukkan kolum kiri","Insert column right":"Masukkan kolum kanan","Insert row above":"Masukkan baris diatas","Insert row below":"Masukkan baris dibawah","Insert table":"Masukkan jadual","Insert table layout":"Sisipkan tataletak jadual",Inset:"Inset","Justify cell text":"Imbang teks sel","Keystrokes that can be used in a table cell":"Ketukan kekunci yang boleh digunakan dalam sel jadual","Layout table":"Jadual tataletak","Merge cell down":"Cantumkan sel kebawah","Merge cell left":"Cantumkan sel kekiri","Merge cell right":"Cantumkan sel kekanan","Merge cell up":"Cantumkan sel keatas","Merge cells":"Cantumkan sel ","Move the selection to the next cell":"Alihkan pilihan ke sel seterusnya","Move the selection to the previous cell":"Alihkan pilihan ke sel sebelumnya","Navigate through the table":"Navigasi melalui jadual",None:"Tiada",Outset:"Outset",Padding:"Ketebalan",Ridge:"Batas",Row:"Baris","Select column":"Pilih kolum","Select row":"Pilih baris",Solid:"Pejal","Split cell horizontally":"Leraikan sel melintang","Split cell vertically":"Leraikan sel menegak",Style:"Gaya",Table:"Jadual","Table Alignment":"Penjajaran Jadual","Table alignment toolbar":"Bar alat capaian jajaran jadual","Table cell text alignment":"Jajaran teks sel jadual","Table layout":"Tataletak jadual","Table properties":"Sifat jadual","Table toolbar":"Bar alat capaian jadual","Table type":"Jenis jadual","Table type options":"Pilihan jenis jadual",'The color is invalid. Try "#FF0000" or "rgb(255,0,0)" or "red".':'Warna tidak sah. Cuba "#FF0000" atau "rgb(255,0,0)" atau "merah".','The value is invalid. Try "10px" or "2em" or simply "2".':'Nilai tidak sah. Cuba "10px" atau "2em" atau "2" sahaja.',"Vertical text alignment toolbar":"Bar alat capaian jajaran teks menegak",Width:"Lebar"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));