<?php

declare(strict_types=1);

namespace Drupal\plugin_test\Plugin\plugin_test\fruit;

/**
 * @Plugin(
 *   id = "orange",
 *   label = "Orange",
 *   color = "orange"
 * )
 */
class Orange implements FruitInterface {}
