!function(e){const n=e.be=e.be||{};n.dictionary=Object.assign(n.dictionary||{},{"Connected users":"","Document outline":"","Enter fullscreen mode":"","Fullscreen mode":"","Hide left sidebar":"","Leave fullscreen mode":"","Show left sidebar":"","Toggle fullscreen mode":"","Toggle sidebar":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));