!function(e){const i=e.km=e.km||{};i.dictionary=Object.assign(i.dictionary||{},{"Blue marker":"","Green marker":"","Green pen":"",Highlight:"","Pink marker":"","Red pen":"","Remove highlight":"","Text highlight toolbar":"","Yellow marker":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));