!function(e){const i=e.bg=e.bg||{};i.dictionary=Object.assign(i.dictionary||{},{Activities:"Дейности","Animals & Nature":"Животни и природа","Dark skin tone":"Тъмен цвят на кожата","Default skin tone":"Цвят на кожата по подразбиране",Emoji:"Емотикон","Emoji picker":"Избор на емотикони","Find an emoji (min. 2 characters)":"Намери емотикон (мин. 2 знака)",Flags:"Знамена","Food & Drinks":"Храна и напитки","Gestures & People":"Мимики и хора","Keep on typing to see the emoji.":"Продължи да пишеш, за да видиш емотикона.","Light skin tone":"Светъл цвят на кожата","Medium Dark skin tone":"Средно тъмен цвят на кожата","Medium Light skin tone":"Средно светъл цвят на кожата","Medium skin tone":"Среден цвят на кожата",'No emojis were found matching "%0".':"Не бяха намерени емотикони, съответстващи на „%0“.",Objects:"Предмети","Select skin tone":"Избери цвят на кожата","Show all emoji...":"Покажи всички емотикони...","Smileys & Expressions":"Усмивки и емоции",Symbols:"Символи","The query must contain at least two characters.":"Заявката трябва да съдържа поне два знака.","Travel & Places":"Пътуване и места"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));