!function(e){const a=e.pt=e.pt||{};a.dictionary=Object.assign(a.dictionary||{},{"Align cell text to the bottom":"Alinhar texto da célula no fundo","Align cell text to the center":"Alinhar texto da célula ao centro","Align cell text to the left":"Alinhar texto da célula à esquerda","Align cell text to the middle":"Alinhar texto da célula ao meio","Align cell text to the right":"Alinhar texto da célula à direita","Align cell text to the top":"Alinhar texto da célula no topo","Align table to the left":"Alinhar tabela à esquerda","Align table to the left with no text wrapping":"Alinhar a tabela à esquerda, sem moldar texto","Align table to the left with text wrapping":"Alinhar a tabela à esquerda e moldar texto","Align table to the right":"Alinhar tabela à direita","Align table to the right with no text wrapping":"Alinhar a tabela à direita, sem moldar texto","Align table to the right with text wrapping":"Alinhar a tabela à direita e moldar texto",Alignment:"Alinhamento",Background:"Fundo",Border:"Borda","Cell properties":"Propriedades da célula","Center table":"Centrar tabela","Center table with no text wrapping":"Centrar a tabela, sem moldar texto","Choose table type":"Escolher tipo de tabela",Color:"Cor",Column:"Coluna","Content table":"Tabela de conteúdo",Dashed:"Tracejado","Delete column":"Eliminar coluna","Delete row":"Eliminar linha",Dimensions:"Dimensões",Dotted:"Pontilhado",Double:"Duplo","Enter table caption":"Introduzir legenda da tabela",Groove:"Sulcos","Header column":"Coluna de cabeçalho","Header row":"Linha de cabeçalho",Height:"Altura","Horizontal text alignment toolbar":"Barra de ferramentas do alinhamento horizontal de texto","Insert a new table row (when in the last cell of a table)":"Inserir uma nova linha de tabela (quando o utilizador estiver na última célula de uma tabela)","Insert column left":"Inserir coluna à esquerda","Insert column right":"Inserir coluna à direita","Insert row above":"Inserir linha acima","Insert row below":"Inserir linha abaixo","Insert table":"Inserir tabela","Insert table layout":"Inserir esquema da tabela",Inset:"Interior","Justify cell text":"Justificar texto da célula","Keystrokes that can be used in a table cell":"Batimentos de teclas que podem ser utilizados numa célula de tabela","Layout table":"Tabela de esquemas","Merge cell down":"Unir célula abaixo","Merge cell left":"Unir célula à esquerda","Merge cell right":"Unir célula à direita","Merge cell up":"Unir célula acima","Merge cells":"Fundir células","Move the selection to the next cell":"Mover a seleção para a célula seguinte","Move the selection to the previous cell":"Mover a seleção para a célula anterior","Navigate through the table":"Navegar pela tabela",None:"Nenhum",Outset:"Exterior",Padding:"Espaçamento",Ridge:"Rebordo",Row:"Linha","Select column":"Selecionar coluna","Select row":"Selecionar linha",Solid:"Sólido","Split cell horizontally":"Dividir célula horizontalmente","Split cell vertically":"Dividir célula verticalmente",Style:"Estilo",Table:"Tabela","Table Alignment":"Alinhamento da Tabela","Table alignment toolbar":"Barra de ferramentas do alinhamento da tabela","Table cell text alignment":"Alinhamento de texto das células da tabela","Table layout":"Esquema da tabela","Table properties":"Propriedades da tabela","Table toolbar":"Barra de ferramentas da tabela","Table type":"Tipo de tabela","Table type options":"Opções tipo de tabela",'The color is invalid. Try "#FF0000" or "rgb(255,0,0)" or "red".':'A cor é inválida. Tente "#FF0000" ou "rgb(255,0,0)" ou "vermelho".','The value is invalid. Try "10px" or "2em" or simply "2".':'O valor é inválido. Tente "10px" ou "2em" ou simplesmente "2".',"Vertical text alignment toolbar":"Barra de ferramentas do alinhamento vertical de texto",Width:"Largura"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));