!function(i){const e=i.it=i.it||{};e.dictionary=Object.assign(e.dictionary||{},{"Disable block editing":"Disabilita modifica blocchi","Disable editing":"Disabilita la modifica","Disable inline editing":"Disabilita modifica lineare","Enable block editing":"Abilita modifica blocchi","Enable editing":"Abilita modifica","Enable inline editing":"Abilita modifica lineare","Navigate editable regions":"Spostati tra regioni modificabili","Next editable region":"Regione modificabile successiva","Previous editable region":"Regione modificabile precedente"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));