!function(a){const n=a.ca=a.ca||{};n.dictionary=Object.assign(n.dictionary||{},{"Choose heading":"Trieu l'encapçalament",Heading:"Encapçalament","Heading 1":"Encapçalament 1","Heading 2":"Encapçalament 2","Heading 3":"Encapçalament 3","Heading 4":"Encapçalament 4","Heading 5":"Encapçalament 5","Heading 6":"Encapçalament 6","Type or paste your content here.":"Escriviu o enganxeu el contingut aquí.","Type your title":"Escriviu el títol"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));