!function(e){const i=e.da=e.da||{};i.dictionary=Object.assign(i.dictionary||{},{"Break text":"Opdel tekst","Caption for image: %0":"Billedtekst for billede: %0","Caption for the image":"Billedtekst for billedet","Centered image":"Centreret billede","Change image text alternative":"Skift alternativ billedtekst",Custom:"Brugerdefineret","Custom image size":"Brugerdefineret billedstørrelse","Enter image caption":"Indtast billedoverskrift","Error during image upload":"Fejl under upload af billede","From computer":"Fra computer","Full size image":"Fuld billedstørrelse",Image:"Billede","Image from computer":"Billede fra computer","Image Resize":"Justering af billedstørrelse","Image resize list":"Liste over justering af billedstørrelse","Image toolbar":"Billedværktøjslinje","Image upload complete":"Billedupload færdig","Image via URL":"Billede via URL","image widget":"billed widget","In line":"På linje","Insert image":"Indsæt billede","Insert image via URL":"Indsæt billede via URL","Insert via URL":"Indsæt via URL","Left aligned image":"Venstrestillet billede",Original:"Original","Replace from computer":"Udskift fra computer","Replace image":"Udskift billede","Replace image from computer":"Udskift billede fra computer","Resize image":"Juster billedstørrelse","Resize image (in %0)":"Ændre størrelse på billede (i %0)","Resize image to %0":"Juster billedstørrelse til %0","Resize image to the original size":"Juster billedstørrelse til original størrelse","Right aligned image":"Højrestillet billede","Side image":"Sidebillede","Text alternative":"Alternativ tekst","Text Alternative":"Tekst","The value must not be empty.":"Værdien må ikke være tom","The value should be a plain number.":"Værdien skal være et almindeligt tal","Update image URL":"Opdater billed-URL","Upload failed":"Upload fejlede","Upload from computer":"Upload fra computer","Upload image from computer":"Upload billede fra computer","Uploading image":"Uploader billede","Via URL":"Via URL","Wrap text":"Ombryd tekst","You have no image upload permissions.":"Du har ikke tilladelser til at uploade billeder."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));