/**
 * @file
 * Styles used by the Update Status module.
 */

.project-update__title {
  font-size: 110%;
  font-weight: bold;
}
.project-update__status {
  float: right; /* LTR */
  font-size: 110%;
}
[dir="rtl"] .project-update__status {
  float: left;
}
.project-update__status--not-supported {
  float: right; /* LTR */
}
[dir="rtl"] .project-update__status--not-supported {
  float: left;
}
.project-update__status--security-error {
  color: #970f00;
  font-weight: bold;
}

.project-update__status-icon {
  padding-left: 0.5em; /* LTR */
}
[dir="rtl"] .project-update__status-icon {
  padding-right: 0.5em;
  padding-left: 0;
}
.project-update__details {
  padding: 1em 1em 0.25em 1em;
}

.project-update__version {
  padding: 1em 0;
}
.project-update__version-date {
  white-space: nowrap;
}
.project-update__version-details {
  padding-right: 0.5em; /* LTR */
}
[dir="rtl"] .project-update__version-details {
  padding-left: 0.5em;
  direction: ltr; /* Version numbers should always be LTR. */
}
.project-update__version-links {
  padding-right: 1em; /* LTR */
  list-style-type: none;
  text-align: right; /* LTR */
}
[dir="rtl"] .project-update__version-links {
  padding-left: 1em;
  text-align: left;
}
.project-update__version--recommended-strong .project-update__version-title {
  font-weight: bold;
}
