!function(e){const n=e.nl=e.nl||{};n.dictionary=Object.assign(n.dictionary||{},{"Cannot access default workspace.":"Geen toegang mogelijk tot standaard werkplek.","Cannot determine a category for the uploaded file.":"Kan geen categorie bepalen voor het geüploade bestand.","Edit image":"Afbeelding bewerken","Failed to determine category of edited image.":"Het is niet gelukt om de categorie van de bewerkte afbeelding te bepalen.","Open file manager":"Open bestandsmanager","Processing the edited image.":"De bewerkte afbeelding verwerken.","Server failed to process the image.":"De server heeft de afbeelding niet verwerkt.","You have no image editing permissions.":"U heeft geen toestemming om afbeeldingen te bewerken."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));