/*
 * @file
 * Provides the layout styles for layout_threecol_33_34_33.
 */

.layout--threecol-33-34-33 {
  display: flex;
  flex-wrap: wrap;
}

.layout--threecol-33-34-33 > .layout__region {
  flex: 0 1 100%;
}

@media screen and (min-width: 40em) {
  .layout--threecol-33-34-33 > .layout__region--first,
  .layout--threecol-33-34-33 > .layout__region--third {
    flex: 0 1 33%;
  }
  .layout--threecol-33-34-33 > .layout__region--second {
    flex: 0 1 34%;
  }
}
