---
label: 'Making your site secure'
top_level: true
---
<h2>{% trans %}What are security updates?{% endtrans %}</h2>
<p>{% trans %}Any software occasionally has bugs, and sometimes these bugs have security implications. When security bugs are fixed in the core software, modules, or themes that your site uses, they are released in a <em>security update</em>. You will need to apply security updates in order to keep your site secure.{% endtrans %}</p>
<h2>{% trans %}What are security advisories?{% endtrans %}</h2>
<p>{% trans %}A security advisory is a public announcement about a reported security problem in the core software. Contributed projects with a shield icon and "Stable releases for this project are covered by the security advisory policy" on their project page are also covered by Drupal's security advisory policy. Security advisories are managed by the <a href="https://www.drupal.org/drupal-security-team">Drupal Security Team</a>.{% endtrans %}</p>
<h2>{% trans %}Security tasks{% endtrans %}</h2>
<p>{% trans %}Keeping track of updates, updating the core software, and updating contributed modules and/or themes are all part of keeping your site secure. See the related topics listed below for specific tasks.{% endtrans %}</p>
<h2>{% trans %}Additional resources{% endtrans %}</h2>
<ul>
    <li>{% trans %}<a href="https://www.drupal.org/docs/user_guide/en/security-chapter.html">Security and Maintenance (Drupal User Guide)</a>{% endtrans %}</li>
  <li>{% trans %}<a href="https://www.drupal.org/drupal-security-team/security-advisory-process-and-permissions-policy">Security advisory process and permissions policy</a>{% endtrans %}</li>
</ul>
