<?php

/**
 * @file
 * Generic transliteration data for the PhpTransliteration class.
 */

$base = [
  0x00 => NULL, 'mian', 'ji', 'fang', 'pei', 'ai', 'fan', 'ao', 'qin', 'qia', 'xiao', 'fen', 'gan', 'qiao', 'ge', 'tong',
  0x10 => 'chan', 'you', 'gao', 'ben', 'fu', 'chu', 'zhu', NULL, 'zhou', NULL, 'hang', 'nin', 'jue', 'chong', 'cha', 'kong',
  0x20 => 'lie', 'li', 'yu', NULL, 'yu', 'hai', 'li', 'hou', 'gong', 'ke', 'yuan', 'de', 'hui', NULL, 'guang', 'jiong',
  0x30 => 'zuo', 'fu', 'qie', 'bei', 'che', 'ci', 'mang', 'han', 'xi', 'qiu', 'huang', NULL, NULL, 'chou', 'san', 'yan',
  0x40 => 'zhi', 'de', 'te', 'men', 'ling', 'shou', 'tui', 'can', 'die', 'che', 'peng', 'yi', 'ju', 'ji', 'lai', 'tian',
  0x50 => 'yuan', NULL, 'cai', 'qi', 'yu', 'lian', 'cong', NULL, NULL, NULL, 'yu', 'ji', 'wei', 'mi', 'sui', 'xie',
  0x60 => 'xu', 'chi', 'qiu', 'hui', NULL, 'yu', 'qie', 'shun', 'shui', 'duo', 'lou', NULL, 'pang', 'tai', 'zhou', 'yin',
  0x70 => 'sao', 'fei', 'chen', 'yuan', 'yi', 'hun', 'se', 'ye', 'min', 'fen', 'he', NULL, 'yin', 'ce', 'ni', 'ao',
  0x80 => 'feng', 'lian', 'chang', 'chan', 'ma', 'die', 'hu', 'lu', NULL, 'yi', 'hua', 'zha', 'hu', 'e', 'huo', 'sun',
  0x90 => 'ni', 'xian', 'li', 'xian', 'yan', 'long', 'men', 'jin', 'ji', NULL, 'bian', 'yu', 'huo', 'miao', 'chou', 'mai',
  0xA0 => NULL, 'le', 'jie', 'wei', 'yi', 'xuan', 'xi', 'can', 'lan', 'yin', 'xie', 'za', 'luo', 'ling', 'qian', 'huo',
  0xB0 => 'jian', 'wo', NULL, NULL, 'ge', 'zhu', 'die', 'yong', 'ji', 'yang', 'ru', 'xi', 'shuang', 'yu', 'yi', 'qian',
  0xC0 => 'ji', 'qu', 'tian', 'shou', 'qian', 'mu', 'jin', 'mao', 'yin', 'gai', 'po', 'xuan', 'mao', 'fang', 'ya', 'gang',
  0xD0 => 'song', 'hui', 'yu', 'gua', 'guai', 'liu', 'e', 'zi', 'zi', 'bi', 'wa', NULL, 'lie', NULL, NULL, 'kuai',
  0xE0 => NULL, 'hai', 'yin', 'zhu', 'chong', 'xian', 'xuan', NULL, 'qiu', 'pei', 'gui', 'er', 'gong', 'qiong', 'hu', 'lao',
  0xF0 => 'li', 'chen', 'san', 'zhuo', 'wo', 'pou', 'keng', 'tun', 'peng', 'te', 'ta', 'zhuo', 'biao', 'gu', 'hu', NULL,
];
