!function(n){const t=n.sv=n.sv||{};t.dictionary=Object.assign(t.dictionary||{},{"Code block":"Kodblock","Entering %0 code snippet":"Anger %0-kodsnutt","Entering code snippet":"Anger kodsnutt","Insert code block":"Infoga kodblock","Leaving %0 code snippet":"Lämnar %0-kodsnutt","Leaving code snippet":"Lämnar kodsnutt","Plain text":"Vanlig text"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));