!function(e){const i=e.de=e.de||{};i.dictionary=Object.assign(i.dictionary||{},{"Disable block editing":"Blockbearbeitung deaktivieren","Disable editing":"Bearbeitung deaktivieren","Disable inline editing":"Inline-Bearbeitung deaktivieren","Enable block editing":"Blockbearbeitung aktivieren","Enable editing":"Bearbeitung zulassen","Enable inline editing":"Inline-Bearbeitung aktivieren","Navigate editable regions":"Durch bearbeitbare Bereiche navigieren","Next editable region":"Nächster bearbeitbarer Bereich","Previous editable region":"Vorheriger bearbeitbarer Bereich"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));