!function(e){const t=e.bn=e.bn||{};t.dictionary=Object.assign(t.dictionary||{},{"Align cell text to the bottom":"নীচে সেল টেক্সট সারিবদ্ধ করুন","Align cell text to the center":"কেন্দ্রে সেল টেক্সট সারিবদ্ধ করুন","Align cell text to the left":"বাম দিকে সেল টেক্সট সারিবদ্ধ করুন","Align cell text to the middle":"মাঝখানে সেল টেক্সট সারিবদ্ধ করুন","Align cell text to the right":"ডানদিকে সেল টেক্সট সারিবদ্ধ করুন ","Align cell text to the top":"শীর্ষে সেল টেক্সট সারিবদ্ধ করুন","Align table to the left":"বাম দিকে টেবিল সারিবদ্ধ করুন","Align table to the left with no text wrapping":"টেক্সট র‍্যাপিং ছাড়া টেবিল বাম দিকে সমন্বয় করুন","Align table to the left with text wrapping":"টেক্সট র‍্যাপিংসহ টেবিল বাম দিকে সমন্বয় করুন","Align table to the right":"ডানদিকে টেবিলটি সারিবদ্ধ করুন","Align table to the right with no text wrapping":"টেক্সট র‍্যাপিং ছাড়া টেবিল ডান দিকে সমন্বয় করুন","Align table to the right with text wrapping":"টেক্সট র‍্যাপিংসহ টেবিল ডান দিকে সমন্বয় করুন",Alignment:"সারিবদ্ধকরণ",Background:"ব্যাকগ্রাউন্ড",Border:"বর্ডার ","Cell properties":"সেল বৈশিষ্ট্য","Center table":"কেন্দ্রের টেবিল","Center table with no text wrapping":"টেক্সট র‍্যাপিং ছাড়া টেবিল কেন্দ্রে সমন্বয় করুন","Choose table type":"টেবিল টাইপ নির্বাচন করুন",Color:"  রং",Column:"কলাম","Content table":"কনটেন্ট টেবিল",Dashed:"ড্যাশড","Delete column":"কলাম মুছে ফেলুন","Delete row":"সারি মুছুন",Dimensions:"মাত্রাগুলো",Dotted:"ডটেড",Double:"দ্বিগুণ","Enter table caption":"টেবিল ক্যাপশন লিখুন",Groove:"খাঁজকাটা","Header column":"হেডার কলাম","Header row":"হেডার সারি",Height:"উচ্চতা","Horizontal text alignment toolbar":"অনুভূমিক টেক্সট সারিবদ্ধকরণ টুলবার","Insert a new table row (when in the last cell of a table)":"একটি নতুন টেবিলের সারি প্রবেশ করুন (কোনো টেবিলের শেষ সেলে থাকা অবস্থায়)","Insert column left":"বাম দিকে কলাম ঢোকান","Insert column right":"ডানদিকে কলাম ঢোকান","Insert row above":"উপরে সারি ঢোকান","Insert row below":"নীচে সারি ঢোকান","Insert table":"টেবিল ঢোকান","Insert table layout":"**টেবিল লেআউট যুক্ত করুন**",Inset:"ইনসেট","Justify cell text":"সেল টেক্সট জাস্টিফাই করুন","Keystrokes that can be used in a table cell":"যে কীস্ট্রোকগুলি টেবিল সেলে ব্যবহার করা যেতে পারে","Layout table":"লেআউট টেবিল","Merge cell down":"নিচে সেল মার্জ করুন","Merge cell left":"বামদিকে সেল মার্জ করুন","Merge cell right":" ডানদিকে সেল মার্জ করুন","Merge cell up":"সেল আপ মার্জ","Merge cells":"সেল একত্রিত করুন","Move the selection to the next cell":"পরবর্তী সেলে সিলেকশন স্থানান্তর করুন","Move the selection to the previous cell":"পূর্ববর্তী সেলে সিলেকশন স্থানান্তর করুন","Navigate through the table":"টেবিলের মধ্যে দিয়ে নেভিগেট করুন",None:"কোনোটিই নয়",Outset:"শুরু",Padding:"প্যাডিং",Ridge:"রিজ",Row:"সারি ","Select column":"কলাম নির্বাচন করুন","Select row":"সারি নির্বাচন করুন",Solid:"সলিড","Split cell horizontally":"অনুভূমিকভাবে সেল বিভক্ত করুন","Split cell vertically":"সেল উল্লম্বভাবে বিভক্ত করুন",Style:"স্টাইল ",Table:"টেবিল","Table Alignment":"টেবিল সমন্বয়","Table alignment toolbar":"টেবিল সারিবদ্ধকরণ টুলবার","Table cell text alignment":"সক্ষম সেল টেক্সট সারিবদ্ধকরণ","Table layout":"টেবিল লেআউট","Table properties":"টেবিল বৈশিষ্ট্য","Table toolbar":"টেবিল টুলবার","Table type":"টেবিল টাইপ","Table type options":"টেবিল টাইপ অপশনসমূহ",'The color is invalid. Try "#FF0000" or "rgb(255,0,0)" or "red".':'রংটি সঠিক নয়। "#FF0000" অথবা "rgb(255,0,0)" অথবা "লাল" ব্যাবহার করুন।\n','The value is invalid. Try "10px" or "2em" or simply "2".':'মানটি সঠিক নয়। "10px" বা "2em" বা সহজভাবে "2" ব্যবহার করে দেখুন।',"Vertical text alignment toolbar":"উল্লম্ব টেক্সট সারিবদ্ধকরণ টুলবার",Width:"প্রস্থ"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));