{#
/**
 * @file
 * Default theme implementation for a missing media error.
 *
 * Available variables
 * - message: The message text.
 * - attributes: HTML attributes for the containing element.
 *
 * When a response from the back end can't be returned, a related error message
 * is displayed from JavaScript.
 *
 * @see Drupal.theme.mediaEmbedPreviewError
 *
 * @ingroup themeable
 */
#}
<div{{ attributes }}>
  {{ message }}
</div>
