<?php

namespace Drupal\Core\Asset\Exception;

/**
 * Defines a custom exception for an invalid libraries-extend specification.
 */
class InvalidLibrariesExtendSpecificationException extends \RuntimeException {

}
