!function(i){const t=i.lt=i.lt||{};t.dictionary=Object.assign(t.dictionary||{},{"Edit source":"Redaguoti šaltinį","Empty snippet content":"Tuščias iškarpos turinys","HTML snippet":"HTML iškarpa","Insert HTML":"Įkelti HTML","No preview available":"Nėra galimų peržiūrų","Paste raw HTML here...":"Įklijuokite neapdorotą HTML čia...","Save changes":"Išsaugoti pakeitimus"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));