!function(e){const t=e.zh=e.zh||{};t.dictionary=Object.assign(t.dictionary||{},{"Align cell text to the bottom":"向下對齊","Align cell text to the center":"置中對齊","Align cell text to the left":"靠左對齊","Align cell text to the middle":"置中對齊","Align cell text to the right":"靠右對齊","Align cell text to the top":"向上對齊","Align table to the left":"靠左對齊","Align table to the left with no text wrapping":"表格靠左對齊，文字不換行","Align table to the left with text wrapping":"表格靠左對齊，文字自動換行","Align table to the right":"靠右對齊","Align table to the right with no text wrapping":"表格靠右對齊，文字自動換行","Align table to the right with text wrapping":"表格靠右對齊，文字自動換行",Alignment:"對齊",Background:"背景顏色",Border:"邊框","Cell properties":"儲存格屬性","Center table":"置中對齊","Center table with no text wrapping":"表格置中，文字不換行","Choose table type":"選擇表格類型",Color:"文字顏色",Column:"欄","Content table":"內容表格",Dashed:"虛線","Delete column":"刪除欄","Delete row":"刪除列",Dimensions:"尺寸",Dotted:"點線",Double:"雙線","Enter table caption":"輸入表標題",Groove:"內凹線","Header column":"標題欄","Header row":"標題列",Height:"高度","Horizontal text alignment toolbar":"水平對齊","Insert a new table row (when in the last cell of a table)":"插入新的表格橫排（當位於表格的最後一個單元格時）","Insert column left":"插入左方欄","Insert column right":"插入右方欄","Insert row above":"插入上方列","Insert row below":"插入下方列","Insert table":"插入表格","Insert table layout":"插入表格版面配置",Inset:"內邊線","Justify cell text":"分散對齊","Keystrokes that can be used in a table cell":"可在表格單元格中使用的按鍵","Layout table":"版面配置表格","Merge cell down":"合併下方儲存格","Merge cell left":"合併左方儲存格","Merge cell right":"合併右方儲存格","Merge cell up":"合併上方儲存格","Merge cells":"合併儲存格","Move the selection to the next cell":"將選取範圍移動到下一個單元格","Move the selection to the previous cell":"將選取範圍移動到上一個單元格","Navigate through the table":"在表格中移動",None:"無",Outset:"外框線",Padding:"儲存格留白",Ridge:"凸起線",Row:"列","Select column":"選擇欄","Select row":"選擇列",Solid:"實線","Split cell horizontally":"水平分割儲存格","Split cell vertically":"垂直分割儲存格",Style:"樣式",Table:"表格","Table Alignment":"表格對齊","Table alignment toolbar":"表格對齊","Table cell text alignment":"儲存格文字對齊","Table layout":"表格版面配置","Table properties":"表格屬性","Table toolbar":"表格工具","Table type":"表格類型","Table type options":"表格類型選項",'The color is invalid. Try "#FF0000" or "rgb(255,0,0)" or "red".':'顏色代碼錯誤。試試看 "#FF0000" 或 "rgb(255, 0, 0)" 或 "red"。','The value is invalid. Try "10px" or "2em" or simply "2".':'尺寸代碼錯誤。試試看 "10px" 或 "2em" 或簡單寫 "2"。',"Vertical text alignment toolbar":"垂直對齊",Width:"寬度"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));