!function(i){const n=i.hr=i.hr||{};n.dictionary=Object.assign(n.dictionary||{},{"Create link":"","Displayed text":"",Downloadable:"Moguće preuzeti","Edit link":"Uredi vezu",Link:"Veza","Link image":"URL slike","Link properties":"","Link URL":"URL veze","Link URL must not be empty.":"","Move out of a link":"","No links available":"","Open in a new tab":"Otvori u novoj kartici","Open link in new tab":"Otvori vezu u novoj kartici",Unlink:"Ukloni vezu"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));