!function(e){const s=e.hu=e.hu||{};s.dictionary=Object.assign(s.dictionary||{},{"Advanced options":"Speciális beállítások",Find:"Keresés","Find and replace":"Keresés és csere","Find in text…":"Keresés szövegben...","Find in the document":"Keresés a dokumentumban","Match case":"Nagybetű érzékeny","Next result":"Következő találat","Previous result":"Előző találat",Replace:"Csere","Replace all":"Mind cserél","Replace with…":"Csere erre...","Text to find must not be empty.":"A keresendő szöveg nem lehet üres.","Tip: Find some text first in order to replace it.":"Tipp: Először keressen egy szöveget, hogy lecserélhesse.","Whole words only":"Csak teljes szavak"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));