!function(e){const i=e.ur=e.ur||{};i.dictionary=Object.assign(i.dictionary||{},{"Blue marker":"نیلا نشان","Green marker":"سبز نشان","Green pen":"سبز قلم",Highlight:"نمایاں","Pink marker":"گلابی نشان","Red pen":"سرخ قلم","Remove highlight":"غیر نمایاں کریں","Text highlight toolbar":"خانہ آلات برائے نمایاں متن","Yellow marker":"پیلا نشان"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));