!function(e){const i=e.id=e.id||{};i.dictionary=Object.assign(i.dictionary||{},{Bold:"Tebal","Bold text":"Teks tebal",Code:"Kode",Italic:"Miring","Italic text":"Teks miring","Move out of an inline code style":"Keluar dari gaya kode sebaris",Strikethrough:"Coret","Strikethrough text":"Teks yang dicoret",Subscript:"Subskrip",Superscript:"Superskrip",Underline:"Garis bawah","Underline text":"Teks bergaris bawah"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));