!function(i){const e=i.bs=i.bs||{};e.dictionary=Object.assign(e.dictionary||{},{"Could not insert image at the current position.":"Nije moguće umetnuti sliku na poziciju.","Could not obtain resized image URL.":"Nije moguće dobiti URL slike.","Insert image or file":"Umetni sliku ili datoteku","Inserting image failed":"Umetanje slike neuspješno","Selecting resized image failed":"Odabir slike nije uspješan"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));