!function(i){const e=i.be=i.be||{};e.dictionary=Object.assign(e.dictionary||{},{"Disable block editing":"","Disable editing":"","Disable inline editing":"","Enable block editing":"","Enable editing":"Уключыць рэдагаванне","Enable inline editing":"","Navigate editable regions":"Навігацыя па рэдагавальных абласцях","Next editable region":"Наступная вобласць для рэдагавання","Previous editable region":"Папярэдняя вобласць для рэдагавання"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));