!function(e){const i=e.gl=e.gl||{};i.dictionary=Object.assign(i.dictionary||{},{"Insert media":"Inserir elemento multimedia",Media:"Elemento multimedia","Media embed":"","Media toolbar":"Barra de ferramentas de multimedia","Media URL":"URL multimedia","media widget":"Trebello multimedia","Open media in new tab":"Abrir multimedia nunha nova lapela","Paste the media URL in the input.":"Pegue o URL do medio na entrada.","The URL must not be empty.":"O URL non debe estar baleiro.","This media URL is not supported.":"Este URL multimedia non é compatible.","Tip: Paste the URL into the content to embed faster.":"Consello: Pegue o URL no contido para incrustalo máis rápido."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));