---
label: 'Using in-line (quick) settings editing'
related:
  - core.ui_components
---
<h2>{% trans %}Goal{% endtrans %}</h2>
<p>{% trans %}Edit settings in place.{% endtrans %}</p>
<h2>{% trans %}What is quick editing?{% endtrans %}</h2>
<p>{% trans %}The core Settings Tray module provides the ability to quickly edit settings inline. It requires the core Contextual Links module in order to expose the links that let you edit in place.{% endtrans %}</p>
<h2>{% trans %}Who can edit settings in place?{% endtrans %}</h2>
<p>{% trans %}In order to follow these steps to edit settings in place, the core Settings Tray module must be installed. Also, either the core Toolbar module or a contributed replacement must be installed. You will need to have <em>Use contextual links</em> permission, as well as permission to edit the particular content or settings.{% endtrans %}</p>
<h2>{% trans %}Steps{% endtrans %}</h2>
<ol>
  <li>{% trans %}Find and visit a page on your site that has the settings that you would like to edit.{% endtrans %}</li>
  <li>{% trans %}Click the contextual links <em>Edit</em> button on the toolbar (in most themes, it looks like a pencil). Contextual <em>Edit</em> links with the same icon will appear all over your page.{% endtrans %}</li>
  <li>{% trans %}Find the contextual link for the part of the page you want to edit. For example, if you want to edit the settings for a block, the link should be in the top-right corner of the block, or top-left for right-to-left languages.{% endtrans %}</li>
  <li>{% trans %}Click the link to open the contextual links menu, and click <em>Quick edit</em>. An editing form for the settings should appear on the page.{% endtrans %}</li>
  <li>{% trans %}Make your edits and submit the form.{% endtrans %}</li>
</ol>
