!function(n){const e=n.sq=n.sq||{};e.dictionary=Object.assign(e.dictionary||{},{"Create link":"","Displayed text":"",Downloadable:"E shkarkueshme","Edit link":"Redakto nyjën",Link:"Shto nyjën","Link image":"Foto e nyjes","Link properties":"","Link URL":"Nyja e URL-së","Link URL must not be empty.":"","Move out of a link":"","No links available":"","Open in a new tab":"Hape në një fletë të re","Open link in new tab":"Hap nyjën në faqe të re",Unlink:"Largo nyjën"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));