!function(t){const i=t.tr=t.tr||{};i.dictionary=Object.assign(i.dictionary||{},{Bold:"Kalın","Bold text":"Kalın yazı",Code:"Kod",Italic:"İtalik","Italic text":"İtalik metin","Move out of an inline code style":"Satır içi kod stilinden çık",Strikethrough:"Üstü çizili","Strikethrough text":"Metnin üstünü çiz",Subscript:"Alt Simge",Superscript:"Üst Simge",Underline:"Altı Çizgili","Underline text":"Metnin altını çiz"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));