!function(i){const e=i.zh=i.zh||{};e.dictionary=Object.assign(e.dictionary||{},{"Disable block editing":"停用區塊編輯","Disable editing":"禁止編輯","Disable inline editing":"停用直接編輯","Enable block editing":"啟用區塊編輯","Enable editing":"啟用編輯","Enable inline editing":"啟用直接編輯","Navigate editable regions":"編輯區域導航","Next editable region":"下一個可編輯區域","Previous editable region":"上一個可編輯區域"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));