!function(e){const a=e.ko=e.ko||{};a.dictionary=Object.assign(a.dictionary||{},{"Break text":"텍스트 분리","Caption for image: %0":"이미지용 캡션: %0","Caption for the image":"이미지용 캡션","Centered image":"가운데 정렬","Change image text alternative":"대체 문구 변경",Custom:"사용자 지정","Custom image size":"사용자 지정 이미지 크기","Enter image caption":"사진 설명을 입력하세요","Error during image upload":"이미지 업로드 중에 오류 발생","From computer":"컴퓨터에서","Full size image":"꽉 찬 크기",Image:"이미지 삽입 기능으로 삽입된 위젯","Image from computer":"컴퓨터에서 이미지 가져오기","Image Resize":"이미지 크기 조절","Image resize list":"사진 크기 목록","Image toolbar":"사진 툴바","Image upload complete":"이미지 업로드 완료","Image via URL":"URL 메뉴를 통한 이미지 삽입","image widget":"사진 위젯","In line":"줄 안에","Insert image":"사진 삽입","Insert image via URL":"URL로 이미지 삽입","Insert via URL":"URL 버튼으로 이미지 삽입","Left aligned image":"왼쪽 정렬",Original:"원본","Replace from computer":"컴퓨터에서 교체","Replace image":"이미지 교체","Replace image from computer":"컴퓨터에서 이미지 교체","Resize image":"사진 크기 조절","Resize image (in %0)":"이미지 크기 조정하기(%0 이내)","Resize image to %0":"사진의 크기를 %0으로 조절","Resize image to the original size":"사진을 원래 크기로 돌려놓기","Right aligned image":"오른쪽 정렬","Side image":"본문 옆에 배치","Text alternative":"대체 문구","Text Alternative":"대체 텍스트","The value must not be empty.":"값은 비워둘 수 없습니다.","The value should be a plain number.":"일반 숫자로 된 값을 입력해야 합니다.","Update image URL":"이미지 URL 업데이트","Upload failed":"업로드 실패","Upload from computer":"컴퓨터에서 업로드","Upload image from computer":"컴퓨터에서 이미지 업로드","Uploading image":"이미지 업로드 중","Via URL":"이미지 메뉴에서 URL을 통한 이미지 삽입","Wrap text":"텍스트 줄 바꿈","You have no image upload permissions.":"이미지를 업로드할 수 있는 권한이 없습니다."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));