!function(e){const t=e.nl=e.nl||{};t.dictionary=Object.assign(t.dictionary||{},{"Block styles":"Blok stijlen","Multiple styles":"Meerdere stijlen",Styles:"Stijlen","Text styles":"Tekst stijlen"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));