/*
 * DO NOT EDIT THIS FILE.
 * See the following change record for more information,
 * https://www.drupal.org/node/3084859
 * @preserve
 */

/**
 * @file
 * Body scroll lock with css only.
 */

[data-admin-toolbar-body-scroll="locked"] {
  overflow: clip;

  body {
    overflow: clip;
  }

  @media (min-width: 64rem) {
    overflow: initial;

    body {
      overflow: initial;
    }
  }
}
