!function(e){const t=e.ro=e.ro||{};t.dictionary=Object.assign(t.dictionary||{},{"Advanced options":"Opțiuni avansate",Find:"Găsire","Find and replace":"Găsire și înlocuire","Find in text…":"Găsire în text...","Find in the document":"Căutare în document","Match case":"Potrivire litere mari și mici","Next result":"Rezultatul următor","Previous result":"Rezultatul anterior",Replace:"Înlocuire","Replace all":"Înlocuire toate","Replace with…":"Înlocuire cu...","Text to find must not be empty.":"„Text de găsit” nu poate fi lăsat gol.","Tip: Find some text first in order to replace it.":"Sfat: Mai întâi găsiți textul pentru a-l înlocui.","Whole words only":"Doar cuvinte întregi"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));