!function(i){const e=i.lt=i.lt||{};e.dictionary=Object.assign(e.dictionary||{},{"Blue marker":"Mėlynas žymeklis","Green marker":"Žalias žymeklis","Green pen":"Žalias žymeklis",Highlight:"Pažymėti žymekliu","Pink marker":"Rožinis žymeklis","Red pen":"Raudonas žymeklis","Remove highlight":"Panaikinti pažymėjimą","Text highlight toolbar":"Teksto paryškinimo įrankių juosta","Yellow marker":"Geltonas žymeklis"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));