{#
/**
 * @file
 * Default theme implementation for a 3x3 grid of checkboxes for image anchors.
 *
 * Available variables:
 * - table: HTML for the table of image anchors.
 *
 * @see \Drupal\image\Hook\ImageThemeHooks::preprocessImageAnchor()
 *
 * @ingroup themeable
 */
#}
{{ table }}
