!function(e){const a=e.sq=e.sq||{};a.dictionary=Object.assign(a.dictionary||{},{"Break text":"","Caption for image: %0":"","Caption for the image":"","Centered image":"Foto e vendosur në mes","Change image text alternative":"Ndrysho tekstin zgjedhor të fotos",Custom:"","Custom image size":"","Enter image caption":"Shto përshkrimin e fotos","Error during image upload":"","From computer":"","Full size image":"Foto me madhësi të plotë",Image:"","Image from computer":"","Image Resize":"","Image resize list":"","Image toolbar":"Kokështrirja e fotos","Image upload complete":"","Image via URL":"","image widget":"Vegla e fotos","In line":"","Insert image":"Shto Foto","Insert image via URL":"","Insert via URL":"","Left aligned image":"Foto e vendosur majtas",Original:"","Replace from computer":"","Replace image":"","Replace image from computer":"","Resize image":"","Resize image (in %0)":"","Resize image to %0":"","Resize image to the original size":"","Right aligned image":"Foto e vendosur djathtas","Side image":"Foto anësore","Text alternative":"Teksti zgjedhor","Text Alternative":"","The value must not be empty.":"","The value should be a plain number.":"","Update image URL":"","Upload failed":"Ngarkimi dështoi","Upload from computer":"","Upload image from computer":"","Uploading image":"","Via URL":"","Wrap text":"","You have no image upload permissions.":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));