<?php

/**
 * @file
 * Generic transliteration data for the PhpTransliteration class.
 */

$base = [
  0x00 => NULL, NULL, 'N', NULL, NULL, 'a', 'a', 'i', 'i', 'u', 'u', NULL, NULL, NULL, NULL, 'e',
  0x10 => 'ai', NULL, NULL, 'o', 'au', 'ka', 'kha', 'ga', 'gha', 'na', 'ca', 'cha', 'ja', 'jha', 'na', 'ta',
  0x20 => 'tha', 'da', 'dha', 'na', 'ta', 'tha', 'da', 'dha', 'na', NULL, 'pa', 'pha', 'ba', 'bha', 'ma', 'ya',
  0x30 => 'ra', NULL, 'la', 'la', NULL, 'va', 'sa', NULL, 'sa', 'ha', NULL, NULL, '\'', NULL, 'a', 'i',
  0x40 => 'i', 'u', 'uu', NULL, NULL, NULL, NULL, 'ee', 'ai', NULL, NULL, 'oo', 'au', '', NULL, NULL,
  0x50 => NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 'kha', 'ga', 'ja', 'ra', NULL, 'pha', NULL,
  0x60 => NULL, NULL, NULL, NULL, NULL, NULL, '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
  0x70 => 'N', 'H', '', '', 'G.E.O.', NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  0x80 => NULL, 'N', 'N', 'h', NULL, 'a', 'a', 'i', 'i', 'u', 'u', 'r', 'l', 'e', NULL, 'e',
  0x90 => 'ai', 'o', NULL, 'o', 'au', 'ka', 'kha', 'ga', 'gha', 'na', 'ca', 'cha', 'ja', 'jha', 'na', 'ta',
  0xA0 => 'tha', 'da', 'dha', 'na', 'ta', 'tha', 'da', 'dha', 'na', NULL, 'pa', 'pha', 'ba', 'bha', 'ma', 'ya',
  0xB0 => 'ra', NULL, 'la', 'la', NULL, 'va', 'sa', 'sa', 'sa', 'ha', NULL, NULL, '\'', '\'', 'a', 'i',
  0xC0 => 'i', 'u', 'uu', 'R', 'RR', 'eN', NULL, 'e', 'ai', 'o', NULL, 'o', 'au', '', NULL, NULL,
  0xD0 => '\'om', NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  0xE0 => 'r', 'l', NULL, NULL, NULL, NULL, '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
  0xF0 => NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
];
