---
label: 'Maintaining and troubleshooting your site'
top_level: true
related:
  - core.cron
  - core.extending
  - core.security
  - system.cache
  - system.config_error
  - system.maintenance_mode
---
<h2>{% trans %}Maintaining and troubleshooting overview{% endtrans %}</h2>
<p>{% trans %}Here are some tasks and hints related to maintaining your site, and troubleshooting problems that may come up on your site. See the related topics below for more information.{% endtrans %}</p>
<ul>
  <li>{% trans %}When performing maintenance, such as installing, uninstalling, or updating a module, put your site in maintenance mode.{% endtrans %}</li>
  <li>{% trans %}Configure your site so that cron runs periodically.{% endtrans %}</li>
  <li>{% trans %}If your site is not behaving as expected, clear the cache before trying to diagnose the problem.{% endtrans %}</li>
  <li>{% trans %}There are several site reports that can help you diagnose problems with your site. There are also two core modules that can be used for error logging: Database Logging and Syslog.{% endtrans %}</li>
</ul>
<h2>{% trans %}Additional resources{% endtrans %}</h2>
<ul>
    <li>{% trans %}<a href="https://www.drupal.org/docs/user_guide/en/prevent-chapter.html">Preventing and Fixing Problems (Drupal User Guide)</a>{% endtrans %}</li>
    <li>{% trans %}<a href="https://www.drupal.org/docs/user_guide/en/security-chapter.html">Security and Maintenance (Drupal User Guide)</a>{% endtrans %}</li>
</ul>
