!function(e){const i=e.bs=e.bs||{};i.dictionary=Object.assign(i.dictionary||{},{"Advanced options":"",Find:"Pronađi","Find and replace":"Pronađi i zamijeni","Find in text…":"Pronađi u tekstu","Find in the document":"","Match case":"Podudaranje","Next result":"","Previous result":"Prethodni rezultat",Replace:"Zamijeni","Replace all":"Zamijeni sve","Replace with…":"Zamijeni sa...","Text to find must not be empty.":"Unesite tekst za pretragu.","Tip: Find some text first in order to replace it.":"","Whole words only":"Samo cijele riječi"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));