!function(e){const t=e.nl=e.nl||{};t.dictionary=Object.assign(t.dictionary||{},{"Bulleted List":"Ongenummerde lijst","Bulleted list styles toolbar":"Toolbar voor stijlen van opsommingslijsten",Circle:"Cirkel",Decimal:"Decimaal","Decimal with leading zero":"Decimaal voorafgegaan door een nul","Decrease list item indent":"Item in de lijst minder ver inspringen",Disc:"Schijf","Entering a to-do list":"Een to-do-lijst invoeren","Increase list item indent":"Item in de lijst verder inspringen","Invalid start index value.":"Ongeldige beginindex.","Keystrokes that can be used in a list":"Toetsaanslagen die gebruikt kunnen worden in een lijst","Leaving a to-do list":"Een to-do-lijst achterlaten","List properties":"Lijsteigenschappen","Lower-latin":"Kleine Latijnse letters","Lower–roman":"Kleine Romeinse cijfers","Numbered List":"Genummerde lijst","Numbered list styles toolbar":"Toolbar voor stijlen van genummerde lijsten","Reversed order":"Omgekeerde volgorde",Square:"Vierkant","Start at":"Begin op","Start index must be greater than 0.":"Begin van index moet groter zijn dan 0.","To-do List":"To-do lijst","Toggle the circle list style":"Schakel de stijl van lijsten met cirkel in","Toggle the decimal list style":"Schakel de stijl van lijsten met cijfers in","Toggle the decimal with leading zero list style":"Schakel de stijl van lijsten met cijfers voorafgegaan door een nul in","Toggle the disc list style":"Schakel de stijl van lijsten met bulletpoint in","Toggle the lower–latin list style":"Schakel de stijl van lijsten met kleine Latijnse letters in","Toggle the lower–roman list style":"Schakel de stijl van lijsten met kleine Romeinse cijfers in","Toggle the square list style":"Schakel de stijl van lijsten met vierkant in","Toggle the upper–latin list style":"Schakel de stijl van lijsten met Latijnse hoofdletters in","Toggle the upper–roman list style":"Schakel de stijl van lijsten met grote Romeinse cijfers in","Upper-latin":"Latijnse hoofdletters","Upper-roman":"Grote Romeinse cijfers"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));