<?php

namespace Drupal\views\Plugin\views\argument;

use Drupal\views\Attribute\ViewsArgument;

/**
 * Default implementation of the base argument plugin.
 *
 * @ingroup views_argument_handlers
  */
#[ViewsArgument(
  id: 'standard',
)]
class Standard extends ArgumentPluginBase {

}
