{#
/**
 * @file
 * Default theme implementation to display a help topic.
 *
 * Available variables:
 * - body: The body of the topic.
 * - related: List of related topic links.
 *
 * @ingroup themeable
 */
#}
<article>
  {{ body }}
  {{ related }}
</article>
