!function(e){const a=e.es=e.es||{};a.dictionary=Object.assign(a.dictionary||{},{"Advanced options":"Opciones avanzadas",Find:"Buscar","Find and replace":"Buscar y reemplazar","Find in text…":"Buscar en el texto...","Find in the document":"Busca en el documento","Match case":"Respetar mayúsculas y minúsculas","Next result":"Siguiente resultado","Previous result":"Resultado anterior",Replace:"Reemplazar","Replace all":"Reemplazar todo","Replace with…":"Reemplazar con...","Text to find must not be empty.":"El texto a buscar no debe estar vacío.","Tip: Find some text first in order to replace it.":"Consejo: Primero busque algún texto para reemplazar.","Whole words only":"Solo palabras completas"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));