!function(n){const g=n.vi=n.vi||{};g.dictionary=Object.assign(g.dictionary||{},{"Choose language":"Chọn ngôn ngữ",Language:"Ngôn ngữ","Remove language":"Xóa ngôn ngữ"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));