!function(i){const e=i.fa=i.fa||{};e.dictionary=Object.assign(e.dictionary||{},{"Could not insert image at the current position.":"نمیتوان تصویر را در موقعیت فعلی وارد کرد","Could not obtain resized image URL.":"نمیتوان آدرس اینترنتی تصویر تغییر اندازه یافته را بدست آورد","Insert image or file":"وارد کردن تصویر یا فایل","Inserting image failed":"وارد کردن تصویر انجام نشد","Selecting resized image failed":"انتخاب تصویر تغییر اندازه یافته انجام نشد"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));