!function(e){const a=e.tr=e.tr||{};a.dictionary=Object.assign(a.dictionary||{},{"Connected users":"Bağlı kullanıcılar","Document outline":"Belge ana hatları","Enter fullscreen mode":"Tam ekran moduna gir","Fullscreen mode":"Tam ekran modu","Hide left sidebar":"Sol kenar çubuğunu gizle","Leave fullscreen mode":"Tam ekran modundan çık","Show left sidebar":"Sol kenar çubuğunu göster","Toggle fullscreen mode":"Tam ekran modunu aç/kapat","Toggle sidebar":"Kenar çubuğunu aç/kapat"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));