!function(e){const t=e["en-gb"]=e["en-gb"]||{};t.dictionary=Object.assign(t.dictionary||{},{"Insert media":"Insert media",Media:"","Media embed":"Media embed","Media toolbar":"","Media URL":"Media URL","media widget":"Media widget","Open media in new tab":"","Paste the media URL in the input.":"Paste the media URL in the input.","The URL must not be empty.":"The URL must not be empty.","This media URL is not supported.":"This media URL is not supported.","Tip: Paste the URL into the content to embed faster.":"Tip: Paste the URL into the content to embed faster."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));