!function(e){const n=e.nb=e.nb||{};n.dictionary=Object.assign(n.dictionary||{},{"Blue marker":"Blå uthevingsfarge","Green marker":"Grønn uthevingsfarge","Green pen":"Grønn penn",Highlight:"Utheving","Pink marker":"Rosa uthevingsfarge","Red pen":"Rød penn","Remove highlight":"Fjern uthevingsfarge","Text highlight toolbar":"","Yellow marker":"Gul uthevingsfarge"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));