!function(t){const o=t.bs=t.bs||{};o.dictionary=Object.assign(o.dictionary||{},{Bold:"Podebljano","Bold text":"",Code:"Kod",Italic:"Zakrivljeno","Italic text":"","Move out of an inline code style":"",Strikethrough:"Precrtano","Strikethrough text":"",Subscript:"",Superscript:"",Underline:"Podcrtano","Underline text":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));