!function(e){const t=e.fa=e.fa||{};t.dictionary=Object.assign(t.dictionary||{},{"Insert media":"وارد کردن رسانه",Media:"","Media embed":"","Media toolbar":"نوارابزار رسانه","Media URL":"آدرس اینترنتی رسانه","media widget":"ویجت رسانه","Open media in new tab":"","Paste the media URL in the input.":"آدرس رسانه را در ورودی قرار دهید","The URL must not be empty.":"آدرس اینترنتی URL نباید خالی باشد.","This media URL is not supported.":"این آدرس اینترنتی رسانه پشتیبانی نمی‌شود","Tip: Paste the URL into the content to embed faster.":"نکته : آدرس را در محتوا قراردهید تا سریع تر جاسازی شود"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));