!function(e){const a=e.sk=e.sk||{};a.dictionary=Object.assign(a.dictionary||{},{"Bulleted List":"Zoznam s odrážkami","Bulleted list styles toolbar":"Panel zoznamu s odrážkami",Circle:"Kruh",Decimal:"Čísla","Decimal with leading zero":"Čísla s nulou na začiatku","Decrease list item indent":"Zmenšiť odsadenie položiek v zozname",Disc:"Plný kruh","Entering a to-do list":"Príchod do zoznamu úloh","Increase list item indent":"Zväčšiť odsadenie položiek v zozname","Invalid start index value.":"Neplatná hodnota začiatočného indexu.","Keystrokes that can be used in a list":"Klávesy, ktoré sa dajú použiť v zozname","Leaving a to-do list":"Odchod zo zoznamu úloh","List properties":"Vlastnosti zoznamu","Lower-latin":"Malé písmená","Lower–roman":"Malé rímske čísla","Numbered List":"Číslovaný zoznam","Numbered list styles toolbar":"Panel so štýlmi číslovaného zoznamu","Reversed order":"Obrátene poradie",Square:"Štvorec","Start at":"Prvá položka","Start index must be greater than 0.":"Prvá položka musí mať index väčší ako 0.","To-do List":"To-do zoznam","Toggle the circle list style":"Prepnúť na zoznam s kruhovým označením","Toggle the decimal list style":"Prepnúť na číselný zoznam","Toggle the decimal with leading zero list style":"Prepnúť na číselný zoznam s nulou na začiatku","Toggle the disc list style":"Prepnúť na zoznam s označením plného kruhu","Toggle the lower–latin list style":"Prepnúť na zoznam s malými písmenami","Toggle the lower–roman list style":"Prepnúť na zoznam s malými rímskymi číslami","Toggle the square list style":"Prepnúť na zoznam so štvorcovým označením","Toggle the upper–latin list style":"Prepnúť na zoznam s veľkými písmenami","Toggle the upper–roman list style":"Prepnúť na zoznam s veľkými rímskymi číslami","Upper-latin":"Veľké písmená","Upper-roman":"Veľké rímske čísla"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));