!function(i){const n=i.fi=i.fi||{};n.dictionary=Object.assign(n.dictionary||{},{"Create link":"Luo linkki","Displayed text":"Näytettävä teksti",Downloadable:"Ladattava","Edit link":"Muokkaa linkkiä",Link:"Linkki","Link image":"Linkkikuva","Link properties":"Linkin ominaisuudet","Link URL":"Linkin osoite","Link URL must not be empty.":"Linkin URL ei voi olla tyhjä.","Move out of a link":"Siirrä linkin ulkopuolelle","No links available":"Linkkejä ei käytettävissä","Open in a new tab":"Avaa uudelle välilehdelle","Open link in new tab":"Avaa linkki uudessa välilehdessä",Unlink:"Poista linkki"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));