!function(a){const n=a.gl=a.gl||{};n.dictionary=Object.assign(n.dictionary||{},{"Create link":"Crear ligazón","Displayed text":"",Downloadable:"Descargábel","Edit link":"Editar a ligazón",Link:"Ligar","Link image":"Ligazón da imaxe","Link properties":"","Link URL":"URL de ligazón","Link URL must not be empty.":"O URL da ligazón non debe estar baleiro.","Move out of a link":"Saír dunha ligazón","No links available":"","Open in a new tab":"Abrir nunha nova lapela","Open link in new tab":"Abrir a ligazón nunha nova lapela",Unlink:"Desligar"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));