{#
/**
 * @file
 * Default theme implementation for a 'checkboxes' #type form element.
 *
 * Available variables
 * - attributes: A list of HTML attributes for the wrapper element.
 * - children: The rendered checkboxes.
 *
 * @see\Drupal\Core\Form\FormPreprocess::preprocessCheckboxes()
 *
 * @ingroup themeable
 */
 @todo: remove this file once https://www.drupal.org/node/1819284 is resolved.
 This is identical to core/modules/system/templates/container.html.twig
#}
<div{{ attributes.addClass('form-checkboxes') }}>{{ children }}</div>
