{#
/**
 * @file
 * Default theme implementation to display a navigation_block.
 *
 * Available variables:
 * - plugin_id: The ID of the navigation_block implementation.
 * - label: The configured label of the navigation_block if visible.
 * - configuration: A list of the navigation_block's configuration values.
 *   - label: The configured label for the navigation_block.
 *   - label_display: The display settings for the label.
 *   - provider: The module or other provider that provided this
       navigation_block plugin.
 *   - Navigation block plugin specific settings will also be stored here.
 * - content: The content of this navigation_block.
 * - attributes: array of HTML attributes populated by modules, intended to
 *   be added to the main container tag of this template.
 *   - id: A valid HTML ID and guaranteed unique.
 * - title_attributes: Same as attributes, except applied to the main title
 *   tag that appears in the template.
 * - title_prefix: Additional output populated by modules, intended to be
 *   displayed in front of the main title tag that appears in the template.
 * - title_suffix: Additional output populated by modules, intended to be
 *   displayed after the main title tag that appears in the template.
 *
 * @see template_preprocess_navigation_block()
 *
 * @ingroup themeable
 */
#}
<div{{ attributes.addClass('toolbar-block') }}>
  {{ title_prefix }}
  {% if configuration.label_display %}
    <h2{{ title_attributes
      .addClass('toolbar-block__title')
      .setAttribute('data-drupal-tooltip', configuration.label)
      .setAttribute('data-drupal-tooltip-class', 'toolbar-block__title-tooltip')
    }}>{{ configuration.label }}</h2>
  {% else %}
    <h2{{ title_attributes
      .addClass('visually-hidden', 'focusable')
    }}>{{ configuration.label }}</h2>
  {% endif %}
  {{ title_suffix }}
  {% block content %}
    {{ content }}
  {% endblock %}
</div>
