!function(t){const i=t.ja=t.ja||{};i.dictionary=Object.assign(i.dictionary||{},{Bold:"ボールド","Bold text":"太字",Code:"コード",Italic:"イタリック","Italic text":"斜体","Move out of an inline code style":"インラインコードスタイルを終了する",Strikethrough:"取り消し線","Strikethrough text":"取り消し線",Subscript:"下付き文字",Superscript:"上付き文字",Underline:"アンダーライン","Underline text":"下線"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));