!function(o){const a=o.el=o.el||{};a.dictionary=Object.assign(a.dictionary||{},{Bookmark:"Σελιδοδείκτης","Bookmark must not be empty.":"Ο σελιδοδείκτης δεν πρέπει να είναι κενός.","Bookmark name":"Όνομα σελιδοδείκτη","Bookmark name already exists.":"Το όνομα σελιδοδείκτη υπάρχει ήδη.","Bookmark name cannot contain space characters.":"Το όνομα του σελιδοδείκτη δεν μπορεί να περιέχει χαρακτήρες κενού διαστήματος.","Bookmark toolbar":"Γραμμή εργαλείων σελιδοδεικτών","bookmark widget":"widget σελιδοδείκτη",Bookmarks:"Σελιδοδείκτες","Edit bookmark":"Επεξεργασία σελιδοδείκτη","Enter the bookmark name without spaces.":"Εισαγάγετε το όνομα του σελιδοδείκτη χωρίς κενά.","No bookmarks available.":"Δεν υπάρχουν διαθέσιμοι σελιδοδείκτες.","Remove bookmark":"Κατάργηση σελειδοδείκτη","Scroll to bookmark":"Κάντε κύλιση στον σελιδοδείκτη"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));