{#
/**
 * @file
 * Default theme implementation for a dropbutton wrapper.
 *
 * Available variables:
 * - children: Contains the child elements of the dropbutton menu.
 *
 * @ingroup themeable
 */
#}
{% if children %}
  <div class="dropbutton-wrapper" data-drupal-ajax-container>
    <div class="dropbutton-widget">{{- children -}}</div>
  </div>
{%- endif %}
