{#
/**
 * @file
 * Default theme implementation for a single navigation top bar page action.
 *
 * Available variables:
 * - link: Link render element.
 */
#}
<li class="toolbar-dropdown__item">
  {{ link|add_class('toolbar-dropdown__link') }}
</li>
