{#
/**
 * @file
 * Default theme implementation of a datetime form element.
 *
 * Available variables:
 * - attributes: HTML attributes for the datetime form element.
 * - content: The datelist form element to be output.
 *
 * @see \Drupal\Core\Datetime\DatePreprocess::preprocessDatetimeForm()
 *
 * @ingroup themeable
 */
#}
<div{{ attributes }}>
  {{ content }}
</div>
