!function(i){const n=i.be=i.be||{};n.dictionary=Object.assign(n.dictionary||{},{"Revert autoformatting action":"Адмяніць дзеянне аўтафарматавання"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));