!function(e){const t=e.ko=e.ko||{};t.dictionary=Object.assign(t.dictionary||{},{"Bulleted List":"불릿 목록","Bulleted list styles toolbar":"글머리 기호 목록 스타일 도구 모음",Circle:"흰 원형",Decimal:"십진수","Decimal with leading zero":"앞에 0이 붙는 십진수","Decrease list item indent":"목록 항목 들여쓰기 줄이기",Disc:"검은 원형","Entering a to-do list":"할 일 목록 입력하는 중","Increase list item indent":"목록 항목 들여쓰기 늘리기","Invalid start index value.":"잘못된 시작 인덱스 값입니다.","Keystrokes that can be used in a list":"목록에서 사용할 수 있는 키 입력","Leaving a to-do list":"할 일 목록 남기는 중","List properties":"목록 속성","Lower-latin":"소문자 알파벳","Lower–roman":"소문자 로마자","Numbered List":"번호 목록","Numbered list styles toolbar":"번호 목록 스타일 도구 모음","Reversed order":"역순",Square:"검은 사각형","Start at":"시작 번호","Start index must be greater than 0.":"시작 번호는 0보다 커야 합니다.","To-do List":"확인 목록","Toggle the circle list style":"검은 원형 목록 스타일 전환","Toggle the decimal list style":"십진수 목록 스타일 전환","Toggle the decimal with leading zero list style":"앞에 0이 붙는 십진수 목록 스타일 전환","Toggle the disc list style":"흰 원형 목록 스타일 전환","Toggle the lower–latin list style":"소문자 알파벳 목록 스타일 전환","Toggle the lower–roman list style":"소문자 로마자 목록 스타일 전환","Toggle the square list style":"검은 사각형 목록 스타일 전환","Toggle the upper–latin list style":"대문자 알파벳 목록 스타일 전환","Toggle the upper–roman list style":"대문자 로마자 목록 스타일 전환","Upper-latin":"대문자 알파벳","Upper-roman":"대문자 로마자"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));