!function(e){const a=e.et=e.et||{};a.dictionary=Object.assign(a.dictionary||{},{"Edit source":"Muuda allikat","Empty snippet content":"Tühi jupi sisu","HTML snippet":"HTML jupp","Insert HTML":"Sisesta HTML","No preview available":"Eelvaade pole saadaval","Paste raw HTML here...":"Kleebi toores HTML siia...","Save changes":"Salvesta muudatused"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));