!function(n){const e=n.no=n.no||{};e.dictionary=Object.assign(e.dictionary||{},{"Code block":"Kodeblokk","Entering %0 code snippet":"Skriver inn %0 kodesnutt","Entering code snippet":"Skriver inn kodesnutt","Insert code block":"Sett inn kodeblokk","Leaving %0 code snippet":"Forlater %0 kodesnutt","Leaving code snippet":"Forlater kodesnutt","Plain text":"Ren tekst"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));