!function(e){const a=e.ug=e.ug||{};a.dictionary=Object.assign(a.dictionary||{},{"Break text":"تېكىست ئۈز","Caption for image: %0":"سۈرەت چۈشەندۈرۈشى: %0","Caption for the image":"سۈرەت چۈشەندۈرۈشى","Centered image":"ئوتتۇردىكى رەسىم","Change image text alternative":"رەسىملىك تېكىست تاللىغۇچنى ئۆزگەرتىش",Custom:"ئىختىيارى","Custom image size":"ئىختىيارى سۈرەت چوڭلۇقى","Enter image caption":"رەسىمنىڭ تېمىسىنى كىرگۈزۈڭ","Error during image upload":"سۈرەت يۈكلەشتە خاتالىق كۆرۈلدى","From computer":"كومپيۇتېردىن","Full size image":"ئەسلى چوڭلۇقتىكى رەسىم",Image:"سۈرەت","Image from computer":"كومپيۇتېردىن سۈرەت تاللاڭ","Image Resize":"","Image resize list":"سۈرەت چوڭلۇقىنى تەڭشەش تىزىمى","Image toolbar":"سۈرەت قورال بالداق","Image upload complete":"سۈرەت يۈكلەش تامام","Image via URL":"تور ئادرېسىدىن كەلگەن سۈرەت","image widget":"رەسىمچىك","In line":"قۇردا","Insert image":"رەسىم قىستۇرۇش","Insert image via URL":"سۈرەتنى تور ئادرېسىدىن قىستۇر","Insert via URL":"تور ئادرېسىدىن قىستۇر","Left aligned image":"سولغا توغۇرلانغان رەسىم",Original:"ئەسلى","Replace from computer":"كومپيۇتېردىن ئالماشتۇر","Replace image":"سۈرەت ئالماشتۇر","Replace image from computer":"سۈرەتنى كومپيۇتېردىن ئالماشتۇرىدۇ","Resize image":"سۈرەت چوڭلۇقىنى تەڭشە","Resize image (in %0)":"سۈرەت چوڭلۇقىنى تەڭشە (بىرلىكى %0)","Resize image to %0":"سۈرەت چوڭلۇقىنى %0 كە تەڭشە","Resize image to the original size":"سۈرەت چوڭلۇقىنى ئەسلى چوڭلۇقىغا تەڭشەيدۇ","Right aligned image":"ئوڭغا توغۇرلانغان رەسىم","Side image":"يان رەسىم","Text alternative":"تېكىست ئاملاشتۇرۇش","Text Alternative":"","The value must not be empty.":"قىممىتى بوش قالدۇرۇلمايدۇ.","The value should be a plain number.":"مەزكۇر قىممەت سان بولۇشى كېرەك.","Update image URL":"سۈرەت تور ئادرېسىنى يېڭىلا","Upload failed":"چىقىرىش مەغلۇپ بولدى","Upload from computer":"كومپيۇتېردىن يۈكلە","Upload image from computer":"سۈرەتنى كومپيۇتېردىن يۈكلە","Uploading image":"سۈرەت يۈكلەۋاتىدۇ","Via URL":"تور ئادرېسى ئارقىلىق","Wrap text":"تېكىست چۆرىدەت","You have no image upload permissions.":"سۈرەت يۈكلەش ئىجازىتىڭىز يوق."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));