!function(o){const a=o.ar=o.ar||{};a.dictionary=Object.assign(a.dictionary||{},{Bookmark:"إشارة مرجعية","Bookmark must not be empty.":"لا يجب أن تكون الإشارة المرجعية فارغة.","Bookmark name":"اسم الإشارة المرجعية","Bookmark name already exists.":"اسم الإشارة المرجعية موجود بالفعل.","Bookmark name cannot contain space characters.":"لا يمكن أن يحتوي اسم الإشارة المرجعية على مسافة.","Bookmark toolbar":"شريط أدوات الإشارة المرجعية","bookmark widget":"أداة الإشارة المرجعية",Bookmarks:"الإشارات المرجعية","Edit bookmark":"تحرير الإشارة المرجعية","Enter the bookmark name without spaces.":"أدخل اسم الإشارة المرجعية بدون مسافات.","No bookmarks available.":"لا توجد إشارات مرجعية متاحة.","Remove bookmark":"إزالة الإشارة المرجعية","Scroll to bookmark":"مرر إلى الإشارة المرجعية"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));