!function(o){const n=o.ru=o.ru||{};n.dictionary=Object.assign(n.dictionary||{},{Big:"Крупный",Default:"По умолчанию","Document colors":"Цвет страницы","Font Background Color":"Цвет фона","Font Color":"Цвет шрифта","Font Family":"Семейство шрифтов","Font Size":"Размер шрифта",Huge:"Очень крупный",Small:"Мелкий",Tiny:"Очень мелкий"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));