!function(e){const t=e.uk=e.uk||{};t.dictionary=Object.assign(t.dictionary||{},{"Align cell text to the bottom":"Вирівняти текст комірки по низу","Align cell text to the center":"Вирівняти текст по центру","Align cell text to the left":"Вирівняти текст по лівому краю","Align cell text to the middle":"Вирівняти текст комірки по середині","Align cell text to the right":"Вирівняти текст по правому краю","Align cell text to the top":"Вирівняти текст комірки по верху ","Align table to the left":"Вирівняти таблицю по лівому краю","Align table to the left with no text wrapping":"Вирівняти таблицю по лівому краю без обтікання текстом","Align table to the left with text wrapping":"Вирівняти таблицю по лівому краю з обтіканням текстом","Align table to the right":"Вирівняти таблицю по правому краю ","Align table to the right with no text wrapping":"Вирівняти таблицю по правому краю без обтікання текстом","Align table to the right with text wrapping":"Вирівняти таблицю по правому краю з обтіканням текстом",Alignment:"Вирівнювання",Background:"Фон",Border:"Межа","Cell properties":"Властивості комірок","Center table":"Відцентрувати таблицю","Center table with no text wrapping":"Вирівняти таблицю по центру без обтікання текстом","Choose table type":"Вибрати тип таблиці",Color:"Колір",Column:"Стовпець","Content table":"Вміст таблиці",Dashed:"Пунктирна","Delete column":"Видалити стовпець","Delete row":"Видалити рядок",Dimensions:"Розміри",Dotted:"Точкова",Double:"Подвійна","Enter table caption":"Введіть підпис таблиці",Groove:"Жолобчаста","Header column":"Заголовок стовпця","Header row":"Заголовок рядка",Height:"Висота","Horizontal text alignment toolbar":"Панель інструментів вирівнювання горизонтального тексту","Insert a new table row (when in the last cell of a table)":"Вставити новий рядок таблиці (якщо в останній клітинці таблиці)","Insert column left":"Вставити стовпець зліва","Insert column right":"Вставити стовпець справа","Insert row above":"Вставити рядок зверху","Insert row below":"Вставити рядок знизу","Insert table":"Вставити таблицю","Insert table layout":"Вставити формат таблиці",Inset:"Угнута","Justify cell text":"Вирівняти текст по ширині ","Keystrokes that can be used in a table cell":"Натискання клавіш, які можна використовувати в комірці таблиці","Layout table":"Формат таблиці","Merge cell down":"Поєднати комірки внизу","Merge cell left":"Поєднати комірки ліворуч","Merge cell right":"Поєднати комірки праворуч","Merge cell up":"Поєднати комірки вгору","Merge cells":"Поєднати комірки","Move the selection to the next cell":"Move the selection to the next cell","Move the selection to the previous cell":"Перемістити виділення в попередню клітинку","Navigate through the table":"Навігація по таблиці",None:"Не вказано",Outset:"Випукла",Padding:"Заповнення",Ridge:"Ребриста",Row:"Рядок","Select column":"Виберіть стовпчик","Select row":"Виберіть рядок",Solid:"Суцільний","Split cell horizontally":"Розділити комірки горизонтально","Split cell vertically":"Розділити комірки вертикально",Style:"Стиль",Table:"Таблиця","Table Alignment":"Вирівнювання таблиці","Table alignment toolbar":"Панель інструментів вирівнювання таблиці","Table cell text alignment":"Вирівнювання тексту комірки","Table layout":"Формат таблиці","Table properties":"Властивості таблиці","Table toolbar":"Панель інструментів таблиці","Table type":"Тип таблиці","Table type options":"Опції типу таблиці",'The color is invalid. Try "#FF0000" or "rgb(255,0,0)" or "red".':'Колір недійсний. Спробуйте "#FF0000" або "rgb(255,0,0)" або "red"','The value is invalid. Try "10px" or "2em" or simply "2".':'Значення недійсне. Спробуйте "10px" або "2em" або просто "2"',"Vertical text alignment toolbar":"Панель інструментів вертикального вирівнювання тексту",Width:"Ширина"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));