{% extends "block.html.twig" %}
{#
/**
 * @file
 * Default theme implementation for a branding block.
 *
 * Each branding element variable (logo, name, slogan) is only available if
 * enabled in the block configuration.
 *
 * Available variables:
 * - site_logo: Logo for site as defined in Appearance or theme settings.
 * - site_name: Name for site as defined in Site information settings.
 * - site_slogan: Slogan for site as defined in Site information settings.
 *
 * @ingroup themeable
 */
#}
{% block content %}
  {% if site_logo %}
    <a href="{{ path('<front>') }}" rel="home">
      <img src="{{ site_logo }}" alt="{{ 'Home'|t }}" fetchpriority="high" />
    </a>
  {% endif %}
  {% if site_name %}
    <a href="{{ path('<front>') }}" rel="home">{{ site_name }}</a>
  {% endif %}
  {{ site_slogan }}
{% endblock %}
