!function(e){const t=e.be=e.be||{};t.dictionary=Object.assign(t.dictionary||{},{"Advanced options":"Дадатковыя параметры",Find:"Знайсці","Find and replace":"Знайсці і замяніць","Find in text…":"Знайсці ў тэксце…","Find in the document":"Знайсці ў дакуменце","Match case":"З улікам рэгістру","Next result":"Наступны вынік","Previous result":"Папярэдні вынік",Replace:"Замяніць","Replace all":"Замяніць усё","Replace with…":"Замяніць на…","Text to find must not be empty.":"Тэкст для пошуку не павінен быць пустым.","Tip: Find some text first in order to replace it.":"Падказка: спачатку знайдзіце тэкст, каб замяніць яго.","Whole words only":"Толькі слова цалкам"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));