!function(n){const a=n.gl=n.gl||{};a.dictionary=Object.assign(a.dictionary||{},{"Decrease indent":"Reducir a sangría","Increase indent":"Aumentar a sangría"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));