!function(a){const e=a.ms=a.ms||{};e.dictionary=Object.assign(e.dictionary||{},{"Insert media":"Masukkan media",Media:"Media","Media embed":"Benam media","Media toolbar":"Bar alat capaian media","Media URL":"URL Media","media widget":"widget media","Open media in new tab":"Buka media dalam tab baru","Paste the media URL in the input.":"Tampal URL media dalam input.","The URL must not be empty.":"URL tidak boleh ditinggalkan kosong.","This media URL is not supported.":"URL media ini tidak disokong.","Tip: Paste the URL into the content to embed faster.":"Petua: Tampal URL kedalam kandungan untuk   membenam dengan lebih pantas."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));