!function(t){const i=t.ti=t.ti||{};i.dictionary=Object.assign(i.dictionary||{},{Bold:"ቦልድ","Bold text":"ጽሑፍ ኣጉልህ",Code:"ኮድ",Italic:"ኢታሊክ","Italic text":"ኢታሊክ ጽሑፍ","Move out of an inline code style":"",Strikethrough:"ሰርዝ","Strikethrough text":"ጽሑፍ ሰርዝ",Subscript:"ሳብስክሪፕት",Superscript:"ሱፐርስክሪፕት",Underline:"ኣስምር","Underline text":"ጽሑፍ ኣስምር"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));