!function(i){const e=i.he=i.he||{};e.dictionary=Object.assign(e.dictionary||{},{"Could not insert image at the current position.":"לא ניתן להוסיף תמונה במיקום הנוכחי","Could not obtain resized image URL.":"לא ניתן להשיג תמונה מוקטנת","Insert image or file":"הוסף תמונה או קובץ","Inserting image failed":"הוספת תמונה נכשלה","Selecting resized image failed":"בחירת תמונה מוקטנת נכשלה"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));