!function(o){const t=o.fi=o.fi||{};t.dictionary=Object.assign(t.dictionary||{},{"Code block":"Koodilohko","Entering %0 code snippet":"Syötetään koodinpätkä %0","Entering code snippet":"Syötetään koodinpätkä","Insert code block":"Liitä koodilohko","Leaving %0 code snippet":"Jätetään koodinpätkä %0","Leaving code snippet":"Jätetään koodinpätkä","Plain text":"Pelkkä teksti"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));