!function(t){const i=t.et=t.et||{};i.dictionary=Object.assign(i.dictionary||{},{Bold:"Rasvane","Bold text":"Paks tekst",Code:"Kood",Italic:"Kaldkiri","Italic text":"Kaldkirjas tekst","Move out of an inline code style":"Välju reasisese koodi stiilist",Strikethrough:"Läbijoonitud","Strikethrough text":"Läbikriipsutatud tekst",Subscript:"Alaindeks",Superscript:"Ülaindeks",Underline:"Allajoonitud","Underline text":"Allakriipsutatud tekst"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));