!function(e){const t=e.az=e.az||{};t.dictionary=Object.assign(t.dictionary||{},{"Align cell text to the bottom":"","Align cell text to the center":"","Align cell text to the left":"","Align cell text to the middle":"","Align cell text to the right":"","Align cell text to the top":"","Align table to the left":"","Align table to the left with no text wrapping":"","Align table to the left with text wrapping":"","Align table to the right":"","Align table to the right with no text wrapping":"","Align table to the right with text wrapping":"",Alignment:"",Background:"Fon",Border:"Sərhəd","Cell properties":"","Center table":"","Center table with no text wrapping":"","Choose table type":"",Color:"Rəng",Column:"Sütun","Content table":"",Dashed:"","Delete column":"Sütunları sil","Delete row":"Sətirləri sil",Dimensions:"Ölçülər",Dotted:"",Double:"","Enter table caption":"",Groove:"","Header column":"Başlıqlı sütun","Header row":"Başlıqlı sətir",Height:"Hündürlük","Horizontal text alignment toolbar":"","Insert a new table row (when in the last cell of a table)":"","Insert column left":"Sola sütun əlavə et","Insert column right":"Sağa sütun əlavə et","Insert row above":"Aşağıya sətir əlavə et","Insert row below":"Yuxarıya sətir əlavə et","Insert table":"Cədvəli əlavə et","Insert table layout":"",Inset:"","Justify cell text":"","Keystrokes that can be used in a table cell":"","Layout table":"","Merge cell down":"Xanaları aşağı birləşdir","Merge cell left":"Xanaları sola birləşdir","Merge cell right":"Xanaları sağa birləşdir","Merge cell up":"Xanaları yuxarı birləşdir","Merge cells":"Xanaları birləşdir","Move the selection to the next cell":"","Move the selection to the previous cell":"","Navigate through the table":"",None:"",Outset:"",Padding:"",Ridge:"",Row:"Sətir","Select column":"","Select row":"",Solid:"","Split cell horizontally":"Xanaları üfüqi böl","Split cell vertically":"Xanaları şaquli böl",Style:"",Table:"","Table Alignment":"","Table alignment toolbar":"","Table cell text alignment":"Cədvəl hüceyrəsi mətninin uyğunlaşdırılması","Table layout":"","Table properties":"Cədvəl xüsusiyyətləri","Table toolbar":"Cədvəl paneli","Table type":"","Table type options":"",'The color is invalid. Try "#FF0000" or "rgb(255,0,0)" or "red".':"",'The value is invalid. Try "10px" or "2em" or simply "2".':"","Vertical text alignment toolbar":"",Width:"Eni"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));