!function(t){const e=t.el=t.el||{};e.dictionary=Object.assign(e.dictionary||{},{Bold:"Έντονα","Bold text":"Έντονο κείμενο",Code:"Κώδικας",Italic:"Πλάγια","Italic text":"Πλάγιο κείμενο","Move out of an inline code style":"Μετακίνηση από ένα στυλ ενσωματωμένου κώδικα",Strikethrough:"Διακριτή διαγραφή","Strikethrough text":"Διακριτική διαγραφή κειμένου",Subscript:"Δείκτης",Superscript:"Εκθέτης",Underline:"Υπογράμμιση","Underline text":"Υπογράμμιση κειμένου"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));