!function(e){const i=e.zh=e.zh||{};i.dictionary=Object.assign(i.dictionary||{},{Activities:"活動","Animals & Nature":"動物與自然","Dark skin tone":"深調膚色","Default skin tone":"預設膚色",Emoji:"表情符號","Emoji picker":"表情符號選擇器","Find an emoji (min. 2 characters)":"尋找表情符號（最少 2 個字元）",Flags:"旗幟","Food & Drinks":"飲食","Gestures & People":"手勢與人物","Keep on typing to see the emoji.":"繼續輸入以查看表情符號。","Light skin tone":"淺調膚色","Medium Dark skin tone":"中深調膚色","Medium Light skin tone":"中淺調膚色","Medium skin tone":"中調膚色",'No emojis were found matching "%0".':"找不到與「%0」相符的表情符號。",Objects:"物件","Select skin tone":"選取膚色","Show all emoji...":"顯示所有表情符號…","Smileys & Expressions":"笑臉與表情",Symbols:"符號","The query must contain at least two characters.":"查詢條件至少須包含兩個字元。","Travel & Places":"旅遊與景點"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));