!function(t){const e=t.et=t.et||{};e.dictionary=Object.assign(e.dictionary||{},{"Advanced options":"Rohkem valikuid",Find:"Otsi","Find and replace":"Otsi ja asenda","Find in text…":"Otsi tekstist...","Find in the document":"Leia dokumendist","Match case":"Erista juhtumit","Next result":"Järgmine tulemus","Previous result":"Eelmine tulemus",Replace:"Asenda","Replace all":"Asenda kõik","Replace with…":"Asenda...","Text to find must not be empty.":"Otsitav tekst ei tohi tühi olla.","Tip: Find some text first in order to replace it.":"Otsi esmalt tekst, mida asendada.","Whole words only":"Ainult terviksõnad"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));