!function(n){const e=n.ar=n.ar||{};e.dictionary=Object.assign(e.dictionary||{},{"Code block":"كتلة تعليمات برمجية","Entering %0 code snippet":"إدخال مقتطف الكود %0","Entering code snippet":"إدخال مقتطف الكود","Insert code block":"إدراج كتلة تعليمات برمجية","Leaving %0 code snippet":"ترك مقتطف الكود %0","Leaving code snippet":"ترك مقتطف الكود","Plain text":"نص عادي"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));