!function(a){const i=a.sq=a.sq||{};i.dictionary=Object.assign(i.dictionary||{},{"Characters: %0":"Karaktere: %0","Words: %0":"Fjalë: %0"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));