!function(t){const e=t.da=t.da||{};e.dictionary=Object.assign(e.dictionary||{},{Bold:"Fed","Bold text":"Fed tekst",Code:"Kode",Italic:"Kursiv","Italic text":"Kursiv tekst","Move out of an inline code style":"Flyt ud af en inline-kodestil",Strikethrough:"Gennemstreg","Strikethrough text":"Gennemstreget tekst",Subscript:"Sænket skrift",Superscript:"Hævet skrift",Underline:"Understreget","Underline text":"Understreget tekst"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));