!function(e){const n=e["de-ch"]=e["de-ch"]||{};n.dictionary=Object.assign(n.dictionary||{},{"Could not insert image at the current position.":"Das Bild konnte an der aktuellen Position nicht eingefügt werden.","Could not obtain resized image URL.":"Die URL des angepassten Bildes konnte nicht abgerufen werden.","Insert image or file":"Bild oder Datei einfügen","Inserting image failed":"Einfügen des Bildes fehlgeschlagen","Selecting resized image failed":"Das angepasste Bild konnte nicht ausgewählt werden."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));