!function(e){const a=e.sr=e.sr||{};a.dictionary=Object.assign(a.dictionary||{},{"Break text":"Прелом текста","Caption for image: %0":"Натпис за слику: %0","Caption for the image":"Натпис за слику","Centered image":"Слика у средини","Change image text alternative":"Измена алтернативног текста",Custom:"Prilagođeno","Custom image size":"Prilagođena veličina slike","Enter image caption":"Одреди текст испод слике","Error during image upload":"Greška tokom otpremanja slike","From computer":"Sa računara","Full size image":"Слика у пуној величини",Image:"Slika","Image from computer":"Slika sa računara","Image Resize":"Promeni veličinu slike","Image resize list":"Листа величине слике","Image toolbar":"Слика трака са алтакама","Image upload complete":"Otpremanje slike je završeno","Image via URL":"Slika putem URL adrese","image widget":"модул са сликом","In line":"У реду","Insert image":"Додај слику","Insert image via URL":"Убаци слику преко УРЛ-а","Insert via URL":"Umetni putem URL adrese","Left aligned image":"Лева слика",Original:"Оригинал","Replace from computer":"Zameni sa računara","Replace image":"Zameni sliku","Replace image from computer":"Zameni sliku sa računara","Resize image":"Промените величину слике","Resize image (in %0)":"Promenite veličinu slike (u %0)","Resize image to %0":"Промените величину слике на% 0","Resize image to the original size":"Промените величину слике до оригиналне величине","Right aligned image":"Десна слика","Side image":"Бочна слика","Text alternative":"Алтернативни текст","Text Alternative":"Alternativni tekst","The value must not be empty.":"Vrednost ne sme biti prazna.","The value should be a plain number.":"Vrednost treba da bude običan broj.","Update image URL":"Ажурирај УРЛ слике","Upload failed":"Постављање неуспешно","Upload from computer":"Otpremi sa računara","Upload image from computer":"Otpremi sliku sa računara","Uploading image":"Otpremanje slike","Via URL":"Putem URL adrese","Wrap text":"Преломити текст","You have no image upload permissions.":"Nemate dozvole za otpremanje slike."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));