!function(e){const t=e.th=e.th||{};t.dictionary=Object.assign(t.dictionary||{},{"Bulleted List":"รายการสัญลักษณ์แสดงหัวข้อย่อย","Bulleted list styles toolbar":"แถบเครื่องมือรูปแบบรายการสัญลักษณ์แสดงหัวข้อย่อย",Circle:"วงกลม",Decimal:"ทศนิยม","Decimal with leading zero":"ทศนิยมที่มีศูนย์นำหน้า","Decrease list item indent":"ลดการเยื้องวัตถุในรายการ",Disc:"ดิสก์","Entering a to-do list":"เข้าสู่รายการสิ่งที่ต้องทำ","Increase list item indent":"เพิ่มการเยื้องวัตถุในรายการ","Invalid start index value.":"ค่าดัชนีเริ่มต้นไม่ถูกต้อง","Keystrokes that can be used in a list":"แป้นพิมพ์ลัดที่สามารถใช้ได้ในรายการ","Leaving a to-do list":"ออกจากรายการสิ่งที่ต้องทำ","List properties":"คุณสมบัติของรายการ","Lower-latin":"อักษรลาตินตัวพิมพ์เล็ก","Lower–roman":"อักษรโรมันตัวพิมพ์เล็ก","Numbered List":"รายการลำดับเลข","Numbered list styles toolbar":"แถบเครื่องมือรูปแบบรายการแบบตัวเลข","Reversed order":"ลำดับที่ย้อนกลับ",Square:"สี่เหลี่ยม","Start at":"เริ่มต้นที่","Start index must be greater than 0.":"ดัชนีเริ่มต้นต้องมากกว่า 0","To-do List":"รายการสิ่งที่จะทำ","Toggle the circle list style":"สลับรูปแบบรายการวงกลม","Toggle the decimal list style":"สลับรูปแบบรายการทศนิยม","Toggle the decimal with leading zero list style":"สลับทศนิยมด้วยรูปแบบรายการศูนย์นำหน้า","Toggle the disc list style":"สลับรูปแบบรายการดิสก์","Toggle the lower–latin list style":"สลับรูปแบบรายการอักษรลาตินตัวพิมพ์เล็ก","Toggle the lower–roman list style":"สลับรูปแบบรายการอักษรโรมันตัวพิมพ์เล็ก","Toggle the square list style":"สลับรูปแบบรายการสี่เหลี่ยม","Toggle the upper–latin list style":"สลับรูปแบบรายการอักษรลาตินตัวพิมพ์ใหญ่","Toggle the upper–roman list style":"สลับรูปแบบรายการอักษรโรมันตัวพิมพ์ใหญ่","Upper-latin":"อักษรลาตินตัวพิมพ์ใหญ่","Upper-roman":"อักษรโรมันตัวพิมพ์ใหญ่"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));