!function(e){const n=e.no=e.no||{};n.dictionary=Object.assign(n.dictionary||{},{"Create link":"Opprett lenke","Displayed text":"Vist tekst",Downloadable:"Nedlastbar","Edit link":"Rediger lenke",Link:"Lenke","Link image":"Bildelenke","Link properties":"Lenkeegenskaper","Link URL":"Lenke-URL","Link URL must not be empty.":"Link-URL kan ikke være tom.","Move out of a link":"Flytt ut fra en lenke","No links available":"Ingen lenker tilgjengelig","Open in a new tab":"Åpne i ny fane","Open link in new tab":"Åpne lenke i ny fane",Unlink:"Fjern lenke"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));