!function(t){const e=t.gl=t.gl||{};e.dictionary=Object.assign(e.dictionary||{},{"Advanced options":"Opcións avanzadas",Find:"Buscar","Find and replace":"Buscar e substituír","Find in text…":"Buscar no texto…","Find in the document":"Atopar no documento","Match case":"Coincidencia de maiúsculas","Next result":"Seguinte resultado","Previous result":"Resultado anterior",Replace:"Substituír","Replace all":"Substituír todo","Replace with…":"Substituír por…","Text to find must not be empty.":"O texto a buscar non debe estar baleiro.","Tip: Find some text first in order to replace it.":"Consello: primeiro busca algo de texto para substituílo.","Whole words only":"Só palabras enteiras"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));