!function(n){const i=n.hi=n.hi||{};i.dictionary=Object.assign(i.dictionary||{},{"Create link":"लिंक बनाएँ","Displayed text":"प्रदर्शित टेक्स्ट",Downloadable:"Downloadable","Edit link":"Edit link",Link:"Link","Link image":"Link image","Link properties":"गुण लिंक करें","Link URL":"Link URL","Link URL must not be empty.":"लिंक का URL रिक्त नहीं होना चाहिए.","Move out of a link":"लिंक के बाहर जाएँ","No links available":"कोई लिंक उपलब्ध नहीं है","Open in a new tab":"Open in a new tab","Open link in new tab":"Open link in new tab",Unlink:"Unlink"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));