!function(e){const i=e.bn=e.bn||{};i.dictionary=Object.assign(i.dictionary||{},{Activities:"কার্যক্রম","Animals & Nature":"প্রাণী ও প্রকৃতি","Dark skin tone":"গাঢ় স্কিন টোন","Default skin tone":"ডিফল্ট স্কিন টোন",Emoji:"ইমোজি","Emoji picker":"ইমোজি পিকার","Find an emoji (min. 2 characters)":"একটি ইমোজির সন্ধান করুন (কমপক্ষে 2 টি ক্যারেক্টার)",Flags:"পতাকা","Food & Drinks":"খাদ্য ও পানীয়","Gestures & People":"অঙ্গভঙ্গি ও মানুষ","Keep on typing to see the emoji.":"ইমোজিটি দেখতে টাইপ করতে থাকুন।","Light skin tone":"হালকা স্কিন টোন","Medium Dark skin tone":"মাঝারি গাঢ় স্কিন টোন","Medium Light skin tone":"মাঝারি হালকা স্কিন টোন","Medium skin tone":"মাঝারি স্কিন টোন",'No emojis were found matching "%0".':'"%0" এর সাথে মেলে এমন কোনো ইমোজি পাওয়া যায়নি।',Objects:"বস্তু","Select skin tone":"স্কিন টোন নির্বাচন করুন","Show all emoji...":"সমস্ত ইমোজি দেখান...","Smileys & Expressions":"স্মাইলি ও অভিব্যক্তি",Symbols:"প্রতীক","The query must contain at least two characters.":"কোয়েরিটিতে কমপক্ষে দুটি ক্যারেক্টার থাকতে হবে।","Travel & Places":"ভ্রমণ ও স্থান"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));