!function(i){const n=i.it=i.it||{};n.dictionary=Object.assign(n.dictionary||{},{"Create link":"Crea un link","Displayed text":"Testo visualizzato",Downloadable:"Scaricabile","Edit link":"Modifica collegamento",Link:"Collegamento","Link image":"Collega immagine","Link properties":"Proprietà del link","Link URL":"URL del collegamento","Link URL must not be empty.":"L'URL del link non può essere lasciato in bianco.","Move out of a link":"Esce da un link","No links available":"Nessun link disponibile","Open in a new tab":"Apri in una nuova scheda","Open link in new tab":"Apri collegamento in nuova scheda",Unlink:"Elimina collegamento"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));