!function(i){const e=i.hr=i.hr||{};e.dictionary=Object.assign(e.dictionary||{},{"Disable block editing":"","Disable editing":"","Disable inline editing":"","Enable block editing":"","Enable editing":"Omogući uređivanje","Enable inline editing":"","Navigate editable regions":"Navigiraj regijama koje je moguće uređivati","Next editable region":"Sljedeća regija koju je moguće uređivati","Previous editable region":"Prethodna regija koju je moguće uređivati"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));