!function(n){const e=n.no=n.no||{};e.dictionary=Object.assign(e.dictionary||{},{"Blue marker":"Blå utheving","Green marker":"Grønn utheving","Green pen":"Grønn penn",Highlight:"Utheving","Pink marker":"Rosa utheving","Red pen":"Rød penn","Remove highlight":"Fjern utheving","Text highlight toolbar":"Verktøylinje for tekstutheving","Yellow marker":"Gul utheving"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));