!function(n){const e=n.he=n.he||{};e.dictionary=Object.assign(e.dictionary||{},{"Code block":"בלוק קוד","Entering %0 code snippet":"נכנס לקטע קוד %0","Entering code snippet":"נכנס לקטע קוד","Insert code block":"הוספת קטע קוד","Leaving %0 code snippet":"יוצא מקטע קוד %0","Leaving code snippet":"יוצא מקטע קוד","Plain text":"טקסט פשוט"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));