/*
 * @file
 * Provides the layout styles for four-column layout section.
 */

.layout--fourcol-section {
  display: flex;
  flex-wrap: wrap;
}

.layout--fourcol-section > .layout__region {
  flex: 0 1 100%;
}

@media screen and (min-width: 40em) {
  .layout--fourcol-section > .layout__region {
    flex: 0 1 25%;
  }
}
