<?php

namespace Drupal\views\Plugin\views\relationship;

use Drupal\views\Attribute\ViewsRelationship;

/**
 * Default implementation of the base relationship plugin.
 *
 * @ingroup views_relationship_handlers
 */
#[ViewsRelationship("standard")]
class Standard extends RelationshipPluginBase {

}
