!function(e){const n=e["zh-cn"]=e["zh-cn"]||{};n.dictionary=Object.assign(n.dictionary||{},{"Connected users":"已连接用户","Document outline":"文档大纲","Enter fullscreen mode":"进入全屏模式","Fullscreen mode":"全屏模式","Hide left sidebar":"隐藏左侧边栏","Leave fullscreen mode":"退出全屏模式","Show left sidebar":"显示左侧边栏","Toggle fullscreen mode":"切换全屏模式","Toggle sidebar":"切换侧边栏"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));