!function(e){const i=e["pt-br"]=e["pt-br"]||{};i.dictionary=Object.assign(i.dictionary||{},{"Edit source":"Editar código","Empty snippet content":"Trecho sem conteúdo","HTML snippet":"Trecho HTML","Insert HTML":"Inserir HTML","No preview available":"Nenhuma visualização disponível","Paste raw HTML here...":"Cole o HTML puro aqui","Save changes":"Salvar alterações"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));