!function(n){const o=n.no=n.no||{};o.dictionary=Object.assign(o.dictionary||{},{"Choose language":"Velg språk",Language:"Språk","Remove language":"Fjern språk"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));