{#
/**
 * @file
 * Default theme implementation to present a list of block types.
 *
 * Available variables:
 * - types: A collection of all the available content block types.
 *   Each block type contains the following:
 *   - link: A link to add a block of this type.
 *   - description: A description of this block type.
 *
 * @see template_preprocess_block_content_add_list()
 *
 * @ingroup themeable
 *
 * @deprecated in drupal:11.3.0 and is removed from drupal:12.0.0. Use entity_add_list instead.
 *
 * @see https://www.drupal.org/node/3530643
 */
#}
<dl>
  {% for type in types %}
    <dt>{{ type.link }}</dt>
    <dd>{{ type.description }}</dd>
  {% endfor %}
</dl>
