!function(e){const i=e.th=e.th||{};i.dictionary=Object.assign(i.dictionary||{},{"Cannot access default workspace.":"ไม่สามารถเข้าถึงพื้นที่ทำงานเริ่มต้น","Cannot determine a category for the uploaded file.":"ไม่สามารถกำหนดหมวดหมู่สำหรับไฟล์ที่อัปโหลด","Edit image":"แก้ไขภาพ","Failed to determine category of edited image.":"ระบุหมวดหมู่ของภาพที่แก้ไขไม่สำเร็จ","Open file manager":"เปิดเครื่องมือจัดการไฟล์","Processing the edited image.":"กำลังประมวลผลภาพที่แก้ไข","Server failed to process the image.":"เซิร์ฟเวอร์ประมวลผลภาพไม่สำเร็จ","You have no image editing permissions.":"คุณไม่มีสิทธิ์ในการแก้ไขภาพ"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));