!function(e){const t=e.sr=e.sr||{};t.dictionary=Object.assign(t.dictionary||{},{"Bulleted List":"Набрајане листе","Bulleted list styles toolbar":"Трака са алаткама за набрајане листе",Circle:"Круг",Decimal:"Децимала","Decimal with leading zero":"Децимала са нулом на почетку","Decrease list item indent":"Smanji uvlačenje liste stavki",Disc:"Диск","Entering a to-do list":"Unošenje liste obaveza","Increase list item indent":"Povećaj uvlačenje liste stavki","Invalid start index value.":"Nevažeća vrednost početnog indeksa.","Keystrokes that can be used in a list":"Tasteri koji se mogu koristiti na listi","Leaving a to-do list":"Ostavljanje liste obaveza","List properties":"Наведите својства ","Lower-latin":"Доњи - латински","Lower–roman":"Доњи - римски","Numbered List":"Листа са бројевима","Numbered list styles toolbar":"Трака са алаткама за листе са бројевима","Reversed order":"Обрнути редослед ",Square:"Квадрат","Start at":"Почети у","Start index must be greater than 0.":"Почетни индекс мора бити већи од 0. ","To-do List":"Листа обавеза","Toggle the circle list style":"Укључи / искључи стил листе кругова","Toggle the decimal list style":"Укључи / искључи стил декадне листе","Toggle the decimal with leading zero list style":"Пребаци децимални знак са водећим стилом листе нула","Toggle the disc list style":"Укључите / искључите стил листе дискова","Toggle the lower–latin list style":"Укључите / искључите стил доње листе латинице","Toggle the lower–roman list style":"Укључите/ искључите стил доње римске листе","Toggle the square list style":"Укључи / искључи стил листе квадрата","Toggle the upper–latin list style":"Укључите / искључите стил горње листе латинице","Toggle the upper–roman list style":"Укључите / искључите стил горње римске листе","Upper-latin":"Горњи - латински","Upper-roman":"Горњи - римски"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));