!function(a){const i=a.fi=a.fi||{};i.dictionary=Object.assign(i.dictionary||{},{"Advanced options":"Lisäasetukset",Find:"Etsi","Find and replace":"Etsi ja korvaa","Find in text…":"Etsi tekstissä…","Find in the document":"Paikanna asiakirjassa","Match case":"Hae tapaukselle vastapari","Next result":"Seuraava tulos","Previous result":"Edellinen tulos",Replace:"Korvaa","Replace all":"Korvaa kaikki","Replace with…":"Korvaa tällä…","Text to find must not be empty.":"Haettava teksti ei saa olla tyhjä.","Tip: Find some text first in order to replace it.":"Vinkki: hae ensin teksti korvataksesi sen.","Whole words only":"Vain kokonaisia sanoja"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));