!function(i){const e=i.ca=i.ca||{};e.dictionary=Object.assign(e.dictionary||{},{"Could not insert image at the current position.":"No s'ha pogut inserir la imatge a la posició actual.","Could not obtain resized image URL.":"No s'ha pogut obtenir l'URL de la imatge redimensionada.","Insert image or file":"Introduir una imatge o un fitxer","Inserting image failed":"No s'ha pogut inserir la imatge","Selecting resized image failed":"S'ha produït un error en seleccionar la imatge redimensionada"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));