<?php

namespace Drupal\Core\Asset\Exception;

/**
 * Defines a custom exception if a library has a remote but no license.
 */
class LibraryDefinitionMissingLicenseException extends \RuntimeException {

}
