<?php

namespace Drupal\Core\Routing\Access;

/**
 * An access check service determines access rules for particular routes.
 */
interface AccessInterface {

  // @todo Remove this interface since it no longer defines any methods?
  // @see https://www.drupal.org/node/2266817.

}
