!function(e){const i=e.fr=e.fr||{};i.dictionary=Object.assign(i.dictionary||{},{Activities:"Activités","Animals & Nature":"Animaux et nature","Dark skin tone":"Teint foncé","Default skin tone":"Teinte de peau par défaut",Emoji:"Émoji","Emoji picker":"Sélecteur d'émojis","Find an emoji (min. 2 characters)":"Trouver un émoji (min. deux caractères)",Flags:"Drapeaux","Food & Drinks":"Nourriture et boissons","Gestures & People":"Gestes et personnes","Keep on typing to see the emoji.":"Continuez à taper pour voir l'émoji.","Light skin tone":"Teint clair","Medium Dark skin tone":"Teint moyennement foncé","Medium Light skin tone":"Teint moyennement clair","Medium skin tone":"Teint moyen",'No emojis were found matching "%0".':"Aucun émoji correspondant à « %0 » n'a été trouvé.",Objects:"Objets","Select skin tone":"Sélectionner la teinte de peau","Show all emoji...":"Afficher tous les émojis...","Smileys & Expressions":"Smileys et expressions",Symbols:"Symboles","The query must contain at least two characters.":"La requête doit contenir au moins deux caractères.","Travel & Places":"Voyages et lieux"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));