<?php

namespace Drupal\Core\Config\Entity\Exception;

use Drupal\Core\Config\ConfigException;

/**
 * Thrown when a storage class is not an instance of ConfigEntityStorage.
 */
class ConfigEntityStorageClassException extends ConfigException {}
