!function(a){const n=a.az=a.az||{};n.dictionary=Object.assign(n.dictionary||{},{"Choose heading":"Başlıqı seç",Heading:"Başlıq","Heading 1":"Başlıq 1","Heading 2":"Başlıq 2","Heading 3":"Başlıq 3","Heading 4":"Başlıq 4","Heading 5":"Başlıq 5","Heading 6":"Başlıq 6","Type or paste your content here.":"","Type your title":"Başlığınızı yazın"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));