!function(e){const t=e.sk=e.sk||{};t.dictionary=Object.assign(t.dictionary||{},{"Align cell text to the bottom":"Zarovnať text bunky nadol","Align cell text to the center":"Zarovnať text bunky na stred","Align cell text to the left":"Zarovnať text bunky doľava","Align cell text to the middle":"Zarovnať text bunky na stred","Align cell text to the right":"Zarovnať text bunky doprava","Align cell text to the top":"Zarovnať text bunky nahor","Align table to the left":"Zarovnať tabuľku doľava","Align table to the left with no text wrapping":"Zarovnať tabuľku doľava bez obtekania textu","Align table to the left with text wrapping":"Zarovnať tabuľku doľava s obtekaním textu","Align table to the right":"Zarovnať tabuľku doprava","Align table to the right with no text wrapping":"Zarovnať tabuľku doprava bez obtekania textu","Align table to the right with text wrapping":"Zarovnať tabuľku doprava s obtekaním textu",Alignment:"Zarovnanie",Background:"Pozadie",Border:"Orámovanie","Cell properties":"Vlastnosti bunky","Center table":"Centrovať tabuľku","Center table with no text wrapping":"Zarovnať tabuľku na stred bez obtekania textu","Choose table type":"Vyberte typ tabuľky",Color:"Farba",Column:"Stĺpec","Content table":"Tabuľka obsahu",Dashed:"Čiarkovaná","Delete column":"Odstrániť stĺpec","Delete row":"Odstrániť riadok",Dimensions:"Rozmery",Dotted:"Bodkovaná",Double:"Dvojitá","Enter table caption":"Zadajte popis tabuľky",Groove:"Drážkovaná","Header column":"Stĺpec hlavičky","Header row":"Riadok hlavičky",Height:"Výška","Horizontal text alignment toolbar":"Horizontálne zarovnanie textu v panely","Insert a new table row (when in the last cell of a table)":"Vložiť nový riadok tabuľky (keď je označená posledná bunka tabuľky)","Insert column left":"Vložiť stĺpec vľavo","Insert column right":"Vložiť stĺpec vpravo","Insert row above":"Vložiť riadok nad","Insert row below":"Vložiť riadok pod","Insert table":"Vložiť tabuľku","Insert table layout":"Vložiť rozloženie tabuľky",Inset:"Vložená z vnútra","Justify cell text":"Zarovnať text bunky z oboch strán","Keystrokes that can be used in a table cell":"Klávesy, ktoré sa dajú použiť v bunke tabuľky","Layout table":"Tabuľka rozloženia","Merge cell down":"Zlúčiť bunku dole","Merge cell left":"Zlúčiť bunku vľavo","Merge cell right":"Zlúčiť bunku vpravo","Merge cell up":"Zlúčiť bunku hore","Merge cells":"Zlúčiť bunky","Move the selection to the next cell":"Presunúť výber do nasledujúcej bunky","Move the selection to the previous cell":"Presunúť výber do predchádzajúcej bunky","Navigate through the table":"Prechádzať tabuľkou",None:"Žiadna",Outset:"Vložená zvonku",Padding:"Vnútorný okraj",Ridge:"Rámovaná",Row:"Riadok","Select column":"Vybrať stĺpec","Select row":"Vybrať riadok",Solid:"Plná","Split cell horizontally":"Rozdeliť bunku vodorovne","Split cell vertically":"Rozdeliť bunku zvislo",Style:"Štýl",Table:"Tabuľka","Table Alignment":"Zarovnanie tabuľky","Table alignment toolbar":"Panel zarovnania tabuľky","Table cell text alignment":"Zarovnanie textu v bunke","Table layout":"Rozloženie tabuľky","Table properties":"Vlastnosti tabuľky","Table toolbar":"Panel nástrojov tabuľky","Table type":"Typ tabuľky","Table type options":"Možnosti typu tabuľky",'The color is invalid. Try "#FF0000" or "rgb(255,0,0)" or "red".':'Farba má nesprávny formát. Skúste "#FF0000", "rgb(255,0,0)" alebo "red".','The value is invalid. Try "10px" or "2em" or simply "2".':'Hodnota je nesprávna. Skúste "10px", "2em" alebo jednoducho "2".',"Vertical text alignment toolbar":"Vertikálne zarovnanie textu v panely",Width:"Šírka"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));