!function(e){const t=e["en-gb"]=e["en-gb"]||{};t.dictionary=Object.assign(t.dictionary||{},{"Align cell text to the bottom":"","Align cell text to the center":"","Align cell text to the left":"","Align cell text to the middle":"","Align cell text to the right":"","Align cell text to the top":"","Align table to the left":"","Align table to the left with no text wrapping":"Align table to the left with no text wrapping","Align table to the left with text wrapping":"Align table to the left with text wrapping","Align table to the right":"","Align table to the right with no text wrapping":"Align table to the right with no text wrapping","Align table to the right with text wrapping":"Align table to the right with text wrapping",Alignment:"",Background:"",Border:"","Cell properties":"","Center table":"","Center table with no text wrapping":"Center table with no text wrapping","Choose table type":"Choose table type",Color:"",Column:"Column","Content table":"Content table",Dashed:"","Delete column":"Delete column","Delete row":"Delete row",Dimensions:"",Dotted:"",Double:"","Enter table caption":"",Groove:"","Header column":"Header column","Header row":"Header row",Height:"","Horizontal text alignment toolbar":"","Insert a new table row (when in the last cell of a table)":"","Insert column left":"Insert column left","Insert column right":"Insert column right","Insert row above":"Insert row above","Insert row below":"Insert row below","Insert table":"Insert table","Insert table layout":"Insert table layout",Inset:"","Justify cell text":"","Keystrokes that can be used in a table cell":"","Layout table":"Layout table","Merge cell down":"Merge cell down","Merge cell left":"Merge cell left","Merge cell right":"Merge cell right","Merge cell up":"Merge cell up","Merge cells":"Merge cells","Move the selection to the next cell":"","Move the selection to the previous cell":"","Navigate through the table":"",None:"",Outset:"",Padding:"",Ridge:"",Row:"Row","Select column":"","Select row":"",Solid:"","Split cell horizontally":"Split cell horizontally","Split cell vertically":"Split cell vertically",Style:"",Table:"","Table Alignment":"Table Alignment","Table alignment toolbar":"","Table cell text alignment":"","Table layout":"Table layout","Table properties":"","Table toolbar":"","Table type":"Table type","Table type options":"Table type options",'The color is invalid. Try "#FF0000" or "rgb(255,0,0)" or "red".':"",'The value is invalid. Try "10px" or "2em" or simply "2".':"","Vertical text alignment toolbar":"",Width:""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));