!function(i){const a=i.fi=i.fi||{};a.dictionary=Object.assign(a.dictionary||{},{"Choose language":"Valitse kieli",Language:"Kieli","Remove language":"Poista kieli"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));