!function(e){const i=e.tr=e.tr||{};i.dictionary=Object.assign(i.dictionary||{},{"Could not insert image at the current position.":"Resim mevcut konumda eklenemedi.","Could not obtain resized image URL.":"Yeniden boyutlandırılmış resim URL’si alınamadı","Insert image or file":"Resim veya dosya ekleyin","Inserting image failed":"Resim eklenemedi","Selecting resized image failed":"Yeniden boyutlandırılan resim seçilemedi"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));