!function(e){const i=e["de-ch"]=e["de-ch"]||{};i.dictionary=Object.assign(i.dictionary||{},{"Break text":"","Caption for image: %0":"","Caption for the image":"","Centered image":"zentriertes Bild","Change image text alternative":"Alternativtext ändern",Custom:"","Custom image size":"","Enter image caption":"Bildunterschrift eingeben","Error during image upload":"","From computer":"","Full size image":"Bild in voller Grösse",Image:"","Image from computer":"","Image Resize":"","Image resize list":"Bildgrössen-Liste","Image toolbar":"Bild Werkzeugleiste","Image upload complete":"","Image via URL":"","image widget":"Bild-Steuerelement","In line":"","Insert image":"Bild einfügen","Insert image via URL":"Bild von URL einfügen","Insert via URL":"","Left aligned image":"linksbündiges Bild",Original:"Original","Replace from computer":"","Replace image":"","Replace image from computer":"","Resize image":"Bildgrösse ändern","Resize image (in %0)":"","Resize image to %0":"Bildgrösse ändern in %0","Resize image to the original size":"Originalgrösse wiederherstellen","Right aligned image":"rechtsbündiges Bild","Side image":"Ausgerichtetes Bild","Text alternative":"Alternativtext","Text Alternative":"","The value must not be empty.":"","The value should be a plain number.":"","Update image URL":"Bild-URL aktualisieren","Upload failed":"Hochladen fehlgeschlagen","Upload from computer":"","Upload image from computer":"","Uploading image":"","Via URL":"","Wrap text":"","You have no image upload permissions.":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));