!function(e){const t=e.ku=e.ku||{};t.dictionary=Object.assign(t.dictionary||{},{"Insert media":"مێدیا دابنێ",Media:"","Media embed":"","Media toolbar":"تووڵامرازی مێدیا","Media URL":"بەستەری مێدیا","media widget":"ویدجێتتی مێدیا","Open media in new tab":"","Paste the media URL in the input.":"بەستەری مێدیاکە لە خانەکە بلکێنە.","The URL must not be empty.":"پێویستە بەستەر بەتاڵ نەبێت.","This media URL is not supported.":"ئەم بەستەری مێدیایە پاڵپشتی ناکرێت.","Tip: Paste the URL into the content to embed faster.":"Tip: Paste the URL into the content to embed faster."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));