!function(o){const n=o["sr-latn"]=o["sr-latn"]||{};n.dictionary=Object.assign(n.dictionary||{},{Big:"Veliko",Default:"Оsnovni","Document colors":"Boje dokumenta","Font Background Color":"Boja pozadine slova","Font Color":"Boja slova","Font Family":"Font","Font Size":"Veličina fonta",Huge:"Ogromno",Small:"Malo",Tiny:"Sitno"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));