!function(e){const t=e.de=e.de||{};t.dictionary=Object.assign(t.dictionary||{},{Bold:"Fett","Bold text":"Fettschrift",Code:"Code",Italic:"Kursiv","Italic text":"Kursivschrift","Move out of an inline code style":"Inline Code Style verlassen",Strikethrough:"Durchgestrichen","Strikethrough text":"Durchgestrichener Text",Subscript:"Tiefgestellt",Superscript:"Hochgestellt",Underline:"Unterstrichen","Underline text":"Text hervorheben"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));