!function(i){const e=i.fi=i.fi||{};e.dictionary=Object.assign(e.dictionary||{},{"Blue marker":"Sininen tussi","Green marker":"Vihreä tussi","Green pen":"Vihreä kynä",Highlight:"Korosta","Pink marker":"Pinkki tussi","Red pen":"Punainen tussi","Remove highlight":"Poista korostus","Text highlight toolbar":"Tekstinkorostuksen työkalupalkki","Yellow marker":"Keltainen tussi"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));