!function(i){const e=i.ca=i.ca||{};e.dictionary=Object.assign(e.dictionary||{},{"Disable block editing":"Desactivar l'edició per blocs","Disable editing":"Desactivar l'edició","Disable inline editing":"Desactivar l'edició en línia","Enable block editing":"Activar l'edició per blocs","Enable editing":"Activar l'edició","Enable inline editing":"Activar l'edició en línia","Navigate editable regions":"Navegar per parts editables","Next editable region":"Següent part editable","Previous editable region":"Part editable anterior"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));