/*
 * @file
 * Provides the layout styles for layout_twocol_bricks.
 */

.layout--twocol-bricks {
  display: flex;
  flex-wrap: wrap;
}

.layout--twocol-bricks > .layout__region {
  flex: 0 1 100%;
}

@media screen and (min-width: 40em) {
  .layout--twocol-bricks > .layout__region--first-above,
  .layout--twocol-bricks > .layout__region--second-above,
  .layout--twocol-bricks > .layout__region--first-below,
  .layout--twocol-bricks > .layout__region--second-below {
    flex: 0 1 50%;
  }
}
