---
label: 'Exporting a single configuration item'
related:
- core.config_overview
- config.export_full
- config.import_full
- config.import_single
---
{% set single_export_link_text %}{% trans %}Single item{% endtrans %}{% endset %}
{% set single_export_link = render_var(help_route_link(single_export_link_text, 'config.export_single')) %}
{% set config_overview_topic = render_var(help_topic_link('core.config_overview')) %}
<h2>{% trans %}Goal{% endtrans %}</h2>
<p>{% trans %}Export a single configuration item to a file. See {{ config_overview_topic }} for more information about configuration.{% endtrans %}</p>
<h2>{% trans %}Steps{% endtrans %}</h2>
<ol>
  <li>{% trans %}In the <em>Manage</em> administrative menu, navigate to <em>Configuration</em> &gt; <em>Development</em> &gt; <em>Configuration synchronization</em> &gt; <em>Export</em> &gt; <em>{{ single_export_link }}</em>.{% endtrans %}</li>
  <li>{% trans %}Select the <em>Configuration type</em> that you want to export, and then select the specific <em>Configuration name</em> to export.{% endtrans %}</li>
  <li>{% trans %}Use your browser to copy the text in the box marked <em>Here is your configuration</em> to the clipboard.{% endtrans %}</li>
  <li>{% trans %}Paste the copied text into a plain-text editor on your computer or other device, and save it using the suggested file name below the text box.{% endtrans %}
  </li>
</ol>
