!function(e){const a=e.es=e.es||{};a.dictionary=Object.assign(a.dictionary||{},{"Choose heading":"Elegir Encabezado",Heading:"Encabezado","Heading 1":"Encabezado 1","Heading 2":"Encabezado 2","Heading 3":"Encabezado 3","Heading 4":"Encabezado 4","Heading 5":"Encabezado 5","Heading 6":"Encabezado 6","Type or paste your content here.":"Introduce o pega tu contenido aquí","Type your title":"Introduce tu título"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));