!function(i){const e=i.it=i.it||{};e.dictionary=Object.assign(e.dictionary||{},{"Edit source":"Modifica sorgente","Empty snippet content":"Contenuto frammento di codice vuoto","HTML snippet":"Frammento HTML","Insert HTML":"Inserisci HTML","No preview available":"Nessuna anteprima disponibile","Paste raw HTML here...":"Incolla qui HTML non elaborato...","Save changes":"Salva modifiche"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));