!function(n){const o=n.vi=n.vi||{};o.dictionary=Object.assign(o.dictionary||{},{Big:"Lớn",Default:"Mặc định","Document colors":"Màu văn bản","Font Background Color":"Màu nền chữ","Font Color":"Màu chữ","Font Family":"Họ chữ","Font Size":"Cỡ chữ",Huge:"Khổng lồ",Small:"Nhỏ",Tiny:"Bé"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));