!function(s){const a=s.ms=s.ms||{};a.dictionary=Object.assign(a.dictionary||{},{"Block styles":"Gaya blok","Multiple styles":"Gaya berbilang",Styles:"Gaya","Text styles":"Gaya teks"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));