<?php

declare(strict_types=1);

namespace Drupal\layout_builder_override;

use Drupal\layout_builder\Entity\LayoutBuilderEntityViewDisplay as BaseLayoutBuilderEntityViewDisplay;

/**
 * Provides an entity view display entity that has a layout.
 */
class LayoutBuilderEntityViewDisplay extends BaseLayoutBuilderEntityViewDisplay {
}
