!function(t){const e=t.pt=t.pt||{};e.dictionary=Object.assign(e.dictionary||{},{"Advanced options":"Opções avançadas",Find:"Encontrar","Find and replace":"Encontrar e substituir","Find in text…":"Encontrar no texto...","Find in the document":"Localizar no documento","Match case":"Corresponder maiúsculas/minúsculas","Next result":"Resultado seguinte","Previous result":"Resultado anterior",Replace:"Substituir","Replace all":"Substituir todos","Replace with…":"Substituir por...","Text to find must not be empty.":"O texto a pesquisar não pode estar vazio.","Tip: Find some text first in order to replace it.":"Dica: Encontre algum texto primeiro para poder substituí-lo.","Whole words only":"Apenas palavras inteiras"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));