!function(n){const i=n["en-au"]=n["en-au"]||{};i.dictionary=Object.assign(i.dictionary||{},{"Create link":"","Displayed text":"Displayed text",Downloadable:"Downloadable","Edit link":"Edit link",Link:"Link","Link image":"Link image","Link properties":"Link properties","Link URL":"Link URL","Link URL must not be empty.":"","Move out of a link":"","No links available":"No links available","Open in a new tab":"Open in a new tab","Open link in new tab":"Open link in new tab",Unlink:"Unlink"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));