!function(s){const t=s.hu=s.hu||{};t.dictionary=Object.assign(t.dictionary||{},{"Block styles":"Blokkstílusok","Multiple styles":"Többféle stílus",Styles:"Stílusok","Text styles":"Szövegstílusok"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));