!function(n){const a=n.ms=n.ms||{};a.dictionary=Object.assign(a.dictionary||{},{"Edit source":"Sunting sumber","Empty snippet content":"Kosongkan kandungan potongan","HTML snippet":"Potongan HTML","Insert HTML":"Masukkan HTML","No preview available":"Tiada pratonton disediakan","Paste raw HTML here...":"Tampal HTML mentah disini...","Save changes":"Simpan perubahan"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));