!function(e){const t=e.hu=e.hu||{};t.dictionary=Object.assign(t.dictionary||{},{"Break text":"Sortörés","Caption for image: %0":"Képfelirat: %0","Caption for the image":"Felirat a képhez","Centered image":"Középre igazított kép","Change image text alternative":"Helyettesítő szöveg módosítása",Custom:"Egyéni","Custom image size":"Egyéni képméret","Enter image caption":"Képaláírás megadása","Error during image upload":"Hiba a kép feltöltése során","From computer":"Számítógépről","Full size image":"Teljes méretű kép",Image:"Kép","Image from computer":"Kép a számítógépről","Image Resize":"Kép átméretezése","Image resize list":"Kép átméretezési lista","Image toolbar":"Kép eszköztár","Image upload complete":"A kép feltöltése befejeződött","Image via URL":"Kép URL-címmel","image widget":"képmodul","In line":"Soron belül","Insert image":"Kép beszúrása","Insert image via URL":"Kép beszúrása URL alapján","Insert via URL":"Beillesztés URL-vel","Left aligned image":"Balra igazított kép",Original:"Eredeti","Replace from computer":"Kicserélés a számítógépről","Replace image":"Kép kicserélése","Replace image from computer":"Cserélje ki a számítógépről","Resize image":"Kép átméretezése","Resize image (in %0)":"Kép átméretezése (%0)","Resize image to %0":"Kép méretezése %0","Resize image to the original size":"Kép méretezése az eredeti méretre","Right aligned image":"Jobbra igazított kép","Side image":"Oldalsó kép","Text alternative":"Helyettesítő szöveg","Text Alternative":"Szöveges alternatíva","The value must not be empty.":"Az érték nem lehet üres.","The value should be a plain number.":"Az érték egy egyszerű szám kell legyen.","Update image URL":"Kép URL módosítása","Upload failed":"A feltöltés nem sikerült","Upload from computer":"Feltöltés a számítógépről","Upload image from computer":"Töltsön fel képet a számítógépről","Uploading image":"Kép feltöltése","Via URL":"URL-vel","Wrap text":"Körbefuttatás","You have no image upload permissions.":"Nincs képfeltöltési engedélyed."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));