!function(e){const i=e.tr=e.tr||{};i.dictionary=Object.assign(i.dictionary||{},{"Advanced options":"Gelişmiş seçenekler",Find:"Bul","Find and replace":"Bul ve değiştir","Find in text…":"Metinde bul...","Find in the document":"Belgede bul","Match case":"Büyük küçük harfe duyarlı","Next result":"Sonraki sonuç","Previous result":"Önceki sonuç",Replace:"Değiştir","Replace all":"Hepsini değiştir","Replace with…":"Şununla değiştir...","Text to find must not be empty.":"Bulunacak metin boş bırakılmamalıdır.","Tip: Find some text first in order to replace it.":"İpucu: Değiştirmek için önce bir metin bul.","Whole words only":"Sadece bütün kelimeler"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));