!function(e){const i=e.ar=e.ar||{};i.dictionary=Object.assign(i.dictionary||{},{"Blue marker":"تحديد ازرق","Green marker":"تحديد اخضر","Green pen":"قلم اخضر",Highlight:"تحديد","Pink marker":"تحديد وردي","Red pen":"تحديد احمر","Remove highlight":"إزالة التحديد","Text highlight toolbar":"شريط أدوات تمييز النص","Yellow marker":"تحديد اصفر"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));