!function(e){const t=e["de-ch"]=e["de-ch"]||{};t.dictionary=Object.assign(t.dictionary||{},{"Align cell text to the bottom":"","Align cell text to the center":"","Align cell text to the left":"","Align cell text to the middle":"","Align cell text to the right":"","Align cell text to the top":"","Align table to the left":"","Align table to the left with no text wrapping":"","Align table to the left with text wrapping":"","Align table to the right":"","Align table to the right with no text wrapping":"","Align table to the right with text wrapping":"",Alignment:"",Background:"",Border:"","Cell properties":"","Center table":"","Center table with no text wrapping":"","Choose table type":"",Color:"",Column:"Spalte","Content table":"",Dashed:"","Delete column":"Spalte löschen","Delete row":"Zeile löschen",Dimensions:"",Dotted:"",Double:"","Enter table caption":"",Groove:"","Header column":"Kopfspalte","Header row":"Kopfspalte",Height:"","Horizontal text alignment toolbar":"","Insert a new table row (when in the last cell of a table)":"","Insert column left":"","Insert column right":"","Insert row above":"Zeile oben einfügen","Insert row below":"Zeile unten einfügen","Insert table":"Tabelle einfügen","Insert table layout":"",Inset:"","Justify cell text":"","Keystrokes that can be used in a table cell":"","Layout table":"","Merge cell down":"Zelle unten verbinden","Merge cell left":"Zelle links verbinden","Merge cell right":"Zele rechts verbinden","Merge cell up":"Zelle oben verbinden","Merge cells":"Zellen verbinden","Move the selection to the next cell":"","Move the selection to the previous cell":"","Navigate through the table":"",None:"",Outset:"",Padding:"",Ridge:"",Row:"Zeile","Select column":"","Select row":"",Solid:"","Split cell horizontally":"Zelle horizontal teilen","Split cell vertically":"Zelle vertikal teilen",Style:"",Table:"","Table Alignment":"","Table alignment toolbar":"","Table cell text alignment":"","Table layout":"","Table properties":"","Table toolbar":"","Table type":"","Table type options":"",'The color is invalid. Try "#FF0000" or "rgb(255,0,0)" or "red".':"",'The value is invalid. Try "10px" or "2em" or simply "2".':"","Vertical text alignment toolbar":"",Width:""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));