<?php

declare(strict_types=1);

namespace Drupal\entity_test_bundle_class\Entity;

use Drupal\entity_test\Entity\EntityTest;

/**
 * A bundle class that shares the same entity type as entity_test.
 */
class SharedEntityTestBundleClassA extends EntityTest {
}
