!function(i){const e=i.th=i.th||{};e.dictionary=Object.assign(e.dictionary||{},{"Could not insert image at the current position.":"ไม่สามารถแทรกภาพที่ตำแหน่งปัจจุบัน","Could not obtain resized image URL.":"ไม่สามารถรับ URL ของภาพที่ปรับขนาด","Insert image or file":"แทรกรูปภาพหรือไฟล์","Inserting image failed":"การแทรกภาพล้มเหลว","Selecting resized image failed":"การเลือกภาพที่ปรับขนาดล้มเหลว"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));