!function(t){const e=t.cs=t.cs||{};e.dictionary=Object.assign(e.dictionary||{},{"Align cell text to the bottom":"Zarovnat text buňky dolů","Align cell text to the center":"Zarovnat text buňky na střed","Align cell text to the left":"Zarovnat text buňky doleva","Align cell text to the middle":"Zarovnat text buňky na střed","Align cell text to the right":"Zarovnat text buňky doprava","Align cell text to the top":"Zarovnat text buňky nahoru","Align table to the left":"Zarovnat tabulku doleva","Align table to the left with no text wrapping":"Zarovnat tabulku vlevo bez zalomení textu","Align table to the left with text wrapping":"Zarovnat tabulku vlevo s obtékáním textu","Align table to the right":"Zarovnat tabulku doprava","Align table to the right with no text wrapping":"Zarovnat tabulku vpravo bez zalomení textu","Align table to the right with text wrapping":"Zarovnat tabulku vpravo s obtékáním textu",Alignment:"Zarovnání",Background:"Pozadí",Border:"Okraj","Cell properties":"Vlastnosti buňky","Center table":"Centrovat tabulku","Center table with no text wrapping":"Vycentrovat tabulku bez zalomení textu","Choose table type":"Zvolte typ tabulky",Color:"Barva",Column:"Sloupec","Content table":"Tabulka obsahu",Dashed:"Čárkovaná","Delete column":"Smazat sloupec","Delete row":"Smazat řádek",Dimensions:"Rozměry",Dotted:"Tečkovaná",Double:"Dvojitá","Enter table caption":"Zadejte titulek tabulky",Groove:"Drážkovaná","Header column":"Sloupec záhlaví","Header row":"Řádek záhlaví",Height:"Výška","Horizontal text alignment toolbar":"Horizontální zarovnání textu v panelu","Insert a new table row (when in the last cell of a table)":"Vložit nový řádek tabulky (když jste v poslední buňce tabulky)","Insert column left":"Vložit sloupec vlevo","Insert column right":"Vložit sloupec vpravo","Insert row above":"Vložit řádek před","Insert row below":"Vložit řádek pod","Insert table":"Vložit tabulku","Insert table layout":"Vložit rozvržení tabulky",Inset:"Vložená zevnitř","Justify cell text":"Zarovnat text buňky z obou stran","Keystrokes that can be used in a table cell":"Klávesy, které lze použít v buňce tabulky","Layout table":"Tabulka rozvržení","Merge cell down":"Sloučit s buňkou pod","Merge cell left":"Sloučit s buňkou vlevo","Merge cell right":"Sloučit s buňkou vpravo","Merge cell up":"Sloučit s buňkou nad","Merge cells":"Sloučit buňky","Move the selection to the next cell":"Přesunout výběr do další buňky","Move the selection to the previous cell":"Přesunout výběr do předchozí buňky","Navigate through the table":"Procházet tabulkou",None:"Žádná",Outset:"Vložená zvenku",Padding:"Vnitřní okraj",Ridge:"Rámovaná",Row:"Řádek","Select column":"Vybrat sloupec","Select row":"Vybrat řádek",Solid:"Plná","Split cell horizontally":"Rozdělit buňky horizontálně","Split cell vertically":"Rozdělit buňky vertikálně",Style:"Styl",Table:"Tabulka","Table Alignment":"Zarovnání tabulky","Table alignment toolbar":"Panel zarovnání tabulky","Table cell text alignment":"Zarovnání textu buňky tabulky","Table layout":"Rozvržení tabulky","Table properties":"Vlastnosti tabulky","Table toolbar":"Panel nástrojů tabulky","Table type":"Typ tabulky","Table type options":"Možnosti typu tabulky",'The color is invalid. Try "#FF0000" or "rgb(255,0,0)" or "red".':'Barva má nesprávný formát. Zkuste "#FF0000", "rgb(255,0,0)" nebo "red".','The value is invalid. Try "10px" or "2em" or simply "2".':'Hodnota je nesprávná. Zkuste "10px", "2em" nebo jednoduše "2".',"Vertical text alignment toolbar":"Vertikální zarovnání textu v panelu",Width:"Šířka"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));