<?php

declare(strict_types=1);

namespace Drupal\plugin_test_extended\Plugin\plugin_test\fruit;

/**
 * @PluginExtended(
 *   id = "big_apple",
 *   label = "Big Apple",
 *   color = "green"
 * )
 */
class BigApple {

}
