---
label: 'Translating user interface text'
related:
  - core.translations
  - locale.import
  - locale.translation_status
  - language.add
---
{% set translate_text %}{% trans %}User interface translation{% endtrans %}{% endset %}
{% set translate_link = render_var(help_route_link(translate_text, 'locale.translate_page')) %}
<h2>{% trans %}Goal{% endtrans %}</h2>
<p>{% trans %}Translate user interface text strings from English into a non-English language that is configured on your site.{% endtrans %}</p>
<h2>{% trans %}Steps{% endtrans %}</h2>
<ol>
  <li>{% trans %}In the <em>Manage</em> administrative menu, navigate to <em>Configuration</em> &gt; <em>Region and language</em> &gt; {{ translate_link }}.{% endtrans %}</li>
  <li>{% trans %}Using the filters, search for a string or set of strings that you want to translate; make sure to select the correct <em>Translation language</em> if you have more than one non-English language on your site.{% endtrans %}</li>
  <li>{% trans %}Enter new translations and click <em>Save translations</em>.{% endtrans %}</li>
  <li>{% trans %}Repeat these steps until all of the desired user interface text is translated for all languages on your site.{% endtrans %}</li>
</ol>
