!function(i){const e=i.ja=i.ja||{};e.dictionary=Object.assign(e.dictionary||{},{"Could not insert image at the current position.":"現在のカーソルの場所への画像の挿入に失敗しました。","Could not obtain resized image URL.":"リサイズした画像のURLの取得に失敗しました。","Insert image or file":"画像やファイルの挿入","Inserting image failed":"画像の挿入に失敗しました。","Selecting resized image failed":"リサイズした画像の選択ができませんでした。"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));