<?php

declare(strict_types=1);

namespace Drupal\install_form_test\Form;

use Drupal\Core\Installer\Form\SiteSettingsForm;

/**
 * Extends the site setting form.
 */
class ExtendedForm extends SiteSettingsForm {
}
