!function(t){const i=t.et=t.et||{};i.dictionary=Object.assign(i.dictionary||{},{"Revert autoformatting action":"Kustuta automaatse vorminduse toiming"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));