{#
/**
 * @file
 * Default theme implementation for the status report counter.
 *
 * Available variables:
 * - amount: The number shown on counter.
 * - text: The text shown on counter.
 * - severity: The severity of the counter.
 *
 * @ingroup themeable
 */
#}
<span class="system-status-counter__status-icon system-status-counter__status-icon--{{ severity }}"></span>
<span>{{ amount }} {{ text }}</span>
<a href="#{{ severity }}"><span class="visually-hidden">{{ text }} </span>{{ 'Details'|t }}</a>
