!function(e){const i=e.no=e.no||{};i.dictionary=Object.assign(i.dictionary||{},{"Disable block editing":"Deaktiver blokkredigering","Disable editing":"Deaktiver redigering","Disable inline editing":"Deaktiver innebygd redigering","Enable block editing":"Aktiver blokkredigering","Enable editing":"Aktivere redigering","Enable inline editing":"Aktiver innebygd redigering","Navigate editable regions":"Naviger redigerbare regioner","Next editable region":"Neste redigerbare region","Previous editable region":"Forrige redigerbare region"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));