!function(s){const t=s.hu=s.hu||{};t.dictionary=Object.assign(t.dictionary||{},{"Bulleted List":"Pontozott lista","Bulleted list styles toolbar":"Felsorolásos lista stílusú eszköztár",Circle:"Kör",Decimal:"Szám","Decimal with leading zero":"Nullával kezdődő szám","Decrease list item indent":"A listaelem behúzás csökkentése",Disc:"Korong","Entering a to-do list":"Belépés a tennivalók listájába","Increase list item indent":"A listaelem behúzás növelése","Invalid start index value.":"Hibás induló indexérték.","Keystrokes that can be used in a list":"A listában használható billentyűk","Leaving a to-do list":"Tennivalók listájának elhagyás","List properties":"Lista tulajdonságai","Lower-latin":"Kisbetűs latin szám","Lower–roman":"Kisbetűs római szám","Numbered List":"Számozott lista","Numbered list styles toolbar":"Számozott lista stílusú eszköztár","Reversed order":"Fordított sorrend",Square:"Négyzet","Start at":"Kezdőpont","Start index must be greater than 0.":"A kezdő index nagyobb kell legyen mint 0.","To-do List":"Tennivaló lista","Toggle the circle list style":"Kör stílusú lista bekapcsolása","Toggle the decimal list style":"Tizedesjegy stílusú lista bekapcsolása","Toggle the decimal with leading zero list style":"Zéróval kezdődő tizedesjegy stílusú lista bekapcsolása","Toggle the disc list style":"Korong stílusú lista bekapcsolása","Toggle the lower–latin list style":"Kis latin betűs stílusú lista bekapcsolása","Toggle the lower–roman list style":"Kis római számok stílusú lista bekapcsolása","Toggle the square list style":"Négyzet stílusú lista bekapcsolása","Toggle the upper–latin list style":"Nagy latin betűs stílusú lista bekapcsolása","Toggle the upper–roman list style":"Nagy római számok stílusú lista bekapcsolása","Upper-latin":"Nagybetűs latin szám","Upper-roman":"Nagybetűs római szám"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));