!function(i){const e=i.bn=i.bn||{};e.dictionary=Object.assign(e.dictionary||{},{"Disable block editing":"ব্লক ইডিটিং নিষ্ক্রিয় করুন","Disable editing":"সম্পাদনা নিষ্ক্রিয় করুন","Disable inline editing":"ইনলাইন ইডিটিং নিষ্ক্রিয় করুন","Enable block editing":"ব্লক ইডিটিং সষ্ক্রিয় করুন","Enable editing":"সম্পাদনা সক্রিয় করুন","Enable inline editing":"ইনলাইন ইডিটিং সষ্ক্রিয় করুন","Navigate editable regions":"সম্পাদনাযোগ্য অংশে নেভিগেট করুন","Next editable region":"পরবর্তী সম্পাদনাযোগ্য অংশ","Previous editable region":"পূর্ববর্তী সম্পাদনাযোগ্য অংশ"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));