!function(t){const i=t.zh=t.zh||{};i.dictionary=Object.assign(i.dictionary||{},{Bold:"粗體","Bold text":"粗體文字",Code:"代碼",Italic:"斜體","Italic text":"斜體文字","Move out of an inline code style":"移出行內程式碼樣式",Strikethrough:"刪除線","Strikethrough text":"刪除線文字",Subscript:"下標",Superscript:"上標",Underline:"底線","Underline text":"底線文字"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));