<?php

namespace Drupal\Core\Config\Entity\Exception;

use Drupal\Core\Config\ConfigException;

/**
 * Defines an exception thrown when a configuration entity ID is too long.
 */
class ConfigEntityIdLengthException extends ConfigException {}
