/* eslint-disable import/prefer-default-export */
/**
 * Used for indicating metadata errors in model.
 *
 * @type {string}
 *
 * @see \Drupal\ckeditor5\Controller\CKEditor5MediaController
 */
export const METADATA_ERROR = 'METADATA_ERROR';
