!function(i){const e=i.sq=i.sq||{};e.dictionary=Object.assign(e.dictionary||{},{"Blue marker":"Shënuesi kaltër","Green marker":"Shënuesi gjelbër","Green pen":"Lapsi gjelbër",Highlight:"Ngjyrimi","Pink marker":"Shënuesi rozë","Red pen":"Lapsi kuq","Remove highlight":"Largo ngjyrimet","Text highlight toolbar":"Shiriti i veglave të nënvizimit të tekstit","Yellow marker":"Shënuesi verdh"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));