!function(i){const e=i.fr=i.fr||{};e.dictionary=Object.assign(e.dictionary||{},{"Could not insert image at the current position.":"Impossible d'insérer l'image à la position courante.","Could not obtain resized image URL.":"Impossible d'obtenir l'image redimensionnée","Insert image or file":"Insérer une image ou un fichier","Inserting image failed":"L'insertion d'image a échoué.","Selecting resized image failed":"La sélection de l'image redimensionnée a échoué."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));