<?php

namespace Drupal\path_alias;

// cspell:ignore whitelist

/**
 * Cache a list of valid alias prefixes.
 *
 * @deprecated in drupal:11.1.0 and is removed from drupal:12.0.0. Use
 * \Drupal\path_alias\AliasPrefixList instead.
 *
 * @see https://www.drupal.org/node/3467559
 */
class AliasWhitelist extends AliasPrefixList {}
