!function(t){const e=t.fi=t.fi||{};e.dictionary=Object.assign(e.dictionary||{},{"Align cell text to the bottom":"Siirrä solun teksti alaosaan","Align cell text to the center":"Siirrä solun teksti keskelle","Align cell text to the left":"Siirrä solun teksti vasemmalle","Align cell text to the middle":"Siirrä solun teksti keskiosaan","Align cell text to the right":"Siirrä solun teksti oikealle","Align cell text to the top":"Siirrä solun teksti yläosaan","Align table to the left":"Siirrä taulukko vasemmalle","Align table to the left with no text wrapping":"Tasaa taulukko vasemmalle ilman tekstin rivitystä","Align table to the left with text wrapping":"Tasaa taulukko vasemmalle tekstin rivityksellä","Align table to the right":"Siirrä taulukko oikealle","Align table to the right with no text wrapping":"Tasaa taulukko oikealle ilman tekstin rivitystä","Align table to the right with text wrapping":"Tasaa taulukko oikealle tekstin rivityksellä",Alignment:"Suunta",Background:"Tausta",Border:"Reunus","Cell properties":"Solun ominaisuudet","Center table":"Keskitä taulukko","Center table with no text wrapping":"Keskitä taulukko ilman tekstin rivitystä","Choose table type":"Valitse taulukon tyyppi",Color:"Väri",Column:"Sarake","Content table":"Sisältötaulukko",Dashed:"Katkoviiva","Delete column":"Poista sarake","Delete row":"Poista rivi",Dimensions:"Mittasuhteet",Dotted:"Pisteviiva",Double:"Kaksinkertainen","Enter table caption":"Syötä taulukon kuvaus",Groove:"Uurrettu","Header column":"Otsikkosarake","Header row":"Otsikkorivi",Height:"Korkeus","Horizontal text alignment toolbar":"Vaakasuoran tekstin suunnan työkalupalkki","Insert a new table row (when in the last cell of a table)":"Liitä uusi taulukkorivi (pätee taulukon viimeiseen soluun)","Insert column left":"Lisää sarake vasemmalle","Insert column right":"Lisää sarake oikealle","Insert row above":"Lisää rivi ylle","Insert row below":"Lisää rivi alle","Insert table":"Lisää taulukko","Insert table layout":"Lisää taulukon asettelu",Inset:"Upote","Justify cell text":"Tasaa solun teksti","Keystrokes that can be used in a table cell":"Painallukset, joita voidaan käyttää taulukon solussa","Layout table":"Asettelutaulukko","Merge cell down":"Yhdistä solu alas","Merge cell left":"Yhdistä solu vasemmalle","Merge cell right":"Yhdistä solu oikealle","Merge cell up":"Yhdistä solu ylös","Merge cells":"Yhdistä tai jaa soluja","Move the selection to the next cell":"Siirrä valittu kohde seuraavaan soluun","Move the selection to the previous cell":"Siirrä valittu kohde edelliseen soluun","Navigate through the table":"Siirry taulukossa",None:"Ei mitään",Outset:"Ulkonema",Padding:"Täyte",Ridge:"Harjanne",Row:"Rivi","Select column":"Valitse sarake","Select row":"Valitse rivi",Solid:"Kiinteä","Split cell horizontally":"Jaa solu vaakasuunnassa","Split cell vertically":"Jaa solu pystysuunnassa",Style:"Tyyli",Table:"Taulukko","Table Alignment":"Taulukon tasaus","Table alignment toolbar":"Taulukon suunnan työkalupalkki","Table cell text alignment":"Taulukkosolun tekstin suunta","Table layout":"Taulukon asettelu","Table properties":"Taulukon ominaisuudet","Table toolbar":"Taulukon työkalupalkki","Table type":"Taulukon tyyppi","Table type options":"Taulukkotyypin vaihtoehdot",'The color is invalid. Try "#FF0000" or "rgb(255,0,0)" or "red".':'Väri ei kelpaa. Ota sen sijaan käyttöön "#FF0000", "rgb(255,0,0)" tai "red".','The value is invalid. Try "10px" or "2em" or simply "2".':'Arvo ei kelpaa. Ota käyttöön "10px", "2em" tai pelkästään "2".',"Vertical text alignment toolbar":"Pystysuoran tekstin suunnan työkalupalkki",Width:"Leveys"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));