!function(e){const i=e.de=e.de||{};i.dictionary=Object.assign(i.dictionary||{},{"Choose heading":"Überschrift auswählen",Heading:"Überschrift","Heading 1":"Überschrift 1","Heading 2":"Überschrift 2","Heading 3":"Überschrift 3","Heading 4":"Überschrift 4","Heading 5":"Überschrift 5","Heading 6":"Überschrift 6","Type or paste your content here.":"Hier Inhalt einfügen.","Type your title":"Titel eingeben"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));