<?php

/**
 * @file
 * Defines the 8001 db update for the "status_report" group.
 */

declare(strict_types=1);

/**
 * Test update.
 */
function entity_test_update_8001(): void {
  // Empty update, we just want to trigger an error in the status report.
}
