{#
/**
 * @file
 * Default theme implementation for menu_link_form.
 *
 * Two-column template for the menu link add/edit form.
 *
 * This template will be used when a menu link form specifies
 * 'menu_link_form' as its #theme callback. Otherwise, by default,
 * menu_link add/edit forms will be themed by form.html.twig.
 *
 * Available variables:
 * - form: The menu link add/edit form.
 *
 * @ingroup themeable
 */
#}
{{ form }}
