!function(t){const i=t.ku=t.ku||{};i.dictionary=Object.assign(i.dictionary||{},{Bold:"قەڵەو","Bold text":"",Code:"کۆد",Italic:"لار","Italic text":"","Move out of an inline code style":"",Strikethrough:"هێڵ بەسەرداهاتوو","Strikethrough text":"",Subscript:"ژێرنووس",Superscript:"سەرنووس",Underline:"ژێرهێڵ","Underline text":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));