!function(n){const e=n.th=n.th||{};e.dictionary=Object.assign(e.dictionary||{},{"Code block":"บล็อกรหัส","Entering %0 code snippet":"รหัสคำสั่งสั้นการเข้าสู่ %0","Entering code snippet":"รหัสคำสั่งสั้นการเข้า","Insert code block":"เพิ่มโค้ดบล็อก","Leaving %0 code snippet":"รหัสคำสั่งสั้นการออกจาก %0","Leaving code snippet":"รหัสคำสั่งสั้นการออก","Plain text":"ข้อความธรรมดา"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));