!function(e){const a=e.lv=e.lv||{};a.dictionary=Object.assign(a.dictionary||{},{"Cannot access default workspace.":"Nevar piekļūt noklusējuma darbvietai.","Cannot determine a category for the uploaded file.":"Nav iespējams noteikt augšupielādētā faila kategoriju","Edit image":"Rediģēt attēlu","Failed to determine category of edited image.":"Neizdevās noteikt rediģētā attēla kategoriju.","Open file manager":"Atvērt failu pārvaldnieku","Processing the edited image.":"Rediģētā attēla apstrāde.","Server failed to process the image.":"Serverim neizdevās apstrādāt attēlu.","You have no image editing permissions.":"Jums nav attēlu rediģēšanas atļaujas."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));