!function(o){const e=o["pt-br"]=o["pt-br"]||{};e.dictionary=Object.assign(e.dictionary||{},{"Code block":"Bloco de código","Entering %0 code snippet":"Inserção do trecho de código %0","Entering code snippet":"Inserção de trecho de código","Insert code block":"Inserir bloco de código","Leaving %0 code snippet":"Deixando o trecho de código %0","Leaving code snippet":"Deixando o trecho de código","Plain text":"Texto sem formatação"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));