!function(e){const a=e.gl=e.gl||{};a.dictionary=Object.assign(a.dictionary||{},{"Bulleted List":"Lista con viñetas","Bulleted list styles toolbar":"Barra de ferramentas de estilos de lista con viñetas",Circle:"Círculos",Decimal:"Decimais","Decimal with leading zero":"Decimais con cero á esquerda","Decrease list item indent":"Diminuir a sangría do elemento da lista",Disc:"Discos","Entering a to-do list":"Introducir unha lista de tarefas pendentes","Increase list item indent":"Aumentar a sangría do elemento da lista","Invalid start index value.":"O valor do índice de inicio non é válido.","Keystrokes that can be used in a list":"Teclas de atallo que se poden usar nunha lista","Leaving a to-do list":"Abandonando unha lista de tarefas pendentes","List properties":"Lista de propiedades","Lower-latin":"Letras minúsculas","Lower–roman":"Núm, romanos en minúscula","Numbered List":"Lista numerada","Numbered list styles toolbar":"Barra de ferramentas de estilos de lista numerada","Reversed order":"Orde inversa",Square:"Cadrados","Start at":"Comezar en","Start index must be greater than 0.":"O índice de inicio debe ser maior que 0.","To-do List":"Lista de cousas por facer","Toggle the circle list style":"Alternar o estilo da lista «círculos»","Toggle the decimal list style":"Alternar o estilo da lista «decimais»","Toggle the decimal with leading zero list style":"Activar/desactivar o estilo «decimais con cero á esquerda»","Toggle the disc list style":"Alternar o estilo da lista «discos»","Toggle the lower–latin list style":"Activar/desactivar o estilo da lista «letras minúsculas»","Toggle the lower–roman list style":"Activar/desactivar o estilo da lista «núm. romanos en minúscula»","Toggle the square list style":"Alternar o estilo da lista «cadrados»","Toggle the upper–latin list style":"Activar/desactivar o estilo da lista «letras maiúsculas»","Toggle the upper–roman list style":"Activar/desactivar o estilo da lista «núm. romanos en maiúscula»","Upper-latin":"Letras maiúsculas","Upper-roman":"Núm, romanos en maiúscula"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));