---
label: 'Managing contact forms'
top_level: true
---
<h2>{% trans %}What are contact forms?{% endtrans %}</h2>
<p>{% trans %}There are two different types of contact forms provided by the core Contact module: personal contact forms, which allow users to contact other users on the site, and site-wide contact forms, which allow users to contact site managers or administrators. A site can have more than one site-wide contact form; each has its own fields to fill out, recipients, and URL; you can also change the fields that are shown on personal contact forms.{% endtrans %}</p>
<h2>{% trans %}Using the personal contact form{% endtrans %}</h2>
<p>{% trans %}Site visitors can email registered users on your site by using the personal contact form, without knowing or learning the email address of the recipient. When a user with the correct permissions is viewing another user's profile, the viewer will see a <em>Contact</em> tab or link, which leads to the personal contact form if the user whose profile is being viewed has their personal contact form enabled (this is a user account setting).{% endtrans %}</p>
<h2>{% trans %}Contact form management tasks{% endtrans %}</h2>
<p>{% trans %}See the related topics below for specific tasks.{% endtrans %}</p>
