{#
/**
 * @file
 * Default theme implementation for a system settings form.
 *
 * This template will be used when a system config form specifies 'config_form'
 * as its #theme callback.  Otherwise, by default, system config forms will be
 * themed by form.html.twig. This does not alter the appearance of a form at
 * all, but is provided as a convenience for themers.
 *
 * Available variables:
 * - form: The confirm form.
 *
 * @ingroup themeable
 */
#}
{{ form }}
