!function(i){const a=i.sk=i.sk||{};a.dictionary=Object.assign(a.dictionary||{},{"Disable block editing":"Zakázať úpravu blokov","Disable editing":"Deaktivovať úpravu","Disable inline editing":"Zakázať vložené úpravy","Enable block editing":"Povoliť úpravu blokov","Enable editing":"Zapnúť režim úprav","Enable inline editing":"Povoliťvložené úpravy","Navigate editable regions":"Navigácia v oblasti úprav","Next editable region":"Nasledujúca oblasť úprav","Previous editable region":"Predchádzajúca oblasť úprav"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));