!function(t){const i=t.bg=t.bg||{};i.dictionary=Object.assign(i.dictionary||{},{Bold:"Удебелен","Bold text":"Почернен текст",Code:"Код",Italic:"Курсив","Italic text":"Наклонен текст","Move out of an inline code style":"Излизане от подравнен стил на кодиране",Strikethrough:"Зачертаване","Strikethrough text":"Зачертан текст",Subscript:"Долен индекс",Superscript:"Горен индекс",Underline:"Подчертаване","Underline text":"Подчертан текст"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));