!function(e){const t=e.hu=e.hu||{};t.dictionary=Object.assign(t.dictionary||{},{"Align cell text to the bottom":"Szöveg igazítása a cellában alulra","Align cell text to the center":"Szöveg igazítása a cellában középre","Align cell text to the left":"Szöveg igazítása a cellában balra","Align cell text to the middle":"Szöveg igazítása a cellában középre","Align cell text to the right":"Szöveg igazítása a cellában jobbra","Align cell text to the top":"Szöveg igazítása a cellában felülre","Align table to the left":"Tábla igazítása balra","Align table to the left with no text wrapping":"Táblázat balra igazítása a szöveg sortördelése nélkül","Align table to the left with text wrapping":"Táblázat balra igazítása a szöveg sortördelésével","Align table to the right":"Tábla igazítása jobbra","Align table to the right with no text wrapping":"Táblázat jobbra igazítása a szöveg sortördelése nélkül","Align table to the right with text wrapping":"Táblázat jobbra igazítása a szöveg sortördelésével",Alignment:"Igazítás",Background:"Háttér",Border:"Keret","Cell properties":"Cella tulajdonságok","Center table":"Tábla igazítása középre","Center table with no text wrapping":"Táblázat középre igazítása a szöveg sortördelése nélkül","Choose table type":"Táblázattípus kiválasztása",Color:"Szín",Column:"Oszlop","Content table":"Tartalomjegyzék",Dashed:"Szaggatott","Delete column":"Oszlop törlése","Delete row":"Sor törlése",Dimensions:"Méretek",Dotted:"Pontozott",Double:"Dupla","Enter table caption":"Táblázat feliratának megadása",Groove:"Árok","Header column":"Oszlop fejléc","Header row":"Sor fejléc",Height:"Magasság","Horizontal text alignment toolbar":"Vízszintes szövegigazítási eszköztár","Insert a new table row (when in the last cell of a table)":"Új sor beillesztése a táblázatba (a táblázat utolsó cellájában)","Insert column left":"Oszlop beszúrása balra","Insert column right":"Oszlop beszúrása jobbra","Insert row above":"Sor beszúrása fölé","Insert row below":"Sor beszúrása alá","Insert table":"Táblázat beszúrása","Insert table layout":"Táblázat elrendezés beillesztése",Inset:"Mélyített","Justify cell text":"Szöveg igazítása a cellában sorkizártra","Keystrokes that can be used in a table cell":"Egy táblázatcellában használható billentyű leütések","Layout table":"Táblázat elrendezése","Merge cell down":"Cellák egyesítése lefelé","Merge cell left":"Cellák egyesítése balra","Merge cell right":"Cellák egyesítése jobbra","Merge cell up":"Cellák egyesítése felfelé","Merge cells":"Cellaegyesítés","Move the selection to the next cell":"A kiválasztás átmozgatása a következő cellába","Move the selection to the previous cell":"A kiválasztás átmozgatása az előző cellába","Navigate through the table":"Navigáció a táblázatban",None:"Nincs",Outset:"Kiemelkedő",Padding:"Térköz",Ridge:"Gerinc",Row:"Sor","Select column":"Oszlop kijelölése","Select row":"Sor kijelölése",Solid:"Tömör","Split cell horizontally":"Cella felosztása vízszintesen","Split cell vertically":"Cella felosztása függőlegesen",Style:"Stílus",Table:"Táblázat","Table Alignment":"Táblázatigazítás","Table alignment toolbar":"Táblázatigazítási eszköztár","Table cell text alignment":"Szöveg igazítása a cellában","Table layout":"Táblázat elrendezés","Table properties":"Táblázat tulajdonságai","Table toolbar":"Táblázat eszköztár","Table type":"Táblázattípus","Table type options":"Táblázattípus lehetőségek",'The color is invalid. Try "#FF0000" or "rgb(255,0,0)" or "red".':'A szín érványtelen. Próbáld így "#FF0000" vagy "rgb(255,0,0)" vagy "red".','The value is invalid. Try "10px" or "2em" or simply "2".':'Az érték érvénytelen. Próbáld így "10px" vagy "2em" vagy csak egyszerűen "2".',"Vertical text alignment toolbar":"Függőleges szövegigazítási eszköztár",Width:"Szélesség"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));