!function(i){const t=i.lt=i.lt||{};t.dictionary=Object.assign(t.dictionary||{},{"Block styles":"Blokuoti stilius","Multiple styles":"Daug stilių",Styles:"Stiliai","Text styles":"Teksto stiliai"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));