!function(e){const t=e.th=e.th||{};t.dictionary=Object.assign(t.dictionary||{},{"Insert media":"แทรกสื่อ",Media:"สื่อ","Media embed":"สื่อที่ฝัง","Media toolbar":"แถบเครื่องมือสื่อ","Media URL":"URL สื่อ","media widget":"วิดเจ็ตสื่อ","Open media in new tab":"เปิดสื่อในแท็บใหม่","Paste the media URL in the input.":"วาง URL สื่อในอินพุต","The URL must not be empty.":"URL ต้องไม่ว่างเปล่า","This media URL is not supported.":"ไม่รองรับ URL ของสื่อนี้","Tip: Paste the URL into the content to embed faster.":"เคล็ดลับ: การวาง URL ลงในเนื้อหาจะช่วยให้ฝังได้เร็วขึ้น"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));