<?php

namespace Drupal\views\Plugin\views\field;

use Drupal\views\Attribute\ViewsField;

/**
 * Default implementation of the base field plugin.
 *
 * @ingroup views_field_handlers
 */
#[ViewsField("standard")]
class Standard extends FieldPluginBase {

}
