!function(t){const s=t["pt-br"]=t["pt-br"]||{};s.dictionary=Object.assign(s.dictionary||{},{"Block styles":"Estilos de bloco","Multiple styles":"Múltiplos estilos",Styles:"Estilos","Text styles":"Estilos de texto"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));