<?php

declare(strict_types=1);

namespace Drupal\entity_test_bundle_class\Entity;

/**
 * An invalid bundle class which does not inherit the main entity class.
 */
class NonInheritingBundleClass {
}
