!function(e){const t=e.ur=e.ur||{};t.dictionary=Object.assign(t.dictionary||{},{"Insert media":"میڈیا نصب کریں",Media:"","Media embed":"","Media toolbar":"آلہ جات برائے میڈیا","Media URL":"میڈیا یو آر ایل","media widget":"آلۂ میڈیا","Open media in new tab":"","Paste the media URL in the input.":"میڈیا یو آر ایل کو چسپاں کریں","The URL must not be empty.":"یو آر ایل خالی نہیں ہونا چاہیے۔","This media URL is not supported.":"میڈیا یو آر ایل  معاونت یافتہ نہیں","Tip: Paste the URL into the content to embed faster.":"نکتہ : یو آر ایل کو جلد ضم کرنے کے لیے ربط مواد میں چسپاں کریں"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));