!function(a){const n=a.id=a.id||{};n.dictionary=Object.assign(n.dictionary||{},{Big:"Besar",Default:"Bawaan","Document colors":"Warna dokumen","Font Background Color":"Warna Latar Huruf","Font Color":"Warna Huruf","Font Family":"Jenis Huruf","Font Size":"Ukuran Huruf",Huge:"Sangat Besar",Small:"Kecil",Tiny:"Sangat Kecil"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));