!function(o){const a=o.ja=o.ja||{};a.dictionary=Object.assign(a.dictionary||{},{Bookmark:"ブックマーク","Bookmark must not be empty.":"ブックマークは空白にできません。","Bookmark name":"ブックマーク名","Bookmark name already exists.":"すでに使われているブックマーク名です。","Bookmark name cannot contain space characters.":"ブックマーク名にスペースを含めることはできません。","Bookmark toolbar":"ブックマークツールバー","bookmark widget":"ブックマークウィジェット",Bookmarks:"ブックマーク","Edit bookmark":"ブックマークを編集","Enter the bookmark name without spaces.":"ブックマーク名をスペースなしで入力して下さい。","No bookmarks available.":"ブックマークはありません。","Remove bookmark":"ブックマークを削除","Scroll to bookmark":"ブックマークまでスクロール"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));