!function(e){const a=e.es=e.es||{};a.dictionary=Object.assign(a.dictionary||{},{"Cannot access default workspace.":"No se puede acceder al espacio de trabajo predeterminado.","Cannot determine a category for the uploaded file.":"No se puede determinar una categoría para el archivo subido.","Edit image":"Editar imagen","Failed to determine category of edited image.":"No se pudo determinar la categoría de la imagen editada.","Open file manager":"Abrir gestor de archivos","Processing the edited image.":"Procesando la imagen editada.","Server failed to process the image.":"El servidor no pudo procesar la imagen.","You have no image editing permissions.":"No tienes permiso para editar imágenes."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));