!function(e){const t=e.sk=e.sk||{};t.dictionary=Object.assign(t.dictionary||{},{Activities:"Aktivity","Animals & Nature":"Zvieratá a príroda","Dark skin tone":"Tmavý tón pleti","Default skin tone":"Predvolený tón pleti",Emoji:"Emotikon","Emoji picker":"Výber emodži","Find an emoji (min. 2 characters)":"Nájsť emotikon (min. 2 znaky)",Flags:"Zástavy","Food & Drinks":"Jedlá a nápoje","Gestures & People":"Gestá a ľudia","Keep on typing to see the emoji.":"Pokračujte v písaní, aby ste videli emotikony.","Light skin tone":"Svetlý tón pleti","Medium Dark skin tone":"Stredne tmavý tón pleti","Medium Light skin tone":"Stredne svetlý tón pleti","Medium skin tone":"Stredný tón pleti",'No emojis were found matching "%0".':"Nenašli sa žiadne emotikony zodpovedajúce výrazu „%0“.",Objects:"Objekty","Select skin tone":"Vyberte tón pleti","Show all emoji...":"Zobraziť všetky emotikony...","Smileys & Expressions":"Smajlíky a výrazy",Symbols:"Symboly","The query must contain at least two characters.":"Dotaz musí obsahovať aspoň dva znaky.","Travel & Places":"Cestovanie a miesta"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));