!function(t){const e=t.tk=t.tk||{};e.dictionary=Object.assign(e.dictionary||{},{"Edit source":"","Empty snippet content":"","HTML snippet":"","Insert HTML":"","No preview available":"","Paste raw HTML here...":"","Save changes":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));