<?php

declare(strict_types=1);

namespace Drupal\plugin_test\Plugin\plugin_test\custom_annotation;

/**
 * Provides a test plugin with a custom annotation.
 *
 * @PluginExample(
 *   id = "example_1",
 *   custom = "John"
 * )
 */
class Example1 {}
