!function(i){const e=i.et=i.et||{};e.dictionary=Object.assign(e.dictionary||{},{"Could not insert image at the current position.":"Pildi sisestamine praegusesse kohta ebaõnnestus.","Could not obtain resized image URL.":"Muudetud suurusega pildi URL-i hankimine ebaõnnestus.","Insert image or file":"Sisesta pilt või fail","Inserting image failed":"Pildi sisestamine ebaõnnestus","Selecting resized image failed":"Muudetud suurusega pildi valimine ebaõnnestus"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));