!function(i){const e=i.et=i.et||{};e.dictionary=Object.assign(e.dictionary||{},{"Disable block editing":"Keela ploki muutmine","Disable editing":"Keela muutmine","Disable inline editing":"Keela teksti muutmine","Enable block editing":"Luba valitud ploki muutmine","Enable editing":"Luba muutmine","Enable inline editing":"Luba valitud teksti muutmine","Navigate editable regions":"Muudetavate piirkondade vahel liikumine","Next editable region":"Järgmine muudetav piirkond","Previous editable region":"Eelmine muudetav piirkond"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));