<?php

namespace Drupal\phpass\Password;

use Drupal\Core\Password\PhpassHashedPasswordBase;

/**
 * Legacy password hashing framework.
 *
 * @see https://www.drupal.org/node/3322420
 */
class PhpassHashedPassword extends PhpassHashedPasswordBase {}
