!function(e){const i=e.et=e.et||{};i.dictionary=Object.assign(i.dictionary||{},{"Cannot access default workspace.":"Puudub juurdepääs vaiketööruumile.","Cannot determine a category for the uploaded file.":"Ei suuda üles laaditud faili kategooriat tuvastada.","Edit image":"Redigeeri pilti","Failed to determine category of edited image.":"Redigeeritud pildi kategooria määramine nurjus.","Open file manager":"Ava failihaldur","Processing the edited image.":"Redigeeritud pildi töötlemine.","Server failed to process the image.":"Serveril ei õnnestunud pilti töödelda.","You have no image editing permissions.":"Puudub pildi muutmise luba"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));