{#
/**
 * @file
 * Default theme implementation for a link to a file.
 *
 * Available variables:
 * - attributes: The HTML attributes for the containing element.
 * - link: A link to the file.
 * - file_size: The size of the file.
 *
 * @see \Drupal\file\Hook\FileThemeHooks::preprocessFileLink()
 *
 * @ingroup themeable
 */
#}
<span{{ attributes }}>{{ link }}</span>
{% if file_size %}
  <span>({{ file_size }})</span>
{% endif %}
