!function(a){const o=a.ca=a.ca||{};o.dictionary=Object.assign(o.dictionary||{},{Bookmark:"Marcador","Bookmark must not be empty.":"El marcador no pot estar buit.","Bookmark name":"Nom del marcador","Bookmark name already exists.":"El nom del marcador ja existeix.","Bookmark name cannot contain space characters.":"El nom del marcador no pot contenir espais.","Bookmark toolbar":"Barra d'eines d'adreces d'interès","bookmark widget":"giny de marcador",Bookmarks:"Adreces d'interès","Edit bookmark":"Edita el marcador","Enter the bookmark name without spaces.":"Introdueix el nom del marcador sense espais.","No bookmarks available.":"No hi ha cap adreça d'interès disponible.","Remove bookmark":"Elimina el marcador","Scroll to bookmark":"Desplaça't fins a les adreces d'interès"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));