{#
/**
 * @file
 * Default theme implementation for a single local action link.
 *
 * Available variables:
 * - attributes: HTML attributes for the wrapper element.
 * - link: A rendered link element.
 *
 * @see \Drupal\Core\Menu\MenuPreprocess::preprocessMenuLocalAction()
 *
 * @ingroup themeable
 */
#}
<li{{ attributes }}>{{ link }}</li>
