!function(i){const n=i.ti=i.ti||{};n.dictionary=Object.assign(n.dictionary||{},{"Choose heading":"ርእሲ ምረጽ",Heading:"ርእሲ","Heading 1":"ርእሲ 1","Heading 2":"ርእሲ 2","Heading 3":"ርእሲ 3","Heading 4":"ርእሲ 4","Heading 5":"ርእሲ 5","Heading 6":"ርእሲ 6","Type or paste your content here.":"ኣብዚ ቦታ ጽሑፍካ ጸሓፍ ወይ ጠቅዕ","Type your title":"ርእሲ ጽሑፍካ ጸሓፍ"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));