!function(o){const a=o["en-gb"]=o["en-gb"]||{};a.dictionary=Object.assign(a.dictionary||{},{Bookmark:"Bookmark","Bookmark must not be empty.":"Bookmark must not be empty.","Bookmark name":"Bookmark name","Bookmark name already exists.":"Bookmark name already exists.","Bookmark name cannot contain space characters.":"Bookmark name cannot contain space characters.","Bookmark toolbar":"Bookmark toolbar","bookmark widget":"bookmark widget",Bookmarks:"Bookmarks","Edit bookmark":"Edit bookmark","Enter the bookmark name without spaces.":"Enter the bookmark name without spaces.","No bookmarks available.":"No bookmarks available.","Remove bookmark":"Remove bookmark","Scroll to bookmark":"Scroll to bookmark"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));