!function(o){const a=o.cs=o.cs||{};a.dictionary=Object.assign(a.dictionary||{},{Bookmark:"Záložka","Bookmark must not be empty.":"Záložka nesmí být prázdná","Bookmark name":"Název záložky","Bookmark name already exists.":"Záložka s tímto názvem už existuje.","Bookmark name cannot contain space characters.":"Název záložky nemůže obsahovat znaky mezery.","Bookmark toolbar":"Panel nástrojů záložek","bookmark widget":"widget záložky",Bookmarks:"Záložky","Edit bookmark":"Upravit záložku","Enter the bookmark name without spaces.":"Zadejte název záložky bez mezer.","No bookmarks available.":"Žádné záložky nejsou k dispozici","Remove bookmark":"Odstranit záložku","Scroll to bookmark":"Přejít na záložku"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));