!function(i){const e=i["zh-cn"]=i["zh-cn"]||{};e.dictionary=Object.assign(e.dictionary||{},{"Disable block editing":"禁用块编辑","Disable editing":"禁用编辑","Disable inline editing":"禁用行内编辑","Enable block editing":"启用块编辑","Enable editing":"允许编辑","Enable inline editing":"启用行内编辑","Navigate editable regions":"导航至可编辑区域","Next editable region":"下一个可编辑区域","Previous editable region":"上一个可编辑区域"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));