!function(i){const e=i.eu=i.eu||{};e.dictionary=Object.assign(e.dictionary||{},{"Could not insert image at the current position.":"","Could not obtain resized image URL.":"","Insert image or file":"","Inserting image failed":"","Selecting resized image failed":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));