!function(i){const e=i.ru=i.ru||{};e.dictionary=Object.assign(e.dictionary||{},{"Could not insert image at the current position.":"Нельзя вставить изображение на текущую позицию.","Could not obtain resized image URL.":"Не удалось получить URL с измененным размером изображения.","Insert image or file":"Вставьте изображение или файл","Inserting image failed":"Вставка изображения не удалась","Selecting resized image failed":"Выбор изображения с измененным размером не удался"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));