!function(t){const i=t.lt=t.lt||{};i.dictionary=Object.assign(i.dictionary||{},{"Advanced options":"Išplėstinės pasirinktys",Find:"Surasti","Find and replace":"Rasti ir pakeisti","Find in text…":"Surasti tekste...","Find in the document":"Ieškoti dokumente","Match case":"Sutapties atvejis","Next result":"Kitas rezultatas","Previous result":"Buvęs rezultatas",Replace:"Pakeisti","Replace all":"Pakeisti visus","Replace with…":"Sukeisti su...","Text to find must not be empty.":"Ieškomas tekstas negali būti tuščias.","Tip: Find some text first in order to replace it.":"Patarimas: Pirmiausiai suraskite teksto, jog jį sukeistumėte.","Whole words only":"Tik vientisus žodžius"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));