!function(e){const n=e["en-gb"]=e["en-gb"]||{};n.dictionary=Object.assign(n.dictionary||{},{"Edit source":"","Empty snippet content":"","HTML snippet":"","Insert HTML":"","No preview available":"","Paste raw HTML here...":"","Save changes":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));