!function(e){const t=e.da=e.da||{};t.dictionary=Object.assign(t.dictionary||{},{"Bulleted List":"Punktopstilling","Bulleted list styles toolbar":"Værktøjslinje til punktopdelt liste",Circle:"Cirkel",Decimal:"Decimal","Decimal with leading zero":"Decimal med nul i starten","Decrease list item indent":"Reducer listeemners indrykning",Disc:"Disk","Entering a to-do list":"Indtastning af en huskeliste","Increase list item indent":"Forøg listemners indrykning","Invalid start index value.":"Ugyldig startindeksværdi.","Keystrokes that can be used in a list":"Tastaturtryk, der kan bruges i en liste","Leaving a to-do list":"At gå fra en huskeliste","List properties":"Listeegenskaber","Lower-latin":"Latinsk med små bogstaver","Lower–roman":"Romertal med små bogstaver","Numbered List":"Opstilling med tal","Numbered list styles toolbar":"Værktøjslinje til nummereret listestil","Reversed order":"Omvendt rækkefølge",Square:"Firkant","Start at":"Start ved","Start index must be greater than 0.":"Startindeks skal være større end 0.","To-do List":"To-do liste","Toggle the circle list style":"Slå listestilen cirkel til og fra","Toggle the decimal list style":"Slå listestilen decimal til og fra","Toggle the decimal with leading zero list style":"Slå listestilen decimal med nul i starten til og fra","Toggle the disc list style":"Slå listestilen disk til og fra","Toggle the lower–latin list style":"Slå listestilen latinsk med små bogstaver til","Toggle the lower–roman list style":"Slå listestilen romertal med små bogstaver til","Toggle the square list style":"Slå listestilen firkantet til og fra","Toggle the upper–latin list style":"Slå listestilen latinsk med store bogstaver til","Toggle the upper–roman list style":"Slå listestilen romertal med store bogstaver til","Upper-latin":"Latinsk med store bogstaver","Upper-roman":"Romertal med store bogstaver"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));