!function(e){const t=e.el=e.el||{};t.dictionary=Object.assign(t.dictionary||{},{"Align cell text to the bottom":"Στοίχιση κειμένου κελιού κάτω","Align cell text to the center":"Στοίχιση κειμένου κελιού στο κέντρο","Align cell text to the left":"Στοίχιση κειμένου κελιού στα αριστερά","Align cell text to the middle":"Στοίχιση κειμένου κελιού στο μέσο","Align cell text to the right":"Στοίχιση κειμένου κελιού στα δεξιά","Align cell text to the top":"Στοίχιση κειμένου κελιού επάνω","Align table to the left":"Στοίχιση πίνακα στα αριστερά","Align table to the left with no text wrapping":"Στοίχιση πίνακα στα αριστερά χωρίς αναδίπλωση κειμένου","Align table to the left with text wrapping":"Στοίχιση πίνακα στα αριστερά με αναδίπλωση κειμένου","Align table to the right":"Στοίχιση πίνακα στα δεξιά","Align table to the right with no text wrapping":"Στοίχιση πίνακα στα δεξιά χωρίς αναδίπλωση κειμένου","Align table to the right with text wrapping":"Στοίχιση πίνακα στα δεξιά με αναδίπλωση κειμένου",Alignment:"Στοίχιση",Background:"Υπόβαθρο",Border:"Περίγραμμα","Cell properties":"Ιδιότητες κελιού","Center table":"Στοίχιση πίνακα στο κέντρο","Center table with no text wrapping":"Κεντράρισμα πίνακα χωρίς αναδίπλωση κειμένου","Choose table type":"Επιλογή τύπου πίνακα",Color:"Χρώμα",Column:"Στήλη","Content table":"Πίνακας περιεχομένων",Dashed:"Με παύλες","Delete column":"Διαγραφή στήλης","Delete row":"Διαγραφή γραμμής",Dimensions:"Διαστάσεις",Dotted:"Διάστικτο",Double:"Διπλό","Enter table caption":"Εισαγωγή λεζάντας πίνακα",Groove:"Βαθουλωτό","Header column":"Στήλη κεφαλίδας","Header row":"Γραμμή κεφαλίδας",Height:"Ύψος","Horizontal text alignment toolbar":"Γραμμή εργαλείων οριζόντιας στοίχισης κειμένου","Insert a new table row (when in the last cell of a table)":"Εισαγωγή νέας σειράς πίνακα (όταν βρίσκεται στο τελευταίο κελί ενός πίνακα)","Insert column left":"Εισαγωγή στήλης αριστερά","Insert column right":"Εισαγωγή στήλης δεξιά","Insert row above":"Εισαγωγή γραμμής πάνω","Insert row below":"Εισαγωγή γραμμής κάτω","Insert table":"Εισαγωγή πίνακα","Insert table layout":"Εισαγωγή διάταξης πίνακα",Inset:"Κείμενο με βάθος","Justify cell text":"Πλήρης στοίχιση κειμένου κελιού","Keystrokes that can be used in a table cell":"Πλήκτρα που μπορείτε να χρησιμοποιήσετε σε ένα κελί πίνακα","Layout table":"Πίνακας διατάξεων","Merge cell down":"Συγχώνευση κελιού κάτω","Merge cell left":"Συγχώνευση κελιού αριστερά","Merge cell right":"Συγχώνευση κελιού δεξιά","Merge cell up":"Συγχώνευση κελιού πάνω","Merge cells":"Συγχώνευση κελιών","Move the selection to the next cell":"Μετακίνηση της επιλογής στο επόμενο κελί","Move the selection to the previous cell":"Μετακίνηση της επιλογής στο προηγούμενο κελί","Navigate through the table":"Πλοήγηση στον πίνακα",None:"Χωρίς περίγραμμα",Outset:"Κείμενο με ύψος",Padding:"Απόσταση κειμένου από το περίγραμμα",Ridge:"Ραχωτό",Row:"Γραμμή","Select column":"Επιλογή στήλης","Select row":"Επιλογή γραμμής",Solid:"Συμπαγές","Split cell horizontally":"Διαχωρισμός κελιού οριζόντια","Split cell vertically":"Διαχωρισμός κελιού κάθετα",Style:"Στυλ",Table:"Πίνακας","Table Alignment":"Στοίχιση πίνακα","Table alignment toolbar":"Γραμμή εργαλείων στοίχισης πίνακα","Table cell text alignment":"Στοίχιση κειμένου κελιού πίνακα","Table layout":"Διάταξη πίνακα","Table properties":"Ιδιότητες πίνακα","Table toolbar":"Γραμμή εργαλείων πίνακα","Table type":"Τύπος πίνακα","Table type options":"Επιλογές τύπου πίνακα",'The color is invalid. Try "#FF0000" or "rgb(255,0,0)" or "red".':"Το χρώμα δεν είναι έγκυρο. Δοκιμάστε «#FF0000» ή «rgb(255,0,0)» ή «red».",'The value is invalid. Try "10px" or "2em" or simply "2".':"Η τιμή δεν είναι έγκυρη. Δοκιμάστε «10px» ή «2em» ή απλά «2».","Vertical text alignment toolbar":"Γραμμή εργαλείων κάθετης στοίχισης κειμένου",Width:"Πλάτος"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));