!function(e){const i=e.et=e.et||{};i.dictionary=Object.assign(i.dictionary||{},{"Connected users":"Ühenduses kasutajad","Document outline":"Dokumendi ülevaade","Enter fullscreen mode":"Sisene täisekraani režiimi","Fullscreen mode":"Täisekraani režiim","Hide left sidebar":"Peida vasakpoolne külgriba","Leave fullscreen mode":"Välju täisekraani režiimist","Show left sidebar":"Kuva vasakpoolne külgriba","Toggle fullscreen mode":"Lülita täisekraani režiim sisse-välja","Toggle sidebar":"Kuva/peida külgriba"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));