!function(e){const n=e.ar=e.ar||{};n.dictionary=Object.assign(n.dictionary||{},{"Connected users":"المستخدمون المتصلون","Document outline":"مخطط المستند","Enter fullscreen mode":"الدخول في وضع ملء الشاشة","Fullscreen mode":"وضع ملء الشاشة","Hide left sidebar":"إخفاء الشريط الجانبي الأيسر","Leave fullscreen mode":"مغادرة وضع ملء الشاشة","Show left sidebar":"إظهار الشريط الجانبي الأيسر","Toggle fullscreen mode":"تبديل وضع ملء الشاشة","Toggle sidebar":"تبديل الشريط الجانبي"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));