!function(e){const i=e.nl=e.nl||{};i.dictionary=Object.assign(i.dictionary||{},{"Could not insert image at the current position.":"Kan afbeelding niet op de huidige positie invoegen.","Could not obtain resized image URL.":"Het is niet gelukt de geschaalde afbeelding URL te verkrijgen.","Insert image or file":"Voeg afbeelding of bestand in","Inserting image failed":"Afbeelding invoegen niet gelukt","Selecting resized image failed":"De geschaalde afbeelding selecteren is niet gelukt"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));