!function(e){const n=e.nl=e.nl||{};n.dictionary=Object.assign(n.dictionary||{},{"Code block":"Codeblok","Entering %0 code snippet":"Codesnippet %0 invoeren","Entering code snippet":"Codesnippet invoeren","Insert code block":"Codeblok invoegen","Leaving %0 code snippet":"Codesnippet %0 verlaten","Leaving code snippet":"Codesnippet verlaten","Plain text":"Platte tekst"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));