!function(o){const i=o.pt=o.pt||{};i.dictionary=Object.assign(i.dictionary||{},{"Code block":"Bloco de código","Entering %0 code snippet":"A introduzir o fragmento de código %0","Entering code snippet":"A introduzir fragmento de código","Insert code block":"Inserir bloco de citação","Leaving %0 code snippet":"A sair do fragmento de código %0","Leaving code snippet":"A sair do fragmento de código","Plain text":"Texto simples"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));