/*
 * DO NOT EDIT THIS FILE.
 * See the following change record for more information,
 * https://www.drupal.org/node/3084859
 * @preserve
 */

/**
 * @file
 * Recreate Drupal admin styling that was removed with reset.
 * @see system.admin.css
 *
 * @internal
 */

#drupal-off-canvas-wrapper {
  & .panel {
    padding: 0.3125rem 0.3125rem 0.9375rem;
  }

  & .panel__description {
    margin: 0 0 0.1875rem;
    padding: 2px 0 0.1875rem 0;
  }

  & .compact-link {
    margin: 0 0 0.625rem 0;
  }

  & small .admin-link::before {
    content: " [";
  }

  & small .admin-link::after {
    content: "]";
  }
}
