!function(a){const e=a.id=a.id||{};e.dictionary=Object.assign(e.dictionary||{},{"Advanced options":"Opsi lanjutan",Find:"Cari","Find and replace":"Cari dan ubah","Find in text…":"Cari di dalam teks...","Find in the document":"Temukan di dokumen","Match case":"Sesuaikan huruf","Next result":"Hasil berikutnya","Previous result":"Hasil sebelumnya",Replace:"Ubah","Replace all":"Ubah semua","Replace with…":"Ubah dengan...","Text to find must not be empty.":"Teks yang dicari tidak boleh kosong.","Tip: Find some text first in order to replace it.":"Tip: Cari suatu teks terlebih dahulu untuk menggantinya.","Whole words only":"Kata utuh saja"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));