!function(e){const n=e.de=e.de||{};n.dictionary=Object.assign(n.dictionary||{},{"Code block":"Codeblock","Entering %0 code snippet":"%0 Code-Ausschnitt eingeben","Entering code snippet":"Code-Ausschnit eingeben","Insert code block":"Code-Block einfügen","Leaving %0 code snippet":"%0 Code-Ausschnitt verlassen","Leaving code snippet":"Code-Ausschnit verlassen","Plain text":"Nur Text"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));