!function(e){const i=e.es=e.es||{};i.dictionary=Object.assign(i.dictionary||{},{"Insert media":"Insertar contenido multimedia",Media:"Multimedia","Media embed":"Inserción de medios","Media toolbar":"Barra de herramientas de contenido multimedia","Media URL":"URL del contenido multimedia","media widget":"Widget de contenido multimedia","Open media in new tab":"Abrir medio en una pestaña nueva","Paste the media URL in the input.":"Pega la URL del contenido multimedia","The URL must not be empty.":"La URL no debe estar vacía","This media URL is not supported.":"La URL de este contenido multimedia no está soportada","Tip: Paste the URL into the content to embed faster.":"Tip: pega la URL dentro del contenido para embeber más rápido"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));