!function(e){const i=e.ne=e.ne||{};i.dictionary=Object.assign(i.dictionary||{},{"Cannot access default workspace.":"","Cannot determine a category for the uploaded file.":"","Edit image":"","Failed to determine category of edited image.":"","Open file manager":"","Processing the edited image.":"","Server failed to process the image.":"","You have no image editing permissions.":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));