!function(i){const t=i.hr=i.hr||{};t.dictionary=Object.assign(t.dictionary||{},{"Block styles":"Blok stilovi","Multiple styles":"Više stilova",Styles:"Stilovi","Text styles":"Tekstualni stilovi"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));