!function(i){const a=i.fi=i.fi||{};a.dictionary=Object.assign(a.dictionary||{},{"Disable block editing":"Poista lohkon muokkaus käytöstä","Disable editing":"Poista muokkaus käytöstä","Disable inline editing":"Poista sisäinen muokkaus käytöstä","Enable block editing":"Ota lohkon muokkaus käyttöön","Enable editing":"Ota muokkaus käyttöön","Enable inline editing":"Ota sisäinen muokkaus käyttöön","Navigate editable regions":"Siirry muokattavilla alueilla","Next editable region":"Seuraava muokattava alue","Previous editable region":"Aikaisempi muokattava alue"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));