!function(i){const e=i.el=i.el||{};e.dictionary=Object.assign(e.dictionary||{},{"Could not insert image at the current position.":"Αδύνατη η εισαγωγή εικόνας στην τρέχουσα θέση.","Could not obtain resized image URL.":"Αδύνατη η λήψη URL εικόνας με αλλαγμένο μέγεθος.","Insert image or file":"Εισαγωγή εικόνας ή αρχείου","Inserting image failed":"Η εισαγωγή εικόνας απέτυχε.","Selecting resized image failed":"Η επιλογή εικόνας με αλλαγμένο μέγεθος απέτυχε"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));