!function(i){const n=i.el=i.el||{};n.dictionary=Object.assign(n.dictionary||{},{"Revert autoformatting action":"Επαναφορά ενέργειας αυτόματης μορφοποίησης"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));