<?php

/**
 * @file
 * Generic transliteration data for the PhpTransliteration class.
 */

$base = [
  0x00 => 'you', 'yi', 'xue', 'sa', 'xu', 'li', 'li', 'yuan', 'dui', 'huo', 'sha', 'leng', 'pou', 'hu', 'guo', 'bu',
  0x10 => 'rui', 'wei', 'sou', 'an', 'yu', 'xiang', 'heng', 'yang', 'xiao', 'yao', NULL, 'bi', NULL, 'heng', 'tao', 'liu',
  0x20 => NULL, 'zhu', NULL, 'xi', 'zan', 'yi', 'dou', 'yuan', 'jiu', NULL, 'bo', 'ti', 'ying', NULL, 'yi', 'nian',
  0x30 => 'shao', 'ben', 'gou', 'ban', 'mo', 'gai', 'en', 'she', NULL, 'zhi', 'yang', 'jian', 'yuan', 'shui', 'ti', 'wei',
  0x40 => 'xun', 'zhi', 'yi', 'ren', 'shi', 'hu', 'ne', 'ye', 'jian', 'sui', 'ying', 'bao', 'hu', 'hu', 'ye', NULL,
  0x50 => 'yang', 'lian', 'xi', 'en', 'dui', 'zan', 'zhu', 'ying', 'ying', 'jin', 'chuang', 'dan', NULL, 'kuai', 'yi', 'ye',
  0x60 => 'jian', 'en', 'ning', 'ci', 'qian', 'xue', 'bo', 'mi', 'shui', 'mo', 'liang', 'qi', 'qi', 'shou', 'fu', 'bo',
  0x70 => 'beng', 'bie', 'yi', 'wei', 'huan', 'fan', 'qi', 'mao', 'bao', 'ang', 'ang', 'fu', 'qi', 'qun', 'tuo', 'yi',
  0x80 => 'bo', 'pian', 'ba', NULL, 'xuan', NULL, NULL, 'yu', 'chi', 'lu', 'yi', 'li', NULL, 'niao', 'xi', 'wu',
  0x90 => NULL, 'lei', 'pu', 'zhuo', 'zui', 'zhuo', 'chang', 'an', 'er', 'yu', 'leng', 'fu', 'zha', 'hun', 'chun', 'sou',
  0xA0 => 'bi', 'bi', 'zha', NULL, 'he', 'li', NULL, 'han', 'zai', 'gu', 'cheng', 'lou', 'mo', 'mi', 'mai', 'ao',
  0xB0 => 'zhe', 'zhu', 'huang', 'fan', 'deng', 'tong', NULL, 'du', 'wo', 'wei', 'ji', 'chi', 'lin', 'biao', 'long', 'jian',
  0xC0 => 'nie', 'luo', 'shen', NULL, 'gua', 'nie', 'yi', 'ku', 'wan', 'wa', 'qia', 'bo', 'kao', 'ling', 'gan', 'gua',
  0xD0 => 'hai', 'kuang', 'heng', 'kui', 'ze', 'ting', 'lang', 'bi', 'huan', 'po', 'yao', 'wan', 'ti', 'sui', 'kua', 'dui',
  0xE0 => 'ao', 'jian', 'mo', 'kui', 'kuai', 'an', 'ma', 'qing', 'qiao', NULL, 'kao', 'hao', 'duo', 'xian', 'nai', 'suo',
  0xF0 => 'jie', 'pi', 'pa', 'song', 'chang', 'nie', 'man', 'song', 'ci', 'xian', 'kuo', NULL, 'di', 'pou', 'tiao', 'zu',
];
