!function(e){const t=e.nl=e.nl||{};t.dictionary=Object.assign(t.dictionary||{},{"Align cell text to the bottom":"Celtekst onder uitlijnen","Align cell text to the center":"Tekst in de cel centreren","Align cell text to the left":"Celtekst links uitlijnen","Align cell text to the middle":"Celtekst in het midden uitlijnen","Align cell text to the right":"Celtekst rechts uitlijnen","Align cell text to the top":"Celtekst boven uitlijnen","Align table to the left":"Tabel links uitlijnen","Align table to the left with no text wrapping":"Tabel links uitlijnen zonder tekstomloop","Align table to the left with text wrapping":"Tabel links uitlijnen met tekstomloop","Align table to the right":"Tabel rechts uitlijnen","Align table to the right with no text wrapping":"Tabel rechts uitlijnen zonder tekstomloop","Align table to the right with text wrapping":"Tabel rechts uitlijnen met tekstomloop",Alignment:"Uitlijning",Background:"Achtergrond",Border:"Rand","Cell properties":"Celeigenschappen","Center table":"Tabel centreren","Center table with no text wrapping":"Tabel centreren zonder tekstomloop","Choose table type":"Tabeltype kiezen",Color:"Kleur",Column:"Kolom","Content table":"Inhoudstabel",Dashed:"Onderbroken","Delete column":"Verwijder kolom","Delete row":"Verwijder rij",Dimensions:"Afmetingen",Dotted:"Stippellijn",Double:"Dubbel","Enter table caption":"Voer tabelbijschrift in",Groove:"Sleuf","Header column":"Titel kolom","Header row":"Titel rij",Height:"Hoogte","Horizontal text alignment toolbar":"Werkbalk voor horizontale tekstuitlijning","Insert a new table row (when in the last cell of a table)":"Voeg een nieuwe rij toe aan de tabel (wanneer in de laatste cel van een tabel)","Insert column left":"Kolom links invoegen","Insert column right":"Kolom rechts invoegen","Insert row above":"Rij hierboven invoegen","Insert row below":"Rij hieronder invoegen","Insert table":"Tabel invoegen","Insert table layout":"Tabellay-out invoegen",Inset:"Ingezet","Justify cell text":"Celtekst uitvullen","Keystrokes that can be used in a table cell":"Toetsaanslagen die in een cel in een tabel gebruikt kunnen worden","Layout table":"Lay-outtabel","Merge cell down":"Cel hieronder samenvoegen","Merge cell left":"Cel hiervoor samenvoegen","Merge cell right":"Cel hierna samenvoegen","Merge cell up":"Cel hierboven samenvoegen","Merge cells":"Cellen samenvoegen","Move the selection to the next cell":"Verplaats de selectie naar de volgende cel","Move the selection to the previous cell":"Verplaats de selectie naar de vorige cel","Navigate through the table":"Navigeer door de tabel",None:"Geen",Outset:"Opliggend",Padding:"Opvulling",Ridge:"Rand",Row:"Rij","Select column":"Selecteer kolom","Select row":"Selecteer rij",Solid:"Ononderbroken","Split cell horizontally":"Splits cel horizontaal","Split cell vertically":"Splits cel verticaal",Style:"Stijl",Table:"Tabel","Table Alignment":"Tabeluitlijning","Table alignment toolbar":"Werkbalk tabeluitlijning","Table cell text alignment":"Tekstuitlijning tabelcel","Table layout":"Tabellay-out","Table properties":"Tabeleigenschappen","Table toolbar":"Tabel werkbalk","Table type":"Tabeltype","Table type options":"Opties voor tabeltype",'The color is invalid. Try "#FF0000" or "rgb(255,0,0)" or "red".':'De kleur in niet correct, probeer "#FF0000" of "rgb(255,0,0)" of "red".','The value is invalid. Try "10px" or "2em" or simply "2".':"De waarde is ongeldig. Probeer '10px' of '2em' of gewoon '2'.","Vertical text alignment toolbar":"Werkbalk voor verticale tekstuitlijning",Width:"Breedte"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));