!function(e){const i=e.da=e.da||{};i.dictionary=Object.assign(i.dictionary||{},{"Insert media":"Indsæt medie",Media:"Medier","Media embed":"Medieindlejring","Media toolbar":"Medie værktøjslinje","Media URL":"Medie URL","media widget":"mediewidget","Open media in new tab":"Åbn medie i ny fane","Paste the media URL in the input.":"Indsæt medie URLen i feltet.","The URL must not be empty.":"URLen kan ikke være tom.","This media URL is not supported.":"Denne medie URL understøttes ikke.","Tip: Paste the URL into the content to embed faster.":"Tip: Indsæt URLen i indholdet for at indlejre hurtigere."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));