!function(e){const r=e.sr=e.sr||{};r.dictionary=Object.assign(r.dictionary||{},{"Connected users":"Povezani korisnici","Document outline":"Struktura dokumenta","Enter fullscreen mode":"Uđite u režim celog ekrana","Fullscreen mode":"Režim celog ekrana","Hide left sidebar":"Sakrij levu bočnu traku","Leave fullscreen mode":"Napustite režim celog ekrana","Show left sidebar":"Prikaži levu bočnu traku","Toggle fullscreen mode":"Uključite režim celog ekrana","Toggle sidebar":"Prebaci na bočnu traku"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));