!function(e){const t=e.es=e.es||{};t.dictionary=Object.assign(t.dictionary||{},{Bold:"Negrita","Bold text":"Texto en negrita",Code:"Código",Italic:"Cursiva","Italic text":"Texto en cursiva","Move out of an inline code style":"Sale de un estilo de código en línea",Strikethrough:"Tachado","Strikethrough text":"Tacha el texto",Subscript:"Subíndice",Superscript:"Superíndice",Underline:"Subrayado","Underline text":"Subraya el texto"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));