!function(e){const t=e["sr-latn"]=e["sr-latn"]||{};t.dictionary=Object.assign(t.dictionary||{},{"Align cell text to the bottom":"Poravnajte tekst ćelije prema dole","Align cell text to the center":"Poravnajte tekst ćelije u sredinu","Align cell text to the left":"Poravnajte tekst ćelije levo","Align cell text to the middle":"Poravnajte tekst ćelije u sredinu","Align cell text to the right":"Poravnajte tekst ćelije desno","Align cell text to the top":"Poravnajte tekst ćelije prema gore","Align table to the left":"Poravnajte tabelu na levu stranu","Align table to the left with no text wrapping":"","Align table to the left with text wrapping":"","Align table to the right":"Poravnajte tabelu na desnu stranu","Align table to the right with no text wrapping":"","Align table to the right with text wrapping":"",Alignment:"Poravnanje",Background:"Pozadina",Border:"Granica","Cell properties":"Svojstva ćelije","Center table":"Centar tabele","Center table with no text wrapping":"","Choose table type":"",Color:"Boja",Column:"Kolona","Content table":"",Dashed:"Razbijeno","Delete column":"Briši kolonu","Delete row":"Briši red",Dimensions:"Dimenzija",Dotted:"Sa tačkama",Double:"Dvostruki","Enter table caption":"Unesite naslov tabele",Groove:"Kolosek","Header column":"Kolona za zaglavlje","Header row":"Red za zaglavlje",Height:"Visina","Horizontal text alignment toolbar":"Horizontalna traka sa alatkama za  poravnavanje teksta","Insert a new table row (when in the last cell of a table)":"","Insert column left":"Dodaj kolonu levo","Insert column right":"Dodaj kolonu desno","Insert row above":"Dodaj red iznad","Insert row below":"Dodaj red ispod","Insert table":"Dodaj tabelu","Insert table layout":"",Inset:"Prilog","Justify cell text":"Opravdajte tekst ćelije","Keystrokes that can be used in a table cell":"","Layout table":"","Merge cell down":"Spoj ćelije na dole","Merge cell left":"Spoj ćelije na levo","Merge cell right":"Spoj ćelije na desno","Merge cell up":"Spoj ćelije na gore","Merge cells":"Spoj ćelije","Move the selection to the next cell":"","Move the selection to the previous cell":"","Navigate through the table":"",None:"Nijedan",Outset:"Početak",Padding:"Postavljanje",Ridge:"Greben",Row:"Red","Select column":"Odaberi kolonu","Select row":"Odaberi red",Solid:"Čvrst","Split cell horizontally":"Deli ćelije vodoravno","Split cell vertically":"Deli ćelije uspravno",Style:"Stil",Table:"","Table Alignment":"","Table alignment toolbar":"Traka sa alatkama za poravnavanje tabele","Table cell text alignment":"Poravnaj tekst u tabeli","Table layout":"","Table properties":"Svojstva tabele","Table toolbar":"Tabela traka sa alatkama","Table type":"","Table type options":"",'The color is invalid. Try "#FF0000" or "rgb(255,0,0)" or "red".':'Boja je nevažeća. Pokušajte sa "# FF0000" ili "rgb (255,0,0)" ili "crvena".','The value is invalid. Try "10px" or "2em" or simply "2".':"Vrednost je nevažeća. Pokušajte sa „10pk“ ili „2em“ ili jednostavno „2“.","Vertical text alignment toolbar":"Vertikalna traka sa alatkama za poravnavanje teksta",Width:"Širina"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));