!function(t){const i=t.ru=t.ru||{};i.dictionary=Object.assign(i.dictionary||{},{Bold:"Жирный","Bold text":"Жирный текст",Code:"Исходный код",Italic:"Курсив","Italic text":"Текст курсивом","Move out of an inline code style":"Выйти из встроенного стиля кода",Strikethrough:"Зачеркнутый","Strikethrough text":"Зачеркнутый текст",Subscript:"Подстрочный",Superscript:"Надстрочный",Underline:"Подчеркнутый","Underline text":"Подчеркнутый текст"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));