<?php

/**
 * @file
 * Generic transliteration data for the PhpTransliteration class.
 */

$base = [
  0x00 => 'hy', 'jy', 'jy', 'khy', 'khy', 'sy', 'sy', 'dy', 'dy', 'shj', 'shh', 'shkh', 'shm', 'shr', 'sr', 'sr',
  0x10 => 'dr', 'ty', 'ty', '', '', 'ghy', 'ghy', 'sy', 'sy', 'shy', 'shy', 'hy', 'hy', 'jy', 'jy', 'khy',
  0x20 => 'khy', 'sy', 'sy', 'dy', 'dy', 'shj', 'shh', 'shkh', 'shm', 'shr', 'sr', 'sr', 'dr', 'shj', 'shh', 'shkh',
  0x30 => 'shm', 'sh', 'shh', 'tm', 'sj', 'sh', 'skh', 'shj', 'shh', 'shkh', 'tm', 'zm', '', '', '', '',
  0x40 => NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  0x50 => 'tjm', 'thj', 'thj', 'thm', 'tkhm', 'tmj', 'tmh', 'tmkh', 'jmh', 'jmh', 'hmy', 'hmy', 'shj', 'sjh', 'sjy', 'smh',
  0x60 => 'smh', 'smj', 'smm', 'smm', 'shh', 'shh', 'smm', 'shhm', 'shhm', 'shjy', 'shmkh', 'shmkh', 'shmm', 'shmm', 'dhy', 'dkhm',
  0x70 => 'dkhm', 'tmh', 'tmh', 'tmm', 'tmy', '', '', '', '', 'ghmm', 'ghmy', 'ghmy', 'fkhm', 'fkhm', 'qmh', 'qmm',
  0x80 => 'lhm', 'lhy', 'lhy', 'ljj', 'ljj', 'lkhm', 'lkhm', 'lmh', 'lmh', 'mhj', 'mhm', 'mhy', 'mjh', 'mjm', 'mkhj', 'mkhm',
  0x90 => NULL, NULL, 'mjkh', 'hmj', 'hmm', 'nhm', 'nhy', 'njm', 'njm', 'njy', 'nmy', 'nmy', 'ymm', 'ymm', 'bkhy', 'tjy',
  0xA0 => 'tjy', 'tkhy', 'tkhy', 'tmy', 'tmy', 'jmy', 'jhy', 'jmy', 'skhy', 'shy', 'shhy', 'dhy', 'ljy', 'lmy', 'yhy', 'yjy',
  0xB0 => 'ymy', 'mmy', 'qmy', 'nhy', 'qmh', 'lhm', '', 'kmy', 'njh', 'mkhy', 'ljm', 'kmm', 'ljm', 'njh', 'jhy', 'hjy',
  0xC0 => 'mjy', 'fmy', 'bhy', 'kmm', '', 'smm', 'skhy', 'njy', NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  0xD0 => NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  0xE0 => NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  0xF0 => '', '', 'allh', 'akbr', 'mhmd', '', 'rswl', '', 'wslm', 'sly', '', 'jl jlalh', 'ryal', NULL, NULL, NULL,
];
