!function(n){const e=n.th=n.th||{};e.dictionary=Object.assign(e.dictionary||{},{"Choose heading":"เลือกขนาดหัวข้อ",Heading:"หัวข้อ","Heading 1":"หัวข้อขนาด 1","Heading 2":"หัวข้อ 2","Heading 3":"หัวข้อ 3","Heading 4":"หัวข้อ 4","Heading 5":"หัวข้อ 5","Heading 6":"หัวข้อ 6","Type or paste your content here.":"พิมพ์หรือวางเนื้อหาของคุณที่นี่","Type your title":"พิมพ์ชื่อเรื่องของคุณ"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));