!function(e){const t=e.pl=e.pl||{};t.dictionary=Object.assign(t.dictionary||{},{Bold:"Pogrubienie","Bold text":"Pogrubia tekst",Code:"Kod",Italic:"Kursywa","Italic text":"Zmienia tekst na kursywę","Move out of an inline code style":"Przenosi zaznaczenie poza styl kodu inline",Strikethrough:"Przekreślenie","Strikethrough text":"Przekreśla tekst",Subscript:"Indeks dolny",Superscript:"Indeks górny",Underline:"Podkreślenie","Underline text":"Podkreśla tekst"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));