/**
 * @file
 * Module styling for user module.
 */
.password-strength__title,
.password-strength__text {
  display: inline;
}
.password-strength__meter {
  height: 0.75em;
  margin-top: 0.5em;
  background-color: lightgray;
}
.password-strength__indicator {
  width: 0;
  height: 100%;
  background-color: gray;
}
.password-confirm-match {
  visibility: hidden;
}
