!function(e){const i=e.ru=e.ru||{};i.dictionary=Object.assign(i.dictionary||{},{"Blue marker":"Выделение синим маркером","Green marker":"Выделение зелёным маркером","Green pen":"Зеленый цвет текста",Highlight:"Выделить","Pink marker":"Выделение розовым маркером","Red pen":"Красный цвет текста","Remove highlight":"Убрать выделение","Text highlight toolbar":"Панель инструментов выделения текста","Yellow marker":"Выделение жёлтым маркером"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));