/*
 * DO NOT EDIT THIS FILE.
 * See the following change record for more information,
 * https://www.drupal.org/node/3084859
 * @preserve
 */

.announcements ul {
  margin-inline-start: 0;
  list-style: none;
}

.announcement:not(.announcement:last-child) {
  margin-block-end: 1rem;
}

.announcement.announcement--featured + .announcement.announcement--standard {
  padding-block-start: 1rem;
  border-top: 1px solid #aaa;
}

.announcements--view-all {
  margin-block-start: 3rem;
}
