!function(e){const n=e.be=e.be||{};n.dictionary=Object.assign(n.dictionary||{},{"Choose heading":"Выбар стылю",Heading:"Стыль","Heading 1":"Загаловак 1","Heading 2":"Загаловак 2","Heading 3":"Загаловак 3","Heading 4":"Загаловак 4","Heading 5":"Загаловак 5","Heading 6":"Загаловак 6","Type or paste your content here.":"Увядзіце або ўстаўце ваш тэкст","Type your title":"Увядзіце загаловак"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));