<?php

/**
 * @file
 */

@trigger_error(__FILE__ . ' is deprecated in drupal:11.3.0 and is removed from drupal:12.0.0. Functions have been deprecated and moved to system.module. See https://www.drupal.org/node/3538089', E_USER_DEPRECATED);
