!function(e){const t=e.cs=e.cs||{};t.dictionary=Object.assign(t.dictionary||{},{"Insert media":"Vložit média",Media:"Média","Media embed":"Vložení médií","Media toolbar":"Panel nástrojů médií","Media URL":"URL adresa","media widget":"ovládací prvek médií","Open media in new tab":"Otevřete média na nové kartě","Paste the media URL in the input.":"Vložte URL média do vstupního pole.","The URL must not be empty.":"URL adresa musí být vyplněna.","This media URL is not supported.":"Tato adresa bohužel není podporována.","Tip: Paste the URL into the content to embed faster.":"Rada: Vložte URL přímo do editoru pro rychlejší vnoření."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));