!function(e){const t=e.be=e.be||{};t.dictionary=Object.assign(t.dictionary||{},{"Insert media":"Уставіць медыя",Media:"Медыя","Media embed":"","Media toolbar":"Панель інструментаў медыя","Media URL":"URL медыя","media widget":"медыя-віджэт","Open media in new tab":"Адкрыць медыя ў новай картцы","Paste the media URL in the input.":"Уставіць URL медыя ў поле ўводу.","The URL must not be empty.":"URL не павінен быць пустым.","This media URL is not supported.":"Гэты медыя URL не падтрымліваецца.","Tip: Paste the URL into the content to embed faster.":"Падказка: Уставіць URL у кантэнт для ўключэння хутчэй."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));