<?php

namespace Drupal\views\Annotation;

/**
 * Defines a Plugin annotation object for views join plugins.
 *
 * @see \Drupal\views\Plugin\views\join\JoinPluginBase
 *
 * @ingroup views_join_handlers
 *
 * @Annotation
 */
class ViewsJoin extends ViewsHandlerAnnotationBase {

}
