!function(a){const o=a.et=a.et||{};o.dictionary=Object.assign(o.dictionary||{},{Bookmark:"Järjehoidja","Bookmark must not be empty.":"Järjehoidja väli ei tohi olla tühi.","Bookmark name":"Järjehoidja nimi","Bookmark name already exists.":"Järjehoidja nimi on juba olemas.","Bookmark name cannot contain space characters.":"Järjehoidja nimi ei tohi sisaldada tühikuid.","Bookmark toolbar":"Järjehoidjariba","bookmark widget":"järjehoidja vidin",Bookmarks:"Järjehoidjad","Edit bookmark":"Muuda järjehoidjat","Enter the bookmark name without spaces.":"Sisestage järjehoidja nimi ilma tühikuteta.","No bookmarks available.":"Ühtegi järjehoidjat pole saadaval.","Remove bookmark":"Eemalda järjehoidja","Scroll to bookmark":"Keri järjehoidjani"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));