<?php

/**
 * @file
 * Generic transliteration data for the PhpTransliteration class.
 */

$base = [
  0x00 => 'bing', 'zhi', 'dong', 'dui', 'zhou', 'nei', 'lin', 'po', 'ji', 'min', 'wei', 'che', 'gou', 'bang', 'ru', 'tan',
  0x10 => 'bu', 'zong', 'kui', 'lao', 'han', 'ying', 'zhi', 'jie', 'xing', 'xie', 'xun', 'shan', 'qian', 'xie', 'su', 'hai',
  0x20 => 'mi', 'hun', 'pi', NULL, 'hui', 'na', 'song', 'ben', 'chou', 'jie', 'huang', 'lan', NULL, 'hu', 'dou', 'huo',
  0x30 => 'gun', 'yao', 'ce', 'gui', 'jian', 'jian', 'dao', 'jin', 'ma', 'hui', 'mian', 'can', 'lue', 'pi', 'yang', 'ju',
  0x40 => 'ju', 'que', NULL, 'qian', 'shai', NULL, 'jiu', 'huo', 'yun', 'da', 'xuan', 'xiao', 'fei', 'ce', 'ye', NULL,
  0x50 => 'den', NULL, 'qin', 'hui', 'tun', NULL, 'qiang', 'xi', 'ni', 'sai', 'meng', 'tuan', 'lan', 'hao', 'ci', 'zhai',
  0x60 => 'ao', 'luo', 'mie', NULL, 'fu', NULL, 'xie', 'bo', 'hui', 'qing', 'xie', NULL, NULL, 'bo', 'qian', 'po',
  0x70 => 'jiao', 'jue', 'kun', 'song', 'ju', 'e', 'nie', 'qian', 'die', 'die', NULL, 'qi', 'zhi', 'qi', 'zhui', 'ku',
  0x80 => 'yu', 'qin', 'ku', 'he', 'fu', NULL, 'di', 'xian', 'gui', 'he', 'qun', 'han', 'tong', 'bo', 'shan', 'bi',
  0x90 => 'lu', 'ye', 'ni', 'chuai', 'san', 'diao', 'lu', 'tou', 'lian', 'ke', 'san', 'zhen', 'chuai', 'lian', 'mao', NULL,
  0xA0 => 'qian', 'kai', 'shao', 'xiao', 'bi', 'zha', 'yin', 'xi', 'shan', 'su', 'sa', 'rui', 'chuo', 'lu', 'ling', 'cha',
  0xB0 => NULL, 'huan', NULL, NULL, 'jia', 'ban', 'hu', 'dou', NULL, 'lou', 'ju', 'juan', 'ke', 'suo', 'luo', 'zhe',
  0xC0 => 'ding', 'duan', 'zhu', 'yan', 'pang', 'cha', NULL, NULL, NULL, NULL, 'yi', NULL, NULL, 'you', 'hui', 'yao',
  0xD0 => 'yao', 'zhi', 'gong', 'qi', 'gen', NULL, NULL, 'hou', 'mi', 'fu', 'hu', 'guang', 'tan', 'di', NULL, 'yan',
  0xE0 => NULL, NULL, 'qu', NULL, 'chang', 'ming', 'tao', 'bao', 'an', NULL, NULL, 'xian', NULL, NULL, NULL, 'mao',
  0xF0 => 'lang', 'nan', 'bei', 'chen', NULL, 'fei', 'zhou', 'ji', 'jie', 'shu', NULL, 'kun', 'die', 'lu', NULL, NULL,
];
