!function(a){const n=a.uz=a.uz||{};n.dictionary=Object.assign(n.dictionary||{},{"Create link":"","Displayed text":"",Downloadable:"Yuklab olinadigan","Edit link":"Havolani tahrirlash",Link:"Havola","Link image":"Rasmga havola","Link properties":"","Link URL":'"Havola URL',"Link URL must not be empty.":"","Move out of a link":"","No links available":"","Open in a new tab":"Yangi oynada oching","Open link in new tab":"Havolani yangi oynada ochish",Unlink:"Havolani olib tashlash"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));