!function(a){const e=a.sk=a.sk||{};e.dictionary=Object.assign(e.dictionary||{},{"Break text":"Zalomenie textu","Caption for image: %0":"Popis k obrázku: %0","Caption for the image":"Popis k obrázku","Centered image":"Zarovnať na stred","Change image text alternative":"Zmeňte alternatívny text obrázka",Custom:"Vlastné","Custom image size":"Vlastná veľkosť obrázka","Enter image caption":"Vložte popis obrázka","Error during image upload":"Chyba pri nahrávaní obrázka","From computer":"Z počítača","Full size image":"Obrázok v plnej veľkosti",Image:"Obrázok","Image from computer":"Obrázok z počítača","Image Resize":"Zmena veľkosti obrázka","Image resize list":"Zoznam možností zmeny veľkosti","Image toolbar":"Panel nástrojov obrázka","Image upload complete":"Nahrávanie obrázka bolo dokončené","Image via URL":"Obrázok cez URL","image widget":"widget obrázka","In line":"V riadku","Insert image":"Vložiť obrázok","Insert image via URL":"Vložiť obrázok pomocou URL","Insert via URL":"Vložiť cez URL","Left aligned image":"Zarovnať vľavo",Original:"Originál","Replace from computer":"Nahradiť z počítača","Replace image":"Nahradiť obrázok","Replace image from computer":"Nahradiť obrázok z počítača","Resize image":"Zmeniť veľkosť obrázka","Resize image (in %0)":"Zmeniť veľkosť obrázka (v %0)","Resize image to %0":"Zmeniť veľkosť na %0","Resize image to the original size":"Zmeniť veľkosť na pôvodnú","Right aligned image":"Zarovnať vpravo","Side image":"Bočný obrázok","Text alternative":"Alternatívny text","Text Alternative":"Alternatíva textu","The value must not be empty.":"Hodnota nesmie byť prázdna.","The value should be a plain number.":"Hodnota by mala byť obyčajné číslo.","Update image URL":"Aktualizovať URL obrázka","Upload failed":"Nahrávanie zlyhalo","Upload from computer":"Nahrať z počítača","Upload image from computer":"Nahrať obrázok z počítača","Uploading image":"Nahrávanie obrázka","Via URL":"Cez URL","Wrap text":"Obtekanie textu","You have no image upload permissions.":"Nemáte žiadne povolenia na nahrávanie obrázkov."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));