!function(i){const e=i["pt-br"]=i["pt-br"]||{};e.dictionary=Object.assign(e.dictionary||{},{"Could not insert image at the current position.":"Não foi possível inserir a imagem na posição atual","Could not obtain resized image URL.":"Não foi possível obter o endereço da imagem redimensionada","Insert image or file":"Inserir imagem ou arquivo","Inserting image failed":"Falha ao inserir imagem","Selecting resized image failed":"Seleção da imagem redimensionada falhou"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));