!function(t){const i=t["pt-br"]=t["pt-br"]||{};i.dictionary=Object.assign(i.dictionary||{},{Bold:"Negrito","Bold text":"Texto em negrito",Code:"Código",Italic:"Itálico","Italic text":"Texto em itálico","Move out of an inline code style":"Sair de um estilo de código inline",Strikethrough:"Tachado","Strikethrough text":"Texto com riscado",Subscript:"Subscrito",Superscript:"Sobrescrito",Underline:"Sublinhado","Underline text":"Texto sublinhado"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));