!function(n){const i=n.vi=n.vi||{};i.dictionary=Object.assign(i.dictionary||{},{"Blue marker":"Bút xanh dương","Green marker":"Bút xanh lá","Green pen":"Mực xanh",Highlight:"Làm nổi","Pink marker":"Bút hồng","Red pen":"Mực đỏ","Remove highlight":"Xóa làm nổi","Text highlight toolbar":"Thanh công cụ làm nổi chữ","Yellow marker":"Bút vàng"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));