!function(e){const t=e.sv=e.sv||{};t.dictionary=Object.assign(t.dictionary||{},{"Align cell text to the bottom":"Bottenjustera celltext","Align cell text to the center":"Centrera celltext","Align cell text to the left":"Vänsterjustera celltext","Align cell text to the middle":"Centrera celltext","Align cell text to the right":"Högerjustera celltext","Align cell text to the top":"Toppjustera celltext","Align table to the left":"Vänsterjustera tabell","Align table to the left with no text wrapping":"Justera tabellen till vänster utan textomslag","Align table to the left with text wrapping":"Justera tabellen till vänster med textomslag","Align table to the right":"Högerjustera tabell","Align table to the right with no text wrapping":"Justera tabellen till höger utan textomslag","Align table to the right with text wrapping":"Justera tabellen till höger med textomslag",Alignment:"Justering",Background:"Bakgrund",Border:"Kant","Cell properties":"Cellegenskaper","Center table":"Centrera tabell","Center table with no text wrapping":"Centrerad tabell utan textomslag","Choose table type":"Välj tabelltyp",Color:"Färg",Column:"Kolumn","Content table":"Innehållstabell",Dashed:"Streckad","Delete column":"Ta bort kolumn","Delete row":"Ta bort rad",Dimensions:"Mått",Dotted:"Prickig",Double:"Dubbel","Enter table caption":"Ange tabellrubrik",Groove:"Skåra","Header column":"Rubrikkolumn","Header row":"Rubrikrad",Height:"Höjd","Horizontal text alignment toolbar":"Verktygsfält för horisontell textjustering","Insert a new table row (when in the last cell of a table)":"Infoga en ny tabellrad (när du är på den sista cellen i en tabell)","Insert column left":"Infoga kolumn till vänster","Insert column right":"Infoga kolumn till höger","Insert row above":"Infoga rad ovanför","Insert row below":"Infoga rad nedanför","Insert table":"Lägg in tabell","Insert table layout":"Infoga tabellayout",Inset:"Infälld","Justify cell text":"Anpassa celltext","Keystrokes that can be used in a table cell":"Tangenter som fungerar i en tabellcell","Layout table":"Layouttabell","Merge cell down":"Sammanfoga celler neråt","Merge cell left":"Sammanfoga celler åt vänster","Merge cell right":"Sammanfoga celler åt höger","Merge cell up":"Sammanfoga celler uppåt","Merge cells":"Sammanfoga celler","Move the selection to the next cell":"Flytta markeringen till nästa cell","Move the selection to the previous cell":"Flytta markeringen till föregående cell","Navigate through the table":"Navigera i tabellen",None:"Inget",Outset:"Utfälld",Padding:"Marginal",Ridge:"Kant",Row:"Rad","Select column":"Välj kolumn","Select row":"Välj rad",Solid:"Enfärgad","Split cell horizontally":"Dela cell horisontellt","Split cell vertically":"Dela cell vertikalt",Style:"Stil",Table:"Tabell","Table Alignment":"Tabelljustering","Table alignment toolbar":"Verktygsfält för tabelljustering","Table cell text alignment":"Textjustering i tabellcell","Table layout":"Tabellayout","Table properties":"Tabellegenskaper","Table toolbar":"Tabellverktygsfält","Table type":"Tabelltyp","Table type options":"Alternativ för tabelltyp",'The color is invalid. Try "#FF0000" or "rgb(255,0,0)" or "red".':'Färgen är ogiltig. Testa "#FF0000" eller "rgb(255,0,0)" eller "röd".','The value is invalid. Try "10px" or "2em" or simply "2".':'Värdet är ogiltigt. Testa "10px" eller "2em" eller helt enkelt "2".',"Vertical text alignment toolbar":"Verktygsfält för vertikal textjustering",Width:"Bredd"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));