---
label: Reordering and disabling menu links
related:
  - menu_ui.menu_item_add
  - menu_ui.menu_operations
  - core.menus
---
{% set structure_menu_text %}{% trans %}Menus{% endtrans %}{% endset %}
{% set structure_menu_link = render_var(help_route_link(structure_menu_text, 'entity.menu.collection')) %}
<h2>{% trans %}Goal{% endtrans %}</h2>
<p>{% trans %}Disable menu links or change the order and hierarchy of menu links.{% endtrans %}</p>
<h2>{% trans %}Steps{% endtrans %}</h2>
<ol>
  <li>{% trans %}In the <em>Manage</em> administration menu, navigate to <em>Structure</em> &gt; {{ structure_menu_link }}.{% endtrans %}</li>
  <li>{% trans %}Click <em>Edit menu</em> for the menu that you want to edit.{% endtrans %}</li>
  <li>{% trans %}Drag menu links into a new order, or check/uncheck <em>Enabled</em> to enable or disable menu links.{% endtrans %}</li>
  <li>{% trans %}Click <em>Save</em> to save your changes.{% endtrans %}</li>
</ol>
