{#
/**
 * @file
 * Default theme implementation for authorize.php operation report templates.
 *
 * This report displays the results of an operation run via authorize.php.
 *
 * Available variables:
 * - messages: A list of result messages.
 * - attributes: HTML attributes for the element.
 *
 * @see template_preprocess_authorize_report()
 *
 * @ingroup themeable
 *
 * @deprecated in drupal:11.2.0 and is removed from drupal:12.0.0. There is no
 *   replacement. Use composer to manage the code for your site.
 *
 * @see https://www.drupal.org/node/3522119
 */
#}
{% if messages %}
  <div{{ attributes.addClass('authorize-results') }}>
    {% for message_group in messages %}
      {{ message_group }}
    {% endfor %}
  </div>
{% endif %}
