!function(n){const i=n["zh-cn"]=n["zh-cn"]||{};i.dictionary=Object.assign(i.dictionary||{},{"Create link":"创建链接","Displayed text":"显示的文本",Downloadable:"可下载","Edit link":"修改链接",Link:"超链接","Link image":"链接图片","Link properties":"链接属性","Link URL":"链接网址","Link URL must not be empty.":"链接 URL 不能为空。","Move out of a link":"移出链接","No links available":"无可用链接","Open in a new tab":"在新标签页中打开","Open link in new tab":"在新标签页中打开链接",Unlink:"取消超链接"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));