!function(e){const n=e["de-ch"]=e["de-ch"]||{};n.dictionary=Object.assign(n.dictionary||{},{"Edit source":"Quelle bearbeiten","Empty snippet content":"Codeschnipsel ohne Inhalt","HTML snippet":"HTML-Codeschnipsel","Insert HTML":"HTML einfügen","No preview available":"Keine Vorschau verfügbar","Paste raw HTML here...":"HTML-Code hier einfügen…","Save changes":"Änderungen speichern"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));