!function(e){const a=e.hu=e.hu||{};a.dictionary=Object.assign(a.dictionary||{},{"Insert media":"Média beszúrása",Media:"Média","Media embed":"Média beágyazása","Media toolbar":"Média eszköztár","Media URL":"Média URL","media widget":"Média widget","Open media in new tab":"Nyissa meg a médiát új lapon","Paste the media URL in the input.":"Illessze be a média URL-jét.","The URL must not be empty.":"Az URL nem lehet üres.","This media URL is not supported.":"Ez a média URL típus nem támogatott.","Tip: Paste the URL into the content to embed faster.":"Tipp: Illessze be a média URL-jét a tartalomba."})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));