!function(t){const i=t.it=t.it||{};i.dictionary=Object.assign(i.dictionary||{},{"Advanced options":"Opzioni avanzate",Find:"Trova","Find and replace":"Trova e sostituisci","Find in text…":"Trova nel testo…","Find in the document":"Trova nel documento","Match case":"Distingui maiuscole e minuscole","Next result":"Risultato successivo","Previous result":"Risultato precedente",Replace:"Sostituisci","Replace all":"Sostituisci tutto","Replace with…":"Sostituisci con…","Text to find must not be empty.":"Il testo da cercare non può essere vuoto.","Tip: Find some text first in order to replace it.":"Consiglio: trova il testo prima di sostituirlo.","Whole words only":"Solo parole intere"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));