/**
 * @file
 * Admin styles for the Database Logging module.
 */
.dblog-filter-form .form-item-type,
.dblog-filter-form .form-item-severity {
  display: inline-block;
  max-width: 30%;
  margin: 0.1em 0.9em 0.1em 0.1em; /* LTR */
}
[dir="rtl"] .dblog-filter-form .form-item-type,
[dir="rtl"] .dblog-filter-form .form-item-severity {
  margin: 0.1em 0.1em 0.1em 0.9em;
}
.dblog-filter-form .form-actions {
  display: inline-block;
  padding: 3ex 0 0;
  vertical-align: top;
}
.admin-dblog .icon,
.admin-dblog .dblog-warning .icon,
.admin-dblog .dblog-error .icon,
.admin-dblog .dblog-critical .icon,
.admin-dblog .dblog-alert .icon,
.admin-dblog .dblog-emergency .icon {
  width: 16px;
  background: no-repeat center;
}
.admin-dblog .dblog-warning .icon {
  background-image: url(../../../misc/icons/e29700/warning.svg);
}
.admin-dblog .dblog-error .icon,
.admin-dblog .dblog-critical .icon,
.admin-dblog .dblog-alert .icon,
.admin-dblog .dblog-emergency .icon {
  background-image: url(../../../misc/icons/e32700/error.svg);
}
