!function(i){const e=i.ar=i.ar||{};e.dictionary=Object.assign(e.dictionary||{},{"Disable block editing":"تعطيل تحرير الكتل","Disable editing":"تعطيل التحرير","Disable inline editing":"تعطيل التحرير المضمن","Enable block editing":"تفعيل تحرير الكتل","Enable editing":"تمكين التحرير","Enable inline editing":"تفعيل التحرير المضمن","Navigate editable regions":"التنقل في مناطق التحرير","Next editable region":"منطقة التحرير التالية","Previous editable region":"منطقة التحرير السابقة"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));