!function(n){const i=n.sv=n.sv||{};i.dictionary=Object.assign(i.dictionary||{},{"Create link":"Skapa länk","Displayed text":"Visad text",Downloadable:"Nedladdningsbar","Edit link":"Redigera länk",Link:"Länk","Link image":"Länka bild","Link properties":"Länkegenskaper","Link URL":"Länkens URL","Link URL must not be empty.":"Länkens URL får inte vara tom.","Move out of a link":"Flytta bort från länken","No links available":"Inga länkar tillgängliga","Open in a new tab":"Öppna i en ny flik","Open link in new tab":"Öppna länk i ny flik",Unlink:"Ta bort länk"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));