!function(i){const e=i.lt=i.lt||{};e.dictionary=Object.assign(e.dictionary||{},{"Could not insert image at the current position.":"Nepavyko įterpti vaizdo į dabartinę vietą.","Could not obtain resized image URL.":"Nepavyko gauti pakeisto dydžio paveiksliuko URL.","Insert image or file":"Įterpti vaizdą ar failą","Inserting image failed":"Nepavyko įterpti vaizdo","Selecting resized image failed":"Nepavyko pasirinkti pakeisto vaizdo"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));