!function(t){const i=t.ar=t.ar||{};i.dictionary=Object.assign(i.dictionary||{},{Bold:"عريض","Bold text":"نص غامق",Code:"شفرة برمجية",Italic:"مائل","Italic text":"نص مائل","Move out of an inline code style":"ابتعدْ عن نمط التعليمات البرمجية المضمّنة",Strikethrough:"يتوسطه خط","Strikethrough text":"توسيط الخط في النص",Subscript:"حرف منخفض",Superscript:"حرف مرتفع",Underline:"تحته خط","Underline text":"تسطير النص"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));