!function(e){const t=e.ru=e.ru||{};t.dictionary=Object.assign(t.dictionary||{},{"Bulleted List":"Маркированный список","Bulleted list styles toolbar":"Стили маркированного списка",Circle:"Окружность",Decimal:"Десятичный","Decimal with leading zero":"Десятичный с ведущим нулем","Decrease list item indent":"Уменьшить отступ элемента списка",Disc:"Диск","Entering a to-do list":"Ввод списка текущих дел","Increase list item indent":"Увеличить отступ элемента списка","Invalid start index value.":"Недопустимое значение начального индекса.","Keystrokes that can be used in a list":"Нажатия клавиш, которые можно использовать в списке","Leaving a to-do list":"Оставляем список текущих дел","List properties":"Свойства списка","Lower-latin":"Малые латинские","Lower–roman":"Малые римские","Numbered List":"Нумерованный список","Numbered list styles toolbar":"Стили нумерованного списка","Reversed order":"Обратный порядок",Square:"Квадрат","Start at":"Начать с","Start index must be greater than 0.":"Начальный индекс должен быть больше 0.","To-do List":"Список задач","Toggle the circle list style":"Переключить на стиль списка с маркерами в виде незакрашенного кружка","Toggle the decimal list style":"Переключить на стиль списка с цифровыми маркерами","Toggle the decimal with leading zero list style":"Переключить на стиль списка с маркерами в виде цифр с нулем в начале","Toggle the disc list style":"Переключить на стиль списка с маркерами в виде закрашенного кружка","Toggle the lower–latin list style":"Переключить на стиль списка с маркерами в виде строчных латинских букв","Toggle the lower–roman list style":"Переключить на стиль списка с маркерами в виде строчных римских цифр","Toggle the square list style":"Переключить на стиль списка с квадратными маркерами","Toggle the upper–latin list style":"Переключить на стиль списка с маркерами в виде заглавных латинских букв","Toggle the upper–roman list style":"Переключить на стиль списка с маркерами в виде заглавных римских цифр","Upper-latin":"Большие латинские","Upper-roman":"Большие римские"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));