!function(e){const n=e.bg=e.bg||{};n.dictionary=Object.assign(n.dictionary||{},{"Edit source":"Редактирай източник","Empty snippet content":"Изпразни съдържанието на съкращението","HTML snippet":"съкратен HTML","Insert HTML":"Въведи HTML","No preview available":"Не е наличен преглед","Paste raw HTML here...":"Постави суров HTML тук...","Save changes":"Запази промените"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));