!function(o){const t=o.ca=o.ca||{};t.dictionary=Object.assign(t.dictionary||{},{Big:"Gran",Default:"Predeterminada","Document colors":"Colors del document","Font Background Color":"Color de fons del tipus de lletra","Font Color":"Color del tipus de lletra","Font Family":"Font","Font Size":"Mida de la font",Huge:"Molt gran",Small:"Peita",Tiny:"Molt petita"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));