!function(r){const a=r.ca=r.ca||{};a.dictionary=Object.assign(a.dictionary||{},{"Blue marker":"Marcador blau","Green marker":"Marcador verd","Green pen":"Bolígraf verd",Highlight:"Destacat","Pink marker":"Marcador rosa","Red pen":"Marcador vermell","Remove highlight":"Esborrar destacat","Text highlight toolbar":"Barra d'eines de ressaltat de text","Yellow marker":"Marcador groc"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));