!function(i){const e=i.uk=i.uk||{};e.dictionary=Object.assign(e.dictionary||{},{"Could not insert image at the current position.":"Не можливо вставити зображення в поточну позицію.","Could not obtain resized image URL.":"Не вдалось отримати URL зміненого зображення.","Insert image or file":"Вставте зображення або файл","Inserting image failed":"Не вдалось вставити зображення","Selecting resized image failed":"Не вдалося вибрати зображення зі зміненим розміром"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));