!function(a){const i=a.gl=a.gl||{};i.dictionary=Object.assign(i.dictionary||{},{"Characters: %0":"Caracteres: %0","Words: %0":"Palabras: %0"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));