<?php

declare(strict_types=1);

namespace Drupal\plugin_test\Plugin\plugin_test\fruit;

/**
 * @Plugin(
 *   id = "cherry",
 *   label = "Cherry",
 *   color = "red"
 * )
 */
class Cherry implements FruitInterface {}
