!function(r){const a=r.gl=r.gl||{};a.dictionary=Object.assign(a.dictionary||{},{"Blue marker":"Marcador azul","Green marker":"Marcador verde","Green pen":"Pluma verde",Highlight:"Resaltado","Pink marker":"Marcador rosa","Red pen":"Pluma vermella","Remove highlight":"Retirar o resaltado","Text highlight toolbar":"Barra de ferramentas para resaltar texto","Yellow marker":"Marcador marelo"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));