!function(i){const n=i.az=i.az||{};n.dictionary=Object.assign(n.dictionary||{},{"Create link":"","Displayed text":"",Downloadable:"Yüklənə bilər","Edit link":"Linki redaktə et",Link:"Əlaqələndir","Link image":"","Link properties":"","Link URL":"Linkin URL","Link URL must not be empty.":"","Move out of a link":"","No links available":"","Open in a new tab":"Yeni pəncərədə aç","Open link in new tab":"Linki yeni pəncərədə aç",Unlink:"Linki sil"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));