!function(e){const t=e["zh-cn"]=e["zh-cn"]||{};t.dictionary=Object.assign(t.dictionary||{},{"Align cell text to the bottom":"使单元格文本对齐到底部","Align cell text to the center":"使单元格文本水平居中","Align cell text to the left":"使单元格文本左对齐","Align cell text to the middle":"使单元格文本垂直居中","Align cell text to the right":"使单元格文本右对齐","Align cell text to the top":"使单元格文本对齐到顶部","Align table to the left":"使表格左对齐","Align table to the left with no text wrapping":"表格左对齐，无文字环绕","Align table to the left with text wrapping":"表格左对齐，文字环绕","Align table to the right":"使表格右对齐","Align table to the right with no text wrapping":"表格右对齐，无文字环绕","Align table to the right with text wrapping":"表格右对齐，文字环绕",Alignment:"对齐",Background:"背景",Border:"边框","Cell properties":"单元格属性","Center table":"表格居中","Center table with no text wrapping":"表格居中，无文字环绕","Choose table type":"选择表格类型",Color:"颜色",Column:"列","Content table":"内容表格",Dashed:"虚线","Delete column":"删除本列","Delete row":"删除本行",Dimensions:"尺寸",Dotted:"点状虚线",Double:"双线","Enter table caption":"输入表标题",Groove:"凹槽边框","Header column":"标题列","Header row":"标题行",Height:"高度","Horizontal text alignment toolbar":"水平文本对齐工具栏","Insert a new table row (when in the last cell of a table)":"插入新的表格行（当位于表格的最后一个单元格时）","Insert column left":"左侧插入列","Insert column right":"右侧插入列","Insert row above":"在上面插入一行","Insert row below":"在下面插入一行","Insert table":"插入表格","Insert table layout":"插入表格布局",Inset:"凹边框","Justify cell text":"对齐单元格文本","Keystrokes that can be used in a table cell":"可在表格单元格中使用的按键","Layout table":"布局表格","Merge cell down":"向下合并单元格","Merge cell left":"向左合并单元格","Merge cell right":"向右合并单元格","Merge cell up":"向上合并单元格","Merge cells":"合并单元格","Move the selection to the next cell":"将所选内容移动到下一个单元格","Move the selection to the previous cell":"将所选内容移至上一个单元格","Navigate through the table":"在表格中进行导览",None:"无",Outset:"凸边框",Padding:"内边距",Ridge:"垄状边框",Row:"行","Select column":"选择列","Select row":"选择行",Solid:"实线","Split cell horizontally":"横向拆分单元格","Split cell vertically":"纵向拆分单元格",Style:"样式",Table:"表格","Table Alignment":"表格对齐方式","Table alignment toolbar":"表格对齐工具栏","Table cell text alignment":"表格单元格中的文本水平对齐","Table layout":"表格布局","Table properties":"表格属性","Table toolbar":"表格工具栏","Table type":"表格类型","Table type options":"表格类型选项",'The color is invalid. Try "#FF0000" or "rgb(255,0,0)" or "red".':'颜色无效。尝试使用"#FF0000"、"rgb(255,0,0)"或者"red"。','The value is invalid. Try "10px" or "2em" or simply "2".':"无效值。尝试使用“10px”、“2ex”或者只写“2”。","Vertical text alignment toolbar":"垂直文本对齐工具栏",Width:"宽度"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));